/******************************************************************************
*
*   File Name:  bridge_top.v
*     Version:  1.0
*        Date:  June 10, 2007
*
* Description:  This file is the top level wrapper for 4 Channel
*               SPI4.2 to Quad SPI3 bridge 
*
*      Author:  Zhe Xia
*     Company:  Xilinx Inc.
*
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*
*     (c) Copyright 2002 Xilinx, Inc.
*     All rights reserved.
*
*******************************************************************************/



module bridge_top (
    Reset_n,
	 Ref_Clk,
    SPI4_RDClk_P,
    SPI4_RDClk_N,
    SPI4_RCtl_P,
    SPI4_RCtl_N,
    SPI4_RDat_P,
    SPI4_RDat_N,
    SPI4_RSClk,
    SPI4_RStat,
    SPI4_TDClk_P,
    SPI4_TDClk_N,
    SPI4_TCtl_P,
    SPI4_TCtl_N,
    SPI4_TDat_P,
    SPI4_TDat_N,
    SPI4_SysClk_P,
    SPI4_SysClk_N,
    SPI4_TSClk,
    SPI4_TStat,
    SPI3_RFClk,
    SPI3_0_RVal,
    SPI3_0_REnb,
    SPI3_0_RSx,
    SPI3_0_RSOP,
    SPI3_0_REOP,
    SPI3_0_RDat,
    SPI3_0_RPrty,
    SPI3_0_RMod,
    SPI3_0_RErr,
    SPI3_TFClk,
    SPI3_0_TEnb,
    SPI3_0_TSx,
    SPI3_0_TSOP,
    SPI3_0_TEOP,
    SPI3_0_TDat,
    SPI3_0_TPrty,
    SPI3_0_TMod,
    SPI3_0_TErr,
    SPI3_0_PTPA,
    SPI3_0_TAdr,
    SPI3_1_RVal,
    SPI3_1_REnb,
    SPI3_1_RSx,
    SPI3_1_RSOP,
    SPI3_1_REOP,
    SPI3_1_RDat,
    SPI3_1_RPrty,
    SPI3_1_RMod,
    SPI3_1_RErr,
    SPI3_1_TEnb,
    SPI3_1_TSx,
    SPI3_1_TSOP,
    SPI3_1_TEOP,
    SPI3_1_TDat,
    SPI3_1_TPrty,
    SPI3_1_TMod,
    SPI3_1_TErr,
    SPI3_1_PTPA,
    SPI3_1_TAdr,
    SPI3_2_RVal,
    SPI3_2_REnb,
    SPI3_2_RSx,
    SPI3_2_RSOP,
    SPI3_2_REOP,
    SPI3_2_RDat,
    SPI3_2_RPrty,
    SPI3_2_RMod,
    SPI3_2_RErr,
    SPI3_2_TEnb,
    SPI3_2_TSx,
    SPI3_2_TSOP,
    SPI3_2_TEOP,
    SPI3_2_TDat,
    SPI3_2_TPrty,
    SPI3_2_TMod,
    SPI3_2_TErr,
    SPI3_2_PTPA,
    SPI3_2_TAdr,
    SPI3_3_RVal,
    SPI3_3_REnb,
    SPI3_3_RSx,
    SPI3_3_RSOP,
    SPI3_3_REOP,
    SPI3_3_RDat,
    SPI3_3_RPrty,
    SPI3_3_RMod,
    SPI3_3_RErr,
    SPI3_3_TEnb,
    SPI3_3_TSx,
    SPI3_3_TSOP,
    SPI3_3_TEOP,
    SPI3_3_TDat,
    SPI3_3_TPrty,
    SPI3_3_TMod,
    SPI3_3_TErr,
    SPI3_3_PTPA,
    SPI3_3_TAdr,
    SPI3_ParityErr
    )/*synthesis syn_edif_bit_format="%n(%i)" */;


    //System signals
    input					Reset_n;

    // SPI4.2 Sink signals, see SPI4.2 core datasheet for details
	 input					 Ref_Clk;
    input				    SPI4_RDClk_P;
    input				    SPI4_RDClk_N;
    input				    SPI4_RCtl_P;
    input				    SPI4_RCtl_N;
    input				    [15:0] SPI4_RDat_P;
    input				    [15:0] SPI4_RDat_N;

    // SPI4.2 Sink Status (LVTTL) input,  see SPI4.2 core datasheet for details
    output				    SPI4_RSClk;
    output				    [1:0]  SPI4_RStat;

    // SPI4.2 Source signals,  see SPI4.2 core datasheet for details
    output				    SPI4_TDClk_P;
    output				    SPI4_TDClk_N;
    output				    SPI4_TCtl_P;
    output				    SPI4_TCtl_N;
    output				    [15:0] SPI4_TDat_P;
    output				    [15:0] SPI4_TDat_N;
    input				    SPI4_SysClk_P;
    input				    SPI4_SysClk_N;
    
    // SPI4.2 Source status(LVTTL) input,  see SPI4.2 core datasheet for details
    input				    SPI4_TSClk;
    input				    [1:0]  SPI4_TStat;

    //SPI3 Clocks - All four SPI3 interfaces share common Rx, Tx Clocks.
    input				    SPI3_RFClk;

    //SPI3 0 Sink Interface,  see SPI3 core datasheet for details
    input				    SPI3_0_RVal;
    output				    SPI3_0_REnb;
    input				    SPI3_0_RSx;
    input				    SPI3_0_RSOP;
    input				    SPI3_0_REOP;
    input				    [31:0] SPI3_0_RDat;
    input				    SPI3_0_RPrty;
    input				    [1:0]  SPI3_0_RMod;
    input				    SPI3_0_RErr;

    //SPi3 0 Source Interface,  see SPI3 core datasheet for details
    input				    SPI3_TFClk;
    output				    SPI3_0_TEnb;
    output				    SPI3_0_TSx;
    output				    SPI3_0_TSOP;
    output				    SPI3_0_TEOP;
    output				    [31:0] SPI3_0_TDat;
    output				    SPI3_0_TPrty;
    output				    [1:0]  SPI3_0_TMod;
    output				    SPI3_0_TErr;
    input				    SPI3_0_PTPA;
    output				    [7:0] SPI3_0_TAdr;


    //SPI3 1 Sink Interface,  see SPI3 core datasheet for details
    input				    SPI3_1_RVal;
    output				    SPI3_1_REnb;
    input				    SPI3_1_RSx;
    input				    SPI3_1_RSOP;
    input				    SPI3_1_REOP;
    input				    [31:0] SPI3_1_RDat;
    input				    SPI3_1_RPrty;
    input				    [1:0]  SPI3_1_RMod;
    input				    SPI3_1_RErr;

    //SPi3 1 Source Interface,  see SPI3 core datasheet for details
    output				    SPI3_1_TEnb;
    output				    SPI3_1_TSx;
    output				    SPI3_1_TSOP;
    output				    SPI3_1_TEOP;
    output				    [31:0] SPI3_1_TDat;
    output				    SPI3_1_TPrty;
    output				    [1:0]  SPI3_1_TMod;
    output				    SPI3_1_TErr;
    input				    SPI3_1_PTPA;
    output				    [7:0]  SPI3_1_TAdr;


    //SPI3 2 Sink Interface,  see SPI3 core datasheet for details
    input				    SPI3_2_RVal;
    output				    SPI3_2_REnb;
    input				    SPI3_2_RSx;
    input				    SPI3_2_RSOP;
    input				    SPI3_2_REOP;
    input				    [31:0] SPI3_2_RDat;
    input				    SPI3_2_RPrty;
    input				    [1:0]  SPI3_2_RMod;
    input				    SPI3_2_RErr;

    //SPI3 2 Source Interface,  see SPI3 core datasheet for details
    output				    SPI3_2_TEnb;
    output				    SPI3_2_TSx;
    output				    SPI3_2_TSOP;
    output				    SPI3_2_TEOP;
    output				    [31:0] SPI3_2_TDat;
    output				    SPI3_2_TPrty;
    output				    [1:0]  SPI3_2_TMod;
    output				    SPI3_2_TErr;
    input				    SPI3_2_PTPA;
    output				    [7:0]  SPI3_2_TAdr;
    
    //SPI3 3 Sink Interface,  see SPI3 core datasheet for details
    input				    SPI3_3_RVal;
    output				    SPI3_3_REnb;
    input				    SPI3_3_RSx;
    input				    SPI3_3_RSOP;
    input				    SPI3_3_REOP;
    input				    [31:0] SPI3_3_RDat;
    input				    SPI3_3_RPrty;
    input				    [1:0]  SPI3_3_RMod;
    input				    SPI3_3_RErr;

    //SPI3 3 Source Interface,  see SPI3 core datasheet for details
    output				    SPI3_3_TEnb;
    output				    SPI3_3_TSx;
    output				    SPI3_3_TSOP;
    output				    SPI3_3_TEOP;
    output				    [31:0] SPI3_3_TDat;
    output				    SPI3_3_TPrty;
    output				    [1:0]  SPI3_3_TMod;
    output				    SPI3_3_TErr;
    input				    SPI3_3_PTPA;
    output				    [7:0]  SPI3_3_TAdr;
    
    //Misc (can be removed, was added only to keep signals from being
    //optimized away)
    output				    SPI3_ParityErr;

  // SPI4 Enable signals
  wire 				    SPI4_SrcEn;
  wire 				    SPI4_SnkEn;

  //SPI4 Clk signals
  wire 				    SPI4_SysClkDiv_GP;
  wire 				    Bridge_Clk;
  wire 				    Reset;
  wire 				    SPI4_TDClk_Locked;
  wire 				    SPI4_TSClk_Locked;
  wire 				    SPI4_RDClk_Locked;

  //SPI4 Calendar signals
  reg 				     [1:0]  count;
  wire				     SPI4CalWr_n;
  wire				     [8:0] SPI4CalAddr;
  wire				     [7:0]  SPI4CalData;
  wire				     SPI4_SnkCalWrEn_n;
  wire				     [8:0]  SPI4_SnkCalAddr;
  wire				     [7:0]  SPI4_SnkCalData;
  wire				     [7:0]  SPI4_SnkCalDataOut;
  wire				     SPI4_SrcCalWrEn_n;
  wire				     [8:0]  SPI4_SrcCalAddr;
  wire				     [7:0]  SPI4_SrcCalData;
  wire				     [7:0]  SPI4_SrcCalDataOut;

  //Static Configuration Parameters
  wire				     [8:0]  SPI4_SnkCalendar_Len;
  wire				     [7:0]  SPI4_SnkCalendar_M;
  wire				     [8:0]  SPI4_SrcCalendar_Len;
  wire				     [7:0]  SPI4_SrcCalendar_M;
  
  //Initialization signals
  wire 				     CalProg;
  reg				     SnkCalProgDone;
  reg 				     SrcCalProgDone;
  wire 				     SPI4_SnkOof;
  wire 				     SPI4_PhaseAlignComplete;
  wire 				     SPI4_PhaseAlignRequest;
  wire 				     SPI4_DcmReset_RDClk;
  wire 				     SPI4_DcmReset_TDClk;
  wire 				     SPI4_DcmReset_TSClk;
  wire                                 SPI4_SnkDPAFailed;
  wire 				     SPI4_SnkFailed;
  wire 				     SPI3_TFClk_bufg_out;

  //SPI-4 Core Paramenters
  assign		SPI4_SnkCalendar_Len   = 3;
  assign		SPI4_SnkCalendar_M     = 0;
  assign		SPI4_SrcCalendar_Len   = 3; 
  assign		SPI4_SrcCalendar_M     = 0;
  
  assign		Bridge_Clk             = SPI4_SysClkDiv_GP;
  
  //Program SPI-4 Calendars
  always @ (negedge Reset_n or posedge Bridge_Clk)
   begin
     if(Reset_n == 0)
       count <= 2'b00;
     else 
	if(CalProg == 1)
         if (count != 2'b11)
           count <= count + 1;
   end


  //Generate SPI-4 Enable signal
  always @ (negedge Reset_n or posedge Bridge_Clk)
   begin
     if(Reset_n == 0)
      begin
       SnkCalProgDone <= 0;
       SrcCalProgDone <= 0;
      end
     else 
	if(count == 2'b11)
       begin
        SnkCalProgDone <= 1;
        SrcCalProgDone <= 1;
       end
     	else
       begin
        SnkCalProgDone <= 0;
        SrcCalProgDone <= 0;
       end
   end

  
 //SPI4 Sink Calendar Signals
  assign		SPI4_SnkCalWrEn_n = ~(CalProg & (~SnkCalProgDone));
  assign		SPI4_SnkCalAddr   = {7'b0000000,count};
  assign		SPI4_SnkCalData   = {6'b000000,count};

 //SPI4 Source Calendar Signals
  assign		SPI4_SrcCalWrEn_n = ~(CalProg & (~SrcCalProgDone));
  assign		SPI4_SrcCalAddr   = {7'b0000000,count};
  assign		SPI4_SrcCalData   = {6'b000000,count};

  assign		Reset = ~Reset_n;

spi3_to_spi4_top spi3_to_spi4_top0 (
    // System signals
    .Reset_n                (Reset_n),
    .SPI3_RFClk             (SPI3_RFClk),
    
    // SPI3 0 Interface
    .SPI3_0_RVal            (SPI3_0_RVal),
    .SPI3_0_REnb_n          (SPI3_0_REnb),   
    .SPI3_0_RSx             (SPI3_0_RSx),
    .SPI3_0_RSOP            (SPI3_0_RSOP),
    .SPI3_0_REOP            (SPI3_0_REOP),
    .SPI3_0_RDat            (SPI3_0_RDat),
    .SPI3_0_RPrty           (SPI3_0_RPrty),
    .SPI3_0_RMod            (SPI3_0_RMod),
    .SPI3_0_RErr            (SPI3_0_RErr),

    // SPI3 1 Interface
    .SPI3_1_RVal            (SPI3_1_RVal),
    .SPI3_1_REnb_n          (SPI3_1_REnb),   
    .SPI3_1_RSx             (SPI3_1_RSx),
    .SPI3_1_RSOP            (SPI3_1_RSOP),
    .SPI3_1_REOP            (SPI3_1_REOP),
    .SPI3_1_RDat            (SPI3_1_RDat),
    .SPI3_1_RPrty           (SPI3_1_RPrty),
    .SPI3_1_RMod            (SPI3_1_RMod),
    .SPI3_1_RErr            (SPI3_1_RErr),

    // SPI3 2 Interface
    .SPI3_2_RVal            (SPI3_2_RVal),
    .SPI3_2_REnb_n          (SPI3_2_REnb),   
    .SPI3_2_RSx             (SPI3_2_RSx),
    .SPI3_2_RSOP            (SPI3_2_RSOP),
    .SPI3_2_REOP            (SPI3_2_REOP),
    .SPI3_2_RDat            (SPI3_2_RDat),
    .SPI3_2_RPrty           (SPI3_2_RPrty),
    .SPI3_2_RMod            (SPI3_2_RMod),
    .SPI3_2_RErr            (SPI3_2_RErr),
    
    .SPI3_ParityErr         (SPI3_ParityErr),

    // SPI3 3 Interface
    .SPI3_3_RVal            (SPI3_3_RVal),
    .SPI3_3_REnb_n          (SPI3_3_REnb),   
    .SPI3_3_RSx             (SPI3_3_RSx),
    .SPI3_3_RSOP            (SPI3_3_RSOP),
    .SPI3_3_REOP            (SPI3_3_REOP),
    .SPI3_3_RDat            (SPI3_3_RDat),
    .SPI3_3_RPrty           (SPI3_3_RPrty),
    .SPI3_3_RMod            (SPI3_3_RMod),
    .SPI3_3_RErr            (SPI3_3_RErr),

    // System clocking input
    .SPI3_DcmReset_RFClk     (Reset),
	 .SPI3_Locked_RFClk    (),
    
    .SPI4_SysClk_P           (SPI4_SysClk_P),
    .SPI4_SysClk_N           (SPI4_SysClk_N),
    .SPI4_DcmReset_TDClk     (SPI4_DcmReset_TDClk),
    .SPI4_DcmReset_TSClk     (SPI4_DcmReset_TSClk),
    .SPI4_Locked_TDClk       (SPI4_TDClk_Locked),
    .SPI4_Locked_TSClk       (SPI4_TSClk_Locked),
    .SPI4_SysClkDiv_GP       (SPI4_SysClkDiv_GP),

    //SPI4.2 Calendar Programming Signals
    .SPI4_SrcCalendar_M      (SPI4_SrcCalendar_M),
    .SPI4_SrcCalendar_Len    (SPI4_SrcCalendar_Len),
    .SPI4_SrcCalClk          (Bridge_Clk),                    
    .SPI4_SrcCalWrEn_n       (SPI4_SrcCalWrEn_n),
    .SPI4_SrcCalAddr         (SPI4_SrcCalAddr),
    .SPI4_SrcCalData         (SPI4_SrcCalData),
    .SPI4_SrcCalDataOut      (SPI4_SrcCalDataOut),
    
    // SPI4.2 Interface
    // SPI4.2 source bus output
    .SPI4_TDClk_P            (SPI4_TDClk_P),
    .SPI4_TDClk_N            (SPI4_TDClk_N),
    .SPI4_TCtl_P             (SPI4_TCtl_P),
    .SPI4_TCtl_N             (SPI4_TCtl_N),
    .SPI4_TDat_P             (SPI4_TDat_P),
    .SPI4_TDat_N             (SPI4_TDat_N),
    
    // SPI4.2 source status (LVTTL) input
    .SPI4_TSClk              (SPI4_TSClk),
    .SPI4_TStat              (SPI4_TStat),
    
    // SPI4 source enable
    .SPI4_SrcEn              (SPI4_SrcEn)
    ) ;
    

spi4_to_spi3_top spi4_to_spi3_top0 (
      // System signals
      .Reset_n              (Reset_n),
      .SPI4_Clk             (Bridge_Clk),
		.CLK200					 (Ref_Clk),
      
      // SPI3 0 Interface
      .SPI3_TFClk         (SPI3_TFClk),
      .SPI3_0_TEnb_n        (SPI3_0_TEnb),
      .SPI3_0_TSx           (SPI3_0_TSx),
      .SPI3_0_TSOP          (SPI3_0_TSOP),
      .SPI3_0_TEOP          (SPI3_0_TEOP),
      .SPI3_0_TDat          (SPI3_0_TDat),
      .SPI3_0_TPrty         (SPI3_0_TPrty),
      .SPI3_0_TMod          (SPI3_0_TMod),
      .SPI3_0_TErr          (SPI3_0_TErr),
      .SPI3_0_PTPA          (SPI3_0_PTPA),
      .SPI3_0_TAdr          (SPI3_0_TAdr),

      // SPI3 1 Interface
      .SPI3_1_TEnb_n        (SPI3_1_TEnb),
      .SPI3_1_TSx           (SPI3_1_TSx),
      .SPI3_1_TSOP          (SPI3_1_TSOP),
      .SPI3_1_TEOP          (SPI3_1_TEOP),
      .SPI3_1_TDat          (SPI3_1_TDat),
      .SPI3_1_TPrty         (SPI3_1_TPrty),
      .SPI3_1_TMod          (SPI3_1_TMod),
      .SPI3_1_TErr          (SPI3_1_TErr),
      .SPI3_1_PTPA          (SPI3_1_PTPA),
      .SPI3_1_TAdr          (SPI3_1_TAdr),

      // SPI3 2 Interface
      .SPI3_2_TEnb_n        (SPI3_2_TEnb),
      .SPI3_2_TSx           (SPI3_2_TSx),
      .SPI3_2_TSOP          (SPI3_2_TSOP),
      .SPI3_2_TEOP          (SPI3_2_TEOP),
      .SPI3_2_TDat          (SPI3_2_TDat),
      .SPI3_2_TPrty         (SPI3_2_TPrty),
      .SPI3_2_TMod          (SPI3_2_TMod),
      .SPI3_2_TErr          (SPI3_2_TErr),
      .SPI3_2_PTPA          (SPI3_2_PTPA),
      .SPI3_2_TAdr          (SPI3_2_TAdr),

      // SPI3 3 Interface
      .SPI3_3_TEnb_n        (SPI3_3_TEnb),
      .SPI3_3_TSx           (SPI3_3_TSx),
      .SPI3_3_TSOP          (SPI3_3_TSOP),
      .SPI3_3_TEOP          (SPI3_3_TEOP),
      .SPI3_3_TDat          (SPI3_3_TDat),
      .SPI3_3_TPrty         (SPI3_3_TPrty),
      .SPI3_3_TMod          (SPI3_3_TMod),
      .SPI3_3_TErr          (SPI3_3_TErr),
      .SPI3_3_PTPA          (SPI3_3_PTPA),
      .SPI3_3_TAdr          (SPI3_3_TAdr),

      // System clocking input
      .SPI3_DcmReset_TFClk  (Reset),
      .SPI4_DcmReset_RDClk  (SPI4_DcmReset_RDClk),
      .SPI4_Locked_RDClk    (SPI4_RDClk_Locked),
    
      // SPI4.2 Interface
      .SPI4_RDClk_P         (SPI4_RDClk_P),
      .SPI4_RDClk_N         (SPI4_RDClk_N),
      .SPI4_RCtl_P          (SPI4_RCtl_P),
      .SPI4_RCtl_N          (SPI4_RCtl_N),
      .SPI4_RDat_P          (SPI4_RDat_P),
      .SPI4_RDat_N          (SPI4_RDat_N),

      // SPI4.2 source status (LVTTL) output
      .SPI4_RSClk           (SPI4_RSClk),
      .SPI4_RStat           (SPI4_RStat),
    
      //SPI4.2 Calendar Programming Signals
      .SPI4_SnkCalendar_M   (SPI4_SnkCalendar_M),
      .SPI4_SnkCalendar_Len (SPI4_SnkCalendar_Len),
      .SPI4_SnkCalClk       (Bridge_Clk),                    
      .SPI4_SnkCalWrEn_n    (SPI4_SnkCalWrEn_n),
      .SPI4_SnkCalAddr      (SPI4_SnkCalAddr),
      .SPI4_SnkCalData      (SPI4_SnkCalData),
      .SPI4_SnkCalDataOut   (SPI4_SnkCalDataOut),
      
      //SPI4.2 Out of Frame
      .SPI4_SnkOof          (SPI4_SnkOof),
      .SPI4_PhaseAlignComplete (SPI4_PhaseAlignComplete),
      .SPI4_PhaseAlignRequest  (SPI4_PhaseAlignRequest),
      .SPI3_TFClk_bufg_out     (SPI3_TFClk_bufg_out),
      .SPI4_SnkDPAFailed       (SPI4_SnkDPAFailed),                                                
      //SPI4 sink enable
      .SPI4_SnkEn            (SPI4_SnkEn)
    ) ;
    
spi_clk_startup spi_clk_startup0 (
    // System signals
    .Reset_n                   (Reset_n),
    .SPI_Clk                   (SPI3_TFClk_bufg_out),

    // SPI4.2 Sink DCM Status
    .SPI4_DcmReset_RDClk     (SPI4_DcmReset_RDClk),
    .SPI4_Locked_RDClk       (SPI4_RDClk_Locked),

    // SPI4.2 Source DCM Status
    .SPI4_DcmReset_TDClk     (SPI4_DcmReset_TDClk),
    .SPI4_Locked_TDClk       (SPI4_TDClk_Locked),
    .SPI4_DcmReset_TSClk     (SPI4_DcmReset_TSClk),
    .SPI4_Locked_TSClk       (SPI4_TSClk_Locked),

    .PL4Rst_n                (),
    
    // Calendar Programming
    .SnkStatClk           (Bridge_Clk),
    .SrcStatClk           (Bridge_Clk),
    .CalProg              (CalProg),
    .SnkCalProgDone       (SnkCalProgDone),
    .SrcCalProgDone       (SrcCalProgDone),
    
    // Sink and Source Core enables
    .SnkEn              (SPI4_SnkEn),
    .SrcEn              (SPI4_SrcEn),
    
    // Dynamic phase aligning signals
    .SnkFFClk           (Bridge_Clk),
    .PhaseAlignRequest  (SPI4_PhaseAlignRequest),
    .PhaseAlignComplete (SPI4_PhaseAlignComplete),
    .SnkOof             (SPI4_SnkFailed)
    
    );

assign SPI4_SnkFailed = SPI4_SnkDPAFailed || SPI4_SnkOof;

endmodule


//module spi3_to_spi4_top (
//    Reset_n,
//    SPI3_RFClk,
//    SPI3_0_RVal,
//    SPI3_0_REnb_n,
//    SPI3_0_RSx,
//    SPI3_0_RSOP,
//    SPI3_0_REOP,
//    SPI3_0_RDat,
//    SPI3_0_RPrty,
//    SPI3_0_RMod,
//    SPI3_0_RErr,
//    SPI3_1_RVal,
//    SPI3_1_REnb_n,
//    SPI3_1_RSx,
//    SPI3_1_RSOP,
//    SPI3_1_REOP,
//    SPI3_1_RDat,
//    SPI3_1_RPrty,
//    SPI3_1_RMod,
//    SPI3_1_RErr,
//    SPI3_2_RVal,
//    SPI3_2_REnb_n,
//    SPI3_2_RSx,
//    SPI3_2_RSOP,
//    SPI3_2_REOP,
//    SPI3_2_RDat,
//    SPI3_2_RPrty,
//    SPI3_2_RMod,
//    SPI3_2_RErr,
//    SPI3_3_RVal,
//    SPI3_3_REnb_n,
//    SPI3_3_RSx,
//    SPI3_3_RSOP,
//    SPI3_3_REOP,
//    SPI3_3_RDat,
//    SPI3_3_RPrty,
//    SPI3_3_RMod,
//    SPI3_3_RErr,
//    SPI4_TDClk_P,
//    SPI4_TDClk_N,
//    SPI4_TCtl_P,
//    SPI4_TCtl_N,
//    SPI4_TDat_P,
//    SPI4_TDat_N,
//    SPI4_TSClk,
//    SPI4_TStat,
//    SPI3_ParityErr,
//    SPI3_DcmReset_RFClk,
//    SPI3_Locked_RFClk,
//    SPI4_SysClk_P,
//    SPI4_SysClk_N,
//    SPI4_DcmReset_TDClk,
//    SPI4_DcmReset_TSClk,
//    SPI4_Locked_TDClk,
//    SPI4_Locked_TSClk,
//    SPI4_SysClkDiv_GP,
//    SPI4_SrcCalendar_M,
//    SPI4_SrcCalendar_Len,
//    SPI4_SrcCalClk,
//    SPI4_SrcCalWrEn_n,
//    SPI4_SrcCalAddr,
//    SPI4_SrcCalData,
//    SPI4_SrcCalDataOut,
//    SPI4_SrcEn,
//    );
//
//    // System signals
//    input                   Reset_n;
//  
//    //SPI3 Rx Clock - common to all 4 SPI3 interfaces   
//    input                   SPI3_RFClk;
//    
//    // SPI3 0 Interface
//    input                   SPI3_0_RVal;
//    output                  SPI3_0_REnb_n;
//    input                   SPI3_0_RSx;
//    input                   SPI3_0_RSOP;
//    input                   SPI3_0_REOP;
//    input          [31:0]   SPI3_0_RDat;
//    input                   SPI3_0_RPrty;
//    input           [1:0]   SPI3_0_RMod;
//    input                   SPI3_0_RErr;
//
//    // SPI3 1 Interface
//    input                   SPI3_1_RVal;
//    output                  SPI3_1_REnb_n;
//    input                   SPI3_1_RSx;
//    input                   SPI3_1_RSOP;
//    input                   SPI3_1_REOP;
//    input           [31:0]  SPI3_1_RDat;
//    input                   SPI3_1_RPrty;
//    input            [1:0]  SPI3_1_RMod;
//    input                   SPI3_1_RErr;
//
//    // SPI3 2 Interface
//    input                   SPI3_2_RVal;
//    output                  SPI3_2_REnb_n;
//    input                   SPI3_2_RSx;
//    input                   SPI3_2_RSOP;
//    input                   SPI3_2_REOP;
//    input           [31:0]  SPI3_2_RDat;
//    input                   SPI3_2_RPrty;
//    input            [1:0]  SPI3_2_RMod;
//    input                   SPI3_2_RErr;
//
//    // SPI3 3 Interface
//    input                   SPI3_3_RVal;
//    output                  SPI3_3_REnb_n;
//    input                   SPI3_3_RSx;
//    input                   SPI3_3_RSOP;
//    input                   SPI3_3_REOP;
//    input           [31:0]  SPI3_3_RDat;
//    input                   SPI3_3_RPrty;
//    input            [1:0]  SPI3_3_RMod;
//    input                   SPI3_3_RErr;
//
//    
//    // SPI4.2 Source Interface Signals
//    output                  SPI4_TDClk_P;
//    output                  SPI4_TDClk_N;
//    output                  SPI4_TCtl_P;
//    output                  SPI4_TCtl_N;
//    output          [15:0]  SPI4_TDat_P;
//    output          [15:0]  SPI4_TDat_N;
//    
//    // SPI4.2 source status (LVTTL) input
//    input                   SPI4_TSClk;
//    input            [1:0]  SPI4_TStat;
//
//    //SPI3 Parity Error 
//    output                  SPI3_ParityErr;
//
//    // SPI3 DCM Clock signals
//    input                   SPI3_DcmReset_RFClk;
//    output                  SPI3_Locked_RFClk;
//
//    // SPI4.2 System clock signals
//    input                   SPI4_SysClk_P;
//    input                   SPI4_SysClk_N;
//
//    //SPI4.2 DCM clock signals
//    input                   SPI4_DcmReset_TDClk;
//    input                   SPI4_DcmReset_TSClk;
//    output                  SPI4_Locked_TDClk;
//    output                  SPI4_Locked_TSClk;
//    output                  SPI4_SysClkDiv_GP;
//    
//    // SPI4.2 Calendar Programming Signals
//    input            [7:0]  SPI4_SrcCalendar_M;
//    input            [8:0]  SPI4_SrcCalendar_Len;
//    input                   SPI4_SrcCalClk;
//    input                   SPI4_SrcCalWrEn_n;
//    input            [8:0]  SPI4_SrcCalAddr;
//    input            [7:0]  SPI4_SrcCalData;
//    output           [7:0]  SPI4_SrcCalDataOut;
//    
//    // SPI4.2 source enable
//    input                   SPI4_SrcEn;
//
//endmodule
//
//
//module spi4_to_spi3_top (
//    Reset_n,
//    SPI4_Clk,
//	 CLK200,
//    SPI3_TFClk,
//    SPI3_0_TEnb_n,
//    SPI3_0_TSx,
//    SPI3_0_TSOP,
//    SPI3_0_TEOP,
//    SPI3_0_TDat,
//    SPI3_0_TPrty,
//    SPI3_0_TMod,
//    SPI3_0_TErr,
//    SPI3_0_PTPA,
//    SPI3_0_TAdr,
//
//    SPI3_1_TEnb_n,
//    SPI3_1_TSx,
//    SPI3_1_TSOP,
//    SPI3_1_TEOP,
//    SPI3_1_TDat,
//    SPI3_1_TPrty,
//    SPI3_1_TMod,
//    SPI3_1_TErr,
//    SPI3_1_PTPA,
//    SPI3_1_TAdr,
//
//    SPI3_2_TEnb_n,
//    SPI3_2_TSx,
//    SPI3_2_TSOP,
//    SPI3_2_TEOP,
//    SPI3_2_TDat,
//    SPI3_2_TPrty,
//    SPI3_2_TMod,
//    SPI3_2_TErr,
//    SPI3_2_PTPA,
//    SPI3_2_TAdr,
//
//    SPI3_3_TEnb_n,
//    SPI3_3_TSx,
//    SPI3_3_TSOP,
//    SPI3_3_TEOP,
//    SPI3_3_TDat,
//    SPI3_3_TPrty,
//    SPI3_3_TMod,
//    SPI3_3_TErr,
//    SPI3_3_PTPA,
//    SPI3_3_TAdr,
//    
//    SPI3_DcmReset_TFClk,
//    SPI3_TFClk_bufg_out,
//
//    SPI4_DcmReset_RDClk,
//    SPI4_Locked_RDClk,
//    SPI4_RDClk_P,
//    SPI4_RDClk_N,
//    SPI4_RCtl_P,
//    SPI4_RCtl_N,
//    SPI4_RDat_P,
//    SPI4_RDat_N,
//    SPI4_RSClk,
//    SPI4_RStat,
//    SPI4_SnkCalendar_M,
//    SPI4_SnkCalendar_Len,
//    SPI4_SnkCalClk,
//    SPI4_SnkCalWrEn_n,
//    SPI4_SnkCalAddr,
//    SPI4_SnkCalData,
//    SPI4_SnkCalDataOut,
//    SPI4_SnkOof,
//    SPI4_PhaseAlignComplete,
//    SPI4_PhaseAlignRequest,
//    SPI4_SnkEn
//    );
//
//    // System signals
//    input                Reset_n;
//    input                SPI4_Clk;
//	 input 					 CLK200;
//
//    // SPI3 0 Interface
//    input                SPI3_TFClk;
//    output               SPI3_0_TEnb_n;
//    output               SPI3_0_TSx;
//    output               SPI3_0_TSOP;
//    output               SPI3_0_TEOP;
//    output       [31:0]  SPI3_0_TDat;
//    output               SPI3_0_TPrty;
//    output        [1:0]  SPI3_0_TMod;
//    output               SPI3_0_TErr;
//    input                SPI3_0_PTPA;
//    output        [7:0]  SPI3_0_TAdr;
//
//    // SPI3 1 Interface
//    output               SPI3_1_TEnb_n;
//    output               SPI3_1_TSx;
//    output               SPI3_1_TSOP;
//    output               SPI3_1_TEOP;
//    output       [31:0]  SPI3_1_TDat;
//    output               SPI3_1_TPrty;
//    output        [1:0]  SPI3_1_TMod;
//    output               SPI3_1_TErr;
//    input                SPI3_1_PTPA;
//    output        [7:0]  SPI3_1_TAdr;
// 
//    // SPI3 2 Interface
//    output               SPI3_2_TEnb_n;
//    output               SPI3_2_TSx;
//    output               SPI3_2_TSOP;
//    output               SPI3_2_TEOP;
//    output       [31:0]  SPI3_2_TDat;
//    output               SPI3_2_TPrty;
//    output        [1:0]  SPI3_2_TMod;
//    output               SPI3_2_TErr;
//    input                SPI3_2_PTPA;
//    output        [7:0]  SPI3_2_TAdr;
//
//    // SPI3 3 Interface
//    output               SPI3_3_TEnb_n;
//    output               SPI3_3_TSx;
//    output               SPI3_3_TSOP;
//    output               SPI3_3_TEOP;
//    output       [31:0]  SPI3_3_TDat;
//    output               SPI3_3_TPrty;
//    output        [1:0]  SPI3_3_TMod;
//    output               SPI3_3_TErr;
//    input                SPI3_3_PTPA;
//    output        [7:0]  SPI3_3_TAdr;
//
//    // System clocking input
//    input                SPI3_DcmReset_TFClk;
//    input                SPI4_DcmReset_RDClk;
//    output               SPI4_Locked_RDClk;
//    
//    // SPI4.2 Interface
//    // SPI4.2 sink bus output
//    input                SPI4_RDClk_P;
//    input                SPI4_RDClk_N;
//    input                SPI4_RCtl_P;
//    input                SPI4_RCtl_N;
//    input        [15:0]  SPI4_RDat_P;
//    input        [15:0]  SPI4_RDat_N;
//
//    // SPI4.2 sink status (LVTTL) input
//    output               SPI4_RSClk;
//    output        [1:0]  SPI4_RStat;
//    
//    //SPI4.2 Calendar Programming Signals
//    input         [7:0]  SPI4_SnkCalendar_M;
//    input         [8:0]  SPI4_SnkCalendar_Len;
//    input                SPI4_SnkCalClk;
//    input                SPI4_SnkCalWrEn_n;
//    input         [8:0]  SPI4_SnkCalAddr;
//    input         [7:0]  SPI4_SnkCalData;
//    output        [7:0]  SPI4_SnkCalDataOut;
//    
//    //SPI4.2 Out of Frame
//    output               SPI4_SnkOof;
//    output               SPI4_PhaseAlignComplete;
//    input                SPI4_PhaseAlignRequest;
//    output               SPI3_TFClk_bufg_out;
//    
//    //PL4 sink enable
//    input                SPI4_SnkEn;
	 
//endmodule
