/******************************************************************************
* SPI3 to SPI4 Burst Storage File
******************************************************************************/

/******************************************************************************
*
*   File Name:  spi3_to_spi4_burst_storage.v
*     Version:  1.0
*        Date:  June 11, 2007
*
* Description:  The purpose of the spi3_to_spi4_burst_storgae.v file is to
*               store a burst of data from the SPI3 interface prior to making 
*               it available to the SPI4.2 interface. 
*
*      Author:  Zhe Xia
*     Company:  Xilinx Inc.
*
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*
*     (c) Copyright 2003 Xilinx, Inc.
*     All rights reserved.
*
*******************************************************************************/

 `include "../../hdl/verilog/spi_pkg.v"  
// `include "../../../hdl/verilog/spi_pkg.v"


/*******************************************************************************
* Module Declaration
*******************************************************************************/

module spi3_to_spi4_burst_storage(
    Reset_n,
    BridgeClk,
    SPI3_Addr,
    SPI3_Data,
    SPI3_REM,
    SPI3_SOP,
    SPI3_EOP,
    SPI3_Err,
	 SPI3_SRC_RDY,
    SPI3_Valid,
    BrstFull_n,
    BrstAddr,
    BrstData,
    BrstMod,
    BrstSOP,
    BrstEOP,
    BrstErr,
    BrstRdEn,
    BrstAlmostEmpty,
    BrstEmpty,
    BrstValid,
    BrstRdEOP,
    PktCntEq0,
    PktCntEq1
    );


    // System signals
    input                  Reset_n;
    input                  BridgeClk;
  
    // SPI3 Burst Storage Interface Output Signals
    input          [7:0]   SPI3_Addr;
    input	  [31:0]   SPI3_Data;
    input	   [1:0]   SPI3_REM;
    input                  SPI3_SOP;
    input                  SPI3_EOP;
    input                  SPI3_Err;
	 input						SPI3_SRC_RDY;
    input                  SPI3_Valid;
    
    // Burst Storage Interface Output Signals
    output                 BrstFull_n;

    // Burst Storage Inteface Output Signals
    output	   [7:0]   BrstAddr;
    output        [63:0]   BrstData;
    output	   [2:0]   BrstMod;
    output                 BrstSOP;
    output                 BrstEOP;
    output                 BrstErr;
    				
    // Burst Storage Read Enable
    input                  BrstRdEn;
    
    output                 BrstAlmostEmpty;
    output                 BrstEmpty;
    output                 BrstValid;
    output                 BrstRdEOP;
    
    //Packet Count Signals
    output                 PktCntEq0;
    output                 PktCntEq1;


/*******************************************************************************
* Definition of Ports:
********************************************************************************
* System signals
*    Reset_n               : System Level Reset (active low)
*    BridgeClk             : BridgeClk Clock 
*
* Burst Storage Input Signals from SPI3 Sink FIFOs
*    SPI3_Addr             : SPI3 Sink Address 
*    SPI3_Data             : SPI3 Sink Data 
*    SPI3_REM              : SPI3 Sink MOD 
*    SPI3_SOP              : SPI3 Sink Start of Packet 
*    SPI3_EOP              : SPI3 Sink End of Packet  
*    SPI3_Err              : SPI3 Sink Error  
*    SPI3_Valid            : SPI3 Sink Data is Valid  
*
* Burst Storage Output Signal - SPI3 Read Interface 
*    BrstFull_n            : Burst Storage FIFO Full (active low) 
*
* Burst Storage Output Signals - SPI4 Write Interface 
*    BrstAddr              : Burst Storage Address 
*    BrstData              : Burst Storage Data 
*    BrstMod               : Burst Storage MOD 
*    BrstSOP               : Burst Storage Start of Packet 
*    BrstEOP               : Burst Storage End of Packet  
*    BrstErr               : Burst Storage Error  
*
* Burst Storage Input Signals - Arbiter Interface
*    BrstRdEn              : Burst Storage Read Enable
*
* Burst Storage Output Signals - Arbiter Interface   
*    BrstAlmostEmpty       : Burst Storage is Almost Empty
*    BrstEmpty             : Burst Storage Empty
*    BrstRdValid           : Burst Storage Read Valid * indicates if Burst Storage
*                            has valid data
*    BrstRdEOP             : Burst Storage Read EOP * indicates if next Burst
*                            Storage word has an EOP
*    BrstPktCntEq0[3:0]    : Burst Storage has zero EOPs (no complete packet)
*    BrstPktCntEq1[3:0]    : Burst Storage has one EOP
*
*******************************************************************************/

  


  /*****************************************************************************
  * Wire and Reg Declarations
  *****************************************************************************/
  
  
  //SPI3 Interface Register signals
  reg	        [31:0]  SPI3_Data_r0;
  reg	         [1:0] 	SPI3_REM_r0;
  reg							SPI3_SRC_RDY_r0;
  reg                   SPI3_Valid_r0;
  reg							SPI3_Valid_r1;
  reg                   SPI3_SOP_r0;
  reg                   SPI3_SOP_r1;
  reg	                SPI3_EOP_r0;
  reg                   SPI3_EOP_r1;
  reg                   SPI3_Err_r0;
  reg                   SPI3_Err_r1;

  //Signal to convert two SPI3 words into one 64 bit word
  reg 	                ConvEn;
  reg	        [31:0]  SPI3_Data_d1;
  reg           [31:0]  SPI3_Data_d2;
  reg	  	 [2:0]  Mod_3bits;

  //Bridge FIFO  signals
  wire                  FIFO_Reset;
  reg	                FIFOWr;
  reg                   FIFOWr_r0;
  reg                   FIFOWr_r1; 
  reg                  FIFO_Write;
  reg	        [71:0]  FIFO_Din;
  wire                  FIFO_RdEn;
  wire          [71:0]  FIFO_Dout;
  wire	                FIFO_Full;
  wire                  FIFO_Empty;
  reg                   FIFO_AlmostEmpty;
  reg	                Empty;
  reg                   Full;
  wire	         [8:0]  DataCount;

  //RegBank signals
  wire                  preload_stage1;
  wire                  preload_stage2;
  reg                   data_valid;
  reg                   data_valid_r0;
  wire                  RdEn;
  wire	                RdEn_reg;
  reg	        [71:0]  RegBank;
  wire                  Taken;

  //Pkt Count signals
  reg                   Wr_EOP_r0;
  reg                   Wr_EOP_r1;
  wire                  Rd_EOP;
  wire                  PktCntEn;
  wire		 [8:0]  PktAdd_Val;
  reg		 [8:0]  PktCnt;
  reg                   PktCnt_Eq0;
  reg                   PktCnt_Eq1;



 /******************************************************************************
 *Register SPI3 Sink FIFO outputs 
 ******************************************************************************/
  // spi3fifo_reg block
  always @ (negedge Reset_n or posedge BridgeClk)
    begin
      if(Reset_n == 0)
       begin
        SPI3_Data_r0  <=    0;
        SPI3_REM_r0   <=    0;
        SPI3_Valid_r0 <=    0;
		  SPI3_Valid_r1 <=    0;
        SPI3_SOP_r0   <=    0;
        SPI3_EOP_r0   <=    0;
        SPI3_Err_r0   <=    0;
		  SPI3_SRC_RDY_r0 <=    0;
       end
      else
       begin
        SPI3_Data_r0  <=    SPI3_Data;
        SPI3_REM_r0   <=    SPI3_REM;
        SPI3_Valid_r0 <=    SPI3_Valid;
		  SPI3_Valid_r1 <=    SPI3_Valid_r0;
        SPI3_SOP_r0   <=    SPI3_SOP && SPI3_SRC_RDY && SPI3_Valid;
        SPI3_EOP_r0   <=    SPI3_EOP && SPI3_SRC_RDY && SPI3_Valid;
        SPI3_Err_r0   <=    SPI3_Err && SPI3_SRC_RDY && SPI3_Valid;
		  SPI3_SRC_RDY_r0 <=  SPI3_SRC_RDY;
       end 
     end

  /****************************************************************************
  * Generate enable signal to convert two SPI-3 FIFO words (32 bits) to one
  * SPI-4 FIFO word (64 bits)
  ****************************************************************************/
  //conv_en block
  always @ (negedge Reset_n or posedge BridgeClk)
    begin
      if(Reset_n == 0)
        ConvEn <=    1; 	
      else if( (SPI3_SOP == 1'b1) && (SPI3_SRC_RDY == 1'b1) && (SPI3_Valid == 1) )
	    ConvEn <=    0;	
      	  else if ((SPI3_SRC_RDY == 1'b1) && (SPI3_Valid == 1))
	    ConvEn <=    ~ConvEn;
    end

  /****************************************************************************
  * Convert two 32-bit SPI-3 Data words to one 64-bit word.
  * If odd SPI3 Word has EOP, assign lower SPI4 word to zeros. Otherwise
  * odd SPI3 words are assigned as 32 MSB bits of SPI4 word  [63:32]and
  * even SPI3 words are assigned to lower LSB bits of SPI4 word [31:0]. 
  ****************************************************************************/
  //spi3data_reg block
  always @ (negedge Reset_n or posedge BridgeClk)
    begin
      if(Reset_n == 0)
       begin
        SPI3_Data_d1 <=    0;
        SPI3_Data_d2 <=    0;
       end
      else 
       if((SPI3_SRC_RDY_r0 == 1'b1) && (SPI3_Valid_r0 == 1'b1))
        begin
         if(ConvEn == 0 && SPI3_EOP_r0 == 1)
          begin
           SPI3_Data_d1 <=    SPI3_Data_r0;
           SPI3_Data_d2 <=    0;
          end
         else if(ConvEn == 0 && (SPI3_EOP_r0 == 0 || SPI3_SOP_r0 == 1))
           SPI3_Data_d1 <=    SPI3_Data_r0;
         else if ( SPI3_Valid_r0 == 1'b1 )
           SPI3_Data_d2 <=    SPI3_Data_r0;
        end
    end

  /*****************************************************************************
  * SPI3 Mod conversion to SPI-4 Mod 
  * This process converts Mod for two SPI-3 FIFO words to Mod of one SPI4 
  * FIFO word. 2-bit Mod of SPI-3 for SPI-3 data [31:0] is converted to 3-bit
  * SPI4 Mod.
  *****************************************************************************/
  // spi3mod_conv block
  always @ (negedge Reset_n or posedge BridgeClk)
  begin
    if (Reset_n == 0)
     Mod_3bits <=    3'b 000;
    else
     if (SPI3_SRC_RDY_r0 == 1 && SPI3_EOP_r0 == 1)
      begin
        if (ConvEn == 0)
          case (SPI3_REM_r0)
				 2'b 11:  Mod_3bits <=    3'b 100;
             2'b 10:  Mod_3bits <=    3'b 011;
             2'b 01:  Mod_3bits <=    3'b 010;
             2'b 00:  Mod_3bits <=    3'b 001;
             default: Mod_3bits <=    Mod_3bits;
           endcase
        else 
          case (SPI3_REM_r0)
             2'b 11:  Mod_3bits <=    3'b 000;
             2'b 10:  Mod_3bits <=    3'b 111;
             2'b 01:  Mod_3bits <=    3'b 110;
             2'b 00:  Mod_3bits <=    3'b 101;
             default: Mod_3bits <=    Mod_3bits;
           endcase           
      end
     else
       Mod_3bits <=    3'b 000;
  end

  /*****************************************************************************
  * Process to capture SOP, EOP, Err bits from SPI3 Interface and
  * write them to Burst Storage FIFO.
  *****************************************************************************/
  // fifopkt_reg block
  always @ (negedge Reset_n or posedge BridgeClk)
    begin
      if(Reset_n == 0)
       begin
        SPI3_SOP_r1 <=    0;
        
       end
      else 
       if (SPI3_SRC_RDY_r0 == 1)
        begin
          if((ConvEn == 0) && (SPI3_Valid_r0 == 1'b1)) begin
  	    		SPI3_SOP_r1 <=    SPI3_SOP_r0;
  	    		end
        	else begin
						SPI3_SOP_r1 <=  SPI3_SOP_r1;
					end 
          
       end
    end

  always @ (negedge Reset_n or posedge BridgeClk)
    begin
      if(Reset_n == 0)
       begin
        SPI3_Err_r1 <=    0;
        SPI3_EOP_r1 <=    0;
       end
      else 
       if (SPI3_SRC_RDY_r0 == 1)
        begin
					
          if ((SPI3_Err_r0 == 1) && (SPI3_Valid_r0 == 1'b1))
     	    	SPI3_Err_r1 <=    1;
 	 				else if ((SPI3_SOP_r0 == 1)&& (SPI3_Valid_r0 == 1'b1))
 	   		  SPI3_Err_r1 <=    0;
 	   		  SPI3_EOP_r1 <=    SPI3_EOP_r0;
       end
    end
 

  /*****************************************************************************
  * Process to generate Burst Storage  FIFO write strobe
  *****************************************************************************/
  // spi3fifowr_reg block
  always @ (negedge Reset_n or posedge BridgeClk)
    begin
      if(Reset_n == 0)
       begin
        FIFOWr    <=    0;
        FIFOWr_r0 <=    0;
       end
      else 
	begin
	 if ((SPI3_Valid_r0 == 1) && (SPI3_SRC_RDY_r0 == 1))
	    if(ConvEn == 0 && SPI3_EOP_r0 == 1)
              FIFOWr <=    1;
            else if(ConvEn == 0)
              FIFOWr <=    0;
            else
              FIFOWr <=    1;
         else
	  FIFOWr <=    0;

        FIFOWr_r0 <=    FIFOWr;
        FIFOWr_r1 <=    FIFOWr_r0;
      end 
     end

     
  /****************************************************************************
  * Combinatorial Burst FIFO  Signal Assignments
  ****************************************************************************/

  //FIFO Data input Bus Bit Mappings

  //FIFO_Din[71:0]  --> Burst Storage FIFO Input
  //   -Packet Data    -> FIFO_Din[63:0]
  //   -Mod            -> FIFO_Din[66:64]
  //   -Err            -> FIFO_Din[67]
  //   -SOP            -> FIFO_Din[68]
  //   -EOP            -> FIFO_Din[69]
  //   -EOP duplicate1 -> FIFO_Din[70] -- To improve timing
  //   -EOP duplicate2 -> FIFO_Din[71] -- To improve timing
   
	always @ (negedge Reset_n or posedge BridgeClk) begin
		if (!Reset_n) begin
			FIFO_Din[71:0] <= 18'h000000000000000000;
			FIFO_Write  <= 1;
		end
		else begin
			FIFO_Din[71:0] <= {SPI3_EOP_r1, SPI3_EOP_r1, SPI3_EOP_r1,
                              SPI3_SOP_r1, SPI3_Err_r1, Mod_3bits,
                              SPI3_Data_d1, SPI3_Data_d2};
//			FIFO_Write <= FIFOWr && (~FIFOWr_r0);
			FIFO_Write <= FIFOWr;
		end
	end
   

    assign  FIFO_Reset = ~Reset_n;
 /*****************************************************************************
 * Burst Storage FIFO
 *****************************************************************************/
 //synthesis attribute box_type of generic_sfifo_512x72 is black_box; 

  generic_sfifo_512x72 burst_fifo
		(.clk(BridgeClk),
		 .rst(FIFO_Reset),
                 .din(FIFO_Din),
		 .wr_en(FIFO_Write),
		 .rd_en(FIFO_RdEn),
		 .dout(FIFO_Dout),
		 .full(FIFO_Full),
		 .empty(FIFO_Empty),
		 .data_count(DataCount)
            );

 
  /****************************************************************************
  * Process to generate Burst Storage  FIFO Almost Empty signal
  ****************************************************************************/
  // spi3fifoae_reg block
  always @ (negedge Reset_n or posedge BridgeClk)
    begin
      if(Reset_n == 0)
        FIFO_AlmostEmpty <=    1;
      else 
	if(DataCount > `SPI3_SPI4BrstAE_Thresh)
	  FIFO_AlmostEmpty <=    0;
	else
	  FIFO_AlmostEmpty <=    1;
     end

  /****************************************************************************
  * Process to generate Burst Storage  FIFO Empty signal
  *****************************************************************************/
  // spi3fifoempty_reg block
  always @ (negedge Reset_n or posedge BridgeClk)
    begin
      if(Reset_n == 0)
        Empty <=    1;
      else 
	if ((FIFO_Empty == 1 && Taken == 1 && data_valid_r0 == 1) || data_valid_r0 == 0)
	  Empty <=    1;
      	else
	  Empty <=     0;
    end

  /****************************************************************************
  * Process to generate Burst Storage  FIFO Full signal
  * Assert FIFO Full if DataCount > 508
  ****************************************************************************/
  // spi3fifofull_reg block
  always @ (negedge Reset_n or posedge BridgeClk)
    begin
      if(Reset_n == 0)
        Full <=    0;
      else 
        if (DataCount[8:3] == 6'b111111)
          Full <=    1;
        else
          Full <=    0;
    end
    
  /****************************************************************************
  * FIFO Read Logic
  * The following logic implement a "Taken FIFO". There is a register bank
  * REGBANK in the FIFO output and this REGBANK is pre-loaded with FIFO Output 
  * Whenever FIFO is not empty
  * the first word is pre-read and loaded into a register bank REGBANK. 
  ****************************************************************************/
  assign  Taken          = BrstRdEn;            
  assign  preload_stage1 = (~FIFO_Empty) & ((~data_valid) | (~data_valid_r0));
  assign  preload_stage2 = ((~data_valid_r0) & data_valid);

  //Only do a read from the FIFO when the FIFO is not currently empty
  //and FIFO wasn't empty on previous cycle.
  assign  RdEn           = (Taken & (~FIFO_Empty)) | preload_stage1;
  assign  RdEn_reg       = (Taken | preload_stage2);
  assign  FIFO_RdEn      = RdEn;

    
  /****************************************************************************
  * Generate valid strobe for Stage 1
  ****************************************************************************/
  // stage1 block
  always @ (negedge Reset_n or posedge BridgeClk)
   begin
    if (Reset_n == 0)
      data_valid <=    0;
    else 
      if (RdEn == 1)           //reading data out of RAM into RAM output register
        data_valid <=    1;
      else if (RdEn_reg == 1)  //reading data out of register and not out of RAM register
        data_valid <=    0;
   end

  /*****************************************************************************
  * Generate data valid strobe to indicate when REGBANK has valid data.
  * Register data from the FIFO into REGBANK
  *****************************************************************************/
  // stage2 block
  always @ (negedge Reset_n or posedge BridgeClk)
   begin
    if (Reset_n == 0)
     begin
      RegBank       <=    0;
      data_valid_r0 <=    0;
     end
    else 
	if (RdEn_reg == 1)   //reading data out of register and 
                             //possibly into register from FIFO
                             //Zero out data if register won't contain valid data
       begin
        if (data_valid == 1)
          RegBank     <=    FIFO_Dout;
        else
          RegBank     <=    0;

        data_valid_r0 <=    data_valid;
       end
   end 

    //Burst Storage Outputs
    assign  BrstEmpty       =  Empty;
    assign  BrstAlmostEmpty =  FIFO_AlmostEmpty;
    assign  BrstFull_n      = ~Full;
    assign  BrstValid       = data_valid_r0;


  //RegBank[71:0]  --> Burst Storage FIFO Output register
  //   -Packet Data    -> RegBank[63:0]
  //   -Mod            -> RegBank[66:64]
  //   -Err            -> RegBank[67]
  //   -SOP            -> RegBank[68]
  //   -EOP            -> RegBank[69]
  //   -EOP duplicate1 -> RegBank[70] ** To improve timing
  //   -EOP duplicate2 -> RegBank[71] ** To improve timing
  
    assign  BrstData   = RegBank[63:0];
    assign  BrstMod    = RegBank[66:64];
    assign  BrstErr    = RegBank[67];          
    assign  BrstSOP    = RegBank[68];         
    assign  BrstEOP    = RegBank[69];
    assign  BrstRdEOP  = RegBank[70];

/******************************************************************************
* Generate EOP count in the Burst Storage FIFO.
* Process to generate write EOP strobe
******************************************************************************/
  //create_wrpkt_strobe block
  always @ (posedge BridgeClk or negedge Reset_n)
   begin
    if (Reset_n == 0)
     begin
       Wr_EOP_r0  <=    0;
       Wr_EOP_r1  <=    0;
     end
    else
      begin
       Wr_EOP_r0  <=    FIFO_Write & FIFO_Din[71];
       Wr_EOP_r1  <=    Wr_EOP_r0 & (~FIFO_Full);
      end
   end 


  /*****************************************************************************
  *Count number of EOPs in the Burst Storage FIFO
  *EOP count : Increment EOP count if no EOP is read out while writing another
  *            EOP into the FIFO
  *            Decrement EOP count if EOP is read out while no other EOP is
  *            being written into the FIFO
  *****************************************************************************/
  assign  Rd_EOP   = Taken & RegBank[71];  //EOP will not be '1' if read not valid
  assign  PktCntEn = (Taken & RegBank[71]) ^ Wr_EOP_r1;

  //add -1 if read & not write,add 1 if not read & write
  assign  PktAdd_Val[0] = 1;
  
  genvar i;
    generate for (i = 1; i < 9; i = i + 1) begin : addval_gen
       assign PktAdd_Val[i] = ~Wr_EOP_r1;
    end
  endgenerate
  
  // create_pkt_count block
  always @ (posedge BridgeClk or negedge Reset_n)
   begin
    if (Reset_n == 0)
      PktCnt <=    0; 
    else 
      if (PktCntEn == 1)
        PktCnt <=    PktCnt + PktAdd_Val;
   end

/******************************************************************************
* Generate Packet Count Equal to Zero and Packet count Equal to one signals
* These signals indicate whether one or more EOPs are present in the Burst FIFO
* PktCntEq0 = 0  indicates No EOPs are present,
*           = 1  indicates atleast  one EOP is present
* PktCntEq1 = 1  indicates only one EOP is present in the Burst FIFO,
*           = 0  indicates either zero EOPs or more than one EOPs are present in the Burst FIFO
******************************************************************************/
  
  // create_pkt_cnt block
  always @ (posedge BridgeClk or negedge Reset_n)
   begin
    if (Reset_n == 0)
     begin
      PktCnt_Eq1 <=    0; 
      PktCnt_Eq0 <=    1; 				
     end
    else 
	if (Rd_EOP == 1 && Wr_EOP_r1 == 0)
	 begin
	  if (PktCnt == 2)
      	    PktCnt_Eq1 <=   1; 				
	  else 
      	    PktCnt_Eq1 <=   0; 
      	    PktCnt_Eq0 <=   PktCnt_Eq1; 
	 end
	else if (Rd_EOP == 0 && Wr_EOP_r1 == 1)
	 begin
	  PktCnt_Eq1 <=   PktCnt_Eq0;
	  PktCnt_Eq0 <=   0;   	
	 end
  end

  assign  PktCntEq0 = PktCnt_Eq0;
  assign  PktCntEq1 = PktCnt_Eq1;
  
endmodule
