/******************************************************************************
*
*   File Name:  spi3_to_spi4_core.v
*     Version:  1.0
*        Date:  June 11, 2007
*
* Description:  The purpose of the spi3_to_spi4_core.v file is to serve as 
*               the wrapper file for all logic associated with the SPI3 to 
*               SPI4 direction of the Bridge.
*
*      Author:  Zhe Xia
*     Company:  Xilinx Inc.
*
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*
*     (c) Copyright 2003 Xilinx, Inc.
*     All rights reserved.
*
*******************************************************************************/

`include "../../hdl/verilog/spi_pkg.v" 

/*******************************************************************************
* Module Declaration
*******************************************************************************/

module spi3_to_spi4_core(
    Reset_n,
    SPI3_Clk,
    SPI4_Stat,
    SPI4_AlmostFull_n,
    SPI4_WrAddr,
    SPI4_WrData,
    SPI4_WrMod,
    SPI4_WrSOP,
    SPI4_WrEOP,
    SPI4_WrErr,
    SPI4_WrEn_n,
    SPI3_0_DST_RDY,
    SPI3_0_SRC_RDY,
    SPI3_0_FFAddr,
    SPI3_0_FFData,
    SPI3_0_FFMod,
    SPI3_0_FFSOP,
    SPI3_0_FFEOP,
    SPI3_0_FFErr,
    SPI3_1_DST_RDY,
    SPI3_1_SRC_RDY,
    SPI3_1_FFAddr,
    SPI3_1_FFData,
    SPI3_1_FFMod,
    SPI3_1_FFSOP,
    SPI3_1_FFEOP,
    SPI3_1_FFErr,
    SPI3_2_DST_RDY,
    SPI3_2_SRC_RDY,
    SPI3_2_FFAddr,
    SPI3_2_FFData,
    SPI3_2_FFMod,
    SPI3_2_FFSOP,
    SPI3_2_FFEOP,
    SPI3_2_FFErr,
    SPI3_3_DST_RDY,
    SPI3_3_SRC_RDY,
    SPI3_3_FFAddr,
    SPI3_3_FFData,
    SPI3_3_FFMod,
    SPI3_3_FFSOP,
    SPI3_3_FFEOP,
    SPI3_3_FFErr
    );


    // System signals
    input                         Reset_n;
    input                         SPI3_Clk;


    // SPI4 Interface Signals
   input                  [7:0]   SPI4_Stat;
   input                          SPI4_AlmostFull_n;
   output                 [7:0]   SPI4_WrAddr;
   output                [63:0]   SPI4_WrData;
   output		  [2:0]   SPI4_WrMod;
   output                         SPI4_WrSOP;
   output                         SPI4_WrEOP;
   output                         SPI4_WrErr;
   output                         SPI4_WrEn_n;
    
    // SPI3 0 Interface Signals
	input									 SPI3_0_SRC_RDY;
   output                         SPI3_0_DST_RDY;
   input 		  [7:0]   SPI3_0_FFAddr;
   input 		 [31:0]   SPI3_0_FFData;
   input 		  [1:0]   SPI3_0_FFMod;
   input                          SPI3_0_FFSOP;
   input                          SPI3_0_FFEOP;
   input                          SPI3_0_FFErr;
//   input                          SPI3_0_FFValid;
  

    // SPI3 1 Interface Signals
	input									 SPI3_1_SRC_RDY;
   output                         SPI3_1_DST_RDY;
   input                  [7:0]   SPI3_1_FFAddr;
   input                 [31:0]   SPI3_1_FFData;
   input                  [1:0]   SPI3_1_FFMod;
   input                          SPI3_1_FFSOP;
   input                          SPI3_1_FFEOP;
   input                          SPI3_1_FFErr;
//   input                          SPI3_1_FFValid;
    
    // SPI3 2 Interface Signals
	input									 SPI3_2_SRC_RDY;
   output                         SPI3_2_DST_RDY;
   input                   [7:0]  SPI3_2_FFAddr;
   input                  [31:0]  SPI3_2_FFData;
   input                   [1:0]  SPI3_2_FFMod;
   input                          SPI3_2_FFSOP;
   input                          SPI3_2_FFEOP;
   input                          SPI3_2_FFErr;
//   input                          SPI3_2_FFValid;

    // SPI3 3 Interface Signals
	input									 SPI3_3_SRC_RDY;
   output                         SPI3_3_DST_RDY;
   input                  [7:0]   SPI3_3_FFAddr;
   input                 [31:0]   SPI3_3_FFData;
   input                  [1:0]   SPI3_3_FFMod;
   input                          SPI3_3_FFSOP;
   input                          SPI3_3_FFEOP;
   input                          SPI3_3_FFErr;
//   input                          SPI3_3_FFValid;


/*******************************************************************************
* Definition of Ports:
********************************************************************************
* System signals
*    Reset_n                 : System Level Reset (active low)
*    SPI3_Clk                : SPI3 Interface Clock 
*
* SPI4 Interface Signals
*    SP4_WrAddr             : SPI4 Interface Address 
*    SP4_WrData             : SPI4 Interface Data 
*    SP4_WrMod              : SPI4 Interface MOD 
*    SP4_WrSOP              : SPI4 Interface Start of Packet 
*    SP4_WrEOP              : SPI4 Interface End of Packet  
*    SP4_WrErr              : SPI4 Interface Error  
*    SP4_WrEn_n             : SPI4 Interface Write Enable (active low) 
*
* SPI3 0 Interface Signals
*    SPI3_0_SRC_RDY          : SPI3 Interface 0 is Ready
*	  SPI3_0_DST_RDY          : SPI3 Interface 0 Data is valid(Affect by SRC_RDY)
*    SPI3_0_Addr             : SPI3 Interface 0 Address
*    SPI3_0_FFData           : SPI3 Interface 0 Data
*    SPI3_0_FFMod            : SPI3 Interface 0 MOD
*    SPI3_0_FFSOP            : SPI3 Interface 0 Start of Packet 
*    SPI3_0_FFEOP            : SPI3 Interface 0 End of Packet 
*    SPI3_0_FFErr            : SPI3 Interface 0 Error 
*    SPI3_0_FFValid          : SPI3 Interface 0 Data is Valid 
*
* SPI3 1 Interface Signals
*    SPI3_1_SRC_RDY          : SPI3 Interface 1 is Ready
*	  SPI3_1_DST_RDY          : SPI3 Interface 1 Data is valid(Affect by SRC_RDY)
*    SPI3_1_Addr             : SPI3 Interface 1 Address
*    SPI3_1_FFData           : SPI3 Interface 1 Data
*    SPI3_1_FFMod            : SPI3 Interface 1 MOD
*    SPI3_1_FFSOP            : SPI3 Interface 1 Start of Packet 
*    SPI3_1_FFEOP            : SPI3 Interface 1 End of Packet 
*    SPI3_1_FFErr            : SPI3 Interface 1 Error 
*    SPI3_1_FFValid          : SPI3 Interface 1 Data is Valid 
*
* SPI3 2 Interface Signals
*    SPI3_2_SRC_RDY          : SPI3 Interface 2 is Ready
*	  SPI3_2_DST_RDY          : SPI3 Interface 2 Data is valid(Affect by SRC_RDY)
*    SPI3_2_Addr             : SPI3 Interface 2 Address
*    SPI3_2_FFData           : SPI3 Interface 2 Data
*    SPI3_2_FFMod            : SPI3 Interface 2 MOD
*    SPI3_2_FFSOP            : SPI3 Interface 2 Start of Packet 
*    SPI3_2_FFEOP            : SPI3 Interface 2 End of Packet 
*    SPI3_2_FFErr            : SPI3 Interface 2 Error 
*    SPI3_2_FFValid          : SPI3 Interface 2 Data is Valid 
*
* SPI3 3 Interface Signals
*    SPI3_3_SRC_RDY          : SPI3 Interface 3 is Ready
*	  SPI3_3_DST_RDY          : SPI3 Interface 3 Data is valid(Affect by SRC_RDY)
*    SPI3_3_Addr             : SPI3 Interface 3 Address
*    SPI3_3_FFData           : SPI3 Interface 3 Data
*    SPI3_3_FFMod            : SPI3 Interface 3 MOD
*    SPI3_3_FFSOP            : SPI3 Interface 3 Start of Packet 
*    SPI3_3_FFEOP            : SPI3 Interface 3 End of Packet 
*    SPI3_3_FFErr            : SPI3 Interface 3 Error 
*    SPI3_3_FFValid          : SPI3 Interface 3 Data is Valid 
*
* SPI3 Status Input Signals
*    SPI3_Stat               : SPI4 Interface Calendar Status
*******************************************************************************/
 
  /*****************************************************************************
  * Wire and Reg Declarations
  *****************************************************************************/

  // SPI3 Read FSM 0 Burst Storage 0 Signals
  wire                          SPI3_0_RdEn_n;
  wire                  [7:0]   SPI3_0_Addr;
  wire 	               [31:0]   SPI3_0_Data;
  wire                  [1:0]   SPI3_0_REM;
  wire                          SPI3_0_SOP;
  wire 	                        SPI3_0_EOP;
  wire 	                        SPI3_0_Err;
  wire 	                        SPI3_0_Valid;
  wire									SPI3_0_FFValid;
  wire 	                        Brst_0_Full_n;


  // SPI3 Read FSM 1 to Burst Storage 1 Signals
  wire 	                        SPI3_1_RdEn_n;
  wire 	                [7:0]   SPI3_1_Addr;
  wire                 [31:0]   SPI3_1_Data;
  wire                  [1:0]   SPI3_1_REM;
  wire 	                        SPI3_1_SOP;
  wire 	                        SPI3_1_EOP;
  wire 	                        SPI3_1_Err;
  wire 	                        SPI3_1_Valid;
  wire									SPI3_1_FFValid;
  wire 	                        Brst_1_Full_n;
  

  // SPI3 Read FSM 2 to Burst Storage 2 Signals
  wire 	                        SPI3_2_RdEn_n;
  wire 	                [7:0]   SPI3_2_Addr;
  wire                 [31:0]   SPI3_2_Data;
  wire 	                [1:0]   SPI3_2_REM;
  wire                          SPI3_2_SOP;
  wire 	                        SPI3_2_EOP;
  wire 	                        SPI3_2_Err;
  wire 	                        SPI3_2_Valid;
  wire									SPI3_2_FFValid;
  wire 	                        Brst_2_Full_n;

  // SPI3 Read FSM 3 to Burst Storage 3 Signals
  wire                          SPI3_3_RdEn_n;
  wire 	                [7:0]   SPI3_3_Addr;
  wire 	               [31:0]   SPI3_3_Data;
  wire 	                [1:0]   SPI3_3_REM;
  wire 	                        SPI3_3_SOP;
  wire 	                        SPI3_3_EOP;
  wire 	                        SPI3_3_Err;
  wire 	                        SPI3_3_Valid;
  wire									SPI3_3_FFValid;
  wire 	                        Brst_3_Full_n;

  // SPI3 Arbiter to Burst Storage Signals
  wire 	                [3:0]   Brst_RdEn;
  wire 	                [3:0]   Brst_RdEOP;
  wire 	                [3:0]   Brst_Valid;
  wire 	                [3:0]   Brst_Empty;
  wire 	                [3:0]   Brst_AlmostEmpty;
  wire 	                [3:0]   Brst_PktCntEq0;
  wire 	                [3:0]   Brst_PktCntEq1;
  wire 	                [1:0]   Arbiter_RdChan;
  wire 	                        Arbiter_WrEn;
  
  // Burst FIFOs - Arbiter signals
  wire 	                        Brst_0_RdEn;
  wire 	                        Brst_0_RdEOP;
  wire 	                        Brst_0_Valid;
  wire 	                        Brst_0_Empty;
  wire 	                        Brst_0_AlmostEmpty;
  wire 	                        Brst_0_PktCntEq0;
  wire 	                        Brst_0_PktCntEq1;
 
  // Burst FIFOs - Arbiter signals
  wire 	                        Brst_1_RdEn;
  wire 	                        Brst_1_RdEOP;
  wire 	                        Brst_1_Valid;
  wire 	                        Brst_1_Empty;
  wire 	                        Brst_1_AlmostEmpty;
  wire 	                        Brst_1_PktCntEq0;
  wire 	                        Brst_1_PktCntEq1;
						
  // Burst FIFOs - Arbiter signals
  wire 	                        Brst_2_RdEn;
  wire 	                        Brst_2_RdEOP;
  wire 	                        Brst_2_Valid;
  wire 	                        Brst_2_Empty;
  wire 	                        Brst_2_AlmostEmpty;
  wire 	                        Brst_2_PktCntEq0;
  wire 	                        Brst_2_PktCntEq1;
  
  // Burst FIFOs - Arbiter signals
  wire 	                        Brst_3_RdEn;
  wire 	                        Brst_3_RdEOP;
  wire 	                        Brst_3_Valid;
  wire                          Brst_3_Empty;
  wire                          Brst_3_AlmostEmpty;
  wire 	                        Brst_3_PktCntEq0;
  wire 	                        Brst_3_PktCntEq1;

  // SPI3 0 Burst Storage Output Signals
  wire 	               [7:0]   Brst_0_Addr;
  wire                [63:0]   Brst_0_Data;
  wire 	               [2:0]   Brst_0_Mod;
  wire                         Brst_0_SOP;
  wire 	                       Brst_0_EOP;
  wire 	                       Brst_0_Err;
  
  // SPI3 1 Burst to Write Logic Signals
  wire                 [7:0]   Brst_1_Addr;
  wire                [63:0]   Brst_1_Data;
  wire                 [2:0]   Brst_1_Mod;
  wire 	                       Brst_1_SOP;
  wire 	                       Brst_1_EOP;
  wire 	                       Brst_1_Err;
 
  // SPI3 2 Burst to Write Logic Signals
  wire                  [7:0]   Brst_2_Addr;
  wire                 [63:0]   Brst_2_Data;
  wire                  [2:0]   Brst_2_Mod;
  wire                          Brst_2_SOP;
  wire                          Brst_2_EOP;
  wire                          Brst_2_Err;
 
  // SPI3 3 Burst to Write Logic Signals
  wire                  [7:0]   Brst_3_Addr;
  wire                 [63:0]   Brst_3_Data;
  wire                  [2:0]   Brst_3_Mod;
  wire                          Brst_3_SOP;
  wire                          Brst_3_EOP;
  wire                          Brst_3_Err;

   
  assign   Brst_0_RdEn          =  Brst_RdEn[0];           
  assign   Brst_RdEOP[0]        =  Brst_0_RdEOP;
  assign   Brst_Valid[0]        =  Brst_0_Valid;
  assign   Brst_Empty[0]        =  Brst_0_Empty;
  assign   Brst_AlmostEmpty[0]  =  Brst_0_AlmostEmpty;
  assign   Brst_PktCntEq0[0]    =  Brst_0_PktCntEq0;
  assign   Brst_PktCntEq1[0]    =  Brst_0_PktCntEq1;

  assign   Brst_1_RdEn          =  Brst_RdEn[1];           
  assign   Brst_RdEOP[1]        =  Brst_1_RdEOP;
  assign   Brst_Valid[1]        =  Brst_1_Valid;
  assign   Brst_Empty[1]        =  Brst_1_Empty;
  assign   Brst_AlmostEmpty[1]  =  Brst_1_AlmostEmpty;
  assign   Brst_PktCntEq0[1]    =  Brst_1_PktCntEq0;
  assign   Brst_PktCntEq1[1]    =  Brst_1_PktCntEq1;  


  assign   Brst_2_RdEn          =  Brst_RdEn[2];           
  assign   Brst_RdEOP[2]        =  Brst_2_RdEOP;
  assign   Brst_Valid[2]        =  Brst_2_Valid;
  assign   Brst_Empty[2]        =  Brst_2_Empty;
  assign   Brst_AlmostEmpty[2]  =  Brst_2_AlmostEmpty;
  assign   Brst_PktCntEq0[2]    =  Brst_2_PktCntEq0;
  assign   Brst_PktCntEq1[2]    =  Brst_2_PktCntEq1;

  assign   Brst_3_RdEn          =  Brst_RdEn[3];           
  assign   Brst_RdEOP[3]        =  Brst_3_RdEOP;
  assign   Brst_Valid[3]        =  Brst_3_Valid;
  assign   Brst_Empty[3]        =  Brst_3_Empty;
  assign   Brst_AlmostEmpty[3]  =  Brst_3_AlmostEmpty;
  assign   Brst_PktCntEq0[3]    =  Brst_3_PktCntEq0;
  assign   Brst_PktCntEq1[3]    =  Brst_3_PktCntEq1;
  
  /*****************************************************************************
  * SPI3 to SPI4 Arbiter Component Instantiation
  *****************************************************************************/
  spi3_to_spi4_arbiter spi3_to_spi4_arbiter0
     (
      // System signals
      .Reset_n(Reset_n), 
      .BridgeClk(SPI3_Clk), 
  
      // SPI4 Source Output signals
      .SPI4_Status(SPI4_Stat),
      .SPI4_AlmostFull_n(SPI4_AlmostFull_n),
      
      // Burst Storage Outputs
      .BrstRdEn(Brst_RdEn), 
      .BrstRdChan(Arbiter_RdChan),
      .SPI4WrEn(Arbiter_WrEn),
   
      // Burst Storage Inputs
      .BrstRdEOP(Brst_RdEOP), 
      .BrstValid(Brst_Valid),
      .BrstEmpty(Brst_Empty), 
      .BrstAlmostEmpty(Brst_AlmostEmpty), 
      .BrstPktCntEq0(Brst_PktCntEq0), 
      .BrstPktCntEq1(Brst_PktCntEq1)
      );

  /*****************************************************************************
  * SPI3 to SPI4 Write Logic Component Instantiation
  *****************************************************************************/
  spi3_to_spi4_write spi3_to_spi4_write0 
     (
      // System signals
      .Reset_n(Reset_n),
      .BridgeClk(SPI3_Clk),
  
      // SPI3 0 Write Logic Inteface Input Signals
      .SPI3_0_BrstData(Brst_0_Data),
      .SPI3_0_BrstMod(Brst_0_Mod),
      .SPI3_0_BrstSOP(Brst_0_SOP),
      .SPI3_0_BrstEOP(Brst_0_EOP),
      .SPI3_0_BrstErr(Brst_0_Err),
  
      // SPI3 1 Write Logic Inteface Input Signals
      .SPI3_1_BrstData(Brst_1_Data),
      .SPI3_1_BrstMod(Brst_1_Mod),
      .SPI3_1_BrstSOP(Brst_1_SOP),
      .SPI3_1_BrstEOP(Brst_1_EOP),
      .SPI3_1_BrstErr(Brst_1_Err),
  
      // SPI3 2 Write Logic Inteface Input Signals
      .SPI3_2_BrstData(Brst_2_Data),
      .SPI3_2_BrstMod(Brst_2_Mod),
      .SPI3_2_BrstSOP(Brst_2_SOP),
      .SPI3_2_BrstEOP(Brst_2_EOP),
      .SPI3_2_BrstErr(Brst_2_Err),
  
      // SPI3 3 Write Logic Inteface Input Signals
      .SPI3_3_BrstData(Brst_3_Data),
      .SPI3_3_BrstMod(Brst_3_Mod),
      .SPI3_3_BrstSOP(Brst_3_SOP),
      .SPI3_3_BrstEOP(Brst_3_EOP),
      .SPI3_3_BrstErr(Brst_3_Err),
  
      // SPI3 Arbiter Interface Input Signals
      .ArbiterRdChan(Arbiter_RdChan),
      .ArbiterWrEn(Arbiter_WrEn),
  
      // SPI4.2 Interface Output Signals
      .SPI4_WrAddr(SPI4_WrAddr),
      .SPI4_WrData(SPI4_WrData),
      .SPI4_WrMod(SPI4_WrMod),
      .SPI4_WrSOP(SPI4_WrSOP),
      .SPI4_WrEOP(SPI4_WrEOP),
      .SPI4_WrErr(SPI4_WrErr),
      .SPI4_WrEn_n(SPI4_WrEn_n)
      );

  /*****************************************************************************
  * SPI3 to SPI4 Burst Storage #0 Component Instantiation
  *****************************************************************************/
  spi3_to_spi4_burst_storage spi3_to_spi4_burst_storage0 (
      // System signals
      .Reset_n(Reset_n),
      .BridgeClk(SPI3_Clk),
    
      // SPI3 Burst Storage Interface Output Signals
      .SPI3_Addr(SPI3_0_Addr),
      .SPI3_Data(SPI3_0_Data),
      .SPI3_REM(SPI3_0_REM),
      .SPI3_SOP(SPI3_0_SOP),
      .SPI3_EOP(SPI3_0_EOP),
      .SPI3_Err(SPI3_0_Err),
		.SPI3_SRC_RDY(SPI3_0_SRC_RDY),
      .SPI3_Valid(SPI3_0_Valid),
    
      // SPI3 Burst Storage Interface Output Signals
      .BrstFull_n(Brst_0_Full_n),
  
      // SPI3 Write Logic Inteface Output Signals
      .BrstAddr(Brst_0_Addr),
      .BrstData(Brst_0_Data),
      .BrstMod(Brst_0_Mod),
      .BrstSOP(Brst_0_SOP),
      .BrstEOP(Brst_0_EOP),
      .BrstErr(Brst_0_Err),
  
      // SPI3 Arbiter Inteface Input Signals
      .BrstRdEn(Brst_0_RdEn),

      // SPI3 Arbiter Interface Output Signals
      .BrstAlmostEmpty(Brst_0_AlmostEmpty),
      .BrstEmpty(Brst_0_Empty),
      .BrstValid(Brst_0_Valid),
      .BrstRdEOP(Brst_0_RdEOP),
      .PktCntEq0(Brst_0_PktCntEq0),
      .PktCntEq1(Brst_0_PktCntEq1)      
      );

  /*****************************************************************************
  * SPI3 to SPI4 Burst Storage #1 Component Instantiation
  *****************************************************************************/
  spi3_to_spi4_burst_storage spi3_to_spi4_burst_storage1 (
      // System signals
      .Reset_n(Reset_n),
      .BridgeClk(SPI3_Clk),
    
      // SPI3 Burst Storage Interface Output Signals
      .SPI3_Addr(SPI3_1_Addr),
      .SPI3_Data(SPI3_1_Data),
      .SPI3_REM(SPI3_1_REM),
      .SPI3_SOP(SPI3_1_SOP),
      .SPI3_EOP(SPI3_1_EOP),
      .SPI3_Err(SPI3_1_Err),
		.SPI3_SRC_RDY(SPI3_1_SRC_RDY),
      .SPI3_Valid(SPI3_1_Valid),
    
      // SPI3 Burst Storage Interface Output Signals
      .BrstFull_n(Brst_1_Full_n),
  
      // SPI3 Write Logic Inteface Output Signals
      .BrstAddr(Brst_1_Addr),
      .BrstData(Brst_1_Data),
      .BrstMod(Brst_1_Mod),
      .BrstSOP(Brst_1_SOP),
      .BrstEOP(Brst_1_EOP),
      .BrstErr(Brst_1_Err),
  
      // SPI3 Arbiter Inteface Input Signals
      .BrstRdEn(Brst_1_RdEn),

      // SPI3 Arbiter Interface Output Signals
      .BrstAlmostEmpty(Brst_1_AlmostEmpty),
      .BrstEmpty(Brst_1_Empty),
      .BrstValid(Brst_1_Valid),
      .BrstRdEOP(Brst_1_RdEOP),
      .PktCntEq0(Brst_1_PktCntEq0),
      .PktCntEq1(Brst_1_PktCntEq1)      
      );

  /*****************************************************************************
  * SPI3 to SPI4 Burst Storage #2 Component Instantiation
  *****************************************************************************/
  spi3_to_spi4_burst_storage spi3_to_spi4_burst_storage2 (
      // System signals
      .Reset_n(Reset_n),
      .BridgeClk(SPI3_Clk),
    
      // SPI3 Burst Storage Interface Output Signals
      .SPI3_Addr(SPI3_2_Addr),
      .SPI3_Data(SPI3_2_Data),
      .SPI3_REM(SPI3_2_REM),
      .SPI3_SOP(SPI3_2_SOP),
      .SPI3_EOP(SPI3_2_EOP),
      .SPI3_Err(SPI3_2_Err),
		.SPI3_SRC_RDY(SPI3_2_SRC_RDY),
      .SPI3_Valid(SPI3_2_Valid),
    
      // SPI3 Burst Storage Interface Output Signals
      .BrstFull_n(Brst_2_Full_n),
  
      // SPI3 Write Logic Inteface Output Signals
      .BrstAddr(Brst_2_Addr),
      .BrstData(Brst_2_Data),
      .BrstMod(Brst_2_Mod),
      .BrstSOP(Brst_2_SOP),
      .BrstEOP(Brst_2_EOP),
      .BrstErr(Brst_2_Err),
  
      // SPI3 Arbiter Inteface Input Signals
      .BrstRdEn(Brst_2_RdEn),

      // SPI3 Arbiter Interface Output Signals
      .BrstAlmostEmpty(Brst_2_AlmostEmpty),
      .BrstEmpty(Brst_2_Empty),
      .BrstValid(Brst_2_Valid),
      .BrstRdEOP(Brst_2_RdEOP),
      .PktCntEq0(Brst_2_PktCntEq0),
      .PktCntEq1(Brst_2_PktCntEq1)      
      );

  /*****************************************************************************
  * SPI3 to SPI4 Burst Storage #3 Component Instantiation
  *****************************************************************************/
  spi3_to_spi4_burst_storage spi3_to_spi4_burst_storage3 (
      // System signals
      .Reset_n(Reset_n),
      .BridgeClk(SPI3_Clk),
    
      // SPI3 Burst Storage Interface Output Signals
      .SPI3_Addr(SPI3_3_Addr),
      .SPI3_Data(SPI3_3_Data),
      .SPI3_REM(SPI3_3_REM),
      .SPI3_SOP(SPI3_3_SOP),
      .SPI3_EOP(SPI3_3_EOP),
      .SPI3_Err(SPI3_3_Err),
		.SPI3_SRC_RDY(SPI3_3_SRC_RDY),
      .SPI3_Valid(SPI3_3_Valid),
    
      // SPI3 Burst Storage Interface Output Signals
      .BrstFull_n(Brst_3_Full_n),
  
      // SPI3 Write Logic Inteface Output Signals
      .BrstAddr(Brst_3_Addr),
      .BrstData(Brst_3_Data),
      .BrstMod(Brst_3_Mod),
      .BrstSOP(Brst_3_SOP),
      .BrstEOP(Brst_3_EOP),
      .BrstErr(Brst_3_Err),
  
      // SPI3 Arbiter Inteface Input Signals
      .BrstRdEn(Brst_3_RdEn),

      // SPI3 Arbiter Interface Output Signals
      .BrstAlmostEmpty(Brst_3_AlmostEmpty),
      .BrstEmpty(Brst_3_Empty),
      .BrstValid(Brst_3_Valid),
      .BrstRdEOP(Brst_3_RdEOP),
      .PktCntEq0(Brst_3_PktCntEq0),
      .PktCntEq1(Brst_3_PktCntEq1)      
      );

  /*****************************************************************************
  * SPI3 to SPI4 #0 Read Component Instantiation
  *****************************************************************************/
	// Generate the data valid of SPI-3 Interface 0
	assign SPI3_0_FFValid = SPI3_0_SRC_RDY && SPI3_0_DST_RDY;

  spi3_to_spi4_read top_spi3_to_spi4_read0 (
      // System signals
      .Reset_n(Reset_n),
      .BridgeClk(SPI3_Clk),

      // SPI3 Core Interface Input Signals
      .SPI3_FFAddr(SPI3_0_FFAddr),
      .SPI3_FFData(SPI3_0_FFData),
		.SPI3_FFMod(SPI3_0_FFMod),
      .SPI3_FFSOP(SPI3_0_FFSOP),
      .SPI3_FFEOP(SPI3_0_FFEOP),
      .SPI3_FFErr(SPI3_0_FFErr),   
      .SPI3_SRC_RDY(SPI3_0_SRC_RDY),

      // SPI3 Core Interface Output Signals
      .SPI3_DST_RDY(SPI3_0_DST_RDY),

      // SPI3 Burst Storage Interface Input Signals
      .SPI3_Full_n(Brst_0_Full_n),

      // SPI3 Burst Storage Interface Output Signals
      .SPI3_Addr(SPI3_0_Addr),
      .SPI3_Data(SPI3_0_Data),
		.SPI3_REM(SPI3_0_REM),
      .SPI3_SOP(SPI3_0_SOP),
      .SPI3_EOP(SPI3_0_EOP),
      .SPI3_Err(SPI3_0_Err),
      .SPI3_Valid(SPI3_0_Valid) 
      );
  
  /*****************************************************************************
  * SPI3 to SPI4 #1 Read Component Instantiation
  *****************************************************************************/
  
  // Generate the data valid of SPI-3 Interface 1
	assign SPI3_1_FFValid = SPI3_1_SRC_RDY && SPI3_1_DST_RDY;
  
  spi3_to_spi4_read top_spi3_to_spi4_read1 (
      // System signals
      .Reset_n(Reset_n),
      .BridgeClk(SPI3_Clk),

      // SPI3 Core Interface Input Signals
      .SPI3_FFAddr(SPI3_1_FFAddr),
      .SPI3_FFData(SPI3_1_FFData),
		.SPI3_FFMod(SPI3_1_FFMod),
      .SPI3_FFSOP(SPI3_1_FFSOP),
      .SPI3_FFEOP(SPI3_1_FFEOP),
      .SPI3_FFErr(SPI3_1_FFErr),
      .SPI3_SRC_RDY(SPI3_1_SRC_RDY),

      // SPI3 Core Interface Output Signals
      .SPI3_DST_RDY(SPI3_1_DST_RDY),

      // SPI3 Burst Storage Interface Input Signals
      .SPI3_Full_n(Brst_1_Full_n),

      // SPI3 Burst Storage Interface Output Signals
      .SPI3_Addr(SPI3_1_Addr),
      .SPI3_Data(SPI3_1_Data),
		.SPI3_REM(SPI3_1_REM),
      .SPI3_SOP(SPI3_1_SOP),
      .SPI3_EOP(SPI3_1_EOP),
      .SPI3_Err(SPI3_1_Err),
      .SPI3_Valid(SPI3_1_Valid) 
      );
  
  /*****************************************************************************
  * SPI3 to SPI4 #2 Read Component Instantiation
  *****************************************************************************/
  
  // Generate the data valid of SPI-3 Interface 2
  assign SPI3_2_FFValid = SPI3_2_SRC_RDY && SPI3_2_DST_RDY;

  spi3_to_spi4_read top_spi3_to_spi4_read2 (
      // System signals
      .Reset_n(Reset_n),
      .BridgeClk(SPI3_Clk),

      // SPI3 Core Interface Input Signals
      .SPI3_FFAddr(SPI3_2_FFAddr),
      .SPI3_FFData(SPI3_2_FFData),
		.SPI3_FFMod(SPI3_2_FFMod),
      .SPI3_FFSOP(SPI3_2_FFSOP),
      .SPI3_FFEOP(SPI3_2_FFEOP),
      .SPI3_FFErr(SPI3_2_FFErr),
      .SPI3_SRC_RDY(SPI3_2_SRC_RDY),

      // SPI3 Core Interface Output Signals
      .SPI3_DST_RDY(SPI3_2_DST_RDY),

      // SPI3 Burst Storage Interface Input Signals
      .SPI3_Full_n(Brst_2_Full_n),

      // SPI3 Burst Storage Interface Output Signals
      .SPI3_Addr(SPI3_2_Addr),
      .SPI3_Data(SPI3_2_Data),
		.SPI3_REM(SPI3_2_REM),
      .SPI3_SOP(SPI3_2_SOP),
      .SPI3_EOP(SPI3_2_EOP),
      .SPI3_Err(SPI3_2_Err),
      .SPI3_Valid(SPI3_2_Valid) 
      );
  
  /*****************************************************************************
  * SPI3 to SPI4 #3 Read Component Instantiation
  *****************************************************************************/
  
  // Generate the data valid of SPI-3 Interface 3
  assign SPI3_3_FFValid = SPI3_3_SRC_RDY && SPI3_3_DST_RDY;
  
  spi3_to_spi4_read top_spi3_to_spi4_read3 (
      // System signals
      .Reset_n(Reset_n),
      .BridgeClk(SPI3_Clk),

      // SPI3 Core Interface Input Signals
      .SPI3_FFAddr(SPI3_3_FFAddr),
      .SPI3_FFData(SPI3_3_FFData),
		.SPI3_FFMod(SPI3_3_FFMod),
      .SPI3_FFSOP(SPI3_3_FFSOP),
      .SPI3_FFEOP(SPI3_3_FFEOP),
      .SPI3_FFErr(SPI3_3_FFErr),
      .SPI3_SRC_RDY(SPI3_3_SRC_RDY),

      // SPI3 Core Interface Output Signals
      .SPI3_DST_RDY(SPI3_3_DST_RDY),

      // SPI3 Burst Storage Interface Input Signals
      .SPI3_Full_n(Brst_3_Full_n),

      // SPI3 Burst Storage Interface Output Signals
      .SPI3_Addr(SPI3_3_Addr),
      .SPI3_Data(SPI3_3_Data),
		.SPI3_REM(SPI3_3_REM),
      .SPI3_SOP(SPI3_3_SOP),
      .SPI3_EOP(SPI3_3_EOP),
      .SPI3_Err(SPI3_3_Err),
      .SPI3_Valid(SPI3_3_Valid)
      );

endmodule
