/*******************************************************************************
*
*   File Name:  spi3_to_spi4_top.v
*     Version:  1.0
*        Date:  June 11, 2007
*
* Description:  The purpose of the spi3_to_spi4_top.v file is to serve as 
*               the wrapper file for SPI3 to SPI4 Bridge Design, as well as 
*               instantiating the four SPI3 Cores, and the SPI4.2 Core.
*
*      Author:  Zhe Xia
*     Company:  Xilinx Inc.
*
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*
*     (c) Copyright 2003 Xilinx, Inc.
*     All rights reserved.
*
*******************************************************************************/

/*******************************************************************************
* Module Declaration
*******************************************************************************/

module spi3_to_spi4_top (
    Reset_n,
    SPI3_RFClk,
    SPI3_0_RVal,
    SPI3_0_REnb_n,
    SPI3_0_RSx,
    SPI3_0_RSOP,
    SPI3_0_REOP,
    SPI3_0_RDat,
    SPI3_0_RPrty,
    SPI3_0_RMod,
    SPI3_0_RErr,
    SPI3_1_RVal,
    SPI3_1_REnb_n,
    SPI3_1_RSx,
    SPI3_1_RSOP,
    SPI3_1_REOP,
    SPI3_1_RDat,
    SPI3_1_RPrty,
    SPI3_1_RMod,
    SPI3_1_RErr,
    SPI3_2_RVal,
    SPI3_2_REnb_n,
    SPI3_2_RSx,
    SPI3_2_RSOP,
    SPI3_2_REOP,
    SPI3_2_RDat,
    SPI3_2_RPrty,
    SPI3_2_RMod,
    SPI3_2_RErr,
    SPI3_3_RVal,
    SPI3_3_REnb_n,
    SPI3_3_RSx,
    SPI3_3_RSOP,
    SPI3_3_REOP,
    SPI3_3_RDat,
    SPI3_3_RPrty,
    SPI3_3_RMod,
    SPI3_3_RErr,
    SPI4_TDClk_P,
    SPI4_TDClk_N,
    SPI4_TCtl_P,
    SPI4_TCtl_N,
    SPI4_TDat_P,
    SPI4_TDat_N,
    SPI4_TSClk,
    SPI4_TStat,
    SPI3_ParityErr,
    SPI3_DcmReset_RFClk,
    SPI3_Locked_RFClk,
    SPI4_SysClk_P,
    SPI4_SysClk_N,
    SPI4_DcmReset_TDClk,
    SPI4_DcmReset_TSClk,
    SPI4_Locked_TDClk,
    SPI4_Locked_TSClk,
    SPI4_SysClkDiv_GP,
    SPI4_SrcCalendar_M,
    SPI4_SrcCalendar_Len,
    SPI4_SrcCalClk,
    SPI4_SrcCalWrEn_n,
    SPI4_SrcCalAddr,
    SPI4_SrcCalData,
    SPI4_SrcCalDataOut,
    SPI4_SrcEn
    );

    // System signals
    input                   Reset_n;
  
    //SPI3 Rx Clock - common to all 4 SPI3 interfaces   
    input                   SPI3_RFClk;
    
    // SPI3 0 Interface
    input                   SPI3_0_RVal;
    output                  SPI3_0_REnb_n;
    input                   SPI3_0_RSx;
    input                   SPI3_0_RSOP;
    input                   SPI3_0_REOP;
    input          [31:0]   SPI3_0_RDat;
    input                   SPI3_0_RPrty;
    input           [1:0]   SPI3_0_RMod;
    input                   SPI3_0_RErr;

    // SPI3 1 Interface
    input                   SPI3_1_RVal;
    output                  SPI3_1_REnb_n;
    input                   SPI3_1_RSx;
    input                   SPI3_1_RSOP;
    input                   SPI3_1_REOP;
    input           [31:0]  SPI3_1_RDat;
    input                   SPI3_1_RPrty;
    input            [1:0]  SPI3_1_RMod;
    input                   SPI3_1_RErr;

    // SPI3 2 Interface
    input                   SPI3_2_RVal;
    output                  SPI3_2_REnb_n;
    input                   SPI3_2_RSx;
    input                   SPI3_2_RSOP;
    input                   SPI3_2_REOP;
    input           [31:0]  SPI3_2_RDat;
    input                   SPI3_2_RPrty;
    input            [1:0]  SPI3_2_RMod;
    input                   SPI3_2_RErr;

    // SPI3 3 Interface
    input                   SPI3_3_RVal;
    output                  SPI3_3_REnb_n;
    input                   SPI3_3_RSx;
    input                   SPI3_3_RSOP;
    input                   SPI3_3_REOP;
    input           [31:0]  SPI3_3_RDat;
    input                   SPI3_3_RPrty;
    input            [1:0]  SPI3_3_RMod;
    input                   SPI3_3_RErr;

    
    // SPI4.2 Source Interface Signals
    output                  SPI4_TDClk_P;
    output                  SPI4_TDClk_N;
    output                  SPI4_TCtl_P;
    output                  SPI4_TCtl_N;
    output          [15:0]  SPI4_TDat_P;
    output          [15:0]  SPI4_TDat_N;
    
    // SPI4.2 source status (LVTTL) input
    input                   SPI4_TSClk;
    input            [1:0]  SPI4_TStat;

    //SPI3 Parity Error 
    output                  SPI3_ParityErr;

    // SPI3 DCM Clock signals
    input                   SPI3_DcmReset_RFClk;
    output                  SPI3_Locked_RFClk;

    // SPI4.2 System clock signals
    input                   SPI4_SysClk_P;
    input                   SPI4_SysClk_N;

    //SPI4.2 DCM clock signals
    input                   SPI4_DcmReset_TDClk;
    input                   SPI4_DcmReset_TSClk;
    output                  SPI4_Locked_TDClk;
    output                  SPI4_Locked_TSClk;
    output                  SPI4_SysClkDiv_GP;
    
    // SPI4.2 Calendar Programming Signals
    input            [7:0]  SPI4_SrcCalendar_M;
    input            [8:0]  SPI4_SrcCalendar_Len;
    input                   SPI4_SrcCalClk;
    input                   SPI4_SrcCalWrEn_n;
    input            [8:0]  SPI4_SrcCalAddr;
    input            [7:0]  SPI4_SrcCalData;
    output           [7:0]  SPI4_SrcCalDataOut;
    
    // SPI4.2 source enable
    input                   SPI4_SrcEn;


/*******************************************************************************
* Definition of Ports:
********************************************************************************
* System signals
*    Reset_n                 : System Level Reset (active low)
*    SPI3_Clk                : SPI3 Interface Clock. All four SPI3 Rx
*                              interfaces share this clock
*
* SPI 3 interfaces, # = 0-3
*    SPI3_#_RFClk            : SPI3 Sink Clock Input 
*    SPI3_#_RVal             : SPI3 Sink Data Valid (active high)
*    SPI3_#_REnb_n           : SPI3 Sink Write Enable (active low)
*    SPI3_#_RSx              : SPI3 Sink Start of Transfer (active high)
*    SPI3_#_RSOP             : SPI3 Sink Start of Packet (active high)
*    SPI3_#_REOP             : SPI3 Sink End of Packet (active high)
*    SPI3_#_RDat             : SPI3 Sink Data
*    SPI3_#_RPrty            : SPI3 Sink Parity
*    SPI3_#_RMod             : SPI3 Sink Word Modulo
*    SPI3_#_RErr             : SPI3 Sink Error
*
* SPI4.2 interfaces
*    SPI4_SysClk_P/N         : SPI4.2 System Clock input
*    SPI4_TDClk_P/N          : SPI4.2 Source Clock
*    SPI4_TCtl_P/N           : SPI4.2 Source Control
*    SPI4_TDat_P/N           : SPI4.2 Source Data
*    SPI4_TSClk              : SPI4.2 Source Status Clock
*    SPI4_TStat              : SPI4.2 Source Status
*
* SPI3 Parity Error Signal
*    SPI3_ParityErr          : SPI3 Parity Error
*   
* SPI3 DCM Clock Signals
*    SPI3_DcmReset_RFClk     : SPI3 RFClk DCM Reset 
*    SPI3_Locked_RFClk       : SPI3 RFClk DCM Lock
*
* SPI4.2 System Clock Signals
*    SPI4_SysClk_P           : SPI4.2 System Clock LVDS P 
*    SPI4_SysClk_N           : SPI4.2 System Clock LVDS N
*
* SPI4.2 DCM clock signals
*    SPI4_DcmReset_TDClk     : SPI4.2 TDClk DCM Reset
*    SPI4_DcmReset_TSClk     : SPI4.2 TSClk DCM Reset
*    SPI4_Locked_TDClk       : SPI4.2 TDClk DCM Lock
*    SPI4_Locked_TSClk       : SPI4.2 TSClk DCM Lock
*    SPI4_SysClkDiv_GP       : SPI4.2 SysClk DCM DIV(divide by 2)
*    
* SPI4.2 Calendar Programming Signals
*    SPI4_SrcCalendar_M      : SPI4.2 Number of repetitions of
*                              SrcCalendar_Len between framing.
* 
*    SPI4_SrcCalendar_Len    : SPI4.2 Source Calendar Length. 
*    SPI4_SrcCalClk          : SPI4.2 Source Calendar Clock              
*    SPI4_SrcCalWrEn_n       : SPI4.2 Source Calendar Write Enable      
*    SPI4_SrcCalAddr         : SPI4.2 Source Calendar Address 
*    SPI4_SrcCalData         : SPI4.2 Source Calendar Write Data 
*    SPI4_SrcCalDataOut      : SPI4.2 Source Calendar Data Out
*    
* SPI4.2 source enable
*    SPI4_SrcEn              : SPI4.2 Source Enable Signal
*    
*******************************************************************************/

  /*****************************************************************************
  * Reg and Wire Declarations
  *****************************************************************************/
  
  // SPI3 0 Core
  wire          [7:0]  SPI3_0_FFAddr;
  wire         [31:0]  SPI3_0_FFData;
  wire          [1:0]  SPI3_0_FFMod;
  wire                 SPI3_0_FFRdEn_n;
  wire                 SPI3_0_FFSOP;
  wire                 SPI3_0_FFEOP;
  wire                 SPI3_0_FFErr;
  wire                 SPI3_0_DST_RDY;
  wire                 SPI3_0_SRC_RDY;
  wire 			[7:0]	  SPI3_0_ERRBUS;
  wire                 SPI3_0_ParitySelect;
  wire                 SPI3_0_AddrParityErr;
  wire                 SPI3_0_DataParityErr;
  wire          [8:0]  SPI3_0_AFThresAssert;
  wire          [8:0]  SPI3_0_AFThresNegate;

  // SPI3 1 Core
  wire          [7:0]  SPI3_1_FFAddr;
  wire         [31:0]  SPI3_1_FFData;
  wire          [1:0]  SPI3_1_FFMod;
  wire                 SPI3_1_FFRdEn_n;
  wire                 SPI3_1_FFSOP;
  wire                 SPI3_1_FFEOP;
  wire                 SPI3_1_FFErr;
  wire                 SPI3_1_DST_RDY;
  wire                 SPI3_1_SRC_RDY;
  wire 			[7:0]	  SPI3_1_ERRBUS;
  wire                 SPI3_1_ParitySelect;
  wire                 SPI3_1_AddrParityErr;
  wire                 SPI3_1_DataParityErr;
  wire          [8:0]  SPI3_1_AFThresAssert;
  wire          [8:0]  SPI3_1_AFThresNegate;

  // SPI3 2 Core
  wire          [7:0]  SPI3_2_FFAddr;
  wire         [31:0]  SPI3_2_FFData;
  wire          [1:0]  SPI3_2_FFMod;
  wire                 SPI3_2_FFRdEn_n;
  wire                 SPI3_2_FFSOP;
  wire                 SPI3_2_FFEOP;
  wire                 SPI3_2_FFErr;
  wire                 SPI3_2_DST_RDY;
  wire                 SPI3_2_SRC_RDY;
  wire 			[7:0]	  SPI3_2_ERRBUS;
  wire                 SPI3_2_ParitySelect;
  wire                 SPI3_2_AddrParityErr;
  wire                 SPI3_2_DataParityErr;
  wire          [8:0]  SPI3_2_AFThresAssert;
  wire          [8:0]  SPI3_2_AFThresNegate;

  // SPI3 3 Core
  wire          [7:0]  SPI3_3_FFAddr;
  wire         [31:0]  SPI3_3_FFData;
  wire          [1:0]  SPI3_3_FFMod;
  wire                 SPI3_3_FFRdEn_n;
  wire                 SPI3_3_FFSOP;
  wire                 SPI3_3_FFEOP;
  wire                 SPI3_3_FFErr;
  wire                 SPI3_3_DST_RDY;
  wire                 SPI3_3_SRC_RDY;
  wire 			[7:0]	  SPI3_3_ERRBUS;
  wire                 SPI3_3_ParitySelect;
  wire                 SPI3_3_AddrParityErr;
  wire                 SPI3_3_DataParityErr;
  wire          [8:0]  SPI3_3_AFThresAssert;
  wire          [8:0]  SPI3_3_AFThresNegate;

  // SPI3 Clocking
  wire 	               SPI3_RFClk_ibufg;
  wire                 SPI3_RFClk_dcm;
  wire 	               SPI3_RFClk_bufg;
  
  // SPI4.2 Static Configuration Signals
  wire          [3:0]  SPI4_NumDip2Errors;
  wire          [3:0]  SPI4_NumDip2Matches;
  wire          [8:0]  SPI4_SrcAFThresAssert;
  wire          [8:0]  SPI4_SrcAFThresNegate;
  wire         [15:0]  SPI4_DataMaxT;
  wire          [7:0]  SPI4_AlphaData;
  wire          [5:0]  SPI4_SrcBurstLen;
  wire                 SPI4_SrcBurstMode; //new

  // SPI4.2 Source FIFO Interface
  wire 	               SPI4_WrEn_n;
  wire 	        [7:0]  SPI4_WrAddr;
  wire 	       [63:0]  SPI4_WrData;
  wire 	        [2:0]  SPI4_WrMod;
  wire 	               SPI4_WrSOP;
  wire 	               SPI4_WrEOP;
  wire 	               SPI4_WrErr;
  wire 	               SPI4_AlmostFull_n;
  wire 	               SPI4_Overflow_n;

  // SPI4.2 Core Control Signals
  wire                SPI4_IdleRequest;
  wire 	          SPI4_TrainingRequest;
  wire 	          SPI4_SrcOof;
  wire                SPI4_SrcDIP2Err;
  wire                SPI4_SrcPatternErr;
 
    // SPI4.2 Source Status Interface
  wire 	     [31:0]  SPI4_Stat;
  wire 	      [3:0]  SPI4_StatAddr;
  wire 	      [7:0]  SPI4_StatCh;
  wire                   SPI4_StatChValid;

  // SPI4.2 Source Status
  wire 	      [1:0]  SPI4_TStat_i;
  wire 	             SPI4_TSClk_i;
  wire 	             SPI4_TSClk_GP;
  wire 	             SysClkDiv_GPint;
  
  // SPI3 to SPI4.2 Bridge Clocking
  wire 	             SPI3_Clk_ibufg;
  wire 	             SPI3_Clk_bufg;
  wire 	             BridgeClk;

  //SPI3 Parity Err Signals
  reg         [7:0]  SPI3ParityErr_r0;
  reg         [7:0]  SPI3ParityErr_r1;
  reg                SPI3ParityErrout;
  
  wire 					  Reset;
  wire 						gnd;
  wire                  vcc;
  assign gnd = 1'b0;
  assign vcc = 1'b1;
   
   assign SPI4_SysClkDiv_GP  = BridgeClk;

  //Bridge does not request Idle sequences or training patterns on SPI4.2 interface
  assign SPI4_IdleRequest     = 0;
  assign SPI4_TrainingRequest = 0;
   
  /*****************************************************************************
  * SPI4.2 Static Configuration Signal Assignments
  *   Modify as required 
  *****************************************************************************/
  assign   SPI4_NumDip2Errors     = 4;
  assign   SPI4_NumDip2Matches    = 4;
  assign   SPI4_SrcAFThresAssert  = 32;
  assign   SPI4_SrcAFThresNegate  = 32;
  assign   SPI4_DataMaxT          = 4096;
  assign   SPI4_AlphaData         = 2;
  assign   SPI4_SrcBurstLen       = 32;
  assign   SPI4_StatAddr          = 0;
  assign   SPI4_SrcBurstMode      = 0;  //new

  /*****************************************************************************
  * SPI3 Static Configuration Signal Assignments
  *   Modify as required 
  *****************************************************************************/
  // SPI3 #0
  assign   SPI3_0_ParitySelect    = 0;
  assign   SPI3_0_AFThresAssert   = 32;
  assign   SPI3_0_AFThresNegate   = 32;

  // SPI3 #1
  assign   SPI3_1_ParitySelect    = 0;
  assign   SPI3_1_AFThresAssert   = 32;
  assign   SPI3_1_AFThresNegate   = 32;

  // SPI3 #2
  assign   SPI3_2_ParitySelect    = 0;
  assign   SPI3_2_AFThresAssert   = 32;
  assign   SPI3_2_AFThresNegate   = 32;

  // SPI3 #3
  assign   SPI3_3_ParitySelect    = 0;
  assign   SPI3_3_AFThresAssert   = 32;
  assign   SPI3_3_AFThresNegate   = 32;


  /*****************************************************************************
  * SPI3 RFClk:  Assumed that all 4 SPI3 interfaces are synchronous to
  *              SPI3_RFClk
  *****************************************************************************/
  IBUFG SPI3_RFClkIBufg (
      .I(SPI3_RFClk),
      .O(SPI3_RFClk_ibufg)
      );

  DCM rfclk_dcm (
        .CLKIN(SPI3_RFClk_ibufg),
        .CLKFB(SPI3_RFClk_bufg),
        .DSSEN(gnd),
        .PSINCDEC(gnd),
        .PSEN(gnd),
        .PSCLK(gnd),
        .RST(SPI3_DcmReset_RFClk),
        .CLK0(SPI3_RFClk_dcm),
        .CLK90(),
        .CLK180(),
        .CLK270(),
        .CLK2X(),
        .CLK2X180(),
        .CLKDV(),
        .CLKFX(),
        .CLKFX180(),
        .LOCKED(SPI3_Locked_RFClk),
        .PSDONE(),
        .STATUS()
       );

  BUFG SPI3_RFClkBufg (
      .I(SPI3_RFClk_dcm),
      .O(SPI3_RFClk_bufg)
      );

  /*****************************************************************************
  * TStat - Instantiate LVTTL IBUF for the TStat and TSClk pins
  *****************************************************************************/
  IBUF TStat_ibuf0 (
      .I(SPI4_TStat[0]),
      .O(SPI4_TStat_i[0])
      );

  IBUF TStat_ibuf1 (
      .I(SPI4_TStat[1]),
      .O(SPI4_TStat_i[1])
      );

  IBUFG TSClk_ibufg0 (
      .I(SPI4_TSClk),
      .O(SPI4_TSClk_i)
      );

  /*****************************************************************************
  * Bridge Core Clock (SPI3_Clk)
  *****************************************************************************/
  //IBUFG SPI3_ClkIBufg (
  //    .I(SPI3_Clk),
  //    .O(SPI3_Clk_ibufg)
  //    );

  //BUFG SPI3_ClkBufg (
  //    .I(SPI3_Clk_ibufg),
  //    .O(SPI3_Clk_bufg)
  //    );


  /****************************************************************************
  * SPI3 Parity Error: This process capture parity errors on any of the
  * four SPI3 Rx cores and flags  SPI_ParityErr output signal
  ****************************************************************************/
  always @ (negedge Reset_n or posedge BridgeClk)
      begin
        if(Reset_n == 0)
         begin
          SPI3ParityErr_r0 <= 0;
          SPI3ParityErr_r1 <= 0;
          SPI3ParityErrout <= 0;
         end
        else
         begin
          SPI3ParityErr_r0 <= {SPI3_3_ERRBUS[2], SPI3_2_ERRBUS[2],
                               SPI3_1_ERRBUS[2], SPI3_0_ERRBUS[2],
										 SPI3_3_ERRBUS[1], SPI3_2_ERRBUS[1],
                               SPI3_1_ERRBUS[1], SPI3_0_ERRBUS[1]};
          SPI3ParityErr_r1 <= SPI3ParityErr_r0;
          SPI3ParityErrout <= SPI3ParityErr_r1[7] | SPI3ParityErr_r1[6] |
                              SPI3ParityErr_r1[5] | SPI3ParityErr_r1[4] |
			      SPI3ParityErr_r1[3] | SPI3ParityErr_r1[2] |
                              SPI3ParityErr_r1[1] | SPI3ParityErr_r1[0];
         end
      end

      assign SPI3_ParityErr = SPI3ParityErrout;
      
  /*****************************************************************************
  * SPI3 0 Sink Instantiation
  *****************************************************************************/
  assign Reset = ~ Reset_n;
     
   spi3_link_v4_1_spi3_link_rx pl3_rx_top0 
     (
      .rst                (Reset),
      .rfclk              (SPI3_RFClk_bufg),
      .renb               (SPI3_0_REnb_n),
      .rval               (SPI3_0_RVal),
      .rdat               (SPI3_0_RDat),
      .rmod               (SPI3_0_RMod),
      .rprty              (SPI3_0_RPrty),
      .rsop               (SPI3_0_RSOP),
      .reop               (SPI3_0_REOP),
      .rerr               (SPI3_0_RErr),
      .rsx                (SPI3_0_RSx),
      .rx_clk             (BridgeClk),
      .rx_dst_rdy         (SPI3_0_DST_RDY),				//new
      .rx_src_rdy         (SPI3_0_SRC_RDY),				//new
      .rx_sob             (), 								//new
      .rx_sof             (SPI3_0_FFSOP),
      .rx_eof             (SPI3_0_FFEOP),
      .rx_addr            (SPI3_0_FFAddr),
      .rx_data            (SPI3_0_FFData),
      .rx_rem             (SPI3_0_FFMod),
      .rx_err             (SPI3_0_FFErr),
      .rx_errbus          (SPI3_0_ERRBUS)
     );
	  
  /*****************************************************************************
  * SPI3 1 Sink Instantiation
  *****************************************************************************/
   spi3_link_v4_1_spi3_link_rx pl3_rx_top1 
     (
      .rst                (Reset),
      .rfclk              (SPI3_RFClk_bufg),
      .renb               (SPI3_1_REnb_n),
      .rval               (SPI3_1_RVal),
      .rdat               (SPI3_1_RDat),
      .rmod               (SPI3_1_RMod),
      .rprty              (SPI3_1_RPrty),
      .rsop               (SPI3_1_RSOP),
      .reop               (SPI3_1_REOP),
      .rerr               (SPI3_1_RErr),
      .rsx                (SPI3_1_RSx),
      .rx_clk             (BridgeClk),
      .rx_dst_rdy         (SPI3_1_DST_RDY),				//new
      .rx_src_rdy         (SPI3_1_SRC_RDY),				//new
      .rx_sob             (), 								//new
      .rx_sof             (SPI3_1_FFSOP),
      .rx_eof             (SPI3_1_FFEOP),
      .rx_addr            (SPI3_1_FFAddr),
      .rx_data            (SPI3_1_FFData),
      .rx_rem             (SPI3_1_FFMod),
      .rx_err             (SPI3_1_FFErr),
      .rx_errbus          (SPI3_1_ERRBUS)
     );  

  /*****************************************************************************
  * SPI3 1 Sink Instantiation
  *****************************************************************************/
   spi3_link_v4_1_spi3_link_rx pl3_rx_top2 
     (
      .rst                (Reset),
      .rfclk              (SPI3_RFClk_bufg),
      .renb               (SPI3_2_REnb_n),
      .rval               (SPI3_2_RVal),
      .rdat               (SPI3_2_RDat),
      .rmod               (SPI3_2_RMod),
      .rprty              (SPI3_2_RPrty),
      .rsop               (SPI3_2_RSOP),
      .reop               (SPI3_2_REOP),
      .rerr               (SPI3_2_RErr),
      .rsx                (SPI3_2_RSx),
      .rx_clk             (BridgeClk),
      .rx_dst_rdy         (SPI3_2_DST_RDY),				//new
      .rx_src_rdy         (SPI3_2_SRC_RDY),				//new
      .rx_sob             (), 								//new
      .rx_sof             (SPI3_2_FFSOP),
      .rx_eof             (SPI3_2_FFEOP),
      .rx_addr            (SPI3_2_FFAddr),
      .rx_data            (SPI3_2_FFData),
      .rx_rem             (SPI3_2_FFMod),
      .rx_err             (SPI3_2_FFErr),
      .rx_errbus          (SPI3_2_ERRBUS)
     );

  /*****************************************************************************
  * SPI3 3 Sink Instantiation
  *****************************************************************************/
   spi3_link_v4_1_spi3_link_rx pl3_rx_top3 
     (
      .rst                (Reset),
      .rfclk              (SPI3_RFClk_bufg),
      .renb               (SPI3_3_REnb_n),
      .rval               (SPI3_3_RVal),
      .rdat               (SPI3_3_RDat),
      .rmod               (SPI3_3_RMod),
      .rprty              (SPI3_3_RPrty),
      .rsop               (SPI3_3_RSOP),
      .reop               (SPI3_3_REOP),
      .rerr               (SPI3_3_RErr),
      .rsx                (SPI3_3_RSx),
      .rx_clk             (BridgeClk),
      .rx_dst_rdy         (SPI3_3_DST_RDY),				//new
      .rx_src_rdy         (SPI3_3_SRC_RDY),				//new
      .rx_sob             (), 								//new
      .rx_sof             (SPI3_3_FFSOP),
      .rx_eof             (SPI3_3_FFEOP),
      .rx_addr            (SPI3_3_FFAddr),
      .rx_data            (SPI3_3_FFData),
      .rx_rem             (SPI3_3_FFMod),
      .rx_err             (SPI3_3_FFErr),
      .rx_errbus          (SPI3_3_ERRBUS)
     );
      
  /*****************************************************************************
  * SPI4.2 Source Instantiation
  *****************************************************************************/
 
	pl4_v8_1_pl4_src_top pl4_src_top0 
		(
		  // common signals
		 .reset_n               (Reset_n),
		 .srctristateen        (gnd),	 //new
		 .srcfiforeset_n       (vcc),  //new
		 .dcmreset_tdclk       (SPI4_DcmReset_TDClk),
		 .dcmreset_tsclk       (SPI4_DcmReset_TSClk),
		 .locked_tdclk         (SPI4_Locked_TDClk),
		 .locked_tsclk         (SPI4_Locked_TSClk),
		 .dcmlost_tdclk        (),    //new
		 .dcmlost_tsclk        (),		//new
		 .srcclksrdy            (),		//new
		 .sysclk_p             (SPI4_SysClk_P),
		 .sysclk_n             (SPI4_SysClk_N),
	 
		 // source general purpose clocks
		 .sysclkdiv_gp         (BridgeClk),
		 .tsclk_gp             (SPI4_TSClk_GP),
		 .sysclk0_gp           (),
	
	
		 // Source Calendar I/O
		 .srccalendar_m        (SPI4_SrcCalendar_M),
		 .srccalendar_len      (SPI4_SrcCalendar_Len),
		 .srccalclk            (SPI4_SrcCalClk),
		 .srccalwren_n         (SPI4_SrcCalWrEn_n),
		 .srccaladdr           (SPI4_SrcCalAddr),
		 .srccaldata           (SPI4_SrcCalData),
		 .srccaldataout        (SPI4_SrcCalDataOut),	
	
				
		 .srcstatclk           (SPI4_TSClk_GP),
		 .srcstataddr          (SPI4_StatAddr),
	
		 .srcstat              (SPI4_Stat),
		 .srcstatch            (SPI4_StatCh),
		 .srcstatchvalid       (SPI4_StatChValid),	
	  
		 // FIFO I/O
		 .srcffclk             (BridgeClk),
		 .srcffwren_n          (SPI4_WrEn_n),
		 .srcffaddr            (SPI4_WrAddr),
		 .srcffdata            (SPI4_WrData),
		 .srcffmod             (SPI4_WrMod),
		 .srcffsop             (SPI4_WrSOP),
		 .srcffeop             (SPI4_WrEOP),
		 .srcfferr             (SPI4_WrErr),
		 .srcffoverflow_n      (SPI4_Overflow_n),
		 .srcffalmostfull_n    (SPI4_AlmostFull_n),
		
		 // Control and status signals
		 .srcen                (SPI4_SrcEn),
		 .idlerequest          (SPI4_IdleRequest),
		 .trainingrequest      (SPI4_TrainingRequest),
		 .srcoofoverride       (gnd),  	//new
		 .srcoof               (SPI4_SrcOof),
		 .srcdip2err           (SPI4_SrcDIP2Err),
		 .srcstatframeerr      (SrcStatFrameErr),
		 .srcpatternerr        (SPI4_SrcPatternErr),		//new
	  
		 // FIFO status interface
		 .tsclk                (SPI4_TSClk_i),
		 .tstat                (SPI4_TStat_i),
	 
		 // PL4 source bus
		 .tdclk_p              (SPI4_TDClk_P),
		 .tdclk_n              (SPI4_TDClk_N),
		 .tdat_p               (SPI4_TDat_P),
		 .tdat_n               (SPI4_TDat_N),
		 .tctl_p               (SPI4_TCtl_P),
		 .tctl_n               (SPI4_TCtl_N),
	 
		 // Configuration signals
		 .srcafthresassert     (SPI4_SrcAFThresAssert),
		 .srcafthresnegate     (SPI4_SrcAFThresNegate),
		 .datamaxt             (SPI4_DataMaxT),
		 .alphadata            (SPI4_AlphaData),
		 .srcburstlen          (SPI4_SrcBurstLen),
		 .srcburstmode         (SPI4_SrcBurstMode),
		 .numdip2errors        (SPI4_NumDip2Errors),
		 .numdip2matches       (SPI4_NumDip2Matches)
		 )/*synthesis syn_black_box*/;


  /****************************************************************************
  * SPI3 to SPI4.2 Bridge Core Instantiation
  *****************************************************************************/
   
  spi3_to_spi4_core spi3_to_spi4_core0 (
      // System signals
      .Reset_n(Reset_n),
      .SPI3_Clk(BridgeClk),
  
      // SPI4 Interface Signals
      .SPI4_Stat(SPI4_Stat[7:0]),
      .SPI4_AlmostFull_n(SPI4_AlmostFull_n),
      .SPI4_WrAddr(SPI4_WrAddr),
      .SPI4_WrData(SPI4_WrData),
      .SPI4_WrMod(SPI4_WrMod),
      .SPI4_WrSOP(SPI4_WrSOP),
      .SPI4_WrEOP(SPI4_WrEOP),
      .SPI4_WrErr(SPI4_WrErr),
      .SPI4_WrEn_n(SPI4_WrEn_n),
  
      // SPI3 0 Interface Signals
      .SPI3_0_DST_RDY(SPI3_0_DST_RDY),
      .SPI3_0_SRC_RDY(SPI3_0_SRC_RDY),
      .SPI3_0_FFAddr(SPI3_0_FFAddr),
      .SPI3_0_FFData(SPI3_0_FFData),
      .SPI3_0_FFMod(SPI3_0_FFMod),
      .SPI3_0_FFSOP(SPI3_0_FFSOP),
      .SPI3_0_FFEOP(SPI3_0_FFEOP),
      .SPI3_0_FFErr(SPI3_0_FFErr),
  
      // SPI3 1 Interface Signals
      .SPI3_1_DST_RDY(SPI3_1_DST_RDY),
      .SPI3_1_SRC_RDY(SPI3_1_SRC_RDY),
      .SPI3_1_FFAddr(SPI3_1_FFAddr),
      .SPI3_1_FFData(SPI3_1_FFData),
      .SPI3_1_FFMod(SPI3_1_FFMod),
      .SPI3_1_FFSOP(SPI3_1_FFSOP),
      .SPI3_1_FFEOP(SPI3_1_FFEOP),
      .SPI3_1_FFErr(SPI3_1_FFErr),
  
      // SPI3 2 Interface Signals
      .SPI3_2_DST_RDY(SPI3_2_DST_RDY),
      .SPI3_2_SRC_RDY(SPI3_2_SRC_RDY),
      .SPI3_2_FFAddr(SPI3_2_FFAddr),
      .SPI3_2_FFData(SPI3_2_FFData),
      .SPI3_2_FFMod(SPI3_2_FFMod),
      .SPI3_2_FFSOP(SPI3_2_FFSOP),
      .SPI3_2_FFEOP(SPI3_2_FFEOP),
      .SPI3_2_FFErr(SPI3_2_FFErr),
  
      // SPI3 3 Interface Signals
      .SPI3_3_DST_RDY(SPI3_3_DST_RDY),
      .SPI3_3_SRC_RDY(SPI3_3_SRC_RDY),
      .SPI3_3_FFAddr(SPI3_3_FFAddr),
      .SPI3_3_FFData(SPI3_3_FFData),
      .SPI3_3_FFMod(SPI3_3_FFMod),
      .SPI3_3_FFSOP(SPI3_3_FFSOP),
      .SPI3_3_FFEOP(SPI3_3_FFEOP),
      .SPI3_3_FFErr(SPI3_3_FFErr)
      );


endmodule
