/******************************************************************************
*
*   File Name:  spi3_to_spi4_write.v
*     Version:  1.0
*        Date:  June 11, 2007
*
* Description:  The purpose of the spi3_to_spi4_write.v file is to take the 
*               burst data from the four SPI-3 burst storage blocks and write 
*               it into the SPI-4.2 interface.
*
*      Author:  Zhe Xia
*     Company:  Xilinx Inc.
*
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*
*     (c) Copyright 2003 Xilinx, Inc.
*     All rights reserved.
*
*******************************************************************************/

`include "../../hdl/verilog/spi_pkg.v" 
`timescale 1 ns/1 ps
/*******************************************************************************
* Module Declaration
*******************************************************************************/

module spi3_to_spi4_write (
    Reset_n,
    BridgeClk,
    SPI3_0_BrstData,
    SPI3_0_BrstMod,
    SPI3_0_BrstSOP,
    SPI3_0_BrstEOP,
    SPI3_0_BrstErr,
    SPI3_1_BrstData,
    SPI3_1_BrstMod,
    SPI3_1_BrstSOP,
    SPI3_1_BrstEOP,
    SPI3_1_BrstErr,
    SPI3_2_BrstData,
    SPI3_2_BrstMod,
    SPI3_2_BrstSOP,
    SPI3_2_BrstEOP,
    SPI3_2_BrstErr,
    SPI3_3_BrstData,
    SPI3_3_BrstMod,
    SPI3_3_BrstSOP,
    SPI3_3_BrstEOP,
    SPI3_3_BrstErr,
    ArbiterRdChan,
    ArbiterWrEn,
    SPI4_WrAddr,
    SPI4_WrData,
    SPI4_WrMod,
    SPI4_WrSOP,
    SPI4_WrEOP,
    SPI4_WrErr,
    SPI4_WrEn_n
    );



    // System signals
    input                     Reset_n;
    input                     BridgeClk;

    // SPI3 0 Write Logic Inteface Input Signals
    input             [63:0]  SPI3_0_BrstData;
    input              [2:0]  SPI3_0_BrstMod;
    input                     SPI3_0_BrstSOP;
    input                     SPI3_0_BrstEOP;
    input                     SPI3_0_BrstErr;

    // SPI3 1 Write Logic Inteface Input Signals
    input            [63:0]  SPI3_1_BrstData;
    input             [2:0]  SPI3_1_BrstMod;
    input                    SPI3_1_BrstSOP;
    input                    SPI3_1_BrstEOP;
    input                    SPI3_1_BrstErr;

    // SPI3 2 Write Logic Inteface Input Signals
    input            [63:0]  SPI3_2_BrstData;
    input             [2:0]  SPI3_2_BrstMod;
    input                    SPI3_2_BrstSOP;
    input                    SPI3_2_BrstEOP;
    input                    SPI3_2_BrstErr;

    // SPI3 3 Write Logic Inteface Input Signals
    input           [63:0]  SPI3_3_BrstData;
    input            [2:0]  SPI3_3_BrstMod;
    input                   SPI3_3_BrstSOP;
    input                   SPI3_3_BrstEOP;
    input                   SPI3_3_BrstErr;

    // SPI3 Arbiter Interface Input Signals
    input            [1:0]  ArbiterRdChan;
    input                   ArbiterWrEn;

    // SPI4.2 Interface Output Signals
    output           [7:0]  SPI4_WrAddr;
    output          [63:0]  SPI4_WrData;
    output           [2:0]  SPI4_WrMod;
    output                  SPI4_WrSOP;
    output                  SPI4_WrEOP;
    output                  SPI4_WrErr;
    output                  SPI4_WrEn_n;


/*******************************************************************************
* Definition of Ports:
********************************************************************************
* System signals
*    Reset_n               : System Level Reset (active low)
*    SPI3_BridgeClk        : SPI3 Interface Clock 
*
* SPI3 Number 0 Write Logic Interface Input Signals
*    SPI3_0_BrstAddr       : Write Logic Address 
*    SPI3_0_BrstData       : Write Logic Data 
*    SPI3_0_BrstMod        : Write Logic MOD 
*    SPI3_0_BrstSOP        : Write Logic Start of Packet 
*    SPI3_0_BrstEOP        : Write Logic End of Packet  
*    SPI3_0_BrstErr        : Write Logic Error  
*    SPI3_0_BrstRdEn_n     : Write Logic Data will be valid on next clock cycle
*                            (active low)
*
* SPI3 Number 1 Write Logic Interface Input Signals
*    SPI3_1_BrstAddr       : Write Logic Address 
*    SPI3_1_BrstData       : Write Logic Data 
*    SPI3_1_BrstMod        : Write Logic MOD 
*    SPI3_1_BrstSOP        : Write Logic Start of Packet 
*    SPI3_1_BrstEOP        : Write Logic End of Packet  
*    SPI3_1_BrstErr        : Write Logic Error  
*    SPI3_1_BrstRdEn_n     : Write Logic Data will be valid on next clock cycle
*                            (active low)
*
* SPI3 Number 2 Write Logic Interface Input Signals
*    SPI3_2_BrstAddr       : Write Logic Address 
*    SPI3_2_BrstData       : Write Logic Data 
*    SPI3_2_BrstMod        : Write Logic MOD 
*    SPI3_2_BrstSOP        : Write Logic Start of Packet 
*    SPI3_2_BrstEOP        : Write Logic End of Packet  
*    SPI3_2_BrstErr        : Write Logic Error  
*    SPI3_2_BrstRdEn_n     : Write Logic Data will be valid on next clock cycle
*                            (active low)
*
* SPI3 Number 3 Write Logic Interface Input Signals
*    SPI3_3_BrstAddr       : Write Logic Address 
*    SPI3_3_BrstData       : Write Logic Data 
*    SPI3_3_BrstMod        : Write Logic MOD 
*    SPI3_3_BrstSOP        : Write Logic Start of Packet 
*    SPI3_3_BrstEOP        : Write Logic End of Packet  
*    SPI3_3_BrstErr        : Write Logic Error  
*    SPI3_3_BrstRdEn_n     : Write Logic Data will be valid on next clock cycle
*                            (active low)
*
* SPI3 Arbiter Interface Input Signals
*    SPI3_0_BrstWrEn_n     : Burst Write Enable for SPI3 0 (active low)
*    SPI3_1_BrstWrEn_n     : Burst Write Enable for SPI3 1 (active low)
*    SPI3_2_BrstWrEn_n     : Burst Write Enable for SPI3 2 (active low)
*    SPI3_3_BrstWrEn_n     : Burst Write Enable for SPI3 3 (active low)
*
* SPI4 Interface Output Signals
*    SPI3_WrAddr           : SPI4 Interface Address 
*    SPI3_WrData           : SPI4 Interface Data 
*    SPI3_WrMod            : SPI4 Interface MOD 
*    SPI3_WrSOP            : SPI4 Interface Start of Packet 
*    SPI3_WrEOP            : SPI4 Interface End of Packet  
*    SPI3_WrErr            : SPI4 Interface Error  
*    SPI3_WrEn_n           : SPI4 Interface Write Enable (active low)  
*
* SPI3 Burst Storage Interface Output Signals
*    SPI3_Full_n           : Burst Logic Full (active low) 
*******************************************************************************/


  /*****************************************************************************
  * Signal Declarations
  *****************************************************************************/
  reg            [7:0]  Sel_SPI3Addr;
  reg           [63:0]  Sel_SPI3Data;
  reg	         [2:0]  Sel_SPI3Mod;
  reg                   Sel_SPI3SOP;
  reg                   Sel_SPI3EOP;
  reg                   Sel_SPI3Err;
  reg                   SPI4WrEn;

  
  /******************************************************************************
  * Process to register selected Channels Burst Storage Outputs
  *
  *****************************************************************************/
  // spi3reg block
  always @(negedge Reset_n or posedge BridgeClk )
   begin
    if( Reset_n == 0)
     begin
      Sel_SPI3Addr <= # `TFF  0;
      Sel_SPI3Data <= # `TFF  0;
      Sel_SPI3Mod  <= # `TFF  0;
      Sel_SPI3SOP  <= # `TFF  0;
      Sel_SPI3EOP  <= # `TFF  0;
      Sel_SPI3Err  <= # `TFF  0;
      //SPI4WrEn     <= # `TFF  0;
     end
    else
      //SPI4WrEn   <=  ArbiterWrEn;

      case (ArbiterRdChan)
        2'b 00:
          begin
            Sel_SPI3Addr <= 0;
            Sel_SPI3Data <= SPI3_0_BrstData;
            Sel_SPI3Mod  <= SPI3_0_BrstMod;
            Sel_SPI3SOP  <= SPI3_0_BrstSOP;
            Sel_SPI3EOP  <= SPI3_0_BrstEOP;
            Sel_SPI3Err  <= SPI3_0_BrstErr;
          end            
        2'b 01:
          begin
            Sel_SPI3Addr <= 1;
            Sel_SPI3Data <= SPI3_1_BrstData;
            Sel_SPI3Mod  <= SPI3_1_BrstMod;
            Sel_SPI3SOP  <= SPI3_1_BrstSOP;
            Sel_SPI3EOP  <= SPI3_1_BrstEOP;
            Sel_SPI3Err  <= SPI3_1_BrstErr;
          end  
        2'b 10:
          begin
            Sel_SPI3Addr <= 2;
            Sel_SPI3Data <= SPI3_2_BrstData;
            Sel_SPI3Mod  <= SPI3_2_BrstMod;
            Sel_SPI3SOP  <= SPI3_2_BrstSOP;
            Sel_SPI3EOP  <= SPI3_2_BrstEOP;
            Sel_SPI3Err  <= SPI3_2_BrstErr;
          end
        2'b 11:
          begin
            Sel_SPI3Addr <= 3;
            Sel_SPI3Data <= SPI3_3_BrstData;
            Sel_SPI3Mod  <= SPI3_3_BrstMod;
            Sel_SPI3SOP  <= SPI3_3_BrstSOP;
            Sel_SPI3EOP  <= SPI3_3_BrstEOP;
            Sel_SPI3Err  <= SPI3_3_BrstErr;
          end
        default: 
          begin
            Sel_SPI3Addr <= Sel_SPI3Addr;
            Sel_SPI3Data <= Sel_SPI3Data;
            Sel_SPI3Mod  <= Sel_SPI3Mod;
            Sel_SPI3SOP  <= Sel_SPI3SOP;
            Sel_SPI3EOP  <= Sel_SPI3EOP;
            Sel_SPI3Err  <= Sel_SPI3Err;
	  end
      endcase
    
  end 

  /*****************************************************************************
  * Combinatorial Output Signal Assignments
  *****************************************************************************/
  assign	SPI4_WrEn_n = ~ArbiterWrEn;
  assign	SPI4_WrAddr = Sel_SPI3Addr;
  assign	SPI4_WrData = Sel_SPI3Data;
  assign	SPI4_WrMod  = Sel_SPI3Mod;
  assign	SPI4_WrSOP  = Sel_SPI3SOP;
  assign	SPI4_WrEOP  = Sel_SPI3EOP;
  assign	SPI4_WrErr  = Sel_SPI3Err;
  
endmodule
