/******************************************************************************
*
*   File Name:  spi4_to_spi3_burst_storage.v
*     Version:  1.0
*        Date:  June 11, 2007
*
* Description:  The purpose of the spi4_to_spi3_burst_storage.v file is to
*               provide an intermediate FIFO between the SPI-4.2 core and the
*               four SPI-3 cores.
*
*      Author:  Zhe Xia
*     Company:  Xilinx Inc.
*
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*
*     (c) Copyright 2003 Xilinx, Inc.
*     All rights reserved.
*
*******************************************************************************/
`timescale 1 ns/1 ps
/*******************************************************************************
* Module Declaration
*******************************************************************************/

module spi4_to_spi3_burst_storage (
    Reset_n,
    SPI4_Clk,
    SPI4_Addr,
    SPI4_Data,
    SPI4_Mod,
    SPI4_SOP,
    SPI4_EOP,
    SPI4_Err,
    SPI4_Valid,
    SPI4_Dip4Err,
    SPI4_PayloadErr,
    SPI3_0_BrstRdEn,
    SPI3_0_BrstData,
    SPI3_1_BrstRdEn,
    SPI3_1_BrstData,
    SPI3_2_BrstRdEn,
    SPI3_2_BrstData,
    SPI3_3_BrstRdEn,
    SPI3_3_BrstData,
    SPI3_0_Brst_Empty,
    SPI3_1_Brst_Empty,
    SPI3_2_Brst_Empty,
    SPI3_3_Brst_Empty,
    SPI3_0_Brst_HalfFull,
    SPI3_1_Brst_HalfFull,
    SPI3_2_Brst_HalfFull,
    SPI3_3_Brst_HalfFull
    );

    //Generic signals
    input                 Reset_n;
    input                 SPI4_Clk;
    
    //Signals coming from the SPI-4.2 core
    input          [7:0]  SPI4_Addr;
    input         [63:0]  SPI4_Data;
    input          [2:0]  SPI4_Mod;
    input                 SPI4_SOP;
    input                 SPI4_EOP;
    input                 SPI4_Err;
    input                 SPI4_Valid;
    input                 SPI4_Dip4Err;
    input                 SPI4_PayloadErr;
    
    //Burst FIFO signals
    input                 SPI3_0_BrstRdEn;
    output        [69:0]  SPI3_0_BrstData;
    input                 SPI3_1_BrstRdEn;
    output        [69:0]  SPI3_1_BrstData;
    input                 SPI3_2_BrstRdEn;
    output        [69:0]  SPI3_2_BrstData;
    input                 SPI3_3_BrstRdEn;
    output        [69:0]  SPI3_3_BrstData;
    output                SPI3_0_Brst_Empty;
    output                SPI3_1_Brst_Empty;
    output                SPI3_2_Brst_Empty;
    output                SPI3_3_Brst_Empty;
    output                SPI3_0_Brst_HalfFull;
    output                SPI3_1_Brst_HalfFull;
    output                SPI3_2_Brst_HalfFull;
    output                SPI3_3_Brst_HalfFull;


/*******************************************************************************
* Definition of Ports:
********************************************************************************
*    --Generic signals
*    Reset_n              : System Level Reset (active low)
*    SPI4_Clk             : SPI4 Interface Clock 
*    
*    --Signals coming from the SPI*4.2 core
*    SPI4_Addr            : Address read from the SPI-4.2 core by read module
*    SPI4_Data            : Data read from the SPI-4.2 core by read module
*    SPI4_Mod             : Mod read from the SPI-4.2 core by read module
*    SPI4_SOP             : SOP read from the SPI-4.2 core by read module
*    SPI4_EOP             : EOP read from the SPI-4.2 core by read module
*    SPI4_Err             : Err read from the SPI-4.2 core by read module
*    SPI4_Valid           : Valid signal read from the SPI-4.2 core by read
*                           module
*    SPI4_Dip4Err         : Dip4Err read from the SPI*4.2 core by read module
*    SPI4_PayloadErr      : PayloadErr signal read from the SPI*4.2 core by
*                           read module
*    
*    --Burst FIFO signals
*    SPI3_0_BrstRdEn      : Read enable for burst FIFO 0 from write module
*    SPI3_0_BrstData      : Data from burst FIFO 0 for write module
*    SPI3_1_BrstRdEn      : Read enable for burst FIFO 1 from write module
*    SPI3_1_BrstData      : Data from burst FIFO 1 for write module
*    SPI3_2_BrstRdEn      : Read enable for burst FIFO 2 from write module
*    SPI3_2_BrstData      : Data from burst FIFO 2 for write module
*    SPI3_3_BrstRdEn      : Read enable for burst FIFO 3 from write module
*    SPI3_3_BrstData      : Data from burst FIFO 3 for write module
*    SPI3_0_Brst_Empty    : Indicates that burst FIFO 0 is empty
*    SPI3_1_Brst_Empty    : Indicates that burst FIFO 1 is empty
*    SPI3_2_Brst_Empty    : Indicates that burst FIFO 2 is empty
*    SPI3_3_Brst_Empty    : Indicates that burst FIFO 3 is empty
*    SPI3_0_Brst_HalfFull : Indicates that burst FIFO 0 is half full
*    SPI3_1_Brst_HalfFull : Indicates that burst FIFO 1 is half full
*    SPI3_2_Brst_HalfFull : Indicates that burst FIFO 2 is half full
*    SPI3_3_Brst_HalfFull : Indicates that burst FIFO 3 is half full
*******************************************************************************/

/*******************************************************************************
* Reg and Wire Declarations
*******************************************************************************/
      
  reg        [7:0]  SPI4_Addr_d1;
  reg       [63:0]  SPI4_Data_d1;
  reg        [2:0]  SPI4_Mod_d1;
  reg               SPI4_SOP_d1;
  reg               SPI4_EOP_d1;
  reg               SPI4_Err_d1;
  reg               SPI4_Valid_d1;
  reg        [7:0]  SPI4_Addr_d2;
  reg       [63:0]  SPI4_Data_d2;
  reg        [2:0]  SPI4_Mod_d2;
  reg 	            SPI4_SOP_d2;
  reg 	            SPI4_EOP_d2;
  reg 	            SPI4_Valid_d2;
  reg 	            BurstFIFO_0_WrEn;
  reg 	            BurstFIFO_1_WrEn;
  reg 	            BurstFIFO_2_WrEn;
  reg 	            BurstFIFO_3_WrEn;
  reg 	            SPI4_Err_d2;
  wire      [71:0]  FIFO_wr_data;
  wire	     [8:0]  FIFO_0_data_count;
  wire	     [8:0]  FIFO_1_data_count;
  wire	     [8:0]  FIFO_2_data_count;
  wire       [8:0]  FIFO_3_data_count;
  wire	            FIFO_reset;
  wire      [71:0]  SPI3_0_BrstData_temp;
  wire      [71:0]  SPI3_1_BrstData_temp;
  wire      [71:0]  SPI3_2_BrstData_temp;
  wire      [71:0]  SPI3_3_BrstData_temp;

 
  //This pipeline stage is done in order to more easily meet timing.  More
  //pipeline stages can be added if timing problems are encountered after
  //changing the design.

  // pipeline_stage block
  always @ (negedge Reset_n or posedge SPI4_Clk)
  begin
    if (Reset_n == 0)
      //pipeline stage 1
     begin
      SPI4_Addr_d1       <= 0;
      SPI4_Data_d1       <= 0;
      SPI4_Mod_d1        <= 0;
      SPI4_SOP_d1        <= 0;
      SPI4_EOP_d1        <= 0;
      SPI4_Err_d1        <= 0;
      SPI4_Valid_d1      <= 0;
      
      //pipeline stage 2
      SPI4_Addr_d2       <= 0;
      SPI4_Data_d2       <= 0;
      SPI4_Mod_d2        <= 0;
      SPI4_SOP_d2        <= 0;
      SPI4_EOP_d2        <= 0;
      SPI4_Valid_d2      <= 0;
     end
    else 
      //pipeline stage 1
     begin
      SPI4_Addr_d1       <= SPI4_Addr;
      SPI4_Data_d1       <= SPI4_Data;
      SPI4_Mod_d1        <= SPI4_Mod;
      SPI4_SOP_d1        <= SPI4_SOP;
      SPI4_EOP_d1        <= SPI4_EOP;
      SPI4_Err_d1        <= SPI4_Err;    
      SPI4_Valid_d1      <= SPI4_Valid;
      
      //pipeline stage 2
      SPI4_Addr_d2       <= SPI4_Addr_d1;
      SPI4_Data_d2       <= SPI4_Data_d1;
      SPI4_Mod_d2        <= SPI4_Mod_d1;
      SPI4_SOP_d2        <= SPI4_SOP_d1;
      SPI4_EOP_d2        <= SPI4_EOP_d1;
      SPI4_Valid_d2      <= SPI4_Valid_d1;
     end
  end
  
  //Decode the address in order to produce write enables for the FIFOs
  //write_enable_proc block
  always @ (negedge Reset_n or posedge SPI4_Clk)
  begin
    if (Reset_n == 0)
     begin
      BurstFIFO_0_WrEn <= 0;
      BurstFIFO_1_WrEn <= 0;
      BurstFIFO_2_WrEn <= 0;
      BurstFIFO_3_WrEn <= 0;
     end
    else
     begin 
      BurstFIFO_0_WrEn <= 0;
      BurstFIFO_1_WrEn <= 0;
      BurstFIFO_2_WrEn <= 0;
      BurstFIFO_3_WrEn <= 0;
      if (SPI4_Addr_d1 == 0)
        BurstFIFO_0_WrEn <= SPI4_Valid_d1;
      else if (SPI4_Addr_d1 == 8'b 00000001)
        BurstFIFO_1_WrEn <= SPI4_Valid_d1;
      else if (SPI4_Addr_d1 == 8'b 00000010)
        BurstFIFO_2_WrEn <= SPI4_Valid_d1;
      else if (SPI4_Addr_d1 == 8'b 00000011)
        BurstFIFO_3_WrEn <= SPI4_Valid_d1;
     end
  end
  
  //A pipeline of the error signal.  If more robust error reporting is required
  //(Dip4, Payload errors), the extra logic can be added to this process.
  always @ (negedge Reset_n or posedge SPI4_Clk)
  begin
    if (Reset_n == 0)
      SPI4_Err_d2 <= 0;
    else
      SPI4_Err_d2 <= SPI4_Err_d1;
  end
  
  assign FIFO_wr_data = {2'b 00, SPI4_Data_d2, SPI4_SOP_d2, SPI4_EOP_d2,
                         SPI4_Mod_d2, SPI4_Err_d2};
  assign FIFO_reset = ~(Reset_n);

  /*****************************************************************************
  * SPI4 to SPI3 Burst FIFO Logic Component Instantiation 0
  *****************************************************************************/
  
  generic_sfifo_512x72 spi4_to_spi3_burst_fifo0 (
	.clk(SPI4_Clk),
	.rst(FIFO_reset),
    	.din(FIFO_wr_data),
    	.wr_en(BurstFIFO_0_WrEn),
    	.rd_en(SPI3_0_BrstRdEn),
    	.dout(SPI3_0_BrstData_temp),
    	.full(),
    	.empty(SPI3_0_Brst_Empty),
    	.data_count(FIFO_0_data_count)
    );

    
  /*****************************************************************************
  * SPI4 to SPI3 Burst FIFO Logic Component Instantiation 1
  *****************************************************************************/
  generic_sfifo_512x72 spi4_to_spi3_burst_fifo1 (
  	.clk(SPI4_Clk),
  	.rst(FIFO_reset),
	.din(FIFO_wr_data),
	.wr_en(BurstFIFO_1_WrEn),
	.rd_en(SPI3_1_BrstRdEn),
	.dout(SPI3_1_BrstData_temp),
	.full(),
	.empty(SPI3_1_Brst_Empty),
    	.data_count(FIFO_1_data_count)
    );
    
  /*****************************************************************************
  * SPI4 to SPI3 Burst FIFO Logic Component Instantiation 2
  *****************************************************************************/
  generic_sfifo_512x72 spi4_to_spi3_burst_fifo2 (
  	.clk(SPI4_Clk),
	.rst(FIFO_reset),
	.din(FIFO_wr_data),
	.wr_en(BurstFIFO_2_WrEn),
    	.rd_en(SPI3_2_BrstRdEn),
    	.dout(SPI3_2_BrstData_temp),
    	.full(),
    	.empty(SPI3_2_Brst_Empty),
    	.data_count(FIFO_2_data_count)
    );
    
  /*****************************************************************************
  * SPI4 to SPI3 Burst FIFO Logic Component Instantiation 3
  *****************************************************************************/
  generic_sfifo_512x72 spi4_to_spi3_burst_fifo3 (
    	.clk(SPI4_Clk),
	.rst(FIFO_reset),
	.din(FIFO_wr_data),
    	.wr_en(BurstFIFO_3_WrEn),
    	.rd_en(SPI3_3_BrstRdEn),
    	.dout(SPI3_3_BrstData_temp),
    	.full(),
    	.empty(SPI3_3_Brst_Empty),
    	.data_count(FIFO_3_data_count)
    );
    
  //Combinatorial outputs
  assign SPI3_0_Brst_HalfFull = FIFO_0_data_count[8];
  assign SPI3_1_Brst_HalfFull = FIFO_1_data_count[8];
  assign SPI3_2_Brst_HalfFull = FIFO_2_data_count[8];
  assign SPI3_3_Brst_HalfFull = FIFO_3_data_count[8];
  
  assign SPI3_0_BrstData = SPI3_0_BrstData_temp[69:0];
  assign SPI3_1_BrstData = SPI3_1_BrstData_temp[69:0];
  assign SPI3_2_BrstData = SPI3_2_BrstData_temp[69:0];
  assign SPI3_3_BrstData = SPI3_3_BrstData_temp[69:0];

endmodule
