/******************************************************************************
*
*   File Name:  spi4_to_spi3_core.v
*     Version:  1.0
*        Date:  June 11, 2007
*
* Description:  The purpose of the spi4_to_spi3_core.v file is to serve as 
*               the wrapper file for all logic associated with the SPI4 to 
*               SPI3 direction of the Bridge.
*
*      Author:  Zhe Xia
*     Company:  Xilinx Inc.
*
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*
*     (c) Copyright 2003 Xilinx, Inc.
*     All rights reserved.
*
*******************************************************************************/
`timescale 1 ns/1 ps
/*******************************************************************************
* Module Declaration
*******************************************************************************/

module spi4_to_spi3_core (
    Reset_n,
    SPI4_Clk,
    SPI4_FFAddr,
    SPI4_FFData,
    SPI4_FFMod,
    SPI4_FFSOP,
    SPI4_FFEOP,
    SPI4_FFErr,
    SPI4_FFValid,
    SPI4_FFPayloadErr,
    SPI4_FFDIP4Err,
    SPI4_FFEmpty_n,
    SPI4_FFAlmostEmpty_n,
    SPI4_FFRdEn_n,

    SPI3_0_WrEn_n,
    SPI3_0_WrAddr,
    SPI3_0_WrData,
    SPI3_0_WrMod,
    SPI3_0_WrSOP,
    SPI3_0_WrEOP,
    SPI3_0_WrErr,
    SPI3_0_TxPTPA,
    SPI3_0_TxTAdr,
	 SPI3_0_DST_RDY,

    SPI3_1_WrEn_n,
    SPI3_1_WrAddr,
    SPI3_1_WrData,
    SPI3_1_WrMod,
    SPI3_1_WrSOP,
    SPI3_1_WrEOP,
    SPI3_1_WrErr,
    SPI3_1_TxPTPA,
    SPI3_1_TxTAdr,
	 SPI3_1_DST_RDY,

    SPI3_2_WrEn_n,
    SPI3_2_WrAddr,
    SPI3_2_WrData,
    SPI3_2_WrMod,
    SPI3_2_WrSOP,
    SPI3_2_WrEOP,
    SPI3_2_WrErr,
    SPI3_2_TxPTPA,
    SPI3_2_TxTAdr,
	 SPI3_2_DST_RDY,

    SPI3_3_WrEn_n,
    SPI3_3_WrAddr,
    SPI3_3_WrData,
    SPI3_3_WrMod,
    SPI3_3_WrSOP,
    SPI3_3_WrEOP,
    SPI3_3_WrErr,
    SPI3_3_TxPTPA,
    SPI3_3_TxTAdr,
	 SPI3_3_DST_RDY,

    SPI4_Stat,
    SPI4_StatWrEn_n
    );

    // System signals
    input			    	Reset_n;
    input			    	SPI4_Clk;

    // SPI4 Interface Signals
    input			[7:0]  	SPI4_FFAddr;
    input		       [63:0] 	SPI4_FFData;
    input		        [2:0] 	SPI4_FFMod;
    input			    	SPI4_FFSOP;
    input			    	SPI4_FFEOP;
    input			    	SPI4_FFErr;
    input			    	SPI4_FFValid;
    input			    	SPI4_FFPayloadErr;
    input			    	SPI4_FFDIP4Err;
    input			    	SPI4_FFEmpty_n;
    input			    	SPI4_FFAlmostEmpty_n;
    output				SPI4_FFRdEn_n;
    
    // SPI3 0 Interface Signals
    output	    		        SPI3_0_WrEn_n;
    output		       [7:0]    SPI3_0_WrAddr;
    output		      [31:0] 	SPI3_0_WrData;
    output		       [1:0]  	SPI3_0_WrMod;
    output				SPI3_0_WrSOP;
    output				SPI3_0_WrEOP;
    output				SPI3_0_WrErr;
    input				SPI3_0_TxPTPA;
    input		      [7:0]  	SPI3_0_TxTAdr;

     // SPI3 1 Interface Signals
    output				SPI3_1_WrEn_n;
    output		      [7:0]  	SPI3_1_WrAddr;
    output		     [31:0] 	SPI3_1_WrData;
    output		      [1:0]  	SPI3_1_WrMod;
    output				SPI3_1_WrSOP;
    output				SPI3_1_WrEOP;
    output				SPI3_1_WrErr;
    input				SPI3_1_TxPTPA;
    input		      [7:0]     SPI3_1_TxTAdr;

    // SPI3 2 Interface Signals
    output				SPI3_2_WrEn_n;
    output		      [7:0]  	SPI3_2_WrAddr;
    output	             [31:0] 	SPI3_2_WrData;
    output	              [1:0]  	SPI3_2_WrMod;
    output				SPI3_2_WrSOP;
    output				SPI3_2_WrEOP;
    output				SPI3_2_WrErr;
    input				SPI3_2_TxPTPA;
    input		      [7:0]     SPI3_2_TxTAdr;

    // SPI3 3 Interface Signals
    output				SPI3_3_WrEn_n;
    output		      [7:0]  	SPI3_3_WrAddr;
    output	             [31:0] 	SPI3_3_WrData;
    output	              [1:0]  	SPI3_3_WrMod;
    output				SPI3_3_WrSOP;
    output				SPI3_3_WrEOP;
    output				SPI3_3_WrErr;
    input				SPI3_3_TxPTPA;
    input		      [7:0]  	SPI3_3_TxTAdr;

    // SPI4 Flow Control
    output	              [7:0]  	SPI4_Stat;
    output				SPI4_StatWrEn_n;

    // SPI3 Flow Control
    input				SPI3_0_DST_RDY;
    input				SPI3_1_DST_RDY;
    input				SPI3_2_DST_RDY;
    input			   SPI3_3_DST_RDY;

   
/*******************************************************************************
* Definition of Ports:
********************************************************************************
* System signals
*    Reset_n                 : System Level Reset (active low)
*    SPI4_Clk                : SPI4 Interface Clock 
*
* SPI4 Core Interface Input Signals
*    SPI4_FFAddr           : SPI4 Interface Address
*    SPI4_FFData           : SPI4 Interface Data
*    SPI4_FFMod            : SPI4 Interface MOD
*    SPI4_FFSOP            : SPI4 Interface Start of Packet 
*    SPI4_FFEOP            : SPI4 Interface End of Packet 
*    SPI4_FFErr            : SPI4 Interface Error 
*    SPI4_FFValid          : SPI4 Interface Data is Valid 
*    SPI4_FFPayloadErr     : SPI4 Interface Payload Error (current data was
*                            not preceded by a payload control word
*    SPI4_FFDIP4Err        : SPI4 Interface DIP*4 Error was detected on current
*                            end of burst (either EOP or N*credit boundary)
*    SPI4_FFEmpty_n        : SPI4 Interface is Empty (active low) 
*    SPI4_FFAlmostEmpty_n  : SPI4 Interface is Almost Empty (active low)
*
* SPI4 Core Interface Output Signals
*    SPI4_FFRdEn_n         : SPI4 Interface Read Enable (active low)
*
* SPI3 Core 0 Interface Write Output Signals
*    SPI3_0_WrEn_n         : SPI3 Interface Write Enable (active low)
*    SPI3_0_Addr           : SPI3 Interface Write Address 
*    SPI3_0_Data           : SPI3 Interface Write Data 
*    SPI3_0_Mod            : SPI3 Interface Write MOD 
*    SPI3_0_SOP            : SPI3 Interface Write Start of Packet 
*    SPI3_0_EOP            : SPI3 Interface Write End of Packet  
*    SPI3_0_Err            : SPI3 Interface Write Error  
*
* SPI3 Core 1 Interface Write Output Signals
*    SPI3_1_WrEn_n         : SPI3 Interface Write Enable (active low)
*    SPI3_1_Addr           : SPI3 Interface Write Address 
*    SPI3_1_Data           : SPI3 Interface Write Data 
*    SPI3_1_Mod            : SPI3 Interface Write MOD 
*    SPI3_1_SOP            : SPI3 Interface Write Start of Packet 
*    SPI3_1_EOP            : SPI3 Interface Write End of Packet  
*    SPI3_1_Err            : SPI3 Interface Write Error  
*
* SPI3 Core 2 Interface Write Output Signals
*    SPI3_2_WrEn_n         : SPI3 Interface Write Enable (active low)
*    SPI3_2_Addr           : SPI3 Interface Write Address 
*    SPI3_2_Data           : SPI3 Interface Write Data 
*    SPI3_2_Mod            : SPI3 Interface Write MOD 
*    SPI3_2_SOP            : SPI3 Interface Write Start of Packet 
*    SPI3_2_EOP            : SPI3 Interface Write End of Packet  
*    SPI3_2_Err            : SPI3 Interface Write Error  
*
* SPI3 Core 3 Interface Write Output Signals
*    SPI3_3_WrEn_n         : SPI3 Interface Write Enable (active low)
*    SPI3_3_Addr           : SPI3 Interface Write Address 
*    SPI3_3_Data           : SPI3 Interface Write Data 
*    SPI3_3_Mod            : SPI3 Interface Write MOD 
*    SPI3_3_SOP            : SPI3 Interface Write Start of Packet 
*    SPI3_3_EOP            : SPI3 Interface Write End of Packet  
*    SPI3_3_Err            : SPI3 Interface Write Error  
*
* SPI4 Status Output Signals
*    SPI4_StatWrEn_n           : SPI4 Status Write Enable (active low)
*    SPI4_Stat                 : SPI4 Status * 2*bits per channel
*                                (7:6) = Channel 3, (1:0) = Channel 0
* SPI3 Core Status Input Signals
*    SPI3_0_DST_RDY        : SPI3 0 is ready. 
*    SPI3_1_DST_RDY        : SPI3 1 is ready. 
*    SPI3_2_DST_RDY        : SPI3 2 is ready. 
*    SPI3_3_DST_RDY        : SPI3 3 is ready. 
*******************************************************************************/

  /*****************************************************************************
  * Wire and Reg Declarations
  *****************************************************************************/

  // SPI4 Read FSM to SPI4 Write Logic Signals
  wire 			[7:0] 	 SPI4_Addr;
  wire 		       [63:0]	 SPI4_Data;
  wire 			[2:0] 	 SPI4_Mod;
  wire 			         SPI4_SOP;
  wire 				 SPI4_EOP;
  wire 				 SPI4_Err;
  wire 				 SPI4_Valid;
  wire 				 SPI4_Full_n;
  
  wire 		      [69:0] 	 SPI3_0_BrstData;
  wire 		      [69:0] 	 SPI3_1_BrstData;
  wire 		      [69:0] 	 SPI3_2_BrstData;
  wire 		      [69:0] 	 SPI3_3_BrstData;
  wire 				 SPI3_0_BrstRdEn;
  wire 				 SPI3_1_BrstRdEn;
  wire 				 SPI3_2_BrstRdEn;
  wire 				 SPI3_3_BrstRdEn;
  wire 				 SPI3_0_Brst_Empty;
  wire 				 SPI3_1_Brst_Empty;
  wire 				 SPI3_2_Brst_Empty;
  wire 				 SPI3_3_Brst_Empty;
  wire 				 SPI3_0_Brst_HalfFull;
  wire 				 SPI3_1_Brst_HalfFull;
  wire 				 SPI3_2_Brst_HalfFull;
  wire 				 SPI3_3_Brst_HalfFull;
  

  /*****************************************************************************
  * SPI4 to SPI3 Read Logic Component Instantiation
  *****************************************************************************/
  spi4_to_spi3_read spi4_to_spi3_read0 (
    .Reset_n(Reset_n),
    .SPI4_Clk(SPI4_Clk),
    .SPI4_FFAddr(SPI4_FFAddr),
    .SPI4_FFData(SPI4_FFData),
    .SPI4_FFMod(SPI4_FFMod),
    .SPI4_FFSOP(SPI4_FFSOP),
    .SPI4_FFEOP(SPI4_FFEOP),
    .SPI4_FFErr(SPI4_FFErr),
    .SPI4_FFValid(SPI4_FFValid),
    .SPI4_FFPayloadErr(SPI4_FFPayloadErr),
    .SPI4_FFDIP4Err(SPI4_FFDIP4Err),
    .SPI4_FFEmpty_n(SPI4_FFEmpty_n),
    .SPI4_FFAlmostEmpty_n(SPI4_FFAlmostEmpty_n),
    .SPI4_FFRdEn_n(SPI4_FFRdEn_n),
    .SPI4_Addr(SPI4_Addr),
    .SPI4_Data(SPI4_Data),
    .SPI4_Mod(SPI4_Mod),
    .SPI4_SOP(SPI4_SOP),
    .SPI4_EOP(SPI4_EOP),
    .SPI4_Err(SPI4_Err),
    .SPI4_Valid(SPI4_Valid) 
    );
  
  /*****************************************************************************
  * SPI4 to SPI3 Write Logic Component Instantiation
  *****************************************************************************/
  spi4_to_spi3_write spi4_to_spi3_write0 (
    .Reset_n(Reset_n) ,
    .SPI4_Clk(SPI4_Clk) ,
    .SPI3_0_BrstRdEn(SPI3_0_BrstRdEn) ,
    .SPI3_0_BrstData(SPI3_0_BrstData) ,
    .SPI3_1_BrstRdEn(SPI3_1_BrstRdEn) ,
    .SPI3_1_BrstData(SPI3_1_BrstData) ,
    .SPI3_2_BrstRdEn(SPI3_2_BrstRdEn) ,
    .SPI3_2_BrstData(SPI3_2_BrstData) ,
    .SPI3_3_BrstRdEn(SPI3_3_BrstRdEn) ,
    .SPI3_3_BrstData(SPI3_3_BrstData) ,
    .SPI3_0_Brst_Empty(SPI3_0_Brst_Empty) ,
    .SPI3_1_Brst_Empty(SPI3_1_Brst_Empty) ,
    .SPI3_2_Brst_Empty (SPI3_2_Brst_Empty) ,
    .SPI3_3_Brst_Empty(SPI3_3_Brst_Empty) ,
    .SPI3_0_DST_RDY(SPI3_0_DST_RDY) ,
    .SPI3_1_DST_RDY(SPI3_1_DST_RDY) ,
    .SPI3_2_DST_RDY(SPI3_2_DST_RDY) ,
    .SPI3_3_DST_RDY(SPI3_3_DST_RDY) ,
    .SPI3_0_WrEn_n(SPI3_0_WrEn_n) ,
    .SPI3_0_WrData(SPI3_0_WrData) ,
    .SPI3_0_WrMod(SPI3_0_WrMod) ,
    .SPI3_0_WrSOP(SPI3_0_WrSOP) ,
    .SPI3_0_WrEOP(SPI3_0_WrEOP) ,
    .SPI3_0_WrErr(SPI3_0_WrErr),
    .SPI3_0_TxPTPA(SPI3_0_TxPTPA) ,
    .SPI3_0_TxTAdr(SPI3_0_TxTAdr) ,
    .SPI3_1_WrEn_n(SPI3_1_WrEn_n) ,
    .SPI3_1_WrData(SPI3_1_WrData) ,
    .SPI3_1_WrMod(SPI3_1_WrMod) ,
    .SPI3_1_WrSOP(SPI3_1_WrSOP) ,
    .SPI3_1_WrEOP(SPI3_1_WrEOP) ,
    .SPI3_1_WrErr(SPI3_1_WrErr) ,
    .SPI3_1_TxPTPA(SPI3_1_TxPTPA) ,
    .SPI3_1_TxTAdr(SPI3_1_TxTAdr) ,
    .SPI3_2_WrEn_n(SPI3_2_WrEn_n) ,
    .SPI3_2_WrData(SPI3_2_WrData) ,
    .SPI3_2_WrMod(SPI3_2_WrMod) ,
    .SPI3_2_WrSOP(SPI3_2_WrSOP) ,
    .SPI3_2_WrEOP(SPI3_2_WrEOP) ,
    .SPI3_2_WrErr(SPI3_2_WrErr) ,
    .SPI3_2_TxPTPA(SPI3_2_TxPTPA) ,
    .SPI3_2_TxTAdr(SPI3_2_TxTAdr) ,
    .SPI3_3_WrEn_n(SPI3_3_WrEn_n) ,
    .SPI3_3_WrData(SPI3_3_WrData) ,
    .SPI3_3_WrMod(SPI3_3_WrMod) ,
    .SPI3_3_WrSOP(SPI3_3_WrSOP) ,
    .SPI3_3_WrEOP(SPI3_3_WrEOP) ,
    .SPI3_3_WrErr(SPI3_3_WrErr) ,
    .SPI3_3_TxPTPA(SPI3_3_TxPTPA) ,
    .SPI3_3_TxTAdr(SPI3_3_TxTAdr) 
    );                  

  /*****************************************************************************
  * SPI4 to SPI3 Flow Control Logic Component Instantiation
  *****************************************************************************/
  spi4_to_spi3_flow_control spi4_to_spi3_flow_control0 (
    .Reset_n(Reset_n) ,
    .SPI4_Clk(SPI4_Clk) ,
    .SPI3_0_DST_RDY(SPI3_0_DST_RDY) ,
    .SPI3_1_DST_RDY(SPI3_1_DST_RDY) ,
    .SPI3_2_DST_RDY(SPI3_2_DST_RDY) ,
    .SPI3_3_DST_RDY(SPI3_3_DST_RDY) ,
    .SPI3_0_Brst_HalfFull(SPI3_0_Brst_HalfFull),
    .SPI3_1_Brst_HalfFull(SPI3_1_Brst_HalfFull),
    .SPI3_2_Brst_HalfFull(SPI3_2_Brst_HalfFull),
    .SPI3_3_Brst_HalfFull(SPI3_3_Brst_HalfFull),
    .SPI4_StatWrEn_n(SPI4_StatWrEn_n) ,
    .SPI4_Stat(SPI4_Stat) 
    );
    
  /*****************************************************************************
  * SPI4 to SPI3 Burst Storage Logic Component Instantiation
  *****************************************************************************/
  spi4_to_spi3_burst_storage spi4_to_spi3_burst_storage0 (
    .Reset_n(Reset_n) ,
    .SPI4_Clk(SPI4_Clk) ,
    .SPI4_Addr(SPI4_Addr),
    .SPI4_Data(SPI4_Data),
    .SPI4_Mod(SPI4_Mod) ,
    .SPI4_SOP(SPI4_SOP) ,
    .SPI4_EOP(SPI4_EOP) ,
    .SPI4_Err(SPI4_Err) ,
    .SPI4_Valid(SPI4_Valid) ,
    .SPI4_Dip4Err(SPI4_FFDip4Err) ,
    .SPI4_PayloadErr(SPI4_FFPayloadErr) ,
    .SPI3_0_BrstRdEn(SPI3_0_BrstRdEn) ,
    .SPI3_0_BrstData(SPI3_0_BrstData) ,
    .SPI3_1_BrstRdEn(SPI3_1_BrstRdEn) ,
    .SPI3_1_BrstData(SPI3_1_BrstData) ,
    .SPI3_2_BrstRdEn(SPI3_2_BrstRdEn) ,
    .SPI3_2_BrstData(SPI3_2_BrstData) ,
    .SPI3_3_BrstRdEn(SPI3_3_BrstRdEn) ,
    .SPI3_3_BrstData(SPI3_3_BrstData) ,
    .SPI3_0_Brst_Empty(SPI3_0_Brst_Empty) ,
    .SPI3_1_Brst_Empty(SPI3_1_Brst_Empty) ,
    .SPI3_2_Brst_Empty(SPI3_2_Brst_Empty) ,
    .SPI3_3_Brst_Empty(SPI3_3_Brst_Empty) ,
    .SPI3_0_Brst_HalfFull(SPI3_0_Brst_HalfFull) ,
    .SPI3_1_Brst_HalfFull(SPI3_1_Brst_HalfFull) ,
    .SPI3_2_Brst_HalfFull(SPI3_2_Brst_HalfFull) ,
    .SPI3_3_Brst_HalfFull(SPI3_3_Brst_HalfFull)
    );
    
  //Only address 0 of the SPI-3 cores is used
  assign SPI3_0_WrAddr = 0;
  assign SPI3_1_WrAddr = 0;
  assign SPI3_2_WrAddr = 0;
  assign SPI3_3_WrAddr = 0;

endmodule
