/******************************************************************************
*
*   File Name:  spi4_to_spi3_top.v
*     Version:  1.0
*        Date:  June 11, 2004
*
* Description:  The purpose of the spi4_to_spi3_top.v file is to serve as 
*               the wrapper file for the SPI4 to SPI3 Bridge Design, as well
*               as instantiating the four SPI3 Cores, and the SPI4.2 Core.
*
*      Author:  Zhe Xia
*     Company:  Xilinx Inc.
*
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*
*     (c) Copyright 2003 Xilinx, Inc.
*     All rights reserved.
*
*******************************************************************************/

/*******************************************************************************
* Module Declaration
*******************************************************************************/

module spi4_to_spi3_top (
    Reset_n,
    SPI4_Clk,
	 CLK200,
    SPI3_TFClk,
    SPI3_0_TEnb_n,
    SPI3_0_TSx,
    SPI3_0_TSOP,
    SPI3_0_TEOP,
    SPI3_0_TDat,
    SPI3_0_TPrty,
    SPI3_0_TMod,
    SPI3_0_TErr,
    SPI3_0_PTPA,
    SPI3_0_TAdr,

    SPI3_1_TEnb_n,
    SPI3_1_TSx,
    SPI3_1_TSOP,
    SPI3_1_TEOP,
    SPI3_1_TDat,
    SPI3_1_TPrty,
    SPI3_1_TMod,
    SPI3_1_TErr,
    SPI3_1_PTPA,
    SPI3_1_TAdr,

    SPI3_2_TEnb_n,
    SPI3_2_TSx,
    SPI3_2_TSOP,
    SPI3_2_TEOP,
    SPI3_2_TDat,
    SPI3_2_TPrty,
    SPI3_2_TMod,
    SPI3_2_TErr,
    SPI3_2_PTPA,
    SPI3_2_TAdr,

    SPI3_3_TEnb_n,
    SPI3_3_TSx,
    SPI3_3_TSOP,
    SPI3_3_TEOP,
    SPI3_3_TDat,
    SPI3_3_TPrty,
    SPI3_3_TMod,
    SPI3_3_TErr,
    SPI3_3_PTPA,
    SPI3_3_TAdr,
    
    SPI3_DcmReset_TFClk,
    SPI3_TFClk_bufg_out,

    SPI4_DcmReset_RDClk,
    SPI4_Locked_RDClk,
    SPI4_RDClk_P,
    SPI4_RDClk_N,
    SPI4_RCtl_P,
    SPI4_RCtl_N,
    SPI4_RDat_P,
    SPI4_RDat_N,
    SPI4_RSClk,
    SPI4_RStat,
    SPI4_SnkCalendar_M,
    SPI4_SnkCalendar_Len,
    SPI4_SnkCalClk,
    SPI4_SnkCalWrEn_n,
    SPI4_SnkCalAddr,
    SPI4_SnkCalData,
    SPI4_SnkCalDataOut,
    SPI4_SnkOof,
    SPI4_PhaseAlignComplete,
    SPI4_PhaseAlignRequest,
    SPI4_SnkDPAFailed,
    SPI4_SnkEn
    );

    // System signals
    input                Reset_n;
    input                SPI4_Clk;
	 input 					 CLK200;

    // SPI3 0 Interface
    input                SPI3_TFClk;
    output               SPI3_0_TEnb_n;
    output               SPI3_0_TSx;
    output               SPI3_0_TSOP;
    output               SPI3_0_TEOP;
    output       [31:0]  SPI3_0_TDat;
    output               SPI3_0_TPrty;
    output        [1:0]  SPI3_0_TMod;
    output               SPI3_0_TErr;
    input                SPI3_0_PTPA;
    output        [7:0]  SPI3_0_TAdr;

    // SPI3 1 Interface
    output               SPI3_1_TEnb_n;
    output               SPI3_1_TSx;
    output               SPI3_1_TSOP;
    output               SPI3_1_TEOP;
    output       [31:0]  SPI3_1_TDat;
    output               SPI3_1_TPrty;
    output        [1:0]  SPI3_1_TMod;
    output               SPI3_1_TErr;
    input                SPI3_1_PTPA;
    output        [7:0]  SPI3_1_TAdr;
 
    // SPI3 2 Interface
    output               SPI3_2_TEnb_n;
    output               SPI3_2_TSx;
    output               SPI3_2_TSOP;
    output               SPI3_2_TEOP;
    output       [31:0]  SPI3_2_TDat;
    output               SPI3_2_TPrty;
    output        [1:0]  SPI3_2_TMod;
    output               SPI3_2_TErr;
    input                SPI3_2_PTPA;
    output        [7:0]  SPI3_2_TAdr;

    // SPI3 3 Interface
    output               SPI3_3_TEnb_n;
    output               SPI3_3_TSx;
    output               SPI3_3_TSOP;
    output               SPI3_3_TEOP;
    output       [31:0]  SPI3_3_TDat;
    output               SPI3_3_TPrty;
    output        [1:0]  SPI3_3_TMod;
    output               SPI3_3_TErr;
    input                SPI3_3_PTPA;
    output        [7:0]  SPI3_3_TAdr;

    // System clocking input
    input                SPI3_DcmReset_TFClk;
    input                SPI4_DcmReset_RDClk;
    output               SPI4_Locked_RDClk;
    
    // SPI4.2 Interface
    // SPI4.2 sink bus output
    input                SPI4_RDClk_P;
    input                SPI4_RDClk_N;
    input                SPI4_RCtl_P;
    input                SPI4_RCtl_N;
    input        [15:0]  SPI4_RDat_P;
    input        [15:0]  SPI4_RDat_N;

    // SPI4.2 sink status (LVTTL) input
    output               SPI4_RSClk;
    output        [1:0]  SPI4_RStat;
    
    //SPI4.2 Calendar Programming Signals
    input         [7:0]  SPI4_SnkCalendar_M;
    input         [8:0]  SPI4_SnkCalendar_Len;
    input                SPI4_SnkCalClk;
    input                SPI4_SnkCalWrEn_n;
    input         [8:0]  SPI4_SnkCalAddr;
    input         [7:0]  SPI4_SnkCalData;
    output        [7:0]  SPI4_SnkCalDataOut;
    
    //SPI4.2 Out of Frame
    output               SPI4_SnkOof;
    output               SPI4_PhaseAlignComplete;
    input                SPI4_PhaseAlignRequest;
    output               SPI3_TFClk_bufg_out;
    output               SPI4_SnkDPAFailed;
    //PL4 sink enable
    input                SPI4_SnkEn;


/*******************************************************************************
* Definition of ports:
********************************************************************************
* System signals
*    Reset_n                 : System Level Reset (active low)
*    SPI4_Clk                : SPI4 Interface Clock 
*
* SPI 3 interfaces, # = 0-3
*    SPI3_#_TFClk            : SPI3 Source Clock Input 
*    SPI3_#_TEnb_n           : SPI3 Source Write Enable (active low)
*    SPI3_#_TSx              : SPI3 Source Start of Transfer (active high)
*    SPI3_#_TSOP             : SPI3 Source Start of Packet (active high)
*    SPI3_#_TEOP             : SPI3 Source End of Packet (active high)
*    SPI3_#_TDat             : SPI3 Source Data
*    SPI3_#_TPrty            : SPI3 Source Parity
*    SPI3_#_TMod             : SPI3 Source Word Modulo
*    SPI3_#_TErr             : SPI3 Source Error
*
* System clocking input
*    SPI3_DcmReset_TFClk     : DCM Reset for the TFClk
*    SPI4_DcmReset_RDClk     : DCM Reset for the RDClk
*    SPI4_Locked_RDClk       : Signal indicating RDClk has locked
*
* SPI4.2 interfaces
*    SPI4_RDClk_P/N          : SPI4.2 Sink Clock
*    SPI4_RCtl_P/N           : SPI4.2 Sink Control
*    SPI4_RDat_P/N           : SPI4.2 Sink Data
*    SPI4_RSClk              : SPI4.2 Sink Status Clock
*    SPI4_RStat              : SPI4.2 Sink Status 
*
* SPI4.2 Calendar Programming Signals
*    SPI4_SnkCalendar_M      : Calendar repitition value
*    SPI4_SnkCalendar_Len    : How long is the calendar
*    SPI4_SnkCalClk          : Clock to program the calendar with          
*    SPI4_SnkCalWrEn_n       : Write enable for the calendar (active low)
*    SPI4_SnkCalAddr         : Calendar Address to be written
*    SPI4_SnkCalData         : Data (channel number) to be written into the calendar
*    SPI4_SnkCalDataOut      : Data that is read from the calendar
*    
*    --SPI4.2 Out of Frame
*    SPI4_SnkOof             : Sink core is out of frame
*    SPI4_PhaseAlignComplete : New phase alignment is complete
*    SPI4_PhaseAlignRequest  : Request for new phase alignment
*    SPI3_TFClk_bufg_out     : Clock that is not associated w/ phase alignment
*                              to use for resetting DCMs
*    
*    --PL4 sink enable
*    SPI4_SnkEn              : Enable for the SPI-4.2 sink core
*******************************************************************************/

  /*****************************************************************************
  * Wire and Reg Declarations
  *****************************************************************************/

  // SPI4.2 Core
  wire           [7:0]  SPI4_FFAddr;
  wire          [63:0]  SPI4_FFData;
  wire           [2:0]  SPI4_FFMod;
  wire                  SPI4_FFSOP;
  wire                  SPI4_FFEOP;
  wire                  SPI4_FFErr;
  wire                  SPI4_FFValid;
  wire                  SPI4_FFPayloadErr;
  wire                  SPI4_FFDIP4Err;
  wire                  SPI4_FFEmpty_n;
  wire                  SPI4_FFAlmostEmpty_n;
  wire                  SPI4_FFRdEn_n;

  // SPI4.2 Sink Control Signals
  wire                  SPI4_SnkBusErr;
  wire           [7:0]  SPI4_SnkBusErrStat;  //new
  wire                  SPI4_SnkTrainValid;
  wire                  SPI4_SnkAlmostFull_n;
  wire                  SPI4_SnkOverflow_n;
  wire           [8:0]  SPI4_SnkAFThresAssert;
  wire           [8:0]  SPI4_SnkAFThresNegate;
  wire           [1:0]  SPI4_FifoAFMode;
  wire           [3:0]  SPI4_NumDip4Errors;
  wire           [3:0]  SPI4_NumTrainSequences;
  wire           [3:0]  SPI4_NumDip2Errors;
  wire           [3:0]  SPI4_NumDip2Matches;
  wire                  SPI4_RSClkPhase;
  wire                  SPI4_RSClkDiv;

  // SPI3 0 Core
  wire                  SPI3_0_WrEn_n;
  wire           [7:0]  SPI3_0_WrAddr;
  wire          [31:0]  SPI3_0_WrData;
  wire           [1:0]  SPI3_0_WrMod;
  wire           [1:0]  SPI3_0_REM;
  wire                  SPI3_0_WrSOP;
  wire                  SPI3_0_WrEOP;
  wire                  SPI3_0_WrErr;
  wire           [7:0]  SPI3_0_TxTAdr;
  wire                  SPI3_0_TxPTPA;
  wire                  SPI3_0_ParitySelect;
  wire           [8:0]  SPI3_0_MaxCalendarLen;
  wire                  SPI3_0_CalWrEn;
  wire 						SPI3_0_CalEn;
  wire           [8:0]  SPI3_0_CalAddr;
  wire           [7:0]  SPI3_0_CalData;
  wire           [7:0]  SPI3_0_CalDataOut;
  wire                  SPI3_0_HaltTDat;
  wire          			SPI3_0_SRC_RDY;
  wire          			SPI3_0_DST_RDY;
  
      
  // SPI3 1 Core
  wire                  SPI3_1_WrEn_n;
  wire           [7:0]  SPI3_1_WrAddr;
  wire          [31:0]  SPI3_1_WrData;
  wire           [1:0]  SPI3_1_WrMod;
  wire           [1:0]  SPI3_1_REM;
  wire                  SPI3_1_WrSOP;
  wire                  SPI3_1_WrEOP;
  wire                  SPI3_1_WrErr;
  wire           [7:0]  SPI3_1_TxTAdr;
  wire                  SPI3_1_TxPTPA;
  wire                  SPI3_1_ParitySelect;
  wire           [8:0]  SPI3_1_MaxCalendarLen;
  wire                  SPI3_1_CalWrEn;
  wire 						SPI3_1_CalEn;
  wire           [8:0]  SPI3_1_CalAddr;
  wire           [7:0]  SPI3_1_CalData;
  wire           [7:0]  SPI3_1_CalDataOut;
  wire                  SPI3_1_HaltTDat;
  wire          			SPI3_1_SRC_RDY;
  wire          			SPI3_1_DST_RDY;
  
  // SPI3 2 Core
  wire                  SPI3_2_WrEn_n;
  wire           [7:0]  SPI3_2_WrAddr;
  wire          [31:0]  SPI3_2_WrData;
  wire           [1:0]  SPI3_2_WrMod;
  wire           [1:0]  SPI3_2_REM;
  wire                  SPI3_2_WrSOP;
  wire                  SPI3_2_WrEOP;
  wire                  SPI3_2_WrErr;
  wire           [7:0]  SPI3_2_TxTAdr;
  wire                  SPI3_2_TxPTPA;
  wire                  SPI3_2_ParitySelect;
  wire           [8:0]  SPI3_2_MaxCalendarLen;
  wire                  SPI3_2_CalWrEn;
  wire 						SPI3_2_CalEn;
  wire           [8:0]  SPI3_2_CalAddr;
  wire           [7:0]  SPI3_2_CalData;
  wire           [7:0]  SPI3_2_CalDataOut;
  wire                  SPI3_2_HaltTDat;
  wire          			SPI3_2_SRC_RDY;
  wire          			SPI3_2_DST_RDY;
  
  // SPI3 3 Core
  wire                  SPI3_3_WrEn_n;
  wire           [7:0]  SPI3_3_WrAddr;
  wire          [31:0]  SPI3_3_WrData;
  wire           [1:0]  SPI3_3_WrMod;
  wire           [1:0]  SPI3_3_REM;
  wire                  SPI3_3_WrSOP;
  wire                  SPI3_3_WrEOP;
  wire                  SPI3_3_WrErr;
  wire           [7:0]  SPI3_3_TxTAdr;
  wire                  SPI3_3_TxPTPA;
  wire                  SPI3_3_ParitySelect;
  wire           [8:0]  SPI3_3_MaxCalendarLen;
  wire                  SPI3_3_CalWrEn;
  wire 						SPI3_3_CalEn;
  wire           [8:0]  SPI3_3_CalAddr;
  wire           [7:0]  SPI3_3_CalData;
  wire           [7:0]  SPI3_3_CalDataOut;
  wire                  SPI3_3_HaltTDat;
  wire          			SPI3_3_SRC_RDY;
  wire          			SPI3_3_DST_RDY;

  // SPI3 Clocking
  wire                  SPI3_TFClk_ibufg;
  wire                  SPI3_TFClk_dcm;
  wire                  SPI3_TFClk_bufg;
  wire                  SPI3_Locked_TFClk;
   
  // SPI4.2 Sink Status Interface
  wire          [31:0]  SPI4_Stat;
  wire           [7:0]  SPI4_Stat_i;
  wire           [3:0]  SPI4_StatAddr;
  wire          [15:0]  SPI4_StatMask;
  wire                  SPI4_StatWrEn_n;

  // PL4 Sink Status
  wire           [1:0]  SPI4_RStat_i;
  wire                  SPI4_RSClk_i;
  
  // SPI3 Core Resets
  wire                  SPI3_Reset;
  wire                  gnd;
  wire                  vcc;
  wire           [1:0]  calendar_wr_cnt;

  assign gnd = 1'b0;
  assign vcc = 1'b1;

  /*****************************************************************************
  * SPI4.2 Dynamic Configuration Signal Assignments
  *   Modify as required 
  *****************************************************************************/
  assign SPI4_NumDip4Errors     = 15;
  assign SPI4_NumTrainSequences = 4;
  assign SPI4_SnkAFThresAssert  = 32;
  assign SPI4_SnkAFThresNegate  = 32;
  assign SPI4_FifoAFMode        = 2'b01;
  assign SPI4_RSClkPhase        = 0;
  assign SPI4_RSClkDiv          = 0;
  assign SPI4_StatAddr          = 0;
  assign SPI4_StatMask          = 16'b 1111111111111111;

  // Convert status into 32-bit vector to interface to SPI4.2 core
  assign SPI4_Stat[31:8] = 0;
  assign SPI4_Stat[7:0]  = SPI4_Stat_i;

  /*****************************************************************************
  * SPI3 Static Configuration Signal Assignments
  *   Modify as required 
  *****************************************************************************/
  // SPI3 #0
  assign  SPI3_0_ParitySelect    = 0;
  assign  SPI3_0_CalWrEn         = 0;
  assign  SPI3_0_CalEn				= 1;
  assign  SPI3_0_HaltTDat        = 0;
  assign  SPI3_0_MaxCalendarLen  = 0;
  assign  SPI3_0_CalAddr         = 0;
  assign  SPI3_0_CalData         = 0;
  assign  SPI3_0_AFThresAssert   = 32;
  assign  SPI3_0_AFThresNegate   = 32;

  // SPI3 #1
  assign  SPI3_1_ParitySelect    = 0;
  assign  SPI3_1_CalWrEn         = 0;
  assign  SPI3_1_CalEn				= 1;
  assign  SPI3_1_HaltTDat        = 0;
  assign  SPI3_1_MaxCalendarLen  = 0;
  assign  SPI3_1_CalAddr         = 0;
  assign  SPI3_1_CalData         = 0;
  assign  SPI3_1_AFThresAssert   = 32;
  assign  SPI3_1_AFThresNegate   = 32;

  // SPI3 #2
  assign  SPI3_2_ParitySelect    = 0;
  assign  SPI3_2_CalWrEn         = 0;
  assign  SPI3_2_CalEn				= 1;
  assign  SPI3_2_HaltTDat        = 0;
  assign  SPI3_2_MaxCalendarLen  = 0;
  assign  SPI3_2_CalAddr         = 0;
  assign  SPI3_2_CalData         = 0;
  assign  SPI3_2_AFThresAssert   = 32;
  assign  SPI3_2_AFThresNegate   = 32;

  // SPI3 #3
  assign  SPI3_3_ParitySelect    = 0;
  assign  SPI3_3_CalWrEn         = 0;
  assign  SPI3_3_CalEn				= 1;
  assign  SPI3_3_HaltTDat        = 0;
  assign  SPI3_3_MaxCalendarLen  = 0;
  assign  SPI3_3_CalAddr         = 0;
  assign  SPI3_3_CalData         = 0;
  assign  SPI3_3_AFThresAssert   = 32;
  assign  SPI3_3_AFThresNegate   = 32;
  
  //Note: Since the SPI-3 calendars default to 0 values, no initialization of the
  //calendars is needed.

  /****************************************************************************
  * SPI3 TFClk
  *   Assumed that all 4 SPI3 interfaces are synchronous to SPI3_RFClk
  *****************************************************************************/
  IBUFG TFClkIBufg (
      .I(SPI3_TFClk),
      .O( SPI3_TFClk_ibufg)
      );

  DCM tfclk_dcm (
        .CLKIN         (SPI3_TFClk_ibufg),
        .CLKFB         (SPI3_TFClk_bufg),
        .DSSEN         (gnd),
        .PSINCDEC      (gnd),
        .PSEN          (gnd),
        .PSCLK         (gnd),
        .RST           (SPI3_DcmReset_TFClk),
        .CLK0          (SPI3_TFClk_dcm),
        .CLK90         (),
        .CLK180        (),
        .CLK270        (),
        .CLK2X         (),
        .CLK2X180      (),
        .CLKDV         (),
        .CLKFX         (),
        .CLKFX180      (),
        .LOCKED        (SPI3_Locked_TFClk),
        .PSDONE        (),
        .STATUS        ()
       );

  BUFG TFClkBuf (
      .I   (SPI3_TFClk_dcm),
      .O   (SPI3_TFClk_bufg)
      );
      
  assign SPI3_TFClk_bufg_out = SPI3_TFClk_bufg;

  /*****************************************************************************
  * RStat
  *****************************************************************************
  * Instantiate LVTTL OBUF for the RStat and RSClk pins
  *****************************************************************************/
  OBUF RStat_obuf0 (
      .I   (SPI4_RStat_i[0]),
      .O   (SPI4_RStat[0])
      );

  OBUF RStat_obuf1 (
      .I   (SPI4_RStat_i[1]),
      .O   (SPI4_RStat[1])
      );

  OBUF RSClk_obuf0 (
      .I   (SPI4_RSClk_i),
      .O   (SPI4_RSClk)
      );


  /*****************************************************************************
  * SPI3 0 Source Instantiation
  *****************************************************************************/
  //Make sure that DCM has locked before SPI-3 cores are taken out of reset

  assign SPI3_Reset = ~(Reset_n & SPI3_Locked_TFClk);
  assign SPI3_0_SRC_RDY = ~SPI3_0_WrEn_n;
  assign SPI3_1_SRC_RDY = ~SPI3_1_WrEn_n;
  assign SPI3_2_SRC_RDY = ~SPI3_2_WrEn_n;
  assign SPI3_3_SRC_RDY = ~SPI3_3_WrEn_n;
  assign SPI3_0_REM = ~ SPI3_0_WrMod;
  assign SPI3_1_REM = ~ SPI3_1_WrMod;
  assign SPI3_2_REM = ~ SPI3_2_WrMod;
  assign SPI3_3_REM = ~ SPI3_3_WrMod;
  
  spi3_link_v4_1_spi3_link_tx pl3_tx_top0  
     (
      .rst                (SPI3_Reset),
      .tfclk              (SPI3_TFClk_bufg),
      .tenb               (SPI3_0_TEnb_n),
      .tdat               (SPI3_0_TDat),
      .tmod               (SPI3_0_TMod),
      .tprty              (SPI3_0_TPrty),
      .tsop               (SPI3_0_TSOP),
      .teop               (SPI3_0_TEOP),
      .terr               (SPI3_0_TErr),
      .tsx                (SPI3_0_TSx),
      .tadr               (SPI3_0_TAdr),
      .ptpa               (SPI3_0_PTPA),
      .tx_clk             (SPI4_Clk),
      .tx_dst_rdy         (SPI3_0_DST_RDY),
      .tx_src_rdy         (SPI3_0_SRC_RDY),
      .tx_sof             (SPI3_0_WrSOP),
      .tx_eof             (SPI3_0_WrEOP),
      .tx_addr            (SPI3_0_WrAddr),
      .tx_data            (SPI3_0_WrData),
      .tx_rem             (SPI3_0_REM),
      .tx_err             (SPI3_0_WrErr),
      .tx_halt_tdat       (SPI3_0_HaltTDat),
      .tx_cal_clk         (SPI4_Clk),
      .tx_cal_en          (SPI3_0_CalEn),
      .tx_cal_wr_en       (SPI3_0_CalWrEn),
      .tx_cal_addr        (SPI3_0_CalAddr),
      .tx_cal_data        (SPI3_0_CalData),
      .tx_cal_data_out    (SPI3_0_CalDataOut),
      .tx_ptpa            (SPI3_0_TxPTPA),
      .tx_tadr            (SPI3_0_TxTAdr)
      );



  /*****************************************************************************
  * SPI3 1 Source Instantiation
  *****************************************************************************/
  spi3_link_v4_1_spi3_link_tx pl3_tx_top1 
     (
      .rst                (SPI3_Reset),
      .tfclk              (SPI3_TFClk_bufg),
      .tenb               (SPI3_1_TEnb_n),
      .tdat               (SPI3_1_TDat),
      .tmod               (SPI3_1_TMod),
      .tprty              (SPI3_1_TPrty),
      .tsop               (SPI3_1_TSOP),
      .teop               (SPI3_1_TEOP),
      .terr               (SPI3_1_TErr),
      .tsx                (SPI3_1_TSx),
      .tadr               (SPI3_1_TAdr),
      .ptpa               (SPI3_1_PTPA),
      .tx_clk             (SPI4_Clk),
      .tx_dst_rdy         (SPI3_1_DST_RDY),
      .tx_src_rdy         (SPI3_1_SRC_RDY),
      .tx_sof             (SPI3_1_WrSOP),
      .tx_eof             (SPI3_1_WrEOP),
      .tx_addr            (SPI3_1_WrAddr),
      .tx_data            (SPI3_1_WrData),
      .tx_rem             (SPI3_1_REM),
      .tx_err             (SPI3_1_WrErr),
      .tx_halt_tdat       (SPI3_1_HaltTDat),
      .tx_cal_clk         (SPI4_Clk),
      .tx_cal_en          (SPI3_1_CalEn),
      .tx_cal_wr_en       (SPI3_1_CalWrEn),
      .tx_cal_addr        (SPI3_1_CalAddr),
      .tx_cal_data        (SPI3_1_CalData),
      .tx_cal_data_out    (SPI3_1_CalDataOut),
      .tx_ptpa            (SPI3_1_TxPTPA),
      .tx_tadr            (SPI3_1_TxTAdr)
      );
  /*****************************************************************************
  * SPI3 2 Source Instantiation
  *****************************************************************************/
  spi3_link_v4_1_spi3_link_tx pl3_tx_top2
     (
      .rst                (SPI3_Reset),
      .tfclk              (SPI3_TFClk_bufg),
      .tenb               (SPI3_2_TEnb_n),
      .tdat               (SPI3_2_TDat),
      .tmod               (SPI3_2_TMod),
      .tprty              (SPI3_2_TPrty),
      .tsop               (SPI3_2_TSOP),
      .teop               (SPI3_2_TEOP),
      .terr               (SPI3_2_TErr),
      .tsx                (SPI3_2_TSx),
      .tadr               (SPI3_2_TAdr),
      .ptpa               (SPI3_2_PTPA),
      .tx_clk             (SPI4_Clk),
      .tx_dst_rdy         (SPI3_2_DST_RDY),
      .tx_src_rdy         (SPI3_2_SRC_RDY),
      .tx_sof             (SPI3_2_WrSOP),
      .tx_eof             (SPI3_2_WrEOP),
      .tx_addr            (SPI3_2_WrAddr),
      .tx_data            (SPI3_2_WrData),
      .tx_rem             (SPI3_2_REM),
      .tx_err             (SPI3_2_WrErr),
      .tx_halt_tdat       (SPI3_2_HaltTDat),
      .tx_cal_clk         (SPI4_Clk),
      .tx_cal_en          (SPI3_2_CalEn),
      .tx_cal_wr_en       (SPI3_2_CalWrEn),
      .tx_cal_addr        (SPI3_2_CalAddr),
      .tx_cal_data        (SPI3_2_CalData),
      .tx_cal_data_out    (SPI3_2_CalDataOut),
      .tx_ptpa            (SPI3_2_TxPTPA),
      .tx_tadr            (SPI3_2_TxTAdr)
      );

  /*****************************************************************************
  * SPI3 3 Source Instantiation
  *****************************************************************************/
  spi3_link_v4_1_spi3_link_tx pl3_tx_top3 
     (
      .rst                (SPI3_Reset),
      .tfclk              (SPI3_TFClk_bufg),
      .tenb               (SPI3_3_TEnb_n),
      .tdat               (SPI3_3_TDat),
      .tmod               (SPI3_3_TMod),
      .tprty              (SPI3_3_TPrty),
      .tsop               (SPI3_3_TSOP),
      .teop               (SPI3_3_TEOP),
      .terr               (SPI3_3_TErr),
      .tsx                (SPI3_3_TSx),
      .tadr               (SPI3_3_TAdr),
      .ptpa               (SPI3_3_PTPA),
      .tx_clk             (SPI4_Clk),
      .tx_dst_rdy         (SPI3_3_DST_RDY),
      .tx_src_rdy         (SPI3_3_SRC_RDY),
      .tx_sof             (SPI3_3_WrSOP),
      .tx_eof             (SPI3_3_WrEOP),
      .tx_addr            (SPI3_3_WrAddr),
      .tx_data            (SPI3_3_WrData),
      .tx_rem             (SPI3_3_REM),
      .tx_err             (SPI3_3_WrErr),
      .tx_halt_tdat       (SPI3_3_HaltTDat),
      .tx_cal_clk         (SPI4_Clk),
      .tx_cal_en          (SPI3_3_CalEn),
      .tx_cal_wr_en       (SPI3_3_CalWrEn),
      .tx_cal_addr        (SPI3_3_CalAddr),
      .tx_cal_data        (SPI3_3_CalData),
      .tx_cal_data_out    (SPI3_3_CalDataOut),
      .tx_ptpa            (SPI3_3_TxPTPA),
      .tx_tadr            (SPI3_3_TxTAdr)
      );

  /*****************************************************************************
  * SPI4.2 Sink Instantiation
  *****************************************************************************/
   
pl4_v8_1_pl4_snk_top pl4_snk_top0 (
    // common signals
    .reset_n            (Reset_n),
    .snkfiforeset_n     (vcc),    	//new

    .dcmlost_rdclk      (),
    .dcmreset_rdclk     (SPI4_DcmReset_RDClk),
    .locked_rdclk       (SPI4_Locked_RDClk),
    .snkclksrdy         (),
    .phasealignrequest  (SPI4_PhaseAlignRequest),
    .phasealigncomplete (SPI4_PhaseAlignComplete),
    .snkdpafailed       (SPI4_SnkDPAFailed) ,  	//new
    .snkdparamdata      () ,		//new
    .snkdparamaddr      () ,		//new
    .snkdparamvalid     () ,   	//new
    .snkidelayrefclk    (CLK200),	//new


    // sink general purpose clocks
    .rdclkdiv_gp        (),
    .rdclk0_gp          (),

    // Sink Calendar I/O
    .snkcalendar_m      (SPI4_SnkCalendar_M),
    .snkcalendar_len    (SPI4_SnkCalendar_Len),
    .snkcalclk          (SPI4_Clk),
    .snkcalwren_n       (SPI4_SnkCalWrEn_n),
    .snkcaladdr         (SPI4_SnkCalAddr),
    .snkcaldata         (SPI4_SnkCalData),
    .snkcaldataout      (SPI4_SnkCalDataOut),			   
    .snkstatclk         (SPI4_Clk),
    .snkstat            (SPI4_Stat),
    .snkstataddr        (SPI4_StatAddr),
    .snkstatwr_n        (SPI4_StatWrEn_n),
    .snkstatmask        (SPI4_StatMask),			   
 			   
    // FIFO I/O
    .snkffclk           (SPI4_Clk),
    .snkffrden_n        (SPI4_FFRdEn_n),
    .snkffaddr          (SPI4_FFAddr),
    .snkffdata          (SPI4_FFData),
    .snkffmod           (SPI4_FFMod),
    .snkffsop           (SPI4_FFSOP),
    .snkffeop           (SPI4_FFEOP),
    .snkffbursterr      (),		//new
    .snkfferr           (SPI4_FFErr),
    .snkffdip4err       (SPI4_FFDIP4Err),
    .snkffpayloaderr    (),  	//new
    .snkffpayloaddip4   (SPI4_FFPayloadErr),

    .snkffvalid         (SPI4_FFValid),
    .snkffempty_n       (SPI4_FFEmpty_n),
    .snkffalmostempty_n (SPI4_FFAlmostEmpty_n),
    .snkalmostfull_n    (SPI4_SnkAlmostFull_n),
    .snkoverflow_n      (SPI4_SnkOverflow_n),		//new

    // Control and status signals
    .snken              (SPI4_SnkEn),
    .snkoof             (SPI4_SnkOof),
    .snkbuserr          (SPI4_SnkBusErr),
    .snkbuserrstat      (),      //new
    .snktrainvalid      (SPI4_SnkTrainValid),
    .snkdip2errrequest  (gnd),   		//new
 
    // FIFO status interface
    .rsclk              (SPI4_RSClk_i),
    .rstat              (SPI4_RStat_i),

    // PL4 sink bus
    .rdclk_p            (SPI4_RDClk_P),
    .rdclk_n            (SPI4_RDClk_N),
    .rdat_p             (SPI4_RDat_P),
    .rdat_n             (SPI4_RDat_N),
    .rctl_p             (SPI4_RCtl_P),
    .rctl_n             (SPI4_RCtl_N),

    // Configuration signals
    .snkafthresassert   (SPI4_SnkAFThresAssert),
    .snkafthresnegate   (SPI4_SnkAFThresNegate),
    .fifoafmode         (SPI4_FifoAFMode),
    .numdip4errors      (SPI4_NumDip4Errors),
    .numtrainsequences  (SPI4_NumTrainSequences),
    .rsclkphase         (SPI4_RSClkPhase),
    .rsclkdiv           (SPI4_RSClkDiv));




  /*****************************************************************************
  * SPI4.2 to SPI3 Core Logic Instantiation
  *****************************************************************************/
    spi4_to_spi3_core spi4_to_spi3_core0 (
      // System signals
      .Reset_n                (Reset_n),
      .SPI4_Clk               (SPI4_Clk),
  
      // SPI4 Interface Signals
      .SPI4_FFAddr            (SPI4_FFAddr),
      .SPI4_FFData            (SPI4_FFData),
      .SPI4_FFMod             (SPI4_FFMod),
      .SPI4_FFSOP             (SPI4_FFSOP),
      .SPI4_FFEOP             (SPI4_FFEOP),
      .SPI4_FFErr             (SPI4_FFErr),
      .SPI4_FFValid           (SPI4_FFValid),
      .SPI4_FFPayloadErr      (SPI4_FFPayloadErr),
      .SPI4_FFDIP4Err         (SPI4_FFDIP4Err),
      .SPI4_FFEmpty_n         (SPI4_FFEmpty_n),
      .SPI4_FFAlmostEmpty_n   (SPI4_FFAlmostEmpty_n),
      .SPI4_FFRdEn_n          (SPI4_FFRdEn_n),
  
      // SPI3 0 Interface Signals
      .SPI3_0_WrEn_n          (SPI3_0_WrEn_n),
      .SPI3_0_WrAddr          (SPI3_0_WrAddr),
      .SPI3_0_WrData          (SPI3_0_WrData),
      .SPI3_0_WrMod           (SPI3_0_WrMod),
      .SPI3_0_WrSOP           (SPI3_0_WrSOP),
      .SPI3_0_WrEOP           (SPI3_0_WrEOP),
      .SPI3_0_WrErr           (SPI3_0_WrErr),
      .SPI3_0_TxPTPA          (SPI3_0_TxPTPA),
      .SPI3_0_TxTAdr          (SPI3_0_TxTAdr),
		.SPI3_0_DST_RDY			(SPI3_0_DST_RDY),

  
      // SPI3 1 Interface Signals
      .SPI3_1_WrEn_n          (SPI3_1_WrEn_n),
      .SPI3_1_WrAddr          (SPI3_1_WrAddr),
      .SPI3_1_WrData          (SPI3_1_WrData),
      .SPI3_1_WrMod           (SPI3_1_WrMod),
      .SPI3_1_WrSOP           (SPI3_1_WrSOP),
      .SPI3_1_WrEOP           (SPI3_1_WrEOP),
      .SPI3_1_WrErr           (SPI3_1_WrErr),
      .SPI3_1_TxPTPA          (SPI3_1_TxPTPA),
      .SPI3_1_TxTAdr          (SPI3_1_TxTAdr),
		.SPI3_1_DST_RDY			(SPI3_0_DST_RDY),
  
      // SPI3 2 Interface Signals
      .SPI3_2_WrEn_n          (SPI3_2_WrEn_n),
      .SPI3_2_WrAddr          (SPI3_2_WrAddr),
      .SPI3_2_WrData          (SPI3_2_WrData),
      .SPI3_2_WrMod           (SPI3_2_WrMod),
      .SPI3_2_WrSOP           (SPI3_2_WrSOP),
      .SPI3_2_WrEOP           (SPI3_2_WrEOP),
      .SPI3_2_WrErr           (SPI3_2_WrErr),
      .SPI3_2_TxPTPA          (SPI3_2_TxPTPA),
      .SPI3_2_TxTAdr          (SPI3_2_TxTAdr),
		.SPI3_2_DST_RDY			(SPI3_0_DST_RDY),
      
      // SPI3 3 Interface Signals
      .SPI3_3_WrEn_n          (SPI3_3_WrEn_n),
      .SPI3_3_WrAddr          (SPI3_3_WrAddr),
      .SPI3_3_WrData          (SPI3_3_WrData),
      .SPI3_3_WrMod           (SPI3_3_WrMod),
      .SPI3_3_WrSOP           (SPI3_3_WrSOP),
      .SPI3_3_WrEOP           (SPI3_3_WrEOP),
      .SPI3_3_WrErr           (SPI3_3_WrErr),
      .SPI3_3_TxPTPA          (SPI3_3_TxPTPA),
      .SPI3_3_TxTAdr          (SPI3_3_TxTAdr),
		.SPI3_3_DST_RDY			(SPI3_0_DST_RDY),
  
  
      // SPI4.2 Status Interface
      .SPI4_Stat              (SPI4_Stat_i),
      .SPI4_StatWrEn_n        (SPI4_StatWrEn_n)
      );
      
endmodule

