/*****************************************************************************
*
*   File Name:  spi4_to_spi3_write.v
*     Version:  1.0
*        Date:  June 11, 2007
*
* Description:  The purpose of the spi4_to_spi3_write.v file is to take the 
*               data from the SPI*4 read logic and write into one of four SPI3
*               Cores.
*
*      Author:  Zhe Xia
*     Company:  Xilinx Inc.
*
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*
*     (c) Copyright 2003 Xilinx, Inc.
*     All rights reserved.
*
*******************************************************************************/

/*******************************************************************************
* Module Declaration
*******************************************************************************/

module spi4_to_spi3_write (
    Reset_n,
    SPI4_Clk,
    SPI3_0_BrstRdEn,
    SPI3_0_BrstData,
    SPI3_1_BrstRdEn,
    SPI3_1_BrstData,
    SPI3_2_BrstRdEn,
    SPI3_2_BrstData,
    SPI3_3_BrstRdEn,
    SPI3_3_BrstData,
    SPI3_0_Brst_Empty,
    SPI3_1_Brst_Empty,
    SPI3_2_Brst_Empty,
    SPI3_3_Brst_Empty,
    SPI3_0_DST_RDY,
    SPI3_1_DST_RDY,
    SPI3_2_DST_RDY,
    SPI3_3_DST_RDY,
    SPI3_0_WrEn_n,
    SPI3_0_WrData,
    SPI3_0_WrMod,
    SPI3_0_WrSOP,
    SPI3_0_WrEOP,
    SPI3_0_WrErr,
    SPI3_0_TxPTPA,
    SPI3_0_TxTAdr,
    SPI3_1_WrEn_n,
    SPI3_1_WrData,
    SPI3_1_WrMod,
    SPI3_1_WrSOP,
    SPI3_1_WrEOP,
    SPI3_1_WrErr,
    SPI3_1_TxPTPA,
    SPI3_1_TxTAdr,
    SPI3_2_WrEn_n,
    SPI3_2_WrData,
    SPI3_2_WrMod,
    SPI3_2_WrSOP,
    SPI3_2_WrEOP,
    SPI3_2_WrErr,
    SPI3_2_TxPTPA,
    SPI3_2_TxTAdr,
    SPI3_3_WrEn_n,
    SPI3_3_WrData,
    SPI3_3_WrMod,
    SPI3_3_WrSOP,
    SPI3_3_WrEOP,
    SPI3_3_WrErr,
    SPI3_3_TxPTPA,
    SPI3_3_TxTAdr
    );

    // System signals
    input                   Reset_n;
    input                   SPI4_Clk;

    // Signals to/from Burst Storage FIFO
    output                  SPI3_0_BrstRdEn;
    input           [69:0]  SPI3_0_BrstData;
    output                  SPI3_1_BrstRdEn;
    input           [69:0]  SPI3_1_BrstData;
    output                  SPI3_2_BrstRdEn;
    input           [69:0]  SPI3_2_BrstData;
    output                  SPI3_3_BrstRdEn;
    input           [69:0]  SPI3_3_BrstData;
    input                   SPI3_0_Brst_Empty;
    input                   SPI3_1_Brst_Empty;
    input                   SPI3_2_Brst_Empty;
    input                   SPI3_3_Brst_Empty;
    
    // SPI3 Status Signals
    input                   SPI3_0_DST_RDY;
    input                   SPI3_1_DST_RDY;
    input                   SPI3_2_DST_RDY;
    input                   SPI3_3_DST_RDY;

    // SPI3 0 Write Logic Interface
    output                  SPI3_0_WrEn_n;
    output          [31:0]  SPI3_0_WrData;
    output           [1:0]  SPI3_0_WrMod;
    output                  SPI3_0_WrSOP;
    output                  SPI3_0_WrEOP;
    output                  SPI3_0_WrErr;
    input                   SPI3_0_TxPTPA;
    input            [7:0]  SPI3_0_TxTAdr;

    // SPI3 1 Write Logic Interface
    output                  SPI3_1_WrEn_n;
    output          [31:0]  SPI3_1_WrData;
    output           [1:0]  SPI3_1_WrMod;
    output                  SPI3_1_WrSOP;
    output                  SPI3_1_WrEOP;
    output                  SPI3_1_WrErr;
    input                   SPI3_1_TxPTPA;
    input            [7:0]  SPI3_1_TxTAdr;

    // SPI3 2 Write Logic Interface
    output                  SPI3_2_WrEn_n;
    output          [31:0]  SPI3_2_WrData;
    output           [1:0]  SPI3_2_WrMod;
    output                  SPI3_2_WrSOP;
    output                  SPI3_2_WrEOP;
    output                  SPI3_2_WrErr;
    input                   SPI3_2_TxPTPA;
    input            [7:0]  SPI3_2_TxTAdr;

    // SPI3 3 Write Logic Interface
    output                  SPI3_3_WrEn_n;
    output          [31:0]  SPI3_3_WrData;
    output           [1:0]  SPI3_3_WrMod;
    output                  SPI3_3_WrSOP;
    output                  SPI3_3_WrEOP;
    output                  SPI3_3_WrErr;
    input                   SPI3_3_TxPTPA;
    input            [7:0]  SPI3_3_TxTAdr;


/*******************************************************************************
* Definition of Ports:
*******************************************************************************
*    -- System signals
*    Reset_n               : Active-Low system reset
*    SPI4_Clk              : Clock that is used for the bridge, derived from a
*                            SPI4 core clock
*
*    -- Signals to/from Burst Storage FIFO
*    SPI3_0_BrstRdEn      : Read Enable for Burst FIFO, channel 0
*    SPI3_0_BrstData      : Data received from Burst FIFO, channel 0
*    SPI3_1_BrstRdEn      : Read Enable for Burst FIFO, channel 1
*    SPI3_1_BrstData      : Data received from Burst FIFO, channel 1
*    SPI3_2_BrstRdEn      : Read Enable for Burst FIFO, channel 2
*    SPI3_2_BrstData      : Data received from Burst FIFO, channel 2
*    SPI3_3_BrstRdEn      : Read Enable for Burst FIFO, channel 3
*    SPI3_3_BrstData      : Data received from Burst FIFO, channel 3
*    SPI3_0_Brst_Empty    : Empty indication from Burst FIFO, channel 0
*    SPI3_1_Brst_Empty    : Empty indication from Burst FIFO, channel 1
*    SPI3_2_Brst_Empty    : Empty indication from Burst FIFO, channel 2
*    SPI3_3_Brst_Empty    : Empty indication from Burst FIFO, channel 3
*    
*    -- SPI3 Status Signals
*    SPI3_0_DST_RDY 		  : The ready signal from SPI3 0 core. Indicates 
*									the core is ready 
*    SPI3_1_DST_RDY 		  : The ready signal from SPI3 1 core. Indicates 
*									the core is ready 
*    SPI3_2_DST_RDY 		  : The ready signal from SPI3 2 core. Indicates 
*									the core is ready 
*    SPI3_3_DST_RDY 		  : The ready signal from SPI3 3 core. Indicates 
*									the core is ready 
*    -- SPI3 0 Write Logic Interface
*    SPI3_0_WrEn_n       : Write Enable signal (active-low) for SPI3 transmit
*                          core 0
*    SPI3_0_WrData       : Data to be written into SPI3 transmit core 0
*    SPI3_0_WrMod        : Mod signal to be written into SPI3 transmit core 0
*    SPI3_0_WrSOP        : SOP signal to be written into SPI3 transmit core 0
*    SPI3_0_WrEOP        : EOP signal to be written into SPI3 transmit core 0
*    SPI3_0_WrErr        : Err signal to be written into SPI3 transmit core 0
*    SPI3_0_TxPTPA       : Polled status from device connected to SPI3 transmit
*                          core 0
*    SPI3_0_TxTAdr       : Polled status address from device connected to SPI3
*                          transmit core 0
*
*    -- SPI3 1 Write Logic Interface
*    SPI3_1_WrEn_n       : Write Enable signal (active-low) for SPI3 transmit
*                          core 1          
*    SPI3_1_WrData       : Data to be written into SPI3 transmit core 1
*    SPI3_1_WrMod        : Mod signal to be written into SPI3 transmit core 1                 
*    SPI3_1_WrSOP        : SOP signal to be written into SPI3 transmit core 1                 
*    SPI3_1_WrEOP        : EOP signal to be written into SPI3 transmit core 1                 
*    SPI3_1_WrErr        : Err signal to be written into SPI3 transmit core 1                 
*    SPI3_1_TxPTPA       : Polled status from device connected to SPI3 transmit
*                          core 1        
*    SPI3_1_TxTAdr       : Polled status address from device connected to SPI3
*                          transmit core 1
*
*    -- SPI3 2 Write Logic Interface
*    SPI3_2_WrEn_n       : Write Enable signal (active*low) for SPI3 transmit
*                          core 2          
*    SPI3_2_WrData       : Data to be written into SPI3 transmit core 2                       
*    SPI3_2_WrMod        : Mod signal to be written into SPI3 transmit core 2                 
*    SPI3_2_WrSOP        : SOP signal to be written into SPI3 transmit core 2                 
*    SPI3_2_WrEOP        : EOP signal to be written into SPI3 transmit core 2                 
*    SPI3_2_WrErr        : Err signal to be written into SPI3 transmit core 2                 
*    SPI3_2_TxPTPA       : Polled status from device connected to SPI3 transmit
*                          core 2        
*    SPI3_2_TxTAdr       : Polled status address from device connected to SPI3
*                          transmit core 2
*
*    -- SPI3 3 Write Logic Interface
*    SPI3_3_WrEn_n       : Write Enable signal (active*low) for SPI3 transmit
*                          core 3          
*    SPI3_3_WrData       : Data to be written into SPI3 transmit core 3                       
*    SPI3_3_WrMod        : Mod signal to be written into SPI3 transmit core 3                 
*    SPI3_3_WrSOP        : SOP signal to be written into SPI3 transmit core 3                 
*    SPI3_3_WrEOP        : EOP signal to be written into SPI3 transmit core 3                 
*    SPI3_3_WrErr        : Err signal to be written into SPI3 transmit core 3                 
*    SPI3_3_TxPTPA       : Polled status from device connected to SPI3 transmit
*                          core 3        
*    SPI3_3_TxTAdr       : Polled status address from device connected to SPI3
*                          transmit core 3
*******************************************************************************/

  /*****************************************************************************
  * Wire and Reg Declarations
  *****************************************************************************/
  reg                SPI3_0_BrstRdEn_int;
  reg                SPI3_0_BrstRdEn_d1;
  reg         [1:0]  wait_for_conversion0;
  reg         [1:0]  pl3_0_conversion_state;
  reg        [31:0]  Holding_Data0;
  reg                Holding_SOP0;
  reg                Holding_EOP0;
  reg         [1:0]  Holding_Mod0;
  reg                Holding_Err0;
  
  reg                SPI3_1_BrstRdEn_int;
  reg                SPI3_1_BrstRdEn_d1;
  reg          [1:0]      wait_for_conversion1;
  reg         [1:0]  pl3_1_conversion_state;
  reg        [31:0]  Holding_Data1;
  reg                Holding_SOP1;
  reg                Holding_EOP1;
  reg         [1:0]  Holding_Mod1;
  reg                Holding_Err1;
  
  reg                SPI3_2_BrstRdEn_int;
  reg                SPI3_2_BrstRdEn_d1;
  reg         [1:0]  wait_for_conversion2;
  reg         [1:0]  pl3_2_conversion_state;
  reg        [31:0]  Holding_Data2;
  reg                Holding_SOP2;
  reg                Holding_EOP2;
  reg         [1:0]  Holding_Mod2;
  reg                Holding_Err2;
  
  reg                SPI3_3_BrstRdEn_int;
  reg                SPI3_3_BrstRdEn_d1;
  reg         [1:0]  wait_for_conversion3;
  reg         [1:0]  pl3_3_conversion_state;
  reg        [31:0]  Holding_Data3;
  reg                Holding_SOP3;
  reg                Holding_EOP3;
  reg         [1:0]  Holding_Mod3;
  reg                Holding_Err3;
  reg                SPI3_0_polled_status;
  reg                SPI3_1_polled_status;
  reg                SPI3_2_polled_status;
  reg                SPI3_3_polled_status;
  reg                SPI3_0_WrEn_n;
  reg        [31:0]  SPI3_0_WrData;
  reg         [1:0]  SPI3_0_WrMod;
  reg                SPI3_0_WrSOP;
  reg                SPI3_0_WrEOP;
  reg                SPI3_0_WrErr;
  reg                SPI3_1_WrEn_n;
  reg        [31:0]  SPI3_1_WrData;
  reg         [1:0]  SPI3_1_WrMod;
  reg                SPI3_1_WrSOP;
  reg                SPI3_1_WrEOP;
  reg                SPI3_1_WrErr;
  reg                SPI3_2_WrEn_n;
  reg        [31:0]  SPI3_2_WrData;
  reg         [1:0]  SPI3_2_WrMod;
  reg                SPI3_2_WrSOP;
  reg                SPI3_2_WrEOP;
  reg                SPI3_2_WrErr;
  reg                SPI3_3_WrEn_n;
  reg         [31:0] SPI3_3_WrData;
  reg         [1:0]  SPI3_3_WrMod;
  reg                SPI3_3_WrSOP;
  reg                SPI3_3_WrEOP;
  reg                SPI3_3_WrErr;


  //Keep track of the status coming from each core
  // status_tracking_proc block
  always @ (negedge Reset_n or posedge SPI4_Clk)
  begin
    if (Reset_n == 0)
     begin
      SPI3_0_polled_status <= 0;
      SPI3_1_polled_status <= 0;
      SPI3_2_polled_status <= 0;
      SPI3_3_polled_status <= 0;
     end
   else 
    begin
      if (SPI3_0_TxTAdr == 0)
      	SPI3_0_polled_status <= SPI3_0_TxPTPA;
      if (SPI3_1_TxTAdr == 0) 
      	SPI3_1_polled_status <= SPI3_1_TxPTPA;
      if (SPI3_2_TxTAdr == 0)
      	SPI3_2_polled_status <= SPI3_2_TxPTPA;
      if (SPI3_3_TxTAdr == 0)
        SPI3_3_polled_status <= SPI3_3_TxPTPA;
    end
  end

  /*****************************************************************************
  * Write SPI-3 core 0 Process
  *
  * Note: the purpose of this process is to write data into the SPI-3 source
  * core, instantiation 0
  *****************************************************************************/
  //If the PL3 core isn't full and the burst FIFO isn't empty
  //then read data
  // write_pl3_core_2_proc block
  always @ (negedge Reset_n or posedge SPI4_Clk)
  begin
    if (Reset_n == 0)
     begin
      SPI3_0_BrstRdEn_int <= 0;
      SPI3_0_BrstRdEn_d1 <= 0;
      wait_for_conversion0 <= 2'b00;
     end
    else
     begin
      SPI3_0_BrstRdEn_d1 <= SPI3_0_BrstRdEn_int;
      if (SPI3_0_DST_RDY == 1 && SPI3_0_Brst_Empty == 0 &&
          wait_for_conversion0 == 2'b00 && SPI3_0_polled_status == 1)
       begin
        SPI3_0_BrstRdEn_int <= 1;
        wait_for_conversion0 <= 2'b01;
       end
      else if (SPI3_0_DST_RDY == 1 && wait_for_conversion0==2'b01) begin
      	SPI3_0_BrstRdEn_int <= 0;
        wait_for_conversion0 <= 2'b10;
      end
      else if (SPI3_0_DST_RDY == 1 && wait_for_conversion0==2'b10) begin
      	SPI3_0_BrstRdEn_int <= 0;
        wait_for_conversion0 <= 2'b00;
      end
      else
       begin
        SPI3_0_BrstRdEn_int <= 0;
        wait_for_conversion0 <= wait_for_conversion0;
       end
     end
  end
  
  always @ (negedge Reset_n or posedge SPI4_Clk)
  begin
    if (Reset_n == 0)
     begin
      pl3_0_conversion_state <= 2'b00;
      SPI3_0_WrData <= 0;
      SPI3_0_WrEn_n <= 1;
      SPI3_0_WrSOP <= 0;
      SPI3_0_WrEOP <= 0;
      SPI3_0_WrMod <= 2'b00;
      SPI3_0_WrErr <= 0;
      Holding_Data0 <= 0;
      Holding_EOP0 <= 0;
      Holding_Mod0 <= 2'b00;
      Holding_Err0 <= 0;
     end
    else
      case (pl3_0_conversion_state)
        2'b00:
          if (SPI3_0_BrstRdEn_d1 == 1)
           begin
           if (SPI3_0_DST_RDY) begin
            pl3_0_conversion_state <= 2'b01;
            SPI3_0_WrData <= SPI3_0_BrstData[69:38];
            SPI3_0_WrEn_n <= 0;
            SPI3_0_WrSOP  <= SPI3_0_BrstData[5];
            Holding_Data0 <= SPI3_0_BrstData[37:6];
            //Look at Mod stuff here
            //If there is an EOP, Mod must be looked at
            if (SPI3_0_BrstData[4] == 1) 
              case (SPI3_0_BrstData[3:1])
                //3'b000 means data bits 63-0 have valid data
                3'b000:
                 begin
                  SPI3_0_WrEOP <= 0;
                  SPI3_0_WrErr <= 0;
                  SPI3_0_WrMod <= 2'b00;
                  Holding_EOP0 <= 1;
                  Holding_Mod0 <= 2'b00;
                  Holding_Err0 <= SPI3_0_BrstData[0];
                  pl3_0_conversion_state <= 2'b01;
                 end
                //3'b111 means data bits 63-8 have valid data
                3'b111:
                 begin
                  SPI3_0_WrEOP <= 0;
                  SPI3_0_WrErr <= 0;
                  SPI3_0_WrMod <= 2'b00;
                  Holding_EOP0 <= 1;
                  Holding_Mod0 <= 2'b01;
                  Holding_Err0 <= SPI3_0_BrstData[0];
                  pl3_0_conversion_state <= 2'b01;
                 end
                //3'b110 means data bits 63-16 have valid data
                3'b110:
                 begin
                  SPI3_0_WrEOP <= 0;
                  SPI3_0_WrErr <= 0;
                  SPI3_0_WrMod <= 2'b00;
                  Holding_EOP0 <= 1;
                  Holding_Mod0 <= 2'b10;
                  Holding_Err0 <= SPI3_0_BrstData[0];
                  pl3_0_conversion_state <= 2'b01;
                 end
                //3'b101 means data bits 6324 have valid data
                3'b101:
                 begin
                  SPI3_0_WrEOP <= 0;
                  SPI3_0_WrErr <= 0;
                  SPI3_0_WrMod <= 2'b00;
                  Holding_EOP0 <= 1;
                  Holding_Mod0 <= 2'b11;
                  Holding_Err0 <= SPI3_0_BrstData[0];
                  pl3_0_conversion_state <= 2'b01;
                 end
                //3'b100 means data bits 63-32 have valid data
                3'b100:
                 begin
                  SPI3_0_WrEOP <= 1;
                  SPI3_0_WrErr <= SPI3_0_BrstData[0];
                  SPI3_0_WrMod <= 2'b00;
                  Holding_EOP0 <= 0;
                  Holding_Mod0 <= 2'b00;
                  Holding_Err0 <= 0;
                  pl3_0_conversion_state <= 2'b10;
                 end
                //3'b011 means data bits 63-40 have valid data
                3'b011:
                 begin
                  SPI3_0_WrEOP <= 1;
                  SPI3_0_WrErr <= SPI3_0_BrstData[0];
                  SPI3_0_WrMod <= 2'b01;
                  Holding_EOP0 <= 0;
                  Holding_Mod0 <= 2'b00;
                  Holding_Err0 <= 0;
                  pl3_0_conversion_state <= 2'b10;
                 end
                //3'b010 means data bits 63-48 have valid data
                3'b010:
                 begin
                  SPI3_0_WrEOP <= 1;
                  SPI3_0_WrErr <= SPI3_0_BrstData[0];
                  SPI3_0_WrMod <= 2'b10;
                  Holding_EOP0 <= 0;
                  Holding_Mod0 <= 2'b00;
                  Holding_Err0 <= 0;
                  pl3_0_conversion_state <= 2'b10;
                 end
                //3'b001 means data bits 63-56 have valid data
                3'b001:
                 begin
                  SPI3_0_WrEOP <= 1;
                  SPI3_0_WrErr <= SPI3_0_BrstData[0];
                  SPI3_0_WrMod <= 2'b11;
                  Holding_EOP0 <= 0;
                  Holding_Mod0 <= 2'b00;
                  Holding_Err0 <= 0;
                  pl3_0_conversion_state <= 2'b10;
                 end
                //We have covered all the cases except X's & U's, so leave null
                default:
                 begin
                  SPI3_0_WrEOP <= SPI3_0_WrEOP;
                  SPI3_0_WrErr <= SPI3_0_WrErr;
                  SPI3_0_WrMod <= SPI3_0_WrMod;
                  Holding_EOP0 <= Holding_EOP0;
                  Holding_Mod0 <= Holding_Mod0;
                  Holding_Err0 <= Holding_Err0;
                  pl3_0_conversion_state <= pl3_0_conversion_state;
                 end
              endcase
            else
             begin
              SPI3_0_WrEOP <= 0;
              SPI3_0_WrErr <= 0;
              SPI3_0_WrMod <= 2'b00;
              Holding_EOP0 <= 0;
              Holding_Mod0 <= 2'b00;
              Holding_Err0 <= 0;
              if (SPI3_0_DST_RDY)
	              pl3_0_conversion_state <= 2'b01;
	              else
	              pl3_0_conversion_state <= 2'b00;
             end
             end
           else begin
             		pl3_0_conversion_state <= 2'b11;
            		SPI3_0_WrEn_n <= 1;
           end
          end
          
          else
           begin
            pl3_0_conversion_state <= 2'b00;
            SPI3_0_WrEn_n <= 1;
           end
          
        2'b01:
         begin
         	if (SPI3_0_DST_RDY) begin
          pl3_0_conversion_state <= 00;
          SPI3_0_WrData <= Holding_Data0;
          SPI3_0_WrEn_n <= 0;
          SPI3_0_WrSOP  <= 0;
          SPI3_0_WrEOP  <= Holding_EOP0;
          SPI3_0_WrMod  <= Holding_Mod0;
          SPI3_0_WrErr  <= Holding_Err0;
          end
         end
        2'b10:
         begin
         	if (SPI3_0_DST_RDY) begin
          pl3_0_conversion_state <= 2'b00;
          SPI3_0_WrEn_n <= 1;
          end
          end
        2'b11:
        	begin
        		if (SPI3_0_DST_RDY) begin
        		pl3_0_conversion_state <= 2'b01;
            SPI3_0_WrData <= SPI3_0_BrstData[69:38];
            SPI3_0_WrEn_n <= 0;
            SPI3_0_WrSOP  <= SPI3_0_BrstData[5];
            Holding_Data0 <= SPI3_0_BrstData[37:6];
            //Look at Mod stuff here
            //If there is an EOP, Mod must be looked at
            if (SPI3_0_BrstData[4] == 1) begin
              case (SPI3_0_BrstData[3:1])
                //3'b000 means data bits 63-0 have valid data
                3'b000:
                 begin
                  SPI3_0_WrEOP <= 0;
                  SPI3_0_WrErr <= 0;
                  SPI3_0_WrMod <= 2'b00;
                  Holding_EOP0 <= 1;
                  Holding_Mod0 <= 2'b00;
                  Holding_Err0 <= SPI3_0_BrstData[0];
                  pl3_0_conversion_state <= 2'b01;
                 end
                //3'b111 means data bits 63-8 have valid data
                3'b111:
                 begin
                  SPI3_0_WrEOP <= 0;
                  SPI3_0_WrErr <= 0;
                  SPI3_0_WrMod <= 2'b00;
                  Holding_EOP0 <= 1;
                  Holding_Mod0 <= 2'b01;
                  Holding_Err0 <= SPI3_0_BrstData[0];
                  pl3_0_conversion_state <= 2'b01;
                 end
                //3'b110 means data bits 63-16 have valid data
                3'b110:
                 begin
                  SPI3_0_WrEOP <= 0;
                  SPI3_0_WrErr <= 0;
                  SPI3_0_WrMod <= 2'b00;
                  Holding_EOP0 <= 1;
                  Holding_Mod0 <= 2'b10;
                  Holding_Err0 <= SPI3_0_BrstData[0];
                  pl3_0_conversion_state <= 2'b01;
                 end
                //3'b101 means data bits 6324 have valid data
                3'b101:
                 begin
                  SPI3_0_WrEOP <= 0;
                  SPI3_0_WrErr <= 0;
                  SPI3_0_WrMod <= 2'b00;
                  Holding_EOP0 <= 1;
                  Holding_Mod0 <= 2'b11;
                  Holding_Err0 <= SPI3_0_BrstData[0];
                  pl3_0_conversion_state <= 2'b01;
                 end
                //3'b100 means data bits 63-32 have valid data
                3'b100:
                 begin
                  SPI3_0_WrEOP <= 1;
                  SPI3_0_WrErr <= SPI3_0_BrstData[0];
                  SPI3_0_WrMod <= 2'b00;
                  Holding_EOP0 <= 0;
                  Holding_Mod0 <= 2'b00;
                  Holding_Err0 <= 0;
                  pl3_0_conversion_state <= 2'b10;
                 end
                //3'b011 means data bits 63-40 have valid data
                3'b011:
                 begin
                  SPI3_0_WrEOP <= 1;
                  SPI3_0_WrErr <= SPI3_0_BrstData[0];
                  SPI3_0_WrMod <= 2'b01;
                  Holding_EOP0 <= 0;
                  Holding_Mod0 <= 2'b00;
                  Holding_Err0 <= 0;
                  pl3_0_conversion_state <= 2'b10;
                 end
                //3'b010 means data bits 63-48 have valid data
                3'b010:
                 begin
                  SPI3_0_WrEOP <= 1;
                  SPI3_0_WrErr <= SPI3_0_BrstData[0];
                  SPI3_0_WrMod <= 2'b10;
                  Holding_EOP0 <= 0;
                  Holding_Mod0 <= 2'b00;
                  Holding_Err0 <= 0;
                  pl3_0_conversion_state <= 2'b10;
                 end
                //3'b001 means data bits 63-56 have valid data
                3'b001:
                 begin
                  SPI3_0_WrEOP <= 1;
                  SPI3_0_WrErr <= SPI3_0_BrstData[0];
                  SPI3_0_WrMod <= 2'b11;
                  Holding_EOP0 <= 0;
                  Holding_Mod0 <= 2'b00;
                  Holding_Err0 <= 0;
                  pl3_0_conversion_state <= 2'b10;
                 end
                //We have covered all the cases except X's & U's, so leave null
                default:
                 begin
                  SPI3_0_WrEOP <= SPI3_0_WrEOP;
                  SPI3_0_WrErr <= SPI3_0_WrErr;
                  SPI3_0_WrMod <= SPI3_0_WrMod;
                  Holding_EOP0 <= Holding_EOP0;
                  Holding_Mod0 <= Holding_Mod0;
                  Holding_Err0 <= Holding_Err0;
                  pl3_0_conversion_state <= pl3_0_conversion_state;
                 end
              endcase
            end  
          else
           begin
            pl3_0_conversion_state <= 2'b11;
            SPI3_0_WrEn_n <= 1;
           end
        	end
        end
        default:
         begin
          pl3_0_conversion_state <= 2'b00;
          SPI3_0_WrEn_n <= 1;
         end
      endcase
  end
  
  assign SPI3_0_BrstRdEn = SPI3_0_BrstRdEn_int;

  
  /*****************************************************************************
  * Write SPI-3 core 1 Process
  *
  * Note: the purpose of this process is to write data into the SPI-3 source
  * core, instantiation 1
  * Note: this is a repeat of the above processes
  *****************************************************************************/
  // write_pl3_core_2_proc block
  always @ (negedge Reset_n or posedge SPI4_Clk)
  begin
    if (Reset_n == 0)
     begin
      SPI3_1_BrstRdEn_int <= 0;
      SPI3_1_BrstRdEn_d1 <= 0;
      wait_for_conversion1 <= 2'b00;
     end
    else
     begin
      SPI3_1_BrstRdEn_d1 <= SPI3_1_BrstRdEn_int;
      if (SPI3_1_DST_RDY == 1 && SPI3_1_Brst_Empty == 0 &&
          wait_for_conversion1 == 2'b00 && SPI3_1_polled_status == 1)
       begin
        SPI3_1_BrstRdEn_int <= 1;
        wait_for_conversion1 <= 2'b01;
       end
      else if (SPI3_1_DST_RDY == 1 && wait_for_conversion1==2'b01) begin
      	SPI3_1_BrstRdEn_int <= 0;
        wait_for_conversion1 <= 2'b10;
      end
      else if (SPI3_1_DST_RDY == 1 && wait_for_conversion1==2'b10) begin
      	SPI3_1_BrstRdEn_int <= 0;
        wait_for_conversion1 <= 2'b00;
      end
      else
       begin
        SPI3_1_BrstRdEn_int <= 0;
        wait_for_conversion1 <= wait_for_conversion1;
       end
     end
  end
  
  always @ (negedge Reset_n or posedge SPI4_Clk)
  begin
    if (Reset_n == 0)
     begin
      pl3_1_conversion_state <= 2'b00;
      SPI3_1_WrData <= 0;
      SPI3_1_WrEn_n <= 1;
      SPI3_1_WrSOP <= 0;
      SPI3_1_WrEOP <= 0;
      SPI3_1_WrMod <= 2'b00;
      SPI3_1_WrErr <= 0;
      Holding_Data1 <= 0;
      Holding_EOP1 <= 0;
      Holding_Mod1 <= 2'b00;
      Holding_Err1 <= 0;
     end
    else
      case (pl3_1_conversion_state)
        2'b00:
          if (SPI3_1_BrstRdEn_d1 == 1)
           begin
           if (SPI3_1_DST_RDY) begin
            pl3_1_conversion_state <= 2'b01;
            SPI3_1_WrData <= SPI3_1_BrstData[69:38];
            SPI3_1_WrEn_n <= 0;
            SPI3_1_WrSOP  <= SPI3_1_BrstData[5];
            Holding_Data1 <= SPI3_1_BrstData[37:6];
            //Look at Mod stuff here
            //If there is an EOP, Mod must be looked at
            if (SPI3_1_BrstData[4] == 1) 
              case (SPI3_1_BrstData[3:1])
                //3'b000 means data bits 63-0 have valid data
                3'b000:
                 begin
                  SPI3_1_WrEOP <= 0;
                  SPI3_1_WrErr <= 0;
                  SPI3_1_WrMod <= 2'b00;
                  Holding_EOP1 <= 1;
                  Holding_Mod1 <= 2'b00;
                  Holding_Err1 <= SPI3_1_BrstData[0];
                  pl3_1_conversion_state <= 2'b01;
                 end
                //3'b111 means data bits 63-8 have valid data
                3'b111:
                 begin
                  SPI3_1_WrEOP <= 0;
                  SPI3_1_WrErr <= 0;
                  SPI3_1_WrMod <= 2'b00;
                  Holding_EOP1 <= 1;
                  Holding_Mod1 <= 2'b01;
                  Holding_Err1 <= SPI3_1_BrstData[0];
                  pl3_1_conversion_state <= 2'b01;
                 end
                //3'b110 means data bits 63-16 have valid data
                3'b110:
                 begin
                  SPI3_1_WrEOP <= 0;
                  SPI3_1_WrErr <= 0;
                  SPI3_1_WrMod <= 2'b00;
                  Holding_EOP1 <= 1;
                  Holding_Mod1 <= 2'b10;
                  Holding_Err1 <= SPI3_1_BrstData[0];
                  pl3_1_conversion_state <= 2'b01;
                 end
                //3'b101 means data bits 6324 have valid data
                3'b101:
                 begin
                  SPI3_1_WrEOP <= 0;
                  SPI3_1_WrErr <= 0;
                  SPI3_1_WrMod <= 2'b00;
                  Holding_EOP1 <= 1;
                  Holding_Mod1 <= 2'b11;
                  Holding_Err1 <= SPI3_1_BrstData[0];
                  pl3_1_conversion_state <= 2'b01;
                 end
                //3'b100 means data bits 63-32 have valid data
                3'b100:
                 begin
                  SPI3_1_WrEOP <= 1;
                  SPI3_1_WrErr <= SPI3_1_BrstData[0];
                  SPI3_1_WrMod <= 2'b00;
                  Holding_EOP1 <= 0;
                  Holding_Mod1 <= 2'b00;
                  Holding_Err1 <= 0;
                  pl3_1_conversion_state <= 2'b10;
                 end
                //3'b011 means data bits 63-40 have valid data
                3'b011:
                 begin
                  SPI3_1_WrEOP <= 1;
                  SPI3_1_WrErr <= SPI3_1_BrstData[0];
                  SPI3_1_WrMod <= 2'b01;
                  Holding_EOP1 <= 0;
                  Holding_Mod1 <= 2'b00;
                  Holding_Err1 <= 0;
                  pl3_1_conversion_state <= 2'b10;
                 end
                //3'b010 means data bits 63-48 have valid data
                3'b010:
                 begin
                  SPI3_1_WrEOP <= 1;
                  SPI3_1_WrErr <= SPI3_1_BrstData[0];
                  SPI3_1_WrMod <= 2'b10;
                  Holding_EOP1 <= 0;
                  Holding_Mod1 <= 2'b00;
                  Holding_Err1 <= 0;
                  pl3_1_conversion_state <= 2'b10;
                 end
                //3'b001 means data bits 63-56 have valid data
                3'b001:
                 begin
                  SPI3_1_WrEOP <= 1;
                  SPI3_1_WrErr <= SPI3_1_BrstData[0];
                  SPI3_1_WrMod <= 2'b11;
                  Holding_EOP1 <= 0;
                  Holding_Mod1 <= 2'b00;
                  Holding_Err1 <= 0;
                  pl3_1_conversion_state <= 2'b10;
                 end
                //We have covered all the cases except X's & U's, so leave null
                default:
                 begin
                  SPI3_1_WrEOP <= SPI3_1_WrEOP;
                  SPI3_1_WrErr <= SPI3_1_WrErr;
                  SPI3_1_WrMod <= SPI3_1_WrMod;
                  Holding_EOP1 <= Holding_EOP1;
                  Holding_Mod1 <= Holding_Mod1;
                  Holding_Err1 <= Holding_Err1;
                  pl3_1_conversion_state <= pl3_1_conversion_state;
                 end
              endcase
            else
             begin
              SPI3_1_WrEOP <= 0;
              SPI3_1_WrErr <= 0;
              SPI3_1_WrMod <= 2'b00;
              Holding_EOP1 <= 0;
              Holding_Mod1 <= 2'b00;
              Holding_Err1 <= 0;
              if (SPI3_1_DST_RDY)
	              pl3_1_conversion_state <= 2'b01;
	              else
	              pl3_1_conversion_state <= 2'b00;
             end
             end
           else begin
             		pl3_1_conversion_state <= 2'b11;
            		SPI3_1_WrEn_n <= 1;
           end
          end
          
          else
           begin
            pl3_1_conversion_state <= 2'b00;
            SPI3_1_WrEn_n <= 1;
           end
          
        2'b01:
         begin
         	if (SPI3_1_DST_RDY) begin
          pl3_1_conversion_state <= 00;
          SPI3_1_WrData <= Holding_Data1;
          SPI3_1_WrEn_n <= 0;
          SPI3_1_WrSOP  <= 0;
          SPI3_1_WrEOP  <= Holding_EOP1;
          SPI3_1_WrMod  <= Holding_Mod1;
          SPI3_1_WrErr  <= Holding_Err1;
          end
         end
        2'b10:
         begin
         	if (SPI3_1_DST_RDY) begin
          pl3_1_conversion_state <= 2'b00;
          SPI3_1_WrEn_n <= 1;
          end
          end
        2'b11:
        	begin
        		if (SPI3_1_DST_RDY) begin
        		pl3_1_conversion_state <= 2'b01;
            SPI3_1_WrData <= SPI3_1_BrstData[69:38];
            SPI3_1_WrEn_n <= 0;
            SPI3_1_WrSOP  <= SPI3_1_BrstData[5];
            Holding_Data1 <= SPI3_1_BrstData[37:6];
            //Look at Mod stuff here
            //If there is an EOP, Mod must be looked at
            if (SPI3_1_BrstData[4] == 1) begin
              case (SPI3_1_BrstData[3:1])
                //3'b000 means data bits 63-0 have valid data
                3'b000:
                 begin
                  SPI3_1_WrEOP <= 0;
                  SPI3_1_WrErr <= 0;
                  SPI3_1_WrMod <= 2'b00;
                  Holding_EOP1 <= 1;
                  Holding_Mod1 <= 2'b00;
                  Holding_Err1 <= SPI3_1_BrstData[0];
                  pl3_1_conversion_state <= 2'b01;
                 end
                //3'b111 means data bits 63-8 have valid data
                3'b111:
                 begin
                  SPI3_1_WrEOP <= 0;
                  SPI3_1_WrErr <= 0;
                  SPI3_1_WrMod <= 2'b00;
                  Holding_EOP1 <= 1;
                  Holding_Mod1 <= 2'b01;
                  Holding_Err1 <= SPI3_1_BrstData[0];
                  pl3_1_conversion_state <= 2'b01;
                 end
                //3'b110 means data bits 63-16 have valid data
                3'b110:
                 begin
                  SPI3_1_WrEOP <= 0;
                  SPI3_1_WrErr <= 0;
                  SPI3_1_WrMod <= 2'b00;
                  Holding_EOP1 <= 1;
                  Holding_Mod1 <= 2'b10;
                  Holding_Err1 <= SPI3_1_BrstData[0];
                  pl3_1_conversion_state <= 2'b01;
                 end
                //3'b101 means data bits 6324 have valid data
                3'b101:
                 begin
                  SPI3_1_WrEOP <= 0;
                  SPI3_1_WrErr <= 0;
                  SPI3_1_WrMod <= 2'b00;
                  Holding_EOP1 <= 1;
                  Holding_Mod1 <= 2'b11;
                  Holding_Err1 <= SPI3_1_BrstData[0];
                  pl3_1_conversion_state <= 2'b01;
                 end
                //3'b100 means data bits 63-32 have valid data
                3'b100:
                 begin
                  SPI3_1_WrEOP <= 1;
                  SPI3_1_WrErr <= SPI3_1_BrstData[0];
                  SPI3_1_WrMod <= 2'b00;
                  Holding_EOP1 <= 0;
                  Holding_Mod1 <= 2'b00;
                  Holding_Err1 <= 0;
                  pl3_1_conversion_state <= 2'b10;
                 end
                //3'b011 means data bits 63-40 have valid data
                3'b011:
                 begin
                  SPI3_1_WrEOP <= 1;
                  SPI3_1_WrErr <= SPI3_1_BrstData[0];
                  SPI3_1_WrMod <= 2'b01;
                  Holding_EOP1 <= 0;
                  Holding_Mod1 <= 2'b00;
                  Holding_Err1 <= 0;
                  pl3_1_conversion_state <= 2'b10;
                 end
                //3'b010 means data bits 63-48 have valid data
                3'b010:
                 begin
                  SPI3_1_WrEOP <= 1;
                  SPI3_1_WrErr <= SPI3_1_BrstData[0];
                  SPI3_1_WrMod <= 2'b10;
                  Holding_EOP1 <= 0;
                  Holding_Mod1 <= 2'b00;
                  Holding_Err1 <= 0;
                  pl3_1_conversion_state <= 2'b10;
                 end
                //3'b001 means data bits 63-56 have valid data
                3'b001:
                 begin
                  SPI3_1_WrEOP <= 1;
                  SPI3_1_WrErr <= SPI3_1_BrstData[0];
                  SPI3_1_WrMod <= 2'b11;
                  Holding_EOP1 <= 0;
                  Holding_Mod1 <= 2'b00;
                  Holding_Err1 <= 0;
                  pl3_1_conversion_state <= 2'b10;
                 end
                //We have covered all the cases except X's & U's, so leave null
                default:
                 begin
                  SPI3_1_WrEOP <= SPI3_1_WrEOP;
                  SPI3_1_WrErr <= SPI3_1_WrErr;
                  SPI3_1_WrMod <= SPI3_1_WrMod;
                  Holding_EOP1 <= Holding_EOP1;
                  Holding_Mod1 <= Holding_Mod1;
                  Holding_Err1 <= Holding_Err1;
                  pl3_1_conversion_state <= pl3_1_conversion_state;
                 end
              endcase
            end  
          else
           begin
            pl3_1_conversion_state <= 2'b11;
            SPI3_1_WrEn_n <= 1;
           end
        	end
        end
        default:
         begin
          pl3_1_conversion_state <= 2'b00;
          SPI3_1_WrEn_n <= 1;
         end
      endcase
  end
  
  assign SPI3_1_BrstRdEn = SPI3_1_BrstRdEn_int;

  
  /*****************************************************************************
  * Write SPI-3 core 2 Process
  *
  * Note: the purpose of this process is to write data into the SPI-3 source
  * core, instantiation 2
  * Note: this is a repeat of the above processes
  *****************************************************************************/
  // write_pl3_core_2_proc block
  always @ (negedge Reset_n or posedge SPI4_Clk)
  begin
    if (Reset_n == 0)
     begin
      SPI3_2_BrstRdEn_int <= 0;
      SPI3_2_BrstRdEn_d1 <= 0;
      wait_for_conversion2 <= 2'b00;
     end
    else
     begin
      SPI3_2_BrstRdEn_d1 <= SPI3_2_BrstRdEn_int;
      if (SPI3_2_DST_RDY == 1 && SPI3_2_Brst_Empty == 0 &&
          wait_for_conversion2 == 2'b00 && SPI3_2_polled_status == 1)
       begin
        SPI3_2_BrstRdEn_int <= 1;
        wait_for_conversion2 <= 2'b01;
       end
      else if (SPI3_2_DST_RDY == 1 && wait_for_conversion2==2'b01) begin
      	SPI3_2_BrstRdEn_int <= 0;
        wait_for_conversion2 <= 2'b10;
      end
      else if (SPI3_2_DST_RDY == 1 && wait_for_conversion2==2'b10) begin
      	SPI3_2_BrstRdEn_int <= 0;
        wait_for_conversion2 <= 2'b00;
      end
      else
       begin
        SPI3_2_BrstRdEn_int <= 0;
        wait_for_conversion2 <= wait_for_conversion2;
       end
     end
  end
  
  always @ (negedge Reset_n or posedge SPI4_Clk)
  begin
    if (Reset_n == 0)
     begin
      pl3_2_conversion_state <= 2'b00;
      SPI3_2_WrData <= 0;
      SPI3_2_WrEn_n <= 1;
      SPI3_2_WrSOP <= 0;
      SPI3_2_WrEOP <= 0;
      SPI3_2_WrMod <= 2'b00;
      SPI3_2_WrErr <= 0;
      Holding_Data2 <= 0;
      Holding_EOP2 <= 0;
      Holding_Mod2 <= 2'b00;
      Holding_Err2 <= 0;
     end
    else
      case (pl3_2_conversion_state)
        2'b00:
          if (SPI3_2_BrstRdEn_d1 == 1)
           begin
           if (SPI3_2_DST_RDY) begin
            pl3_2_conversion_state <= 2'b01;
            SPI3_2_WrData <= SPI3_2_BrstData[69:38];
            SPI3_2_WrEn_n <= 0;
            SPI3_2_WrSOP  <= SPI3_2_BrstData[5];
            Holding_Data2 <= SPI3_2_BrstData[37:6];
            //Look at Mod stuff here
            //If there is an EOP, Mod must be looked at
            if (SPI3_2_BrstData[4] == 1) 
              case (SPI3_2_BrstData[3:1])
                //3'b000 means data bits 63-0 have valid data
                3'b000:
                 begin
                  SPI3_2_WrEOP <= 0;
                  SPI3_2_WrErr <= 0;
                  SPI3_2_WrMod <= 2'b00;
                  Holding_EOP2 <= 1;
                  Holding_Mod2 <= 2'b00;
                  Holding_Err2 <= SPI3_2_BrstData[0];
                  pl3_2_conversion_state <= 2'b01;
                 end
                //3'b111 means data bits 63-8 have valid data
                3'b111:
                 begin
                  SPI3_2_WrEOP <= 0;
                  SPI3_2_WrErr <= 0;
                  SPI3_2_WrMod <= 2'b00;
                  Holding_EOP2 <= 1;
                  Holding_Mod2 <= 2'b01;
                  Holding_Err2 <= SPI3_2_BrstData[0];
                  pl3_2_conversion_state <= 2'b01;
                 end
                //3'b110 means data bits 63-16 have valid data
                3'b110:
                 begin
                  SPI3_2_WrEOP <= 0;
                  SPI3_2_WrErr <= 0;
                  SPI3_2_WrMod <= 2'b00;
                  Holding_EOP2 <= 1;
                  Holding_Mod2 <= 2'b10;
                  Holding_Err2 <= SPI3_2_BrstData[0];
                  pl3_2_conversion_state <= 2'b01;
                 end
                //3'b101 means data bits 6324 have valid data
                3'b101:
                 begin
                  SPI3_2_WrEOP <= 0;
                  SPI3_2_WrErr <= 0;
                  SPI3_2_WrMod <= 2'b00;
                  Holding_EOP2 <= 1;
                  Holding_Mod2 <= 2'b11;
                  Holding_Err2 <= SPI3_2_BrstData[0];
                  pl3_2_conversion_state <= 2'b01;
                 end
                //3'b100 means data bits 63-32 have valid data
                3'b100:
                 begin
                  SPI3_2_WrEOP <= 1;
                  SPI3_2_WrErr <= SPI3_2_BrstData[0];
                  SPI3_2_WrMod <= 2'b00;
                  Holding_EOP2 <= 0;
                  Holding_Mod2 <= 2'b00;
                  Holding_Err2 <= 0;
                  pl3_2_conversion_state <= 2'b10;
                 end
                //3'b011 means data bits 63-40 have valid data
                3'b011:
                 begin
                  SPI3_2_WrEOP <= 1;
                  SPI3_2_WrErr <= SPI3_2_BrstData[0];
                  SPI3_2_WrMod <= 2'b01;
                  Holding_EOP2 <= 0;
                  Holding_Mod2 <= 2'b00;
                  Holding_Err2 <= 0;
                  pl3_2_conversion_state <= 2'b10;
                 end
                //3'b010 means data bits 63-48 have valid data
                3'b010:
                 begin
                  SPI3_2_WrEOP <= 1;
                  SPI3_2_WrErr <= SPI3_2_BrstData[0];
                  SPI3_2_WrMod <= 2'b10;
                  Holding_EOP2 <= 0;
                  Holding_Mod2 <= 2'b00;
                  Holding_Err2 <= 0;
                  pl3_2_conversion_state <= 2'b10;
                 end
                //3'b001 means data bits 63-56 have valid data
                3'b001:
                 begin
                  SPI3_2_WrEOP <= 1;
                  SPI3_2_WrErr <= SPI3_2_BrstData[0];
                  SPI3_2_WrMod <= 2'b11;
                  Holding_EOP2 <= 0;
                  Holding_Mod2 <= 2'b00;
                  Holding_Err2 <= 0;
                  pl3_2_conversion_state <= 2'b10;
                 end
                //We have covered all the cases except X's & U's, so leave null
                default:
                 begin
                  SPI3_2_WrEOP <= SPI3_2_WrEOP;
                  SPI3_2_WrErr <= SPI3_2_WrErr;
                  SPI3_2_WrMod <= SPI3_2_WrMod;
                  Holding_EOP2 <= Holding_EOP2;
                  Holding_Mod2 <= Holding_Mod2;
                  Holding_Err2 <= Holding_Err2;
                  pl3_2_conversion_state <= pl3_2_conversion_state;
                 end
              endcase
            else
             begin
              SPI3_2_WrEOP <= 0;
              SPI3_2_WrErr <= 0;
              SPI3_2_WrMod <= 2'b00;
              Holding_EOP2 <= 0;
              Holding_Mod2 <= 2'b00;
              Holding_Err2 <= 0;
              if (SPI3_2_DST_RDY)
	              pl3_2_conversion_state <= 2'b01;
	              else
	              pl3_2_conversion_state <= 2'b00;
             end
             end
           else begin
             		pl3_2_conversion_state <= 2'b11;
            		SPI3_2_WrEn_n <= 1;
           end
          end
          
          else
           begin
            pl3_2_conversion_state <= 2'b00;
            SPI3_2_WrEn_n <= 1;
           end
          
        2'b01:
         begin
         	if (SPI3_2_DST_RDY) begin
          pl3_2_conversion_state <= 00;
          SPI3_2_WrData <= Holding_Data2;
          SPI3_2_WrEn_n <= 0;
          SPI3_2_WrSOP  <= 0;
          SPI3_2_WrEOP  <= Holding_EOP2;
          SPI3_2_WrMod  <= Holding_Mod2;
          SPI3_2_WrErr  <= Holding_Err2;
          end
         end
        2'b10:
         begin
         	if (SPI3_2_DST_RDY) begin
          pl3_2_conversion_state <= 2'b00;
          SPI3_2_WrEn_n <= 1;
          end
          end
        2'b11:
        	begin
        		if (SPI3_2_DST_RDY) begin
        		pl3_2_conversion_state <= 2'b01;
            SPI3_2_WrData <= SPI3_2_BrstData[69:38];
            SPI3_2_WrEn_n <= 0;
            SPI3_2_WrSOP  <= SPI3_2_BrstData[5];
            Holding_Data2 <= SPI3_2_BrstData[37:6];
            //Look at Mod stuff here
            //If there is an EOP, Mod must be looked at
            if (SPI3_2_BrstData[4] == 1) begin
              case (SPI3_2_BrstData[3:1])
                //3'b000 means data bits 63-0 have valid data
                3'b000:
                 begin
                  SPI3_2_WrEOP <= 0;
                  SPI3_2_WrErr <= 0;
                  SPI3_2_WrMod <= 2'b00;
                  Holding_EOP2 <= 1;
                  Holding_Mod2 <= 2'b00;
                  Holding_Err2 <= SPI3_2_BrstData[0];
                  pl3_2_conversion_state <= 2'b01;
                 end
                //3'b111 means data bits 63-8 have valid data
                3'b111:
                 begin
                  SPI3_2_WrEOP <= 0;
                  SPI3_2_WrErr <= 0;
                  SPI3_2_WrMod <= 2'b00;
                  Holding_EOP2 <= 1;
                  Holding_Mod2 <= 2'b01;
                  Holding_Err2 <= SPI3_2_BrstData[0];
                  pl3_2_conversion_state <= 2'b01;
                 end
                //3'b110 means data bits 63-16 have valid data
                3'b110:
                 begin
                  SPI3_2_WrEOP <= 0;
                  SPI3_2_WrErr <= 0;
                  SPI3_2_WrMod <= 2'b00;
                  Holding_EOP2 <= 1;
                  Holding_Mod2 <= 2'b10;
                  Holding_Err2 <= SPI3_2_BrstData[0];
                  pl3_2_conversion_state <= 2'b01;
                 end
                //3'b101 means data bits 6324 have valid data
                3'b101:
                 begin
                  SPI3_2_WrEOP <= 0;
                  SPI3_2_WrErr <= 0;
                  SPI3_2_WrMod <= 2'b00;
                  Holding_EOP2 <= 1;
                  Holding_Mod2 <= 2'b11;
                  Holding_Err2 <= SPI3_2_BrstData[0];
                  pl3_2_conversion_state <= 2'b01;
                 end
                //3'b100 means data bits 63-32 have valid data
                3'b100:
                 begin
                  SPI3_2_WrEOP <= 1;
                  SPI3_2_WrErr <= SPI3_2_BrstData[0];
                  SPI3_2_WrMod <= 2'b00;
                  Holding_EOP2 <= 0;
                  Holding_Mod2 <= 2'b00;
                  Holding_Err2 <= 0;
                  pl3_2_conversion_state <= 2'b10;
                 end
                //3'b011 means data bits 63-40 have valid data
                3'b011:
                 begin
                  SPI3_2_WrEOP <= 1;
                  SPI3_2_WrErr <= SPI3_2_BrstData[0];
                  SPI3_2_WrMod <= 2'b01;
                  Holding_EOP2 <= 0;
                  Holding_Mod2 <= 2'b00;
                  Holding_Err2 <= 0;
                  pl3_2_conversion_state <= 2'b10;
                 end
                //3'b010 means data bits 63-48 have valid data
                3'b010:
                 begin
                  SPI3_2_WrEOP <= 1;
                  SPI3_2_WrErr <= SPI3_2_BrstData[0];
                  SPI3_2_WrMod <= 2'b10;
                  Holding_EOP2 <= 0;
                  Holding_Mod2 <= 2'b00;
                  Holding_Err2 <= 0;
                  pl3_2_conversion_state <= 2'b10;
                 end
                //3'b001 means data bits 63-56 have valid data
                3'b001:
                 begin
                  SPI3_2_WrEOP <= 1;
                  SPI3_2_WrErr <= SPI3_2_BrstData[0];
                  SPI3_2_WrMod <= 2'b11;
                  Holding_EOP2 <= 0;
                  Holding_Mod2 <= 2'b00;
                  Holding_Err2 <= 0;
                  pl3_2_conversion_state <= 2'b10;
                 end
                //We have covered all the cases except X's & U's, so leave null
                default:
                 begin
                  SPI3_2_WrEOP <= SPI3_2_WrEOP;
                  SPI3_2_WrErr <= SPI3_2_WrErr;
                  SPI3_2_WrMod <= SPI3_2_WrMod;
                  Holding_EOP2 <= Holding_EOP2;
                  Holding_Mod2 <= Holding_Mod2;
                  Holding_Err2 <= Holding_Err2;
                  pl3_2_conversion_state <= pl3_2_conversion_state;
                 end
              endcase
            end  
          else
           begin
            pl3_2_conversion_state <= 2'b11;
            SPI3_2_WrEn_n <= 1;
           end
        	end
        end
        default:
         begin
          pl3_2_conversion_state <= 2'b00;
          SPI3_2_WrEn_n <= 1;
         end
      endcase
  end
  
  assign SPI3_2_BrstRdEn = SPI3_2_BrstRdEn_int;

  
  /*****************************************************************************
  * Write SPI-3 core 3 Process
  *
  * Note: the purpose of this process is to write data into the SPI-3 source
  * core, instantiation 3
  * Note: this is a repeat of the above processes
  *****************************************************************************/
  // write_pl3_core_3_proc block
  always @ (negedge Reset_n or posedge SPI4_Clk)
  begin
    if (Reset_n == 0)
     begin
      SPI3_3_BrstRdEn_int <= 0;
      SPI3_3_BrstRdEn_d1 <= 0;
      wait_for_conversion3 <= 2'b00;
     end
    else
     begin
      SPI3_3_BrstRdEn_d1 <= SPI3_3_BrstRdEn_int;
      if (SPI3_3_DST_RDY == 1 && SPI3_3_Brst_Empty == 0 &&
          wait_for_conversion3 == 2'b00 && SPI3_3_polled_status == 1)
       begin
        SPI3_3_BrstRdEn_int <= 1;
        wait_for_conversion3 <= 2'b01;
       end
      else if (SPI3_3_DST_RDY == 1 && wait_for_conversion3==2'b01) begin
      	SPI3_3_BrstRdEn_int <= 0;
        wait_for_conversion3 <= 2'b10;
      end
      else if (SPI3_3_DST_RDY == 1 && wait_for_conversion3==2'b10) begin
      	SPI3_3_BrstRdEn_int <= 0;
        wait_for_conversion3 <= 2'b00;
      end
      else
       begin
        SPI3_3_BrstRdEn_int <= 0;
        wait_for_conversion3 <= wait_for_conversion3;
       end
     end
  end
  
  always @ (negedge Reset_n or posedge SPI4_Clk)
  begin
    if (Reset_n == 0)
     begin
      pl3_3_conversion_state <= 2'b00;
      SPI3_3_WrData <= 0;
      SPI3_3_WrEn_n <= 1;
      SPI3_3_WrSOP <= 0;
      SPI3_3_WrEOP <= 0;
      SPI3_3_WrMod <= 2'b00;
      SPI3_3_WrErr <= 0;
      Holding_Data3 <= 0;
      Holding_EOP3 <= 0;
      Holding_Mod3 <= 2'b00;
      Holding_Err3 <= 0;
     end
    else
      case (pl3_3_conversion_state)
        2'b00:
          if (SPI3_3_BrstRdEn_d1 == 1)
           begin
           if (SPI3_3_DST_RDY) begin
            pl3_3_conversion_state <= 2'b01;
            SPI3_3_WrData <= SPI3_3_BrstData[69:38];
            SPI3_3_WrEn_n <= 0;
            SPI3_3_WrSOP  <= SPI3_3_BrstData[5];
            Holding_Data3 <= SPI3_3_BrstData[37:6];
            //Look at Mod stuff here
            //If there is an EOP, Mod must be looked at
            if (SPI3_3_BrstData[4] == 1) 
              case (SPI3_3_BrstData[3:1])
                //3'b000 means data bits 63-0 have valid data
                3'b000:
                 begin
                  SPI3_3_WrEOP <= 0;
                  SPI3_3_WrErr <= 0;
                  SPI3_3_WrMod <= 2'b00;
                  Holding_EOP3 <= 1;
                  Holding_Mod3 <= 2'b00;
                  Holding_Err3 <= SPI3_3_BrstData[0];
                  pl3_3_conversion_state <= 2'b01;
                 end
                //3'b111 means data bits 63-8 have valid data
                3'b111:
                 begin
                  SPI3_3_WrEOP <= 0;
                  SPI3_3_WrErr <= 0;
                  SPI3_3_WrMod <= 2'b00;
                  Holding_EOP3 <= 1;
                  Holding_Mod3 <= 2'b01;
                  Holding_Err3 <= SPI3_3_BrstData[0];
                  pl3_3_conversion_state <= 2'b01;
                 end
                //3'b110 means data bits 63-16 have valid data
                3'b110:
                 begin
                  SPI3_3_WrEOP <= 0;
                  SPI3_3_WrErr <= 0;
                  SPI3_3_WrMod <= 2'b00;
                  Holding_EOP3 <= 1;
                  Holding_Mod3 <= 2'b10;
                  Holding_Err3 <= SPI3_3_BrstData[0];
                  pl3_3_conversion_state <= 2'b01;
                 end
                //3'b101 means data bits 6324 have valid data
                3'b101:
                 begin
                  SPI3_3_WrEOP <= 0;
                  SPI3_3_WrErr <= 0;
                  SPI3_3_WrMod <= 2'b00;
                  Holding_EOP3 <= 1;
                  Holding_Mod3 <= 2'b11;
                  Holding_Err3 <= SPI3_3_BrstData[0];
                  pl3_3_conversion_state <= 2'b01;
                 end
                //3'b100 means data bits 63-32 have valid data
                3'b100:
                 begin
                  SPI3_3_WrEOP <= 1;
                  SPI3_3_WrErr <= SPI3_3_BrstData[0];
                  SPI3_3_WrMod <= 2'b00;
                  Holding_EOP3 <= 0;
                  Holding_Mod3 <= 2'b00;
                  Holding_Err3 <= 0;
                  pl3_3_conversion_state <= 2'b10;
                 end
                //3'b011 means data bits 63-40 have valid data
                3'b011:
                 begin
                  SPI3_3_WrEOP <= 1;
                  SPI3_3_WrErr <= SPI3_3_BrstData[0];
                  SPI3_3_WrMod <= 2'b01;
                  Holding_EOP3 <= 0;
                  Holding_Mod3 <= 2'b00;
                  Holding_Err3 <= 0;
                  pl3_3_conversion_state <= 2'b10;
                 end
                //3'b010 means data bits 63-48 have valid data
                3'b010:
                 begin
                  SPI3_3_WrEOP <= 1;
                  SPI3_3_WrErr <= SPI3_3_BrstData[0];
                  SPI3_3_WrMod <= 2'b10;
                  Holding_EOP3 <= 0;
                  Holding_Mod3 <= 2'b00;
                  Holding_Err3 <= 0;
                  pl3_3_conversion_state <= 2'b10;
                 end
                //3'b001 means data bits 63-56 have valid data
                3'b001:
                 begin
                  SPI3_3_WrEOP <= 1;
                  SPI3_3_WrErr <= SPI3_3_BrstData[0];
                  SPI3_3_WrMod <= 2'b11;
                  Holding_EOP3 <= 0;
                  Holding_Mod3 <= 2'b00;
                  Holding_Err3 <= 0;
                  pl3_3_conversion_state <= 2'b10;
                 end
                //We have covered all the cases except X's & U's, so leave null
                default:
                 begin
                  SPI3_3_WrEOP <= SPI3_3_WrEOP;
                  SPI3_3_WrErr <= SPI3_3_WrErr;
                  SPI3_3_WrMod <= SPI3_3_WrMod;
                  Holding_EOP3 <= Holding_EOP3;
                  Holding_Mod3 <= Holding_Mod3;
                  Holding_Err3 <= Holding_Err3;
                  pl3_3_conversion_state <= pl3_3_conversion_state;
                 end
              endcase
            else
             begin
              SPI3_3_WrEOP <= 0;
              SPI3_3_WrErr <= 0;
              SPI3_3_WrMod <= 2'b00;
              Holding_EOP3 <= 0;
              Holding_Mod3 <= 2'b00;
              Holding_Err3 <= 0;
              if (SPI3_3_DST_RDY)
	              pl3_3_conversion_state <= 2'b01;
	              else
	              pl3_3_conversion_state <= 2'b00;
             end
             end
           else begin
             		pl3_3_conversion_state <= 2'b11;
            		SPI3_3_WrEn_n <= 1;
           end
          end
          
          else
           begin
            pl3_3_conversion_state <= 2'b00;
            SPI3_3_WrEn_n <= 1;
           end
          
        2'b01:
         begin
         	if (SPI3_3_DST_RDY) begin
          pl3_3_conversion_state <= 00;
          SPI3_3_WrData <= Holding_Data3;
          SPI3_3_WrEn_n <= 0;
          SPI3_3_WrSOP  <= 0;
          SPI3_3_WrEOP  <= Holding_EOP3;
          SPI3_3_WrMod  <= Holding_Mod3;
          SPI3_3_WrErr  <= Holding_Err3;
          end
         end
        2'b10:
         begin
         	if (SPI3_3_DST_RDY) begin
          pl3_3_conversion_state <= 2'b00;
          SPI3_3_WrEn_n <= 1;
          end
          end
        2'b11:
        	begin
        		if (SPI3_3_DST_RDY) begin
        		pl3_3_conversion_state <= 2'b01;
            SPI3_3_WrData <= SPI3_3_BrstData[69:38];
            SPI3_3_WrEn_n <= 0;
            SPI3_3_WrSOP  <= SPI3_3_BrstData[5];
            Holding_Data3 <= SPI3_3_BrstData[37:6];
            //Look at Mod stuff here
            //If there is an EOP, Mod must be looked at
            if (SPI3_3_BrstData[4] == 1) begin
              case (SPI3_3_BrstData[3:1])
                //3'b000 means data bits 63-0 have valid data
                3'b000:
                 begin
                  SPI3_3_WrEOP <= 0;
                  SPI3_3_WrErr <= 0;
                  SPI3_3_WrMod <= 2'b00;
                  Holding_EOP3 <= 1;
                  Holding_Mod3 <= 2'b00;
                  Holding_Err3 <= SPI3_3_BrstData[0];
                  pl3_3_conversion_state <= 2'b01;
                 end
                //3'b111 means data bits 63-8 have valid data
                3'b111:
                 begin
                  SPI3_3_WrEOP <= 0;
                  SPI3_3_WrErr <= 0;
                  SPI3_3_WrMod <= 2'b00;
                  Holding_EOP3 <= 1;
                  Holding_Mod3 <= 2'b01;
                  Holding_Err3 <= SPI3_3_BrstData[0];
                  pl3_3_conversion_state <= 2'b01;
                 end
                //3'b110 means data bits 63-16 have valid data
                3'b110:
                 begin
                  SPI3_3_WrEOP <= 0;
                  SPI3_3_WrErr <= 0;
                  SPI3_3_WrMod <= 2'b00;
                  Holding_EOP3 <= 1;
                  Holding_Mod3 <= 2'b10;
                  Holding_Err3 <= SPI3_3_BrstData[0];
                  pl3_3_conversion_state <= 2'b01;
                 end
                //3'b101 means data bits 6324 have valid data
                3'b101:
                 begin
                  SPI3_3_WrEOP <= 0;
                  SPI3_3_WrErr <= 0;
                  SPI3_3_WrMod <= 2'b00;
                  Holding_EOP3 <= 1;
                  Holding_Mod3 <= 2'b11;
                  Holding_Err3 <= SPI3_3_BrstData[0];
                  pl3_3_conversion_state <= 2'b01;
                 end
                //3'b100 means data bits 63-32 have valid data
                3'b100:
                 begin
                  SPI3_3_WrEOP <= 1;
                  SPI3_3_WrErr <= SPI3_3_BrstData[0];
                  SPI3_3_WrMod <= 2'b00;
                  Holding_EOP3 <= 0;
                  Holding_Mod3 <= 2'b00;
                  Holding_Err3 <= 0;
                  pl3_3_conversion_state <= 2'b10;
                 end
                //3'b011 means data bits 63-40 have valid data
                3'b011:
                 begin
                  SPI3_3_WrEOP <= 1;
                  SPI3_3_WrErr <= SPI3_3_BrstData[0];
                  SPI3_3_WrMod <= 2'b01;
                  Holding_EOP3 <= 0;
                  Holding_Mod3 <= 2'b00;
                  Holding_Err3 <= 0;
                  pl3_3_conversion_state <= 2'b10;
                 end
                //3'b010 means data bits 63-48 have valid data
                3'b010:
                 begin
                  SPI3_3_WrEOP <= 1;
                  SPI3_3_WrErr <= SPI3_3_BrstData[0];
                  SPI3_3_WrMod <= 2'b10;
                  Holding_EOP3 <= 0;
                  Holding_Mod3 <= 2'b00;
                  Holding_Err3 <= 0;
                  pl3_3_conversion_state <= 2'b10;
                 end
                //3'b001 means data bits 63-56 have valid data
                3'b001:
                 begin
                  SPI3_3_WrEOP <= 1;
                  SPI3_3_WrErr <= SPI3_3_BrstData[0];
                  SPI3_3_WrMod <= 2'b11;
                  Holding_EOP3 <= 0;
                  Holding_Mod3 <= 2'b00;
                  Holding_Err3 <= 0;
                  pl3_3_conversion_state <= 2'b10;
                 end
                //We have covered all the cases except X's & U's, so leave null
                default:
                 begin
                  SPI3_3_WrEOP <= SPI3_3_WrEOP;
                  SPI3_3_WrErr <= SPI3_3_WrErr;
                  SPI3_3_WrMod <= SPI3_3_WrMod;
                  Holding_EOP3 <= Holding_EOP3;
                  Holding_Mod3 <= Holding_Mod3;
                  Holding_Err3 <= Holding_Err3;
                  pl3_3_conversion_state <= pl3_3_conversion_state;
                 end
              endcase
            end  
          else
           begin
            pl3_3_conversion_state <= 2'b11;
            SPI3_3_WrEn_n <= 1;
           end
        	end
        end
        default:
         begin
          pl3_3_conversion_state <= 2'b00;
          SPI3_3_WrEn_n <= 1;
         end
      endcase
  end
  
  assign SPI3_3_BrstRdEn = SPI3_3_BrstRdEn_int;


endmodule