-------------------------------------------------------------------------------
--
--   File Name:  Bridge_4Channel_top.vhd
--     Version:  1.0
--        Date:  June 11, 2007
--
-- Description:  This file is the top level wrapper for 4 Channel
--               SPI4.2 to SPI3 bridge 
--
--      Author:  Zhe Xia
--     Company:  Xilinx Inc.
--
--
--     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
--     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
--     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
--     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
--     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
--     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
--     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
--     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
--     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
--     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
--     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
--     FOR A PARTICULAR PURPOSE.
--
--     (c) Copyright 2002 Xilinx, Inc.
--     All rights reserved.
--
--------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- Library Declarations
-------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity bridge_top is
  port (
    --System signals
    Reset_n                 : in  std_logic;
	 Ref_Clk					    : in  std_logic;	

    -- SPI4.2 Sink signals, see SPI4.2 core datasheet for details
    SPI4_RDClk_P            : in  std_logic;
    SPI4_RDClk_N            : in  std_logic;
    SPI4_RCtl_P             : in  std_logic;
    SPI4_RCtl_N             : in  std_logic;
    SPI4_RDat_P             : in  std_logic_vector(15 downto 0);
    SPI4_RDat_N             : in  std_logic_vector(15 downto 0);

    -- SPI4.2 Sink Status (LVTTL) input,  see SPI4.2 core datasheet for details
    SPI4_RSClk              : out std_logic;
    SPI4_RStat              : out std_logic_vector(1 downto 0);

    -- SPI4.2 Source signals,  see SPI4.2 core datasheet for details
    SPI4_TDClk_P            : out std_logic;
    SPI4_TDClk_N            : out std_logic;
    SPI4_TCtl_P             : out std_logic;
    SPI4_TCtl_N             : out std_logic;
    SPI4_TDat_P             : out std_logic_vector(15 downto 0);
    SPI4_TDat_N             : out std_logic_vector(15 downto 0);
    SPI4_SysClk_P           : in  std_logic;
    SPI4_SysClk_N           : in  std_logic;
    
    -- SPI4.2 Source status(LVTTL) input,  see SPI4.2 core datasheet for details
    SPI4_TSClk              : in  std_logic;
    SPI4_TStat              : in  std_logic_vector(1 downto 0);

    --SPI3 Clocks - All four SPI3 interfaces share common Rx, Tx Clocks.
    SPI3_RFClk              : in  std_logic;

    --SPI3 0 Sink Interface,  see SPI3 core datasheet for details
    SPI3_0_RVal            : in  std_logic;
    SPI3_0_REnb            : out std_logic;
    SPI3_0_RSx             : in  std_logic;
    SPI3_0_RSOP            : in  std_logic;
    SPI3_0_REOP            : in  std_logic;
    SPI3_0_RDat            : in  std_logic_vector(31 downto 0);
    SPI3_0_RPrty           : in  std_logic;
    SPI3_0_RMod            : in  std_logic_vector(1 downto 0);
    SPI3_0_RErr            : in  std_logic;

    --SPi3 0 Source Interface,  see SPI3 core datasheet for details
    SPI3_TFClk           : in  std_logic;
    SPI3_0_TEnb          : out std_logic;
    SPI3_0_TSx             : out std_logic;
    SPI3_0_TSOP            : out std_logic;
    SPI3_0_TEOP            : out std_logic;
    SPI3_0_TDat            : out std_logic_vector(31 downto 0);
    SPI3_0_TPrty           : out std_logic;
    SPI3_0_TMod            : out std_logic_vector(1 downto 0);
    SPI3_0_TErr            : out std_logic;
    SPI3_0_PTPA            : in  std_logic;
    SPI3_0_TAdr            : out std_logic_vector(7 downto 0);


    --SPI3 1 Sink Interface,  see SPI3 core datasheet for details
    SPI3_1_RVal            : in  std_logic;
    SPI3_1_REnb          : out std_logic;
    SPI3_1_RSx             : in  std_logic;
    SPI3_1_RSOP            : in  std_logic;
    SPI3_1_REOP            : in  std_logic;
    SPI3_1_RDat            : in  std_logic_vector(31 downto 0);
    SPI3_1_RPrty           : in  std_logic;
    SPI3_1_RMod            : in  std_logic_vector(1 downto 0);
    SPI3_1_RErr            : in  std_logic;

    --SPi3 1 Source Interface,  see SPI3 core datasheet for details
    SPI3_1_TEnb            : out std_logic;
    SPI3_1_TSx             : out std_logic;
    SPI3_1_TSOP            : out std_logic;
    SPI3_1_TEOP            : out std_logic;
    SPI3_1_TDat            : out std_logic_vector(31 downto 0);
    SPI3_1_TPrty           : out std_logic;
    SPI3_1_TMod            : out std_logic_vector(1 downto 0);
    SPI3_1_TErr            : out std_logic;
    SPI3_1_PTPA            : in  std_logic;
    SPI3_1_TAdr            : out std_logic_vector(7 downto 0);


    --SPI3 2 Sink Interface,  see SPI3 core datasheet for details
    SPI3_2_RVal            : in  std_logic;
    SPI3_2_REnb          : out std_logic;
    SPI3_2_RSx             : in  std_logic;
    SPI3_2_RSOP            : in  std_logic;
    SPI3_2_REOP            : in  std_logic;
    SPI3_2_RDat            : in  std_logic_vector(31 downto 0);
    SPI3_2_RPrty           : in  std_logic;
    SPI3_2_RMod            : in  std_logic_vector(1 downto 0);
    SPI3_2_RErr            : in  std_logic;

    --SPi3 2 Source Interface,  see SPI3 core datasheet for details
    SPI3_2_TEnb          : out std_logic;
    SPI3_2_TSx             : out std_logic;
    SPI3_2_TSOP            : out std_logic;
    SPI3_2_TEOP            : out std_logic;
    SPI3_2_TDat            : out std_logic_vector(31 downto 0);
    SPI3_2_TPrty           : out std_logic;
    SPI3_2_TMod            : out std_logic_vector(1 downto 0);
    SPI3_2_TErr            : out std_logic;
    SPI3_2_PTPA            : in  std_logic;
    SPI3_2_TAdr            : out std_logic_vector(7 downto 0);
    
    --SPI3 3 Sink Interface,  see SPI3 core datasheet for details
    SPI3_3_RVal            : in  std_logic;
    SPI3_3_REnb            : out std_logic;
    SPI3_3_RSx             : in  std_logic;
    SPI3_3_RSOP            : in  std_logic;
    SPI3_3_REOP            : in  std_logic;
    SPI3_3_RDat            : in  std_logic_vector(31 downto 0);
    SPI3_3_RPrty           : in  std_logic;
    SPI3_3_RMod            : in  std_logic_vector(1 downto 0);
    SPI3_3_RErr            : in  std_logic;

    --SPi3 3 Source Interface,  see SPI3 core datasheet for details
    SPI3_3_TEnb          : out std_logic;
    SPI3_3_TSx             : out std_logic;
    SPI3_3_TSOP            : out std_logic;
    SPI3_3_TEOP            : out std_logic;
    SPI3_3_TDat            : out std_logic_vector(31 downto 0);
    SPI3_3_TPrty           : out std_logic;
    SPI3_3_TMod            : out std_logic_vector(1 downto 0);
    SPI3_3_TErr            : out std_logic;
    SPI3_3_PTPA            : in  std_logic;
    SPI3_3_TAdr            : out std_logic_vector(7 downto 0);
    
    --Misc (can be removed, was added only to keep signals from being
    --optimized away)
    SPI3_ParityErr         : out std_logic
    );

end bridge_top;


architecture xilinx of bridge_top is

component spi3_to_spi4_top
  port (
    -- System signals
    Reset_n                 : in  std_logic;
	 SPI4_SrcFifoReset_n     : in std_logic;

    --SPI3 Rx Clock - common to all 4 SPI3 interfaces   
    SPI3_RFClk             : in  std_logic;
    
    -- SPI3 0 Interface
    SPI3_0_RVal            : in  std_logic;
    SPI3_0_REnb          : out std_logic;
    SPI3_0_RSx             : in  std_logic;
    SPI3_0_RSOP            : in  std_logic;
    SPI3_0_REOP            : in  std_logic;
    SPI3_0_RDat            : in  std_logic_vector(31 downto 0);
    SPI3_0_RPrty           : in  std_logic;
    SPI3_0_RMod            : in  std_logic_vector(1 downto 0);
    SPI3_0_RErr            : in  std_logic;

    -- SPI3 1 Interface
    SPI3_1_RVal            : in  std_logic;
    SPI3_1_REnb          : out std_logic;
    SPI3_1_RSx             : in  std_logic;
    SPI3_1_RSOP            : in  std_logic;
    SPI3_1_REOP            : in  std_logic;
    SPI3_1_RDat            : in  std_logic_vector(31 downto 0);
    SPI3_1_RPrty           : in  std_logic;
    SPI3_1_RMod            : in  std_logic_vector(1 downto 0);
    SPI3_1_RErr            : in  std_logic;

    -- SPI3 2 Interface
    SPI3_2_RVal            : in  std_logic;
    SPI3_2_REnb            : out std_logic;
    SPI3_2_RSx             : in  std_logic;
    SPI3_2_RSOP            : in  std_logic;
    SPI3_2_REOP            : in  std_logic;
    SPI3_2_RDat            : in  std_logic_vector(31 downto 0);
    SPI3_2_RPrty           : in  std_logic;
    SPI3_2_RMod            : in  std_logic_vector(1 downto 0);
    SPI3_2_RErr            : in  std_logic;

    -- SPI3 3 Interface
    SPI3_3_RVal            : in  std_logic;
    SPI3_3_REnb          : out std_logic;
    SPI3_3_RSx             : in  std_logic;
    SPI3_3_RSOP            : in  std_logic;
    SPI3_3_REOP            : in  std_logic;
    SPI3_3_RDat            : in  std_logic_vector(31 downto 0);
    SPI3_3_RPrty           : in  std_logic;
    SPI3_3_RMod            : in  std_logic_vector(1 downto 0);
    SPI3_3_RErr            : in  std_logic;

    
    -- SPI4.2 Source Interface Signals
    SPI4_TDClk_P            : out std_logic;
    SPI4_TDClk_N            : out std_logic;
    SPI4_TCtl_P             : out std_logic;
    SPI4_TCtl_N             : out std_logic;
    SPI4_TDat_P             : out std_logic_vector(15 downto 0);
    SPI4_TDat_N             : out std_logic_vector(15 downto 0);
    
    -- SPI4.2 source status (LVTTL) input
    SPI4_TSClk              : in  std_logic;
    SPI4_TStat              : in  std_logic_vector(1 downto 0);

    --SPI3 Parity Error 
    SPI3_ParityErr         : out std_logic;

    -- SPI3 DCM Clock signals
    SPI3_DcmReset_RFClk     : in  std_logic;
    SPI3_Locked_RFClk       : out std_logic;

    --SPI4.2 System clock signals
    SPI4_SysClk_P           : in  std_logic;
    SPI4_SysClk_N           : in  std_logic;

    --SPI4.2 DCM clock signals
    SPI4_DcmReset_TDClk     : in  std_logic;
    SPI4_DcmReset_TSClk     : in  std_logic;
    SPI4_Locked_TDClk       : out std_logic;
    SPI4_Locked_TSClk       : out std_logic;
    SPI4_SysClkDiv_GP       : out std_logic;
    
    --SPI4.2 Calendar Programming Signals
    SPI4_SrcCalendar_M      : in  std_logic_vector(7 downto 0); 
    SPI4_SrcCalendar_Len    : in  std_logic_vector(8 downto 0); 
    SPI4_SrcCalClk          : in  std_logic;                    
    SPI4_SrcCalWrEn_n       : in  std_logic;                    
    SPI4_SrcCalAddr         : in  std_logic_vector(8 downto 0); 
    SPI4_SrcCalData         : in  std_logic_vector(7 downto 0); 
    SPI4_SrcCalDataOut      : out std_logic_vector(7 downto 0);
    
    --SPI4.2 source enable
    SPI4_SrcEn              : in  std_logic
    );
end component;

component spi4_to_spi3_top
  port (
    -- System signals
    Reset_n                 : in  std_logic;
	 SPI4_FFReset_n			 : in  std_logic;	
    SPI4_Clk                : in  std_logic;
	 Clk200						 : in  std_logic;

    -- SPI3 0 Interface
    SPI3_TFClk             : in  std_logic;
    SPI3_0_TEnb            : out std_logic;
    SPI3_0_TSx             : out std_logic;
    SPI3_0_TSOP            : out std_logic;
    SPI3_0_TEOP            : out std_logic;
    SPI3_0_TDat            : out std_logic_vector(31 downto 0);
    SPI3_0_TPrty           : out std_logic;
    SPI3_0_TMod            : out std_logic_vector(1 downto 0);
    SPI3_0_TErr            : out std_logic;
    SPI3_0_PTPA            : in  std_logic;
    SPI3_0_TAdr            : out std_logic_vector(7 downto 0);

    -- SPI3 1 Interface
    SPI3_1_TEnb            : out std_logic;
    SPI3_1_TSx             : out std_logic;
    SPI3_1_TSOP            : out std_logic;
    SPI3_1_TEOP            : out std_logic;
    SPI3_1_TDat            : out std_logic_vector(31 downto 0);
    SPI3_1_TPrty           : out std_logic;
    SPI3_1_TMod            : out std_logic_vector(1 downto 0);
    SPI3_1_TErr            : out std_logic;
    SPI3_1_PTPA            : in  std_logic;
    SPI3_1_TAdr            : out std_logic_vector(7 downto 0);

    -- SPI3 2 Interface
    SPI3_2_TEnb            : out std_logic;
    SPI3_2_TSx             : out std_logic;
    SPI3_2_TSOP            : out std_logic;
    SPI3_2_TEOP            : out std_logic;
    SPI3_2_TDat            : out std_logic_vector(31 downto 0);
    SPI3_2_TPrty           : out std_logic;
    SPI3_2_TMod            : out std_logic_vector(1 downto 0);
    SPI3_2_TErr            : out std_logic;
    SPI3_2_PTPA            : in  std_logic;
    SPI3_2_TAdr            : out std_logic_vector(7 downto 0);

    -- SPI3 3 Interface
    SPI3_3_TEnb            : out std_logic;
    SPI3_3_TSx             : out std_logic;
    SPI3_3_TSOP            : out std_logic;
    SPI3_3_TEOP            : out std_logic;
    SPI3_3_TDat            : out std_logic_vector(31 downto 0);
    SPI3_3_TPrty           : out std_logic;
    SPI3_3_TMod            : out std_logic_vector(1 downto 0);
    SPI3_3_TErr            : out std_logic;
    SPI3_3_PTPA            : in  std_logic;
    SPI3_3_TAdr            : out std_logic_vector(7 downto 0);

    -- System clocking input
    SPI3_DcmReset_TFClk     : in  std_logic;
    SPI4_DcmReset_RDClk     : in  std_logic;
    SPI4_Locked_RDClk       : out std_logic;
    
    -- SPI4.2 Interface
    -- SPI4.2 sink bus output
    SPI4_RDClk_P            : in  std_logic;
    SPI4_RDClk_N            : in  std_logic;
    SPI4_RCtl_P             : in  std_logic;
    SPI4_RCtl_N             : in  std_logic;
    SPI4_RDat_P             : in  std_logic_vector(15 downto 0);
    SPI4_RDat_N             : in  std_logic_vector(15 downto 0);

    -- SPI4.2 sink status (LVTTL) input
    SPI4_RSClk              : out std_logic;
    SPI4_RStat              : out std_logic_vector(1 downto 0);
    
    --SPI4.2 Calendar Programming Signals
    SPI4_SnkCalendar_M      : in  std_logic_vector(7 downto 0); 
    SPI4_SnkCalendar_Len    : in  std_logic_vector(8 downto 0); 
--    SPI4_SnkCalClk          : in  std_logic;                    
    SPI4_SnkCalWrEn_n       : in  std_logic;                    
    SPI4_SnkCalAddr         : in  std_logic_vector(8 downto 0); 
    SPI4_SnkCalData         : in  std_logic_vector(7 downto 0); 
    SPI4_SnkCalDataOut      : out std_logic_vector(7 downto 0); 
    
    --SPI4.2 Out of Frame
    SPI4_SnkOof             : out std_logic;
    SPI4_PhaseAlignComplete : out std_logic;
    SPI4_PhaseAlignRequest  : in  std_logic;
    SPI3_TFClk_bufg_out     : out std_logic;
    
    --PL4 sink enable
    SPI4_SnkEn              : in  std_logic;
	 SPI4_SnkTrainValid     : out std_logic;
	 SPI4_SnkDPAFailed      : out std_logic;
	 SPI4_SnkClksRdy        : out std_logic
    ); 
end component;

component spi_clk_startup
  port (
    -- System signals
    Reset_n                   : in  std_logic;
    SPI_Clk                   : in  std_logic;

    -- SPI4.2 Sink DCM Status
    SPI4_DcmReset_RDClk     : out std_logic;
    SPI4_Locked_RDClk       : in  std_logic;

    -- SPI4.2 Source DCM Status
    SPI4_DcmReset_TDClk     : out std_logic;
    SPI4_Locked_TDClk       : in  std_logic;
    SPI4_DcmReset_TSClk     : out std_logic;
    SPI4_Locked_TSClk       : in  std_logic;

    PL4Rst_n                : out std_logic;
    
    -- Calendar Programming
    SnkStatClk           : in  std_logic;
    SrcStatClk           : in  std_logic;
    CalProg              : out std_logic;
    SnkCalProgDone       : in  std_logic;
    SrcCalProgDone       : in  std_logic;
    
    -- Sink and Source Core enables
    SnkEn              : out std_logic;
    SrcEn              : out std_logic;
    
    -- Dynamic phase aligning signals
    SnkFFClk           : in  std_logic;
    PhaseAlignRequest  : out std_logic;
    PhaseAlignComplete : in  std_logic;
    SnkOof             : in  std_logic
    
    );
end component;

  -- SPI4 Enable signals
  signal SPI4_SrcEn             : std_logic;
  signal SPI4_SnkEn             : std_logic;

  --SPI4 Clk signals
  signal SPI4_SysClkDiv_GP      : std_logic;
  signal Bridge_Clk              : std_logic;
  signal Reset                  : std_logic;
  signal SPI4_TDClk_Locked      : std_logic;
  signal SPI4_TSClk_Locked      : std_logic;
  signal SPI4_RDClk_Locked      : std_logic;

  --SPI4 Calendar signals
  signal count                  : std_logic_vector(1 downto 0);
  signal SPI4CalWr_n            : std_logic;
  signal SPI4CalAddr            : std_logic_vector(8 downto 0);
  signal SPI4CalData            : std_logic_vector(7 downto 0);
  signal SPI4_SnkCalWrEn_n      : std_logic;
  signal SPI4_SnkCalAddr        : std_logic_vector(8 downto 0);
  signal SPI4_SnkCalData        : std_logic_vector(7 downto 0);
  signal SPI4_SnkCalDataOut     : std_logic_vector(7 downto 0);
  signal SPI4_SrcCalWrEn_n      : std_logic;
  signal SPI4_SrcCalAddr        : std_logic_vector(8 downto 0);
  signal SPI4_SrcCalData        : std_logic_vector(7 downto 0);
  signal SPI4_SrcCalDataOut     : std_logic_vector(7 downto 0);

  --Static Configuration Parameters
  signal SPI4_SnkCalendar_Len   : std_logic_vector(8 downto 0);
  signal SPI4_SnkCalendar_M     : std_logic_vector(7 downto 0);
  signal SPI4_SrcCalendar_Len   : std_logic_vector(8 downto 0);
  signal SPI4_SrcCalendar_M     : std_logic_vector(7 downto 0);
  
  --Initialization signals
  signal CalProg                : std_logic;
  signal SnkCalProgDone         : std_logic;
  signal SrcCalProgDone         : std_logic;
  signal SPI4_SnkOof            : std_logic;
  signal SPI4_PhaseAlignComplete : std_logic;
  signal SPI4_PhaseAlignRequest : std_logic;
  signal SPI4_DcmReset_RDClk    : std_logic;
  signal SPI4_DcmReset_TDClk    : std_logic;
  signal SPI4_DcmReset_TSClk    : std_logic;
  signal SPI3_TFClk_bufg_out    : std_logic;
  signal SPI4_SnkDPAFailed      : std_logic;
  signal SPI4_SnkFailed      : std_logic;

  --Synthesis Attributes
--  attribute syn_black_box                     : boolean;
--  attribute black_box_pad_pin                 : string ;
--  attribute syn_black_box of spi3_to_spi4_top : component is true;
--  attribute syn_black_box of spi4_to_spi3_top : component is true;
--  attribute black_box_pad_pin of spi4_to_spi3_top : component is "SPI4_RDClk_P,SPI4_RDClk_N,  SPI4_RDat_P(15:0),SPI4_RDat_N(15:0), SPI4_RCtl_P,SPI4_RCtl_N, SPI4_RSClk, SPI4_RStat(1:0), SPI3_TFClk, SPI3_TFClk_bufg_out";
--  
--  attribute black_box_pad_pin of spi3_to_spi4_top : component is "SPI4_TDClk_P,SPI4_TDClk_N, SPI4_TDat_P(15:0),SPI4_TDat_N(15:0),SPI4_TCtl_P,SPI4_TCtl_N, SPI4_TSClk, SPI4_TStat(1:0), SPI4_SysClk_P, SPI4_SysClk_N, SPI3_RFClk";
  
begin
  --SPI-4 Core Paramenters
  SPI4_SnkCalendar_Len   <= conv_std_logic_vector(3, 9);
  SPI4_SnkCalendar_M     <= conv_std_logic_vector(0, 8);
  SPI4_SrcCalendar_Len   <= conv_std_logic_vector(3, 9); 
  SPI4_SrcCalendar_M     <= conv_std_logic_vector(0, 8);
  
  Bridge_Clk <=     SPI4_SysClkDiv_GP;
  
  --Program SPI-4 Calendars
prog_cal:  process(Reset_n, Bridge_Clk)
  begin
    if(Reset_n = '0') then
      count <= "00";
    elsif(Bridge_Clk'event and Bridge_Clk = '1') then
      if(CalProg = '1') then
        if (count /= "11") then
          count <= count + '1';
        end if;
      end if;
    end if;
  end process;


  --Generate SPI-4 Enable signal
gen_spi4en: process(Reset_n, Bridge_Clk)
  begin
    if(Reset_n = '0') then
      SnkCalProgDone <= '0';
      SrcCalProgDone <= '0';
    elsif(Bridge_Clk'event and Bridge_Clk = '1') then
      if(count = "11") then
        SnkCalProgDone <= '1';
        SrcCalProgDone <= '1';
      else
        SnkCalProgDone <= '0';
        SrcCalProgDone <= '0';
      end if;
    end if;
  end process;

  
 --SPI4 Sink Calendar Signals
  SPI4_SnkCalWrEn_n <= not (CalProg and not SnkCalProgDone);
  SPI4_SnkCalAddr   <= "0000000" & count;
  SPI4_SnkCalData   <= "000000" & count;

 --SPI4 Source Calendar Signals
  SPI4_SrcCalWrEn_n <= not (CalProg and not SrcCalProgDone);
  SPI4_SrcCalAddr   <= "0000000" & count;
  SPI4_SrcCalData   <= "000000" & count;

  Reset <= not Reset_n;

spi3_to_spi4_top0 : spi3_to_spi4_top
  port map (
    -- System signals
    Reset_n                => Reset_n,
	 SPI4_SrcFifoReset_n    => Reset_n,
    SPI3_RFClk             => SPI3_RFClk,
    
    -- SPI3 0 Interface
    SPI3_0_RVal            => SPI3_0_RVal,
    SPI3_0_REnb            => SPI3_0_REnb,   
    SPI3_0_RSx             => SPI3_0_RSx,
    SPI3_0_RSOP            => SPI3_0_RSOP,
    SPI3_0_REOP            => SPI3_0_REOP,
    SPI3_0_RDat            => SPI3_0_RDat,
    SPI3_0_RPrty           => SPI3_0_RPrty,
    SPI3_0_RMod            => SPI3_0_RMod,
    SPI3_0_RErr            => SPI3_0_RErr,

    -- SPI3 1 Interface
    SPI3_1_RVal            => SPI3_1_RVal,
    SPI3_1_REnb            => SPI3_1_REnb,   
    SPI3_1_RSx             => SPI3_1_RSx,
    SPI3_1_RSOP            => SPI3_1_RSOP,
    SPI3_1_REOP            => SPI3_1_REOP,
    SPI3_1_RDat            => SPI3_1_RDat,
    SPI3_1_RPrty           => SPI3_1_RPrty,
    SPI3_1_RMod            => SPI3_1_RMod,
    SPI3_1_RErr            => SPI3_1_RErr,

    -- SPI3 2 Interface
    SPI3_2_RVal            => SPI3_2_RVal,
    SPI3_2_REnb            => SPI3_2_REnb,   
    SPI3_2_RSx             => SPI3_2_RSx,
    SPI3_2_RSOP            => SPI3_2_RSOP,
    SPI3_2_REOP            => SPI3_2_REOP,
    SPI3_2_RDat            => SPI3_2_RDat,
    SPI3_2_RPrty           => SPI3_2_RPrty,
    SPI3_2_RMod            => SPI3_2_RMod,
    SPI3_2_RErr            => SPI3_2_RErr,
    
    SPI3_ParityErr         => SPI3_ParityErr,

    -- SPI3 3 Interface
    SPI3_3_RVal            => SPI3_3_RVal,
    SPI3_3_REnb            => SPI3_3_REnb,   
    SPI3_3_RSx             => SPI3_3_RSx,
    SPI3_3_RSOP            => SPI3_3_RSOP,
    SPI3_3_REOP            => SPI3_3_REOP,
    SPI3_3_RDat            => SPI3_3_RDat,
    SPI3_3_RPrty           => SPI3_3_RPrty,
    SPI3_3_RMod            => SPI3_3_RMod,
    SPI3_3_RErr            => SPI3_3_RErr,

    -- System clocking input
    SPI3_DcmReset_RFClk     => Reset,
    
    SPI4_SysClk_P           => SPI4_SysClk_P,
    SPI4_SysClk_N           => SPI4_SysClk_N,
    SPI4_DcmReset_TDClk     => SPI4_DcmReset_TDClk,
    SPI4_DcmReset_TSClk     => SPI4_DcmReset_TSClk,
    SPI4_Locked_TDClk       => SPI4_TDClk_Locked,
    SPI4_Locked_TSClk       => SPI4_TSClk_Locked,
    SPI4_SysClkDiv_GP       => SPI4_SysClkDiv_GP,

    --SPI4.2 Calendar Programming Signals
    SPI4_SrcCalendar_M      => SPI4_SrcCalendar_M,
    SPI4_SrcCalendar_Len    => SPI4_SrcCalendar_Len,
    SPI4_SrcCalClk          => Bridge_Clk,                    
    SPI4_SrcCalWrEn_n       => SPI4_SrcCalWrEn_n,
    SPI4_SrcCalAddr         => SPI4_SrcCalAddr,
    SPI4_SrcCalData         => SPI4_SrcCalData,
    SPI4_SrcCalDataOut      => SPI4_SrcCalDataOut,
    
    -- SPI4.2 Interface
    -- SPI4.2 source bus output
    SPI4_TDClk_P            => SPI4_TDClk_P,
    SPI4_TDClk_N            => SPI4_TDClk_N,
    SPI4_TCtl_P             => SPI4_TCtl_P,
    SPI4_TCtl_N             => SPI4_TCtl_N,
    SPI4_TDat_P             => SPI4_TDat_P,
    SPI4_TDat_N             => SPI4_TDat_N,
    
    -- SPI4.2 source status (LVTTL) input
    SPI4_TSClk              => SPI4_TSClk,
    SPI4_TStat              => SPI4_TStat,
    
    -- SPI4 source enable
    SPI4_SrcEn              => SPI4_SrcEn
    );   


spi4_to_spi3_top0 : spi4_to_spi3_top
  port map (
     -- System signals
      Reset_n              => Reset_n,
	   SPI4_FFReset_n			=> Reset_n,
      SPI4_Clk             => Bridge_Clk,
		Clk200					=> Ref_Clk,
      
      -- SPI3 0 Interface
      SPI3_TFClk           => SPI3_TFClk,
      SPI3_0_TEnb          => SPI3_0_TEnb,
      SPI3_0_TSx           => SPI3_0_TSx,
      SPI3_0_TSOP          => SPI3_0_TSOP,
      SPI3_0_TEOP          => SPI3_0_TEOP,
      SPI3_0_TDat          => SPI3_0_TDat,
      SPI3_0_TPrty         => SPI3_0_TPrty,
      SPI3_0_TMod          => SPI3_0_TMod,
      SPI3_0_TErr          => SPI3_0_TErr,
      SPI3_0_PTPA          => SPI3_0_PTPA,
      SPI3_0_TAdr          => SPI3_0_TAdr,

      -- SPI3 1 Interface
      SPI3_1_TEnb          => SPI3_1_TEnb,
      SPI3_1_TSx           => SPI3_1_TSx,
      SPI3_1_TSOP          => SPI3_1_TSOP,
      SPI3_1_TEOP          => SPI3_1_TEOP,
      SPI3_1_TDat          => SPI3_1_TDat,
      SPI3_1_TPrty         => SPI3_1_TPrty,
      SPI3_1_TMod          => SPI3_1_TMod,
      SPI3_1_TErr          => SPI3_1_TErr,
      SPI3_1_PTPA          => SPI3_1_PTPA,
      SPI3_1_TAdr          => SPI3_1_TAdr,

      -- SPI3 2 Interface
      SPI3_2_TEnb          => SPI3_2_TEnb,
      SPI3_2_TSx           => SPI3_2_TSx,
      SPI3_2_TSOP          => SPI3_2_TSOP,
      SPI3_2_TEOP          => SPI3_2_TEOP,
      SPI3_2_TDat          => SPI3_2_TDat,
      SPI3_2_TPrty         => SPI3_2_TPrty,
      SPI3_2_TMod          => SPI3_2_TMod,
      SPI3_2_TErr          => SPI3_2_TErr,
      SPI3_2_PTPA          => SPI3_2_PTPA,
      SPI3_2_TAdr          => SPI3_2_TAdr,

      -- SPI3 3 Interface
      SPI3_3_TEnb          => SPI3_3_TEnb,
      SPI3_3_TSx           => SPI3_3_TSx,
      SPI3_3_TSOP          => SPI3_3_TSOP,
      SPI3_3_TEOP          => SPI3_3_TEOP,
      SPI3_3_TDat          => SPI3_3_TDat,
      SPI3_3_TPrty         => SPI3_3_TPrty,
      SPI3_3_TMod          => SPI3_3_TMod,
      SPI3_3_TErr          => SPI3_3_TErr,
      SPI3_3_PTPA          => SPI3_3_PTPA,
      SPI3_3_TAdr          => SPI3_3_TAdr,

      -- System clocking input
      SPI3_DcmReset_TFClk  => Reset,
      SPI4_DcmReset_RDClk  => SPI4_DcmReset_RDClk,
      SPI4_Locked_RDClk    => SPI4_RDClk_Locked,
    
      -- SPI4.2 Interface
      SPI4_RDClk_P         => SPI4_RDClk_P,
      SPI4_RDClk_N         => SPI4_RDClk_N,
      SPI4_RCtl_P          => SPI4_RCtl_P,
      SPI4_RCtl_N          => SPI4_RCtl_N,
      SPI4_RDat_P          => SPI4_RDat_P,
      SPI4_RDat_N          => SPI4_RDat_N,

      -- SPI4.2 source status (LVTTL) output
      SPI4_RSClk           => SPI4_RSClk,
      SPI4_RStat           => SPI4_RStat,
    
      --SPI4.2 Calendar Programming Signals
      SPI4_SnkCalendar_M   => SPI4_SnkCalendar_M,
      SPI4_SnkCalendar_Len => SPI4_SnkCalendar_Len,
--      SPI4_SnkCalClk       => Bridge_Clk,                    
      SPI4_SnkCalWrEn_n    => SPI4_SnkCalWrEn_n,
      SPI4_SnkCalAddr      => SPI4_SnkCalAddr,
      SPI4_SnkCalData      => SPI4_SnkCalData,
      SPI4_SnkCalDataOut   => SPI4_SnkCalDataOut,
      
      --SPI4.2 Out of Frame
      SPI4_SnkOof          => SPI4_SnkOof,
      SPI4_PhaseAlignComplete => SPI4_PhaseAlignComplete,
      SPI4_PhaseAlignRequest  => SPI4_PhaseAlignRequest,
      SPI3_TFClk_bufg_out     => SPI3_TFClk_bufg_out,
                                                       
      --SPI4 sink enable
      SPI4_SnkEn            => SPI4_SnkEn,
		SPI4_SnkTrainValid    => open,
	   SPI4_SnkDPAFailed     => SPI4_SnkDPAFailed,
	   SPI4_SnkClksRdy 		 => open
    );  
    
spi_clk_startup0: spi_clk_startup
  port map(
    -- System signals
    Reset_n                   => Reset_n,
    SPI_Clk                   => SPI3_TFClk_bufg_out,

    -- SPI4.2 Sink DCM Status
    SPI4_DcmReset_RDClk     => SPI4_DcmReset_RDClk,
    SPI4_Locked_RDClk       => SPI4_RDClk_Locked,

    -- SPI4.2 Source DCM Status
    SPI4_DcmReset_TDClk     => SPI4_DcmReset_TDClk,
    SPI4_Locked_TDClk       => SPI4_TDClk_Locked,
    SPI4_DcmReset_TSClk     => SPI4_DcmReset_TSClk,
    SPI4_Locked_TSClk       => SPI4_TSClk_Locked,

    PL4Rst_n                => OPEN,
    
    -- Calendar Programming
    SnkStatClk           => Bridge_Clk,
    SrcStatClk           => Bridge_Clk,
    CalProg              => CalProg,
    SnkCalProgDone       => SnkCalProgDone,
    SrcCalProgDone       => SrcCalProgDone,
    
    -- Sink and Source Core enables
    SnkEn              => SPI4_SnkEn,
    SrcEn              => SPI4_SrcEn,
    
    -- Dynamic phase aligning signals
    SnkFFClk           => Bridge_Clk,
    PhaseAlignRequest  => SPI4_PhaseAlignRequest,
    PhaseAlignComplete => SPI4_PhaseAlignComplete,
    SnkOof             => SPI4_SnkOof
    
    );

SPI4_SnkFailed <= SPI4_SnkDPAFailed or SPI4_SnkOof;

end xilinx;
