-------------------------------------------------------------------------------
--
--   File Name:  spi3_to_spi4_core.vhd
--     Version:  1.0
--        Date:  June 11, 2007
--
-- Description:  The purpose of the spi3_to_spi4_core.vhd file is to serve as 
--               the wrapper file for all logic associated with the SPI3 to 
--               SPI4 direction of the Bridge.
--
--      Author:  Zhe Xia
--     Company:  Xilinx Inc.
--
--
--     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
--     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
--     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
--     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
--     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
--     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
--     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
--     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
--     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
--     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
--     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
--     FOR A PARTICULAR PURPOSE.
--
--     (c) Copyright 2003 Xilinx, Inc.
--     All rights reserved.
--
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Library Declarations
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use work.spi_pkg.all;

--------------------------------------------------------------------------------
-- Entity Declaration
--------------------------------------------------------------------------------

entity spi3_to_spi4_core is  
  port (
    -- System signals
    Reset_n                 : in  std_logic;
    SPI3_Clk                : in  std_logic;

    -- SPI4 Interface Signals
    SPI4_Stat               : in  std_logic_vector(7 downto 0);
    SPI4_AlmostFull_n       : in  std_logic;
    SPI4_WrAddr             : out std_logic_vector(7 downto 0);
    SPI4_WrData             : out std_logic_vector(63 downto 0);
    SPI4_WrMod              : out std_logic_vector(2 downto 0);
    SPI4_WrSOP              : out std_logic;
    SPI4_WrEOP              : out std_logic;
    SPI4_WrErr              : out std_logic;
    SPI4_WrEn_n             : out std_logic;
    
    -- SPI3 0 Interface Signals
    SPI3_0_src_rdy          : in std_logic;
    SPI3_0_dst_rdy          : out  std_logic;
    SPI3_0_FFAddr           : in  std_logic_vector(7 downto 0);
    SPI3_0_FFData           : in  std_logic_vector(31 downto 0);
    SPI3_0_FFMod            : in  std_logic_vector(1 downto 0);
    SPI3_0_FFSOP            : in  std_logic;
    SPI3_0_FFEOP            : in  std_logic;
    SPI3_0_FFErr            : in  std_logic;
    SPI3_0_FFValid          : in  std_logic;

    -- SPI3 1 Interface Signals
    SPI3_1_src_rdy          : in std_logic;
    SPI3_1_dst_rdy          : out  std_logic;
    SPI3_1_FFAddr           : in  std_logic_vector(7 downto 0);
    SPI3_1_FFData           : in  std_logic_vector(31 downto 0);
    SPI3_1_FFMod            : in  std_logic_vector(1 downto 0);
    SPI3_1_FFSOP            : in  std_logic;
    SPI3_1_FFEOP            : in  std_logic;
    SPI3_1_FFErr            : in  std_logic;
    SPI3_1_FFValid          : in  std_logic;
    
    -- SPI3 2 Interface Signals
    SPI3_2_src_rdy          : in std_logic;
    SPI3_2_dst_rdy          : out  std_logic;
    SPI3_2_FFAddr           : in  std_logic_vector(7 downto 0);
    SPI3_2_FFData           : in  std_logic_vector(31 downto 0);
    SPI3_2_FFMod            : in  std_logic_vector(1 downto 0);
    SPI3_2_FFSOP            : in  std_logic;
    SPI3_2_FFEOP            : in  std_logic;
    SPI3_2_FFErr            : in  std_logic;
    SPI3_2_FFValid          : in  std_logic;

    -- SPI3 3 Interface Signals
--    SPI3_3_FFRdEn_n         : out std_logic;
--    SPI3_3_FFEmpty_n        : in  std_logic;
--    SPI3_3_FFAlmostEmpty_n  : in  std_logic;
	 SPI3_3_src_rdy          : in std_logic;
    SPI3_3_dst_rdy          : out  std_logic;
    SPI3_3_FFAddr           : in  std_logic_vector(7 downto 0);
    SPI3_3_FFData           : in  std_logic_vector(31 downto 0);
    SPI3_3_FFMod            : in  std_logic_vector(1 downto 0);
    SPI3_3_FFSOP            : in  std_logic;
    SPI3_3_FFEOP            : in  std_logic;
    SPI3_3_FFErr            : in  std_logic;
    SPI3_3_FFValid          : in  std_logic
    );
end spi3_to_spi4_core;

--------------------------------------------------------------------------------
-- Definition of Ports:
--------------------------------------------------------------------------------
-- System signals
--    Reset_n                 : System Level Reset (active low)
--    SPI3_Clk                : SPI3 Interface Clock 
--
-- SPI4 Interface Signals
--    SP4_WrAddr             : SPI4 Interface Address 
--    SP4_WrData             : SPI4 Interface Data 
--    SP4_WrMod              : SPI4 Interface MOD 
--    SP4_WrSOP              : SPI4 Interface Start of Packet 
--    SP4_WrEOP              : SPI4 Interface End of Packet  
--    SP4_WrErr              : SPI4 Interface Error  
--    SP4_WrEn_n             : SPI4 Interface Write Enable (active low) 
--
-- SPI3 0 Interface Signals
--    SPI3_0_src_rdy          : SPI3 Interface 0 source ready
--		SPI3_0_dst_rdy   			: SPI3 Interface 0 Destination ready
--    SPI3_0_Addr             : SPI3 Interface 0 Address
--    SPI3_0_FFData           : SPI3 Interface 0 Data
--    SPI3_0_FFMod            : SPI3 Interface 0 MOD
--    SPI3_0_FFSOP            : SPI3 Interface 0 Start of Packet 
--    SPI3_0_FFEOP            : SPI3 Interface 0 End of Packet 
--    SPI3_0_FFErr            : SPI3 Interface 0 Error 
--    SPI3_0_FFValid          : SPI3 Interface 0 Data is Valid 
--    SPI3_0_FFEmpty_n        : SPI3 Interface 0 is Empty (active low) 
--    SPI3_0_FFAlmostEmpty_n  : SPI3 Interface 0 is Almost Empty (active low)
--
-- SPI3 1 Interface Signals
--    SPI3_1_src_rdy          : SPI3 Interface 1 source ready
--		SPI3_1_dst_rdy   			: SPI3 Interface 1 Destination ready
--    SPI3_1_Addr             : SPI3 Interface 1 Address
--    SPI3_1_FFData           : SPI3 Interface 1 Data
--    SPI3_1_FFMod            : SPI3 Interface 1 MOD
--    SPI3_1_FFSOP            : SPI3 Interface 1 Start of Packet 
--    SPI3_1_FFEOP            : SPI3 Interface 1 End of Packet 
--    SPI3_1_FFErr            : SPI3 Interface 1 Error 
--    SPI3_1_FFValid          : SPI3 Interface 1 Data is Valid 
--    SPI3_1_FFEmpty_n        : SPI3 Interface 1 is Empty (active low) 
--    SPI3_1_FFAlmostEmpty_n  : SPI3 Interface 1 is Almost Empty (active low)
--
-- SPI3 2 Interface Signals
--    SPI3_2_src_rdy          : SPI3 Interface 2 source ready
--		SPI3_2_dst_rdy   			: SPI3 Interface 2 Destination ready
--    SPI3_2_Addr             : SPI3 Interface 2 Address
--    SPI3_2_FFData           : SPI3 Interface 2 Data
--    SPI3_2_FFMod            : SPI3 Interface 2 MOD
--    SPI3_2_FFSOP            : SPI3 Interface 2 Start of Packet 
--    SPI3_2_FFEOP            : SPI3 Interface 2 End of Packet 
--    SPI3_2_FFErr            : SPI3 Interface 2 Error 
--    SPI3_2_FFValid          : SPI3 Interface 2 Data is Valid 
--    SPI3_2_FFEmpty_n        : SPI3 Interface 2 is Empty (active low) 
--    SPI3_2_FFAlmostEmpty_n  : SPI3 Interface 2 is Almost Empty (active low)
--
-- SPI3 3 Interface Signals
--    SPI3_3_src_rdy          : SPI3 Interface 3 source ready
--		SPI3_3_dst_rdy   			: SPI3 Interface 3 Destination ready
--    SPI3_3_Addr             : SPI3 Interface 3 Address
--    SPI3_3_FFData           : SPI3 Interface 3 Data
--    SPI3_3_FFMod            : SPI3 Interface 3 MOD
--    SPI3_3_FFSOP            : SPI3 Interface 3 Start of Packet 
--    SPI3_3_FFEOP            : SPI3 Interface 3 End of Packet 
--    SPI3_3_FFErr            : SPI3 Interface 3 Error 
--    SPI3_3_FFValid          : SPI3 Interface 3 Data is Valid 
--    SPI3_3_FFEmpty_n        : SPI3 Interface 3 is Empty (active low) 
--    SPI3_3_FFAlmostEmpty_n  : SPI3 Interface 3 is Almost Empty (active low)
--
-- SPI3 Status Input Signals
--    SPI3_Stat               : SPI4 Interface Calendar Status
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
-- spi3_to_spi4_read Architecture
--------------------------------------------------------------------------------
architecture xilinx of spi3_to_spi4_core is
  
  ------------------------------------------------------------------------------
  -- SPI3 to SPI4 Read Logic Component Declarations
  ------------------------------------------------------------------------------
--  component spi3_to_spi4_read
--    port (
--      Reset_n               : in  std_logic;
--      BridgeClk             : in  std_logic;
--  
--      SPI3_FFAddr           : in  std_logic_vector(7 downto 0);
--      SPI3_FFData           : in  std_logic_vector(31 downto 0);
--      SPI3_FFMod            : in  std_logic_vector(1 downto 0);
--      SPI3_FFSOP            : in  std_logic;
--      SPI3_FFEOP            : in  std_logic;
--      SPI3_FFErr            : in  std_logic;
--      SPI3_FFValid          : in  std_logic;
--      SPI3_FFEmpty_n        : in  std_logic;
--      SPI3_FFAlmostEmpty_n  : in  std_logic;
--  
--      SPI3_FFRdEn_n         : out std_logic;
--      
--      SPI3_Full_n           : in  std_logic;
--  
--      SPI3_Addr             : out std_logic_vector(7 downto 0);
--      SPI3_Data             : out std_logic_vector(31 downto 0);
--      SPI3_Mod              : out std_logic_vector(1 downto 0);
--      SPI3_SOP              : out std_logic;
--      SPI3_EOP              : out std_logic;
--      SPI3_Err              : out std_logic;
--      SPI3_Valid            : out std_logic
--      );
--  end component;
	COMPONENT spi3_to_spi4_read
	PORT(
		Reset_n : IN std_logic;
		BridgeClk : IN std_logic;
		SPI3_FFAddr : IN std_logic_vector(7 downto 0);
		SPI3_FFData : IN std_logic_vector(31 downto 0);
		SPI3_FFMod : IN std_logic_vector(1 downto 0);
		SPI3_FFSOP : IN std_logic;
		SPI3_FFEOP : IN std_logic;
		SPI3_FFErr : IN std_logic;
		SPI3_Src_Rdy : IN std_logic;
		SPI3_Full_n : IN std_logic;          
		SPI3_Dst_Rdy : OUT std_logic;
		SPI3_Addr : OUT std_logic_vector(7 downto 0);
		SPI3_Data : OUT std_logic_vector(31 downto 0);
		SPI3_Mod : OUT std_logic_vector(1 downto 0);
		SPI3_SOP : OUT std_logic;
		SPI3_EOP : OUT std_logic;
		SPI3_Err : OUT std_logic;
		SPI3_Valid : OUT std_logic
		);
	END COMPONENT;

  ------------------------------------------------------------------------------
  -- SPI3 to SPI4 Burst Storage Component Declarations
  ------------------------------------------------------------------------------
  Component spi3_to_spi4_burst_storage
    port (
      -- System signals
      Reset_n               : in  std_logic;
      BridgeClk             : in  std_logic;
  
      -- SPI3 Burst Storage Interface Output Signals
      SPI3_Addr             : in  std_logic_vector(7 downto 0);
      SPI3_Data             : in  std_logic_vector(31 downto 0);
      SPI3_Mod              : in  std_logic_vector(1 downto 0);
      SPI3_SOP              : in  std_logic;
      SPI3_EOP              : in  std_logic;
      SPI3_Err              : in  std_logic;
      SPI3_Valid            : in  std_logic;
		SPI3_Src_Rdy			 : in  std_logic;
      -- Burst Storage Interface Output Signals
      BrstFull_n            : out std_logic;

      -- Burst Storage Inteface Output Signals
      BrstAddr              : out std_logic_vector(7 downto 0);
      BrstData              : out std_logic_vector(63 downto 0);
      BrstMod               : out std_logic_vector(2 downto 0);
      BrstSOP               : out std_logic;
      BrstEOP               : out std_logic;
      BrstErr               : out std_logic;
    
      -- Burst Storage Read Enable
      BrstRdEn              : in  std_logic;

      BrstAlmostEmpty       : out std_logic;
      BrstEmpty             : out std_logic;
      BrstValid             : out std_logic;
      BrstRdEOP             : out std_logic;
      
      --Packet Count Signals
      PktCntEq0             : out std_logic;
      PktCntEq1             : out std_logic
    );
  end Component;
  
  ------------------------------------------------------------------------------
  -- SPI3 to SPI4 Arbiter Component Declarations
  ------------------------------------------------------------------------------
  Component spi3_to_spi4_arbiter
    port (
      -- System signals
      Reset_n                : in  std_logic;
      BridgeClk              : in  std_logic;
      SPI4_Status            : in  std_logic_vector(7 downto 0);
      SPI4_AlmostFull_n      : in  std_logic;
    
      -- Burst Storage Outputs
      BrstRdEn               : out  std_logic_vector(3 downto 0);
      BrstRdChan             : out  std_logic_vector(1 downto 0);
      SPI4WrEn               : out std_logic;
      
      -- Burst Storage Inputs    
      BrstRdEOP              : in   std_logic_vector(3 downto 0);
      BrstValid              : in   std_logic_vector(3 downto 0);
      BrstEmpty              : in   std_logic_vector(3 downto 0);
      BrstAlmostEmpty        : in   std_logic_vector(3 downto 0);
      BrstPktCntEq0          : in   std_logic_vector(3 downto 0);
      BrstPktCntEq1          : in   std_logic_vector(3 downto 0)
     );
    end Component;

  ------------------------------------------------------------------------------
  -- SPI3 to SPI4 Write Component Declarations
  ------------------------------------------------------------------------------

  Component spi3_to_spi4_write  
    port (
      -- System signals
      Reset_n               : in  std_logic;
      BridgeClk             : in  std_logic;

      -- SPI3 0 Write Logic Inteface Input Signals
      SPI3_0_BrstData       : in  std_logic_vector(63 downto 0);
      SPI3_0_BrstMod        : in  std_logic_vector(2 downto 0);
      SPI3_0_BrstSOP        : in  std_logic;
      SPI3_0_BrstEOP        : in  std_logic;
      SPI3_0_BrstErr        : in  std_logic;

      -- SPI3 1 Write Logic Inteface Input Signals
      SPI3_1_BrstData       : in  std_logic_vector(63 downto 0);
      SPI3_1_BrstMod        : in  std_logic_vector(2 downto 0);
      SPI3_1_BrstSOP        : in  std_logic;
      SPI3_1_BrstEOP        : in  std_logic;
      SPI3_1_BrstErr        : in  std_logic;

      -- SPI3 2 Write Logic Inteface Input Signals
      SPI3_2_BrstData       : in  std_logic_vector(63 downto 0);
      SPI3_2_BrstMod        : in  std_logic_vector(2 downto 0);
      SPI3_2_BrstSOP        : in  std_logic;
      SPI3_2_BrstEOP        : in  std_logic;
      SPI3_2_BrstErr        : in  std_logic;

      -- SPI3 3 Write Logic Inteface Input Signals
      SPI3_3_BrstData       : in  std_logic_vector(63 downto 0);
      SPI3_3_BrstMod        : in  std_logic_vector(2 downto 0);
      SPI3_3_BrstSOP        : in  std_logic;
      SPI3_3_BrstEOP        : in  std_logic;
      SPI3_3_BrstErr        : in  std_logic;

      -- SPI3 Arbiter Interface Input Signals
      ArbiterRdChan          : in  std_logic_vector(1 downto 0);
      ArbiterWrEn           : in  std_logic;

      -- SPI4.2 Interface Output Signals
      SPI4_WrAddr           : out std_logic_vector(7 downto 0);
      SPI4_WrData           : out std_logic_vector(63 downto 0);
      SPI4_WrMod            : out std_logic_vector(2 downto 0);
      SPI4_WrSOP            : out std_logic;
      SPI4_WrEOP            : out std_logic;
      SPI4_WrErr            : out std_logic;
      SPI4_WrEn_n           : out std_logic
    );
  end Component;      


  ------------------------------------------------------------------------------
  -- Signal Declarations
  ------------------------------------------------------------------------------

  -- SPI3 Read FSM 0 Burst Storage 0 Signals
  signal SPI3_0_RdEn_n            : std_logic;
  signal SPI3_0_Addr              : std_logic_vector(7 downto 0);
  signal SPI3_0_Data              : std_logic_vector(31 downto 0);
  signal SPI3_0_Mod               : std_logic_vector(1 downto 0);
  signal SPI3_0_SOP               : std_logic;
  signal SPI3_0_EOP               : std_logic;
  signal SPI3_0_Err               : std_logic;
  signal SPI3_0_Valid             : std_logic;
  signal Brst_0_Full_n            : std_logic;

  -- SPI3 Read FSM 1 to Burst Storage 1 Signals
  signal SPI3_1_RdEn_n            : std_logic;
  signal SPI3_1_Addr              : std_logic_vector(7 downto 0);
  signal SPI3_1_Data              : std_logic_vector(31 downto 0);
  signal SPI3_1_Mod               : std_logic_vector(1 downto 0);
  signal SPI3_1_SOP               : std_logic;
  signal SPI3_1_EOP               : std_logic;
  signal SPI3_1_Err               : std_logic;
  signal SPI3_1_Valid             : std_logic;
  signal Brst_1_Full_n            : std_logic;

  -- SPI3 Read FSM 2 to Burst Storage 2 Signals
  signal SPI3_2_RdEn_n            : std_logic;
  signal SPI3_2_Addr              : std_logic_vector(7 downto 0);
  signal SPI3_2_Data              : std_logic_vector(31 downto 0);
  signal SPI3_2_Mod               : std_logic_vector(1 downto 0);
  signal SPI3_2_SOP               : std_logic;
  signal SPI3_2_EOP               : std_logic;
  signal SPI3_2_Err               : std_logic;
  signal SPI3_2_Valid             : std_logic;
  signal Brst_2_Full_n            : std_logic;

  -- SPI3 Read FSM 3 to Burst Storage 3 Signals
  signal SPI3_3_RdEn_n            : std_logic;
  signal SPI3_3_Addr              : std_logic_vector(7 downto 0);
  signal SPI3_3_Data              : std_logic_vector(31 downto 0);
  signal SPI3_3_Mod               : std_logic_vector(1 downto 0);
  signal SPI3_3_SOP               : std_logic;
  signal SPI3_3_EOP               : std_logic;
  signal SPI3_3_Err               : std_logic;
  signal SPI3_3_Valid             : std_logic;
  signal Brst_3_Full_n            : std_logic;

  -- SPI3 Arbiter to Burst Storage Signals
  signal Brst_RdEn                : std_logic_vector(3 downto 0); 
  signal Brst_RdEOP               : std_logic_vector(3 downto 0);
  signal Brst_Valid               : std_logic_vector(3 downto 0);
  signal Brst_Empty               : std_logic_vector(3 downto 0);
  signal Brst_AlmostEmpty         : std_logic_vector(3 downto 0);
  signal Brst_PktCntEq0           : std_logic_vector(3 downto 0);
  signal Brst_PktCntEq1           : std_logic_vector(3 downto 0);
  signal Arbiter_RdChan           : std_logic_vector(1 downto 0);
  signal Arbiter_WrEn             : std_logic;
  
  -- Burst FIFOs - Arbiter signals
  signal Brst_0_RdEn                : std_logic; 
  signal Brst_0_RdEOP               : std_logic;
  signal Brst_0_Valid               : std_logic;
  signal Brst_0_Empty               : std_logic;
  signal Brst_0_AlmostEmpty         : std_logic;
  signal Brst_0_PktCntEq0           : std_logic;
  signal Brst_0_PktCntEq1           : std_logic;
 
 -- Burst FIFOs - Arbiter signals
  signal Brst_1_RdEn                : std_logic; 
  signal Brst_1_RdEOP               : std_logic;
  signal Brst_1_Valid               : std_logic;
  signal Brst_1_Empty               : std_logic;
  signal Brst_1_AlmostEmpty         : std_logic;
  signal Brst_1_PktCntEq0           : std_logic;
  signal Brst_1_PktCntEq1           : std_logic;

 -- Burst FIFOs - Arbiter signals
  signal Brst_2_RdEn                : std_logic; 
  signal Brst_2_RdEOP               : std_logic;
  signal Brst_2_Valid               : std_logic;
  signal Brst_2_Empty               : std_logic;
  signal Brst_2_AlmostEmpty         : std_logic;
  signal Brst_2_PktCntEq0           : std_logic;
  signal Brst_2_PktCntEq1           : std_logic;
  
 -- Burst FIFOs - Arbiter signals
  signal Brst_3_RdEn                : std_logic; 
  signal Brst_3_RdEOP               : std_logic;
  signal Brst_3_Valid               : std_logic;
  signal Brst_3_Empty               : std_logic;
  signal Brst_3_AlmostEmpty         : std_logic;
  signal Brst_3_PktCntEq0           : std_logic;
  signal Brst_3_PktCntEq1           : std_logic;

  -- SPI3 0 Burst Storage Output Signals
  signal Brst_0_Addr                : std_logic_vector(7 downto 0);  
  signal Brst_0_Data                : std_logic_vector(63 downto 0);
  signal Brst_0_Mod                 : std_logic_vector(2 downto 0);
  signal Brst_0_SOP                 : std_logic;
  signal Brst_0_EOP                 : std_logic;
  signal Brst_0_Err                 : std_logic;
  
  -- SPI3 1 Burst to Write Logic Signals
  signal Brst_1_Addr                : std_logic_vector(7 downto 0);  
  signal Brst_1_Data                : std_logic_vector(63 downto 0);
  signal Brst_1_Mod                 : std_logic_vector(2 downto 0);
  signal Brst_1_SOP                 : std_logic;
  signal Brst_1_EOP                 : std_logic;
  signal Brst_1_Err                 : std_logic;
 
  -- SPI3 2 Burst to Write Logic Signals
  signal Brst_2_Addr                : std_logic_vector(7 downto 0);  
  signal Brst_2_Data                : std_logic_vector(63 downto 0);
  signal Brst_2_Mod                 : std_logic_vector(2 downto 0);
  signal Brst_2_SOP                 : std_logic;
  signal Brst_2_EOP                 : std_logic;
  signal Brst_2_Err                 : std_logic;
 
  -- SPI3 3 Burst to Write Logic Signals
  signal Brst_3_Addr                : std_logic_vector(7 downto 0);  
  signal Brst_3_Data                : std_logic_vector(63 downto 0);
  signal Brst_3_Mod                 : std_logic_vector(2 downto 0);
  signal Brst_3_SOP                 : std_logic;
  signal Brst_3_EOP                 : std_logic;
  signal Brst_3_Err                 : std_logic;

begin
  Brst_0_RdEn          <=  Brst_RdEn(0);           
  Brst_RdEOP(0)        <=  Brst_0_RdEOP;
  Brst_Valid(0)        <=  Brst_0_Valid;
  Brst_Empty(0)        <=  Brst_0_Empty;
  Brst_AlmostEmpty(0)  <=  Brst_0_AlmostEmpty;
  Brst_PktCntEq0(0)    <=  Brst_0_PktCntEq0;
  Brst_PktCntEq1(0)    <=  Brst_0_PktCntEq1;

  Brst_1_RdEn          <=  Brst_RdEn(1);           
  Brst_RdEOP(1)        <=  Brst_1_RdEOP;
  Brst_Valid(1)        <=  Brst_1_Valid;
  Brst_Empty(1)        <=  Brst_1_Empty;
  Brst_AlmostEmpty(1)  <=  Brst_1_AlmostEmpty;
  Brst_PktCntEq0(1)    <=  Brst_1_PktCntEq0;
  Brst_PktCntEq1(1)    <=  Brst_1_PktCntEq1;  


  Brst_2_RdEn          <=  Brst_RdEn(2);           
  Brst_RdEOP(2)        <=  Brst_2_RdEOP;
  Brst_Valid(2)        <=  Brst_2_Valid;
  Brst_Empty(2)        <=  Brst_2_Empty;
  Brst_AlmostEmpty(2)  <=  Brst_2_AlmostEmpty;
  Brst_PktCntEq0(2)    <=  Brst_2_PktCntEq0;
  Brst_PktCntEq1(2)    <=  Brst_2_PktCntEq1;

  Brst_3_RdEn          <=  Brst_RdEn(3);           
  Brst_RdEOP(3)        <=  Brst_3_RdEOP;
  Brst_Valid(3)        <=  Brst_3_Valid;
  Brst_Empty(3)        <=  Brst_3_Empty;
  Brst_AlmostEmpty(3)  <=  Brst_3_AlmostEmpty;
  Brst_PktCntEq0(3)    <=  Brst_3_PktCntEq0;
  Brst_PktCntEq1(3)    <=  Brst_3_PktCntEq1;
  
  ------------------------------------------------------------------------------
  -- SPI3 to SPI4 Arbiter Component Instantiation
  ------------------------------------------------------------------------------
  spi3_to_spi4_arbiter0 : spi3_to_spi4_arbiter
    port map (
      -- System signals
      Reset_n                   => Reset_n, 
      BridgeClk                 => SPI3_Clk, 
  
      -- SPI4 Source Output signals
      SPI4_Status               => SPI4_Stat,
      SPI4_AlmostFull_n         => SPI4_AlmostFull_n,
      
      -- Burst Storage Outputs
      BrstRdEn                  => Brst_RdEn, 
      BrstRdChan                => Arbiter_RdChan,
      SPI4WrEn                  => Arbiter_WrEn,
   
      -- Burst Storage Inputs
      BrstRdEOP                 => Brst_RdEOP, 
      BrstValid                 => Brst_Valid,
      BrstEmpty                 => Brst_Empty, 
      BrstAlmostEmpty           => Brst_AlmostEmpty, 
      BrstPktCntEq0             => Brst_PktCntEq0, 
      BrstPktCntEq1             => Brst_PktCntEq1
      );

  ------------------------------------------------------------------------------
  -- SPI3 to SPI4 Write Logic Component Instantiation
  ------------------------------------------------------------------------------
  spi3_to_spi4_write0 : spi3_to_spi4_write
    port map (
      -- System signals
      Reset_n               => Reset_n,
      BridgeClk             => SPI3_Clk,
  
      -- SPI3 0 Write Logic Inteface Input Signals
      SPI3_0_BrstData       => Brst_0_Data,
      SPI3_0_BrstMod        => Brst_0_Mod,
      SPI3_0_BrstSOP        => Brst_0_SOP,
      SPI3_0_BrstEOP        => Brst_0_EOP,
      SPI3_0_BrstErr        => Brst_0_Err,
  
      -- SPI3 1 Write Logic Inteface Input Signals
      SPI3_1_BrstData       => Brst_1_Data,
      SPI3_1_BrstMod        => Brst_1_Mod,
      SPI3_1_BrstSOP        => Brst_1_SOP,
      SPI3_1_BrstEOP        => Brst_1_EOP,
      SPI3_1_BrstErr        => Brst_1_Err,
  
      -- SPI3 2 Write Logic Inteface Input Signals
      SPI3_2_BrstData       => Brst_2_Data,
      SPI3_2_BrstMod        => Brst_2_Mod,
      SPI3_2_BrstSOP        => Brst_2_SOP,
      SPI3_2_BrstEOP        => Brst_2_EOP,
      SPI3_2_BrstErr        => Brst_2_Err,
  
      -- SPI3 3 Write Logic Inteface Input Signals
      SPI3_3_BrstData       => Brst_3_Data,
      SPI3_3_BrstMod        => Brst_3_Mod,
      SPI3_3_BrstSOP        => Brst_3_SOP,
      SPI3_3_BrstEOP        => Brst_3_EOP,
      SPI3_3_BrstErr        => Brst_3_Err,
  
      -- SPI3 Arbiter Interface Input Signals
      ArbiterRdChan         => Arbiter_RdChan,
      ArbiterWrEn           => Arbiter_WrEn,
  
      -- SPI4.2 Interface Output Signals
      SPI4_WrAddr           => SPI4_WrAddr,
      SPI4_WrData           => SPI4_WrData,
      SPI4_WrMod            => SPI4_WrMod,
      SPI4_WrSOP            => SPI4_WrSOP,
      SPI4_WrEOP            => SPI4_WrEOP,
      SPI4_WrErr            => SPI4_WrErr,
      SPI4_WrEn_n           => SPI4_WrEn_n
      );

  ------------------------------------------------------------------------------
  -- SPI3 to SPI4 Burst Storage #0 Component Instantiation
  ------------------------------------------------------------------------------
  spi3_to_spi4_burst_storage0 : spi3_to_spi4_burst_storage
    port map(
      -- System signals
      Reset_n               => Reset_n,
      BridgeClk             => SPI3_Clk,
    
      -- SPI3 Burst Storage Interface Output Signals
      SPI3_Addr             => SPI3_0_Addr,
      SPI3_Data             => SPI3_0_Data,
      SPI3_Mod              => SPI3_0_Mod,
      SPI3_SOP              => SPI3_0_SOP,
      SPI3_EOP              => SPI3_0_EOP,
      SPI3_Err              => SPI3_0_Err,
      SPI3_Valid            => SPI3_0_Valid,
		SPI3_Src_Rdy			 => SPI3_0_Src_Rdy,
    
      -- SPI3 Burst Storage Interface Output Signals
      BrstFull_n            => Brst_0_Full_n,
  
      -- SPI3 Write Logic Inteface Output Signals
      BrstAddr              => Brst_0_Addr,
      BrstData              => Brst_0_Data,
      BrstMod               => Brst_0_Mod,
      BrstSOP               => Brst_0_SOP,
      BrstEOP               => Brst_0_EOP,
      BrstErr               => Brst_0_Err,
  
      -- SPI3 Arbiter Inteface Input Signals
      BrstRdEn              => Brst_0_RdEn,

      -- SPI3 Arbiter Interface Output Signals
      BrstAlmostEmpty       => Brst_0_AlmostEmpty,
      BrstEmpty             => Brst_0_Empty,
      BrstValid             => Brst_0_Valid,
      BrstRdEOP             => Brst_0_RdEOP,
      PktCntEq0             => Brst_0_PktCntEq0,
      PktCntEq1             => Brst_0_PktCntEq1      
      );

  ------------------------------------------------------------------------------
  -- SPI3 to SPI4 Burst Storage #1 Component Instantiation
  ------------------------------------------------------------------------------
  spi3_to_spi4_burst_storage1 : spi3_to_spi4_burst_storage
    port map(
      -- System signals
      Reset_n               => Reset_n,
      BridgeClk             => SPI3_Clk,
    
      -- SPI3 Burst Storage Interface Output Signals
      SPI3_Addr             => SPI3_1_Addr,
      SPI3_Data             => SPI3_1_Data,
      SPI3_Mod              => SPI3_1_Mod,
      SPI3_SOP              => SPI3_1_SOP,
      SPI3_EOP              => SPI3_1_EOP,
      SPI3_Err              => SPI3_1_Err,
      SPI3_Valid            => SPI3_1_Valid,
		SPI3_Src_Rdy			 => SPI3_1_Src_Rdy,
    
      -- SPI3 Burst Storage Interface Output Signals
      BrstFull_n            => Brst_1_Full_n,
  
      -- SPI3 Write Logic Inteface Output Signals
      BrstAddr              => Brst_1_Addr,
      BrstData              => Brst_1_Data,
      BrstMod               => Brst_1_Mod,
      BrstSOP               => Brst_1_SOP,
      BrstEOP               => Brst_1_EOP,
      BrstErr               => Brst_1_Err,
  
      -- SPI3 Arbiter Inteface Input Signals
      BrstRdEn              => Brst_1_RdEn,

      -- SPI3 Arbiter Interface Output Signals
      BrstAlmostEmpty       => Brst_1_AlmostEmpty  ,
      BrstEmpty             => Brst_1_Empty  ,
      BrstValid             => Brst_1_Valid,
      BrstRdEOP             => Brst_1_RdEOP,
      PktCntEq0             => Brst_1_PktCntEq0,
      PktCntEq1             => Brst_1_PktCntEq1      
      );

  ------------------------------------------------------------------------------
  -- SPI3 to SPI4 Burst Storage #2 Component Instantiation
  ------------------------------------------------------------------------------
  spi3_to_spi4_burst_storage2 : spi3_to_spi4_burst_storage
    port map(
      -- System signals
      Reset_n               => Reset_n,
      BridgeClk             => SPI3_Clk,
    
      -- SPI3 Burst Storage Interface Output Signals
      SPI3_Addr             => SPI3_2_Addr,
      SPI3_Data             => SPI3_2_Data,
      SPI3_Mod              => SPI3_2_Mod,
      SPI3_SOP              => SPI3_2_SOP,
      SPI3_EOP              => SPI3_2_EOP,
      SPI3_Err              => SPI3_2_Err,
      SPI3_Valid            => SPI3_2_Valid,
		SPI3_Src_Rdy			 => SPI3_2_Src_Rdy,
    
      -- SPI3 Burst Storage Interface Output Signals
      BrstFull_n            => Brst_2_Full_n,
  
      -- SPI3 Write Logic Inteface Output Signals
      BrstAddr              => Brst_2_Addr,
      BrstData              => Brst_2_Data,
      BrstMod               => Brst_2_Mod,
      BrstSOP               => Brst_2_SOP,
      BrstEOP               => Brst_2_EOP,
      BrstErr               => Brst_2_Err,
  
      -- SPI3 Arbiter Inteface Input Signals
      BrstRdEn              => Brst_2_RdEn,

      -- SPI3 Arbiter Interface Output Signals
      BrstAlmostEmpty       => Brst_2_AlmostEmpty  ,
      BrstEmpty             => Brst_2_Empty  ,
      BrstValid             => Brst_2_Valid,
      BrstRdEOP             => Brst_2_RdEOP,
      PktCntEq0             => Brst_2_PktCntEq0,
      PktCntEq1             => Brst_2_PktCntEq1      
      );

  ------------------------------------------------------------------------------
  -- SPI3 to SPI4 Burst Storage #3 Component Instantiation
  ------------------------------------------------------------------------------
  spi3_to_spi4_burst_storage3 : spi3_to_spi4_burst_storage
    port map(
      -- System signals
      Reset_n               => Reset_n,
      BridgeClk             => SPI3_Clk,
    
      -- SPI3 Burst Storage Interface Output Signals
      SPI3_Addr             => SPI3_3_Addr,
      SPI3_Data             => SPI3_3_Data,
      SPI3_Mod              => SPI3_3_Mod,
      SPI3_SOP              => SPI3_3_SOP,
      SPI3_EOP              => SPI3_3_EOP,
      SPI3_Err              => SPI3_3_Err,
      SPI3_Valid            => SPI3_3_Valid,
		SPI3_Src_Rdy			 => SPI3_3_Src_Rdy,
    
      -- SPI3 Burst Storage Interface Output Signals
      BrstFull_n            => Brst_3_Full_n,
  
      -- SPI3 Write Logic Inteface Output Signals
      BrstAddr              => Brst_3_Addr,
      BrstData              => Brst_3_Data,
      BrstMod               => Brst_3_Mod,
      BrstSOP               => Brst_3_SOP,
      BrstEOP               => Brst_3_EOP,
      BrstErr               => Brst_3_Err,
  
      -- SPI3 Arbiter Inteface Input Signals
      BrstRdEn             => Brst_3_RdEn,

      -- SPI3 Arbiter Interface Output Signals
      BrstAlmostEmpty       => Brst_3_AlmostEmpty  ,
      BrstEmpty             => Brst_3_Empty  ,
      BrstValid             => Brst_3_Valid,
      BrstRdEOP             => Brst_3_RdEOP,
      PktCntEq0             => Brst_3_PktCntEq0,
      PktCntEq1             => Brst_3_PktCntEq1      
      );

  ------------------------------------------------------------------------------
  -- SPI3 to SPI4 #0 Read Component Instantiation
  ------------------------------------------------------------------------------
--  top_spi3_to_spi4_read0 : spi3_to_spi4_read 
--    port map (                          
--      -- System signals
--      Reset_n               => Reset_n,
--      BridgeClk             => SPI3_Clk,
--
--      -- SPI3 Core Interface Input Signals
--      SPI3_FFAddr           => SPI3_0_FFAddr,
--      SPI3_FFData           => SPI3_0_FFData,
--      SPI3_FFMod            => SPI3_0_FFMod,
--      SPI3_FFSOP            => SPI3_0_FFSOP,
--      SPI3_FFEOP            => SPI3_0_FFEOP,
--      SPI3_FFErr            => SPI3_0_FFErr,   
--      SPI3_FFValid          => SPI3_0_FFValid,
--      SPI3_FFEmpty_n        => SPI3_0_FFEmpty_n,
--      SPI3_FFAlmostEmpty_n  => SPI3_0_FFAlmostEmpty_n,
--
--      -- SPI3 Core Interface Output Signals
--      SPI3_FFRdEn_n         => SPI3_0_FFRdEn_n,
--
--      -- SPI3 Burst Storage Interface Input Signals
--      SPI3_Full_n           => Brst_0_Full_n,
--      -- SPI3 Burst Storage Interface Output Signals
--      SPI3_Addr             => SPI3_0_Addr,
--      SPI3_Data             => SPI3_0_Data,
--      SPI3_Mod              => SPI3_0_Mod,
--      SPI3_SOP              => SPI3_0_SOP,
--      SPI3_EOP              => SPI3_0_EOP,
--      SPI3_Err              => SPI3_0_Err,
--      SPI3_Valid            => SPI3_0_Valid 
--      );
	top_spi3_to_spi4_read0 : spi3_to_spi4_read PORT MAP(
		Reset_n => Reset_n,
		BridgeClk => SPI3_Clk,
		
		-- SPI3 Core Interface Input Signals
		SPI3_FFAddr => SPI3_0_FFAddr,
		SPI3_FFData => SPI3_0_FFData,
		SPI3_FFMod => SPI3_0_FFMod,
		SPI3_FFSOP => SPI3_0_FFSOP,
		SPI3_FFEOP => SPI3_0_FFEOP,
		SPI3_FFErr => SPI3_0_FFErr,
		SPI3_Src_Rdy => SPI3_0_Src_Rdy,
		
		-- SPI3 Core Interface Output Signals		
		SPI3_Dst_Rdy => SPI3_0_Dst_Rdy,
		
		-- SPI3 Burst Storage Interface Input Signals
		SPI3_Full_n => Brst_0_Full_n,
		
		-- SPI3 Burst Storage Interface Output Signals
		SPI3_Addr => SPI3_0_Addr,
		SPI3_Data => SPI3_0_Data,
		SPI3_Mod => SPI3_0_Mod,
		SPI3_SOP => SPI3_0_SOP,
		SPI3_EOP => SPI3_0_EOP,
		SPI3_Err => SPI3_0_Err,
		SPI3_Valid => SPI3_0_Valid 
	);
  ------------------------------------------------------------------------------
  -- SPI3 to SPI4 #1 Read Component Instantiation
  ------------------------------------------------------------------------------
	top_spi3_to_spi4_read1 : spi3_to_spi4_read PORT MAP(
		Reset_n => Reset_n,
		BridgeClk => SPI3_Clk,
		
		-- SPI3 Core Interface Input Signals
		SPI3_FFAddr => SPI3_1_FFAddr,
		SPI3_FFData => SPI3_1_FFData,
		SPI3_FFMod => SPI3_1_FFMod,
		SPI3_FFSOP => SPI3_1_FFSOP,
		SPI3_FFEOP => SPI3_1_FFEOP,
		SPI3_FFErr => SPI3_1_FFErr,
		SPI3_Src_Rdy => SPI3_1_Src_Rdy,
		
		-- SPI3 Core Interface Output Signals		
		SPI3_Dst_Rdy => SPI3_1_Dst_Rdy,
		
		-- SPI3 Burst Storage Interface Input Signals
		SPI3_Full_n => Brst_1_Full_n,
		
		-- SPI3 Burst Storage Interface Output Signals
		SPI3_Addr => SPI3_1_Addr,
		SPI3_Data => SPI3_1_Data,
		SPI3_Mod => SPI3_1_Mod,
		SPI3_SOP => SPI3_1_SOP,
		SPI3_EOP => SPI3_1_EOP,
		SPI3_Err => SPI3_1_Err,
		SPI3_Valid => SPI3_1_Valid 
	);
  
  ------------------------------------------------------------------------------
  -- SPI3 to SPI4 #2 Read Component Instantiation
  ------------------------------------------------------------------------------
	top_spi3_to_spi4_read2 : spi3_to_spi4_read PORT MAP(
		Reset_n => Reset_n,
		BridgeClk => SPI3_Clk,
		
		-- SPI3 Core Interface Input Signals
		SPI3_FFAddr => SPI3_2_FFAddr,
		SPI3_FFData => SPI3_2_FFData,
		SPI3_FFMod => SPI3_2_FFMod,
		SPI3_FFSOP => SPI3_2_FFSOP,
		SPI3_FFEOP => SPI3_2_FFEOP,
		SPI3_FFErr => SPI3_2_FFErr,
		SPI3_Src_Rdy => SPI3_2_Src_Rdy,
		
		-- SPI3 Core Interface Output Signals		
		SPI3_Dst_Rdy => SPI3_2_Dst_Rdy,
		
		-- SPI3 Burst Storage Interface Input Signals
		SPI3_Full_n => Brst_2_Full_n,
		
		-- SPI3 Burst Storage Interface Output Signals
		SPI3_Addr => SPI3_2_Addr,
		SPI3_Data => SPI3_2_Data,
		SPI3_Mod => SPI3_2_Mod,
		SPI3_SOP => SPI3_2_SOP,
		SPI3_EOP => SPI3_2_EOP,
		SPI3_Err => SPI3_2_Err,
		SPI3_Valid => SPI3_2_Valid 
	);
  
  ------------------------------------------------------------------------------
  -- SPI3 to SPI4 #3 Read Component Instantiation
  ------------------------------------------------------------------------------
	top_spi3_to_spi4_read3 : spi3_to_spi4_read PORT MAP(
		Reset_n => Reset_n,
		BridgeClk => SPI3_Clk,
		
		-- SPI3 Core Interface Input Signals
		SPI3_FFAddr => SPI3_3_FFAddr,
		SPI3_FFData => SPI3_3_FFData,
		SPI3_FFMod => SPI3_3_FFMod,
		SPI3_FFSOP => SPI3_3_FFSOP,
		SPI3_FFEOP => SPI3_3_FFEOP,
		SPI3_FFErr => SPI3_3_FFErr,
		SPI3_Src_Rdy => SPI3_3_Src_Rdy,
		
		-- SPI3 Core Interface Output Signals		
		SPI3_Dst_Rdy => SPI3_3_Dst_Rdy,
		
		-- SPI3 Burst Storage Interface Input Signals
		SPI3_Full_n => Brst_3_Full_n,
		
		-- SPI3 Burst Storage Interface Output Signals
		SPI3_Addr => SPI3_3_Addr,
		SPI3_Data => SPI3_3_Data,
		SPI3_Mod => SPI3_3_Mod,
		SPI3_SOP => SPI3_3_SOP,
		SPI3_EOP => SPI3_3_EOP,
		SPI3_Err => SPI3_3_Err,
		SPI3_Valid => SPI3_3_Valid 
	);

end xilinx;
