-------------------------------------------------------------------------------
--
--   File Name:  spi3_to_spi4_top.vhd
--     Version:  1.0
--        Date:  June 11, 2007
--
-- Description:  The purpose of the spi3_to_spi4_top.vhd file is to serve as 
--               the wrapper file for SPI3 to SPI4 Bridge Design, as well as 
--               instantiating the four SPI3 Cores, and the SPI4.2 Core.
--
--      Author:  Zhe Xia
--     Company:  Xilinx Inc.
--
--
--     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
--     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
--     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
--     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
--     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
--     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
--     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
--     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
--     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
--     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
--     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
--     FOR A PARTICULAR PURPOSE.
--
--     (c) Copyright 2003 Xilinx, Inc.
--     All rights reserved.
--
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Library Declarations
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.conv_std_logic_vector;

-- synthesis translate_off
library unisim;
use unisim.vcomponents.all;
-- synthesis translate_on

--------------------------------------------------------------------------------
-- Entity Declaration
--------------------------------------------------------------------------------

entity spi3_to_spi4_top is
  port (
    -- System signals
    Reset_n                 : in  std_logic;
	 SPI4_SrcFifoReset_n     : in std_logic;

    --SPI3 Rx Clock - common to all 4 SPI3 interfaces   
    SPI3_RFClk             : in  std_logic;
    
    -- SPI3 0 Interface
    SPI3_0_RVal            : in  std_logic;
    SPI3_0_REnb          : out std_logic;
    SPI3_0_RSx             : in  std_logic;
    SPI3_0_RSOP            : in  std_logic;
    SPI3_0_REOP            : in  std_logic;
    SPI3_0_RDat            : in  std_logic_vector(31 downto 0);
    SPI3_0_RPrty           : in  std_logic;
    SPI3_0_RMod            : in  std_logic_vector(1 downto 0);
    SPI3_0_RErr            : in  std_logic;

    -- SPI3 1 Interface
    SPI3_1_RVal            : in  std_logic;
    SPI3_1_REnb          : out std_logic;
    SPI3_1_RSx             : in  std_logic;
    SPI3_1_RSOP            : in  std_logic;
    SPI3_1_REOP            : in  std_logic;
    SPI3_1_RDat            : in  std_logic_vector(31 downto 0);
    SPI3_1_RPrty           : in  std_logic;
    SPI3_1_RMod            : in  std_logic_vector(1 downto 0);
    SPI3_1_RErr            : in  std_logic;

    -- SPI3 2 Interface
    SPI3_2_RVal            : in  std_logic;
    SPI3_2_REnb            : out std_logic;
    SPI3_2_RSx             : in  std_logic;
    SPI3_2_RSOP            : in  std_logic;
    SPI3_2_REOP            : in  std_logic;
    SPI3_2_RDat            : in  std_logic_vector(31 downto 0);
    SPI3_2_RPrty           : in  std_logic;
    SPI3_2_RMod            : in  std_logic_vector(1 downto 0);
    SPI3_2_RErr            : in  std_logic;

    -- SPI3 3 Interface
    SPI3_3_RVal            : in  std_logic;
    SPI3_3_REnb          : out std_logic;
    SPI3_3_RSx             : in  std_logic;
    SPI3_3_RSOP            : in  std_logic;
    SPI3_3_REOP            : in  std_logic;
    SPI3_3_RDat            : in  std_logic_vector(31 downto 0);
    SPI3_3_RPrty           : in  std_logic;
    SPI3_3_RMod            : in  std_logic_vector(1 downto 0);
    SPI3_3_RErr            : in  std_logic;

    
    -- SPI4.2 Source Interface Signals
    SPI4_TDClk_P            : out std_logic;
    SPI4_TDClk_N            : out std_logic;
    SPI4_TCtl_P             : out std_logic;
    SPI4_TCtl_N             : out std_logic;
    SPI4_TDat_P             : out std_logic_vector(15 downto 0);
    SPI4_TDat_N             : out std_logic_vector(15 downto 0);
    
    -- SPI4.2 source status (LVTTL) input
    SPI4_TSClk              : in  std_logic;
    SPI4_TStat              : in  std_logic_vector(1 downto 0);

    --SPI3 Parity Error 
    SPI3_ParityErr         : out std_logic;

    -- SPI3 DCM Clock signals
    SPI3_DcmReset_RFClk     : in  std_logic;
    SPI3_Locked_RFClk       : out std_logic;

    --SPI4.2 System clock signals
    SPI4_SysClk_P           : in  std_logic;
    SPI4_SysClk_N           : in  std_logic;

    --SPI4.2 DCM clock signals
    SPI4_DcmReset_TDClk     : in  std_logic;
    SPI4_DcmReset_TSClk     : in  std_logic;
    SPI4_Locked_TDClk       : out std_logic;
    SPI4_Locked_TSClk       : out std_logic;
    SPI4_SysClkDiv_GP       : out std_logic;
    
    --SPI4.2 Calendar Programming Signals
    SPI4_SrcCalendar_M      : in  std_logic_vector(7 downto 0); 
    SPI4_SrcCalendar_Len    : in  std_logic_vector(8 downto 0); 
    SPI4_SrcCalClk          : in  std_logic;                    
    SPI4_SrcCalWrEn_n       : in  std_logic;                    
    SPI4_SrcCalAddr         : in  std_logic_vector(8 downto 0); 
    SPI4_SrcCalData         : in  std_logic_vector(7 downto 0); 
    SPI4_SrcCalDataOut      : out std_logic_vector(7 downto 0);
    
    --SPI4.2 source enable
    SPI4_SrcEn              : in  std_logic
    );
end spi3_to_spi4_top;

--------------------------------------------------------------------------------
-- Definition of Ports:
--------------------------------------------------------------------------------
-- System signals
--    Reset_n                 : System Level Reset (active low)
--    SPI3_Clk                : SPI3 Interface Clock. All four SPI3 Rx
--                              interfaces share this clock
--
-- SPI 3 interfaces, # = 0-3
--    SPI3_#_RFClk            : SPI3 Sink Clock Input 
--    SPI3_#_RVal             : SPI3 Sink Data Valid (active high)
--    SPI3_#_REnb_n           : SPI3 Sink Write Enable (active low)
--    SPI3_#_RSx              : SPI3 Sink Start of Transfer (active high)
--    SPI3_#_RSOP             : SPI3 Sink Start of Packet (active high)
--    SPI3_#_REOP             : SPI3 Sink End of Packet (active high)
--    SPI3_#_RDat             : SPI3 Sink Data
--    SPI3_#_RPrty            : SPI3 Sink Parity
--    SPI3_#_RMod             : SPI3 Sink Word Modulo
--    SPI3_#_RErr             : SPI3 Sink Error
--
-- SPI4.2 interfaces
--    SPI4_SysClk_P/N         : SPI4.2 System Clock input
--    SPI4_TDClk_P/N          : SPI4.2 Source Clock
--    SPI4_TCtl_P/N           : SPI4.2 Source Control
--    SPI4_TDat_P/N           : SPI4.2 Source Data
--    SPI4_TSClk              : SPI4.2 Source Status Clock
--    SPI4_TStat              : SPI4.2 Source Status
--
-- SPI3 Parity Error Signal
--    SPI3_ParityErr          : SPI3 Parity Error
--   
-- SPI3 DCM Clock Signals
--    SPI3_DcmReset_RFClk     : SPI3 RFClk DCM Reset 
--    SPI3_Locked_RFClk       : SPI3 RFClk DCM Lock
--
-- SPI4.2 System Clock Signals
--    SPI4_SysClk_P           : SPI4.2 System Clock LVDS P 
--    SPI4_SysClk_N           : SPI4.2 System Clock LVDS N
--
-- SPI4.2 DCM clock signals
--    SPI4_DcmReset_TDClk     : SPI4.2 TDClk DCM Reset
--    SPI4_DcmReset_TSClk     : SPI4.2 TSClk DCM Reset
--    SPI4_Locked_TDClk       : SPI4.2 TDClk DCM Lock
--    SPI4_Locked_TSClk       : SPI4.2 TSClk DCM Lock
--    SPI4_SysClkDiv_GP       : SPI4.2 SysClk DCM DIV(divide by 2)
--    
-- SPI4.2 Calendar Programming Signals
--    SPI4_SrcCalendar_M      : SPI4.2 Number of repetitions of
--                              SrcCalendar_Len between framing.
-- 
--    SPI4_SrcCalendar_Len    : SPI4.2 Source Calendar Length. 
--    SPI4_SrcCalClk          : SPI4.2 Source Calendar Clock              
--    SPI4_SrcCalWrEn_n       : SPI4.2 Source Calendar Write Enable      
--    SPI4_SrcCalAddr         : SPI4.2 Source Calendar Address 
--    SPI4_SrcCalData         : SPI4.2 Source Calendar Write Data 
--    SPI4_SrcCalDataOut      : SPI4.2 Source Calendar Data Out
--    
-- SPI4.2 source enable
--    SPI4_SrcEn              : SPI4.2 Source Enable Signal
--    
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
-- spi3_to_spi4_read Architecture
--------------------------------------------------------------------------------
architecture xilinx of spi3_to_spi4_top is

  -- SPI3 Sink Component Declaration
  component spi3_link_v4_1_spi3_link_rx is
 port (
      RST             : in  std_logic;
      RFCLK           : in  std_logic;
      RENB            : out std_logic;
      RDAT            : in  std_logic_vector(31 downto 0);
      RMOD            : in  std_logic_vector(1 downto 0);
      RPRTY           : in  std_logic;
      RVAL            : in  std_logic;
      RSOP            : in  std_logic;
      REOP            : in  std_logic;
      RERR            : in  std_logic;
      RSX             : in  std_logic;
      RX_CLK          : in  std_logic;
      RX_SRC_RDY      : out std_logic;
      RX_DST_RDY      : in  std_logic;
      RX_SOB          : out std_logic;
      RX_SOF          : out std_logic;
      RX_EOF          : out std_logic;
      RX_ADDR         : out std_logic_vector(7 downto 0);
      RX_DATA         : out std_logic_vector(31 downto 0);
      RX_REM          : out std_logic_vector(1 downto 0);
      RX_ERR          : out std_logic;
      RX_ERRBUS       : out std_logic_vector(7 downto 0)
    );
  end component;


  -- SPI4 Source Component Declaration
   component pl4_v8_1_pl4_src_top 
  ------------------------------------------------------------------------------
  -- Port Declarations
  ------------------------------------------------------------------------------
  port (
    -- common signals
    Reset_n           : in  std_logic;
    SrcTriStateEn     : in  std_logic;
    SrcFifoReset_n    : in  std_logic;
    DcmReset_TDClk    : in  std_logic;
    DcmReset_TSClk    : in  std_logic;
    Locked_TDClk      : out std_logic;
    Locked_TSClk      : out std_logic;
    DcmLost_TDClk        : out std_logic;
    DcmLost_TSClk        : out std_logic;
    SrcClksRdy        : out std_logic;
    SysClk_P          : in  std_logic;
    SysClk_N          : in  std_logic;

    -- source general purpose clocks
    SysClkDiv_GP      : out std_logic;
    TSClk_GP          : out std_logic;
    SysClk0_GP        : out std_logic;

 

    -- Source Calendar I/O
    SrcCalendar_M     : in  std_logic_vector(7 downto 0);
    SrcCalendar_Len   : in  std_logic_vector(8 downto 0);
    SrcCalClk         : in  std_logic;
    SrcCalWrEn_n      : in  std_logic;
    SrcCalAddr        : in  std_logic_vector(8 downto 0);
    SrcCalData        : in  std_logic_vector(7 downto 0);
    SrcCalDataOut     : out std_logic_vector(7 downto 0);

    SrcStatClk        : in  std_logic;
    SrcStat           : out std_logic_vector(31 downto 0);
    SrcStatAddr       : in  std_logic_vector(3 downto 0);
    SrcStatCh         : out std_logic_vector(7 downto 0);
    SrcStatChValid    : out std_logic; 

    -- FIFO I/O
    SrcFFClk          : in  std_logic;
    SrcFFWrEn_n       : in  std_logic;
    SrcFFAddr         : in  std_logic_vector(7 downto 0);
    SrcFFData         : in  std_logic_vector(63 downto 0);
    SrcFFMod          : in  std_logic_vector(2 downto 0);
    SrcFFSOP          : in  std_logic;
    SrcFFEOP          : in  std_logic;
    SrcFFErr          : in  std_logic;
    SrcFFOverflow_n   : out std_logic;
    SrcFFAlmostFull_n : out std_logic;
 
    -- Control and status signals
    SrcEn             : in  std_logic;
    IdleRequest       : in std_logic;
    TrainingRequest   : in std_logic;
    SrcOofOverride    : in std_logic;
    SrcOof            : out std_logic;
    SrcDIP2Err        : out std_logic;
    SrcStatFrameErr   : out std_logic;
    SrcPatternErr     : out std_logic;
 
    -- FIFO status interface
    TSClk             : in  std_logic;
    TStat             : in  std_logic_vector(1 downto 0);

    -- PL4 source bus
    TDClk_P           : out std_logic;
    TDClk_N           : out std_logic;
    TDat_P            : out std_logic_vector(15 downto 0);
    TDat_N            : out std_logic_vector(15 downto 0);
    TCtl_P            : out std_logic;
    TCtl_N            : out std_logic;

    -- Configuration signals
    SrcAFThresAssert  : in std_logic_vector(8 downto 0);
    SrcAFThresNegate  : in std_logic_vector(8 downto 0);
    DataMaxT          : in std_logic_vector(15 downto 0);
    AlphaData         : in std_logic_vector(7 downto 0);
    SrcBurstMode      : in std_logic;
    SrcBurstLen       : in std_logic_vector(5 downto 0);
    NumDip2Errors     : in std_logic_vector(3 downto 0);
    NumDip2Matches    : in std_logic_vector(3 downto 0)

    );
  end component;



  -- SPI3 to SPI4.2 Bridge Logic
  component spi3_to_spi4_core
  port (
    Reset_n                 : in  std_logic;
    SPI3_Clk                : in  std_logic;

    SPI4_Stat               : in  std_logic_vector(7 downto 0);
    SPI4_AlmostFull_n       : in  std_logic;
    SPI4_WrAddr             : out std_logic_vector(7 downto 0);
    SPI4_WrData             : out std_logic_vector(63 downto 0);
    SPI4_WrMod              : out std_logic_vector(2 downto 0);
    SPI4_WrSOP              : out std_logic;
    SPI4_WrEOP              : out std_logic;
    SPI4_WrErr              : out std_logic;
    SPI4_WrEn_n             : out std_logic;
    
    SPI3_0_Src_Rdy          : in std_logic;
    SPI3_0_Dst_Rdy			 : out std_logic;
    SPI3_0_FFAddr           : in  std_logic_vector(7 downto 0);
    SPI3_0_FFData           : in  std_logic_vector(31 downto 0);
    SPI3_0_FFMod            : in  std_logic_vector(1 downto 0);
    SPI3_0_FFSOP            : in  std_logic;
    SPI3_0_FFEOP            : in  std_logic;
    SPI3_0_FFErr            : in  std_logic;
    SPI3_0_FFValid          : in  std_logic;

    SPI3_1_Src_Rdy          : in std_logic;
    SPI3_1_Dst_Rdy			 : out std_logic;
    SPI3_1_FFAddr           : in  std_logic_vector(7 downto 0);
    SPI3_1_FFData           : in  std_logic_vector(31 downto 0);
    SPI3_1_FFMod            : in  std_logic_vector(1 downto 0);
    SPI3_1_FFSOP            : in  std_logic;
    SPI3_1_FFEOP            : in  std_logic;
    SPI3_1_FFErr            : in  std_logic;
    SPI3_1_FFValid          : in  std_logic;
    
    SPI3_2_Src_Rdy          : in std_logic;
    SPI3_2_Dst_Rdy			 : out std_logic;
    SPI3_2_FFAddr           : in  std_logic_vector(7 downto 0);
    SPI3_2_FFData           : in  std_logic_vector(31 downto 0);
    SPI3_2_FFMod            : in  std_logic_vector(1 downto 0);
    SPI3_2_FFSOP            : in  std_logic;
    SPI3_2_FFEOP            : in  std_logic;
    SPI3_2_FFErr            : in  std_logic;
    SPI3_2_FFValid          : in  std_logic;

    SPI3_3_Src_Rdy          : in std_logic;
    SPI3_3_Dst_Rdy			 : out std_logic;
    SPI3_3_FFAddr           : in  std_logic_vector(7 downto 0);
    SPI3_3_FFData           : in  std_logic_vector(31 downto 0);
    SPI3_3_FFMod            : in  std_logic_vector(1 downto 0);
    SPI3_3_FFSOP            : in  std_logic;
    SPI3_3_FFEOP            : in  std_logic;
    SPI3_3_FFErr            : in  std_logic;
    SPI3_3_FFValid          : in  std_logic
    );
  end component;


  component IBUFG
  port (
    I : in  std_logic;
    O : out std_logic
  );
  end component;

  component BUFG
  port (
    I : in  std_logic;
    O : out std_logic
  );
  end component;

  component IBUF
  port (
    I : in  std_logic;
    O : out std_logic
  );
  end component;

  component DCM
  port (
    CLKIN   : in  std_logic ;
    CLKFB   : in  std_logic ;
    DSSEN   : in  std_logic ;
    PSINCDEC: in  std_logic ;
    PSEN    : in  std_logic ;
    PSCLK   : in  std_logic ;
    RST     : in  std_logic ;
    CLK0    : out std_logic ;
    CLK90   : out std_logic ;
    CLK180  : out std_logic ;
    CLK270  : out std_logic ;
    CLK2X   : out std_logic ;
    CLK2X180: out std_logic ;
    CLKDV   : out std_logic ;
    CLKFX   : out std_logic ;
    CLKFX180: out std_logic ;
    LOCKED  : out std_logic ;
    PSDONE  : out std_logic ;
    STATUS  : out std_logic_vector(7 downto 0));
  end component;


  ------------------------------------------------------------------------------
  -- Signal Declarations
  ------------------------------------------------------------------------------
  
  -- SPI3 0 Core
  signal SPI3_0_FFAddr          : std_logic_vector(7 downto 0); 
  signal SPI3_0_FFData          : std_logic_vector(31 downto 0); 
  signal SPI3_0_FFMod           : std_logic_vector(1 downto 0); 
  signal SPI3_0_Src_Rdy         : std_logic;
  signal SPI3_0_Dst_Rdy			  : std_logic;
  signal SPI3_0_FFSOP           : std_logic; 
  signal SPI3_0_FFEOP           : std_logic; 
  signal SPI3_0_FFErr           : std_logic; 
  signal SPI3_0_FFValid         : std_logic;      
  signal SPI3_0_FFEmpty_n       : std_logic;      
  signal SPI3_0_FFAlmostEmpty_n : std_logic;      
  signal SPI3_0_ParitySelect    : std_logic;
  signal SPI3_0_ParityErr       : std_logic;
  signal SPI3_0_ErrBus  		  : std_logic_vector(7 downto 0);
  signal SPI3_0_AFThresAssert   : std_logic_vector(8 downto 0);
  signal SPI3_0_AFThresNegate   : std_logic_vector(8 downto 0);
 

  -- SPI3 1 Core
  signal SPI3_1_FFAddr          : std_logic_vector(7 downto 0); 
  signal SPI3_1_FFData          : std_logic_vector(31 downto 0); 
  signal SPI3_1_FFMod           : std_logic_vector(1 downto 0); 
  signal SPI3_1_Src_Rdy         : std_logic;
  signal SPI3_1_Dst_Rdy			  : std_logic;
  signal SPI3_1_FFSOP           : std_logic; 
  signal SPI3_1_FFEOP           : std_logic; 
  signal SPI3_1_FFErr           : std_logic; 
  signal SPI3_1_FFValid         : std_logic;      
  signal SPI3_1_FFEmpty_n       : std_logic;      
  signal SPI3_1_FFAlmostEmpty_n : std_logic;      
  signal SPI3_1_ParitySelect    : std_logic;
  signal SPI3_1_ParityErr       : std_logic;
  signal SPI3_1_ErrBus  		  : std_logic_vector(7 downto 0);
  signal SPI3_1_AFThresAssert   : std_logic_vector(8 downto 0);
  signal SPI3_1_AFThresNegate   : std_logic_vector(8 downto 0);

  -- SPI3 2 Core
  signal SPI3_2_FFAddr          : std_logic_vector(7 downto 0); 
  signal SPI3_2_FFData          : std_logic_vector(31 downto 0); 
  signal SPI3_2_FFMod           : std_logic_vector(1 downto 0); 
  signal SPI3_2_Src_Rdy         : std_logic;
  signal SPI3_2_Dst_Rdy			  : std_logic;
  signal SPI3_2_FFSOP           : std_logic; 
  signal SPI3_2_FFEOP           : std_logic; 
  signal SPI3_2_FFErr           : std_logic; 
  signal SPI3_2_FFValid         : std_logic;      
  signal SPI3_2_FFEmpty_n       : std_logic;      
  signal SPI3_2_FFAlmostEmpty_n : std_logic;      
  signal SPI3_2_ParitySelect    : std_logic;
  signal SPI3_2_ParityErr       : std_logic;
  signal SPI3_2_ErrBus  		  : std_logic_vector(7 downto 0);
  signal SPI3_2_AFThresAssert   : std_logic_vector(8 downto 0);
  signal SPI3_2_AFThresNegate   : std_logic_vector(8 downto 0);

  -- SPI3 3 Core
  signal SPI3_3_FFAddr          : std_logic_vector(7 downto 0); 
  signal SPI3_3_FFData          : std_logic_vector(31 downto 0); 
  signal SPI3_3_FFMod           : std_logic_vector(1 downto 0);
  signal SPI3_3_Src_Rdy         : std_logic;
  signal SPI3_3_Dst_Rdy			  : std_logic;
  signal SPI3_3_FFSOP           : std_logic; 
  signal SPI3_3_FFEOP           : std_logic; 
  signal SPI3_3_FFErr           : std_logic; 
  signal SPI3_3_FFValid         : std_logic;      
  signal SPI3_3_FFEmpty_n       : std_logic;      
  signal SPI3_3_FFAlmostEmpty_n : std_logic;      
  signal SPI3_3_ParitySelect    : std_logic;
  signal SPI3_3_ParityErr       : std_logic;
  signal SPI3_3_ErrBus  		  : std_logic_vector(7 downto 0);
  signal SPI3_3_AFThresAssert   : std_logic_vector(8 downto 0);
  signal SPI3_3_AFThresNegate   : std_logic_vector(8 downto 0);

  -- SPI3 Clocking
  signal SPI3_RFClk_ibufg       : std_logic;
  signal SPI3_RFClk_dcm         : std_logic;
  signal SPI3_RFClk_bufg        : std_logic;
  
  -- SPI4.2 Static Configuration Signals
  signal SPI4_NumDip2Errors    : std_logic_vector(3 downto 0);
  signal SPI4_NumDip2Matches   : std_logic_vector(3 downto 0);
  signal SPI4_SrcAFThresAssert : std_logic_vector(8 downto 0);
  signal SPI4_SrcAFThresNegate : std_logic_vector(8 downto 0);
  signal SPI4_DataMaxT         : std_logic_vector(15 downto 0);
  signal SPI4_AlphaData        : std_logic_vector(7 downto 0);
  signal SPI4_SrcBurstLen      : std_logic_vector(5 downto 0);


  -- SPI4.2 Source FIFO Interface
  signal SPI4_WrEn_n           : std_logic;
  signal SPI4_WrAddr           : std_logic_vector(7 downto 0);
  signal SPI4_WrData           : std_logic_vector(63 downto 0);
  signal SPI4_WrMod            : std_logic_vector(2 downto 0);
  signal SPI4_WrSOP            : std_logic;
  signal SPI4_WrEOP            : std_logic;
  signal SPI4_WrErr            : std_logic;
  signal SPI4_AlmostFull_n     : std_logic;
  signal SPI4_Overflow_n       : std_logic;

  -- SPI4.2 Core Control Signals
  signal SPI4_IdleRequest      : std_logic;
  signal SPI4_TrainingRequest  : std_logic;
  signal SPI4_SrcOof           : std_logic;
  signal SPI4_SrcDIP2Err       : std_logic;
  signal SPI4_SrcPatternErr    : std_logic;
  
  -- SPI4.2 Source Status Interface
  signal SPI4_Stat             : std_logic_vector(31 downto 0);
  signal SPI4_StatAddr         : std_logic_vector(3 downto 0);
  signal SPI4_StatCh           : std_logic_vector(7 downto 0);
  signal SPI4_StatChValid      : std_logic;

  -- SPI4.2 Source Status
  signal SPI4_TStat_i          : std_logic_vector(1 downto 0);
  signal SPI4_TSClk_i          : std_logic;
  signal SPI4_TSClk_GP         : std_logic;
  signal SysClkDiv_GPint      : std_logic;
  
  -- SPI3 to SPI4.2 Bridge Clocking
  signal SPI3_Clk_ibufg        : std_logic;
  signal SPI3_Clk_bufg         : std_logic;
  signal BridgeClk             : std_logic;

  --SPI3 Parity Err Signals
  signal SPI3ErrBus_r0      : std_logic_vector(15 downto 0);
  signal SPI3ErrBus_r1      : std_logic_vector(15 downto 0);
  signal SPI3ParityErrout      : std_logic;
  
  signal Reset						 : std_logic;
  signal GND                   : std_logic;

  -- Synthesis Attributes for SPI4.2 Core
  -- Used to prevent I/O buffers from being instantiated on the SPI4.2 Core
 
  --------------------------
  -- Synplify Constraints: comment out if using different synthesis tool 
  --------------------------
  attribute syn_black_box                    : boolean;
  attribute black_box_pad_pin                : string;
  attribute syn_black_box of pl4_v8_1_pl4_src_top     : component is true;
--  attribute black_box_pad_pin of pl4_src_top : component is "SysClk_P, SysClk_N,TDClk_P,TDClk_N,TDat_P(15:0), TDat_N(15:0),TCtl_P,TCtl_N";
  attribute syn_black_box of IBUFG           : component is true;
  attribute syn_black_box of DCM             : component is true;
  attribute syn_black_box of BUFG            : component is true;
  attribute syn_black_box of IBUF            : component is true;

begin
   Reset <= NOT Reset_n;
	
   GND <= '0';
   
   SPI4_SysClkDiv_GP  <= BridgeClk;

  --Bridge does not request Idle sequences or training patterns on SPI4.2 interface
  SPI4_IdleRequest     <= '0';
  SPI4_TrainingRequest <= '0';

   
  ------------------------------------------------------------------------------
  -- SPI4.2 Static Configuration Signal Assignments
  --   Modify as required 
  ------------------------------------------------------------------------------
  SPI4_NumDip2Errors     <= conv_std_logic_vector( 4 ,4);
  SPI4_NumDip2Matches    <= conv_std_logic_vector( 4 ,4);
  SPI4_SrcAFThresAssert  <= conv_std_logic_vector( 32 ,9);
  SPI4_SrcAFThresNegate  <= conv_std_logic_vector( 32 ,9);
  SPI4_DataMaxT          <= conv_std_logic_vector( 4096 ,16);
  SPI4_AlphaData         <= conv_std_logic_vector( 2 ,8);
  SPI4_SrcBurstLen       <= conv_std_logic_vector( 32 ,6);
  SPI4_StatAddr          <= conv_std_logic_vector( 0, 4);

  ------------------------------------------------------------------------------
  -- SPI3 Static Configuration Signal Assignments
  --   Modify as required 
  ------------------------------------------------------------------------------
  -- SPI3 #0
  SPI3_0_ParitySelect    <= '0';
  SPI3_0_AFThresAssert   <= conv_std_logic_vector( 32, 9);
  SPI3_0_AFThresNegate   <= conv_std_logic_vector( 32, 9);

  -- SPI3 #1
  SPI3_1_ParitySelect    <= '0';
  SPI3_1_AFThresAssert   <= conv_std_logic_vector( 32, 9);
  SPI3_1_AFThresNegate   <= conv_std_logic_vector( 32, 9);

  -- SPI3 #2
  SPI3_2_ParitySelect    <= '0';
  SPI3_2_AFThresAssert   <= conv_std_logic_vector( 32, 9);
  SPI3_2_AFThresNegate   <= conv_std_logic_vector( 32, 9);

  -- SPI3 #3
  SPI3_3_ParitySelect    <= '0';
  SPI3_3_AFThresAssert   <= conv_std_logic_vector( 32, 9);
  SPI3_3_AFThresNegate   <= conv_std_logic_vector( 32, 9);


  ------------------------------------------------------------------------------
  -- SPI3 RFClk:  Assumed that all 4 SPI3 interfaces are synchronous to
  --              SPI3_RFClk
  ------------------------------------------------------------------------------
  SPI3_RFClkIBufg: IBUFG
    port map (
      I => SPI3_RFClk,
      O => SPI3_RFClk_ibufg
      );

  rfclk_dcm: DCM
  port map (
        CLKIN      =>  SPI3_RFClk_ibufg,
        CLKFB      =>  SPI3_RFClk_bufg,
        DSSEN      =>  GND,
        PSINCDEC   =>  GND,
        PSEN       =>  GND,
        PSCLK      =>  GND,
        RST        =>  SPI3_DcmReset_RFClk,
        CLK0       =>  SPI3_RFClk_dcm,
        CLK90      =>  OPEN,
        CLK180     =>  OPEN,
        CLK270     =>  OPEN,
        CLK2X      =>  OPEN,
        CLK2X180   =>  OPEN,
        CLKDV      =>  OPEN,
        CLKFX      =>  OPEN,
        CLKFX180   =>  OPEN,
        LOCKED     =>  SPI3_Locked_RFClk,
        PSDONE     =>  OPEN,
        STATUS     =>  OPEN
       ) ;

  SPI3_RFClkBufg: BUFG
    port map (
      I => SPI3_RFClk_dcm,
      O => SPI3_RFClk_bufg
      );

  ------------------------------------------------------------------------------
  -- TStat - Instantiate LVTTL IBUF for the TStat and TSClk pins
  ------------------------------------------------------------------------------
  TStat_ibuf0 : IBUF
    port map (
      I => SPI4_TStat(0),
      O => SPI4_TStat_i(0));

  TStat_ibuf1 : IBUF
    port map (
      I => SPI4_TStat(1),
      O => SPI4_TStat_i(1));

  TSClk_ibufg0 : IBUFG
    port map (
      I => SPI4_TSClk,
      O => SPI4_TSClk_i);

  ------------------------------------------------------------------------------
  -- Bridge Core Clock (SPI3_Clk)
  ------------------------------------------------------------------------------
  --SPI3_ClkIBufg: IBUFG
  --  port map (
  --    I => SPI3_Clk,
  --    O => SPI3_Clk_ibufg
  --    );

  --SPI3_ClkBufg: BUFG
  --  port map (
  --    I => SPI3_Clk_ibufg,
  --    O => SPI3_Clk_bufg
  --    );


  -----------------------------------------------------------------------------
  -- SPI3 Parity Error: This process capture parity errors on any of the
  -- four SPI3 Rx cores and flags  SPI_ParityErr output signal
  -----------------------------------------------------------------------------
    process(Reset_n, BridgeClk)
      begin
        if(Reset_n = '0') then
          SPI3ErrBus_r0 <= (others => '0');
          SPI3ErrBus_r1 <= (others => '0');
          SPI3ParityErrout <= '0';
        elsif(BridgeClk'event and BridgeClk = '1') then
          SPI3ErrBus_r0 <= SPI3_0_ErrBus(3 downto 0) & SPI3_1_ErrBus(3 downto 0) &
                              SPI3_2_ErrBus(3 downto 0) & SPI3_3_ErrBus(3 downto 0);
          SPI3ErrBus_r1 <= SPI3ErrBus_r0;
          SPI3ParityErrout <= SPI3ErrBus_r1(13) or SPI3ErrBus_r1(9) or
                              SPI3ErrBus_r1(5) or SPI3ErrBus_r1(1);
        end if;
      end process;


      SPI3_ParityErr <= SPI3ParityErrout;
      
  ------------------------------------------------------------------------------
  -- SPI3 0 Sink Instantiation
  ------------------------------------------------------------------------------
  
  pl3_rx_top0 : spi3_link_v4_1_spi3_link_rx
    port map (
      RST             => Reset,
      RFCLK           => SPI3_RFClk_bufg,
      RENB            => SPI3_0_REnb,
      RDAT            => SPI3_0_RDat,
      RMOD            => SPI3_0_RMod,
      RPRTY           => SPI3_0_RPrty,
      RVAL            => SPI3_0_RVal,
      RSOP            => SPI3_0_RSOP,
      REOP            => SPI3_0_REOP,
      RERR            => SPI3_0_RErr,
      RSX             => SPI3_0_RSx,
      RX_CLK          => BridgeClk,
      RX_SRC_RDY      => SPI3_0_SRC_RDY,
      RX_DST_RDY      => SPI3_0_DST_RDY,
      RX_SOB          => open,
      RX_SOF          => SPI3_0_FFSOP,
      RX_EOF          => SPI3_0_FFEOP,
      RX_ADDR         => SPI3_0_FFAddr,
      RX_DATA         => SPI3_0_FFData,
      RX_REM          => SPI3_0_FFMod,
      RX_ERR          => SPI3_0_FFErr,
      RX_ERRBUS       => SPI3_0_ErrBus
      );  	
  ------------------------------------------------------------------------------
  -- SPI3 1 Sink Instantiation
  ------------------------------------------------------------------------------
  pl3_rx_top1 : spi3_link_v4_1_spi3_link_rx
    port map (
      RST             => Reset,
      RFCLK           => SPI3_RFClk_bufg,
      RENB            => SPI3_1_REnb,
      RDAT            => SPI3_1_RDat,
      RMOD            => SPI3_1_RMod,
      RPRTY           => SPI3_1_RPrty,
      RVAL            => SPI3_1_RVal,
      RSOP            => SPI3_1_RSOP,
      REOP            => SPI3_1_REOP,
      RERR            => SPI3_1_RErr,
      RSX             => SPI3_1_RSx,
      RX_CLK          => BridgeClk,
      RX_SRC_RDY      => SPI3_1_SRC_RDY,
      RX_DST_RDY      => SPI3_1_DST_RDY,
      RX_SOB          => open,
      RX_SOF          => SPI3_1_FFSOP,
      RX_EOF          => SPI3_1_FFEOP,
      RX_ADDR         => SPI3_1_FFAddr,
      RX_DATA         => SPI3_1_FFData,
      RX_REM          => SPI3_1_FFMod,
      RX_ERR          => SPI3_1_FFErr,
      RX_ERRBUS       => SPI3_1_ErrBus
      );   

  ------------------------------------------------------------------------------
  -- SPI3 1 Sink Instantiation
  ------------------------------------------------------------------------------
  pl3_rx_top2 : spi3_link_v4_1_spi3_link_rx
    port map (
      RST             => Reset,
      RFCLK           => SPI3_RFClk_bufg,
      RENB            => SPI3_2_REnb,
      RDAT            => SPI3_2_RDat,
      RMOD            => SPI3_2_RMod,
      RPRTY           => SPI3_2_RPrty,
      RVAL            => SPI3_2_RVal,
      RSOP            => SPI3_2_RSOP,
      REOP            => SPI3_2_REOP,
      RERR            => SPI3_2_RErr,
      RSX             => SPI3_2_RSx,
      RX_CLK          => BridgeClk,
      RX_SRC_RDY      => SPI3_2_SRC_RDY,
      RX_DST_RDY      => SPI3_2_DST_RDY,
      RX_SOB          => open,
      RX_SOF          => SPI3_2_FFSOP,
      RX_EOF          => SPI3_2_FFEOP,
      RX_ADDR         => SPI3_2_FFAddr,
      RX_DATA         => SPI3_2_FFData,
      RX_REM          => SPI3_2_FFMod,
      RX_ERR          => SPI3_2_FFErr,
      RX_ERRBUS       => SPI3_2_ErrBus
      );    

  ------------------------------------------------------------------------------
  -- SPI3 3 Sink Instantiation
  ------------------------------------------------------------------------------
  pl3_rx_top3 : spi3_link_v4_1_spi3_link_rx
    port map (
      RST             => Reset,
      RFCLK           => SPI3_RFClk_bufg,
      RENB            => SPI3_3_REnb,
      RDAT            => SPI3_3_RDat,
      RMOD            => SPI3_3_RMod,
      RPRTY           => SPI3_3_RPrty,
      RVAL            => SPI3_3_RVal,
      RSOP            => SPI3_3_RSOP,
      REOP            => SPI3_3_REOP,
      RERR            => SPI3_3_RErr,
      RSX             => SPI3_3_RSx,
      RX_CLK          => BridgeClk,
      RX_SRC_RDY      => SPI3_3_SRC_RDY,
      RX_DST_RDY      => SPI3_3_DST_RDY,
      RX_SOB          => open,
      RX_SOF          => SPI3_3_FFSOP,
      RX_EOF          => SPI3_3_FFEOP,
      RX_ADDR         => SPI3_3_FFAddr,
      RX_DATA         => SPI3_3_FFData,
      RX_REM          => SPI3_3_FFMod,
      RX_ERR          => SPI3_3_FFErr,
      RX_ERRBUS       => SPI3_3_ErrBus
      ); 
      
  ------------------------------------------------------------------------------
  -- SPI4.2 Source Instantiation
  ------------------------------------------------------------------------------
   
  pl4_src_top0 : pl4_v8_1_pl4_src_top
  port map (

    -- common signals
    Reset_n           => Reset_n,
    SrcFifoReset_n    => SPI4_SrcFifoReset_n,
    SrcTriStateEn     => '0',
    DcmReset_TDClk    => SPI4_DcmReset_TDClk,
    DcmReset_TSClk    => SPI4_DcmReset_TSClk,
    Locked_TDClk      => SPI4_Locked_TDClk,
    Locked_TSClk      => SPI4_Locked_TSClk,
    DcmLost_TDClk     => open,
    DcmLost_TSClk     => open,
    SrcClksRdy        => open,
    SysClk_P          => SPI4_SysClk_P,
    SysClk_N          => SPI4_SysClk_N,

    -- source general purpose clocks
    SysClkDiv_GP      => BridgeClk,
    TSClk_GP          => SPI4_TSClk_GP,
    SysClk0_GP        => open,

    -- Source Calendar I/O
	 
    SrcCalendar_M     => SPI4_SrcCalendar_M,
    SrcCalendar_Len   => SPI4_SrcCalendar_Len,
    SrcCalClk         => SPI4_SrcCalClk,
    SrcCalWrEn_n      => SPI4_SrcCalWrEn_n,
    SrcCalAddr        => SPI4_SrcCalAddr,
    SrcCalData        => SPI4_SrcCalData,
    SrcCalDataOut     => SPI4_SrcCalDataOut,  
  
    SrcStatClk        => SPI4_TSClk_GP,
    SrcStatAddr       => SPI4_StatAddr,
    SrcStat           => SPI4_Stat,
    SrcStatCh         => SPI4_StatCh,
    SrcStatChValid    => SPI4_StatChValid,
 
    -- FIFO I/O
    SrcFFClk          => BridgeClk,
    SrcFFWrEn_n       => SPI4_WrEn_n,
    SrcFFAddr         => SPI4_WrAddr,
    SrcFFData         => SPI4_WrData,
    SrcFFMod          => SPI4_WrMod,
    SrcFFSOP          => SPI4_WrSOP,
    SrcFFEOP          => SPI4_WrEOP,
    SrcFFErr          => SPI4_WrErr,
    SrcFFOverflow_n   => SPI4_Overflow_n,
    SrcFFAlmostFull_n => SPI4_AlmostFull_n,
 
    -- Control and status signals
    SrcEn             => SPI4_SrcEn,
    IdleRequest       => SPI4_IdleRequest,
    TrainingRequest   => SPI4_TrainingRequest,
    SrcOofOverride    => '0',
    SrcOof            => SPI4_SrcOof,
    SrcDIP2Err        => SPI4_SrcDIP2Err,
    SrcStatFrameErr   => open,
    SrcPatternErr     => SPI4_SrcPatternErr,

    -- FIFO status interface
    TSClk             => SPI4_TSClk_i,
    TStat             => SPI4_TStat_i,

    -- PL4 source bus
    TDClk_P           => SPI4_TDClk_P,
    TDClk_N           => SPI4_TDClk_N,
    TDat_P            => SPI4_TDat_P,
    TDat_N            => SPI4_TDat_N,
    TCtl_P            => SPI4_TCtl_P,
    TCtl_N            => SPI4_TCtl_N,

    -- Configuration signals
    SrcAFThresAssert  => SPI4_SrcAFThresAssert,
    SrcAFThresNegate  => SPI4_SrcAFThresNegate,
    DataMaxT          => SPI4_DataMaxT,
    AlphaData         => SPI4_AlphaData,
    SrcBurstLen       => SPI4_SrcBurstLen,
    SrcBurstMode      => '0',
    NumDip2Errors     => SPI4_NumDip2Errors,
    NumDip2Matches    => SPI4_NumDip2Matches);   
	 

  ------------------------------------------------------------------------------
  -- -- SPI3 to SPI4.2 Bridge Core Instantiation
  ------------------------------------------------------------------------------
   
  spi3_to_spi4_core0 : spi3_to_spi4_core
    port map (
      -- System signals
      Reset_n                => Reset_n,
      SPI3_Clk               => BridgeClk,
  
      -- SPI4 Interface Signals
      SPI4_Stat              => SPI4_Stat(7 downto 0),
      SPI4_AlmostFull_n      => SPI4_AlmostFull_n,
      SPI4_WrAddr            => SPI4_WrAddr,
      SPI4_WrData            => SPI4_WrData ,
      SPI4_WrMod             => SPI4_WrMod,
      SPI4_WrSOP             => SPI4_WrSOP,
      SPI4_WrEOP             => SPI4_WrEOP,
      SPI4_WrErr             => SPI4_WrErr,
      SPI4_WrEn_n            => SPI4_WrEn_n,
  
      -- SPI3 0 Interface Signals
      SPI3_0_Src_Rdy         => SPI3_0_Src_Rdy ,
		SPI3_0_Dst_Rdy			  => SPI3_0_Dst_Rdy,
      SPI3_0_FFAddr          => SPI3_0_FFAddr,
      SPI3_0_FFData          => SPI3_0_FFData,
      SPI3_0_FFMod           => SPI3_0_FFMod,
      SPI3_0_FFSOP           => SPI3_0_FFSOP,
      SPI3_0_FFEOP           => SPI3_0_FFEOP,
      SPI3_0_FFErr           => SPI3_0_FFErr,
      SPI3_0_FFValid         => SPI3_0_FFValid,
  
      -- SPI3 1 Interface Signals
      SPI3_1_Src_Rdy         => SPI3_1_Src_Rdy,
		SPI3_1_Dst_Rdy			  => SPI3_1_Dst_Rdy,
      SPI3_1_FFAddr          => SPI3_1_FFAddr,
      SPI3_1_FFData          => SPI3_1_FFData,
      SPI3_1_FFMod           => SPI3_1_FFMod,
      SPI3_1_FFSOP           => SPI3_1_FFSOP,
      SPI3_1_FFEOP           => SPI3_1_FFEOP,
      SPI3_1_FFErr           => SPI3_1_FFErr,
      SPI3_1_FFValid         => SPI3_1_FFValid,
  
      -- SPI3 2 Interface Signals
      SPI3_2_Src_Rdy         => SPI3_2_Src_Rdy,
		SPI3_2_Dst_Rdy			  => SPI3_2_Dst_Rdy,
      SPI3_2_FFAddr          => SPI3_2_FFAddr,
      SPI3_2_FFData          => SPI3_2_FFData,
      SPI3_2_FFMod           => SPI3_2_FFMod,
      SPI3_2_FFSOP           => SPI3_2_FFSOP,
      SPI3_2_FFEOP           => SPI3_2_FFEOP,
      SPI3_2_FFErr           => SPI3_2_FFErr,
      SPI3_2_FFValid         => SPI3_2_FFValid,
  
      -- SPI3 3 Interface Signals
      SPI3_3_Src_Rdy         => SPI3_3_Src_Rdy,
		SPI3_3_Dst_Rdy			  => SPI3_3_Dst_Rdy,
      SPI3_3_FFAddr          => SPI3_3_FFAddr,
      SPI3_3_FFData          => SPI3_3_FFData,
      SPI3_3_FFMod           => SPI3_3_FFMod,
      SPI3_3_FFSOP           => SPI3_3_FFSOP,
      SPI3_3_FFEOP           => SPI3_3_FFEOP,
      SPI3_3_FFErr           => SPI3_3_FFErr,
      SPI3_3_FFValid         => SPI3_3_FFValid 
      );


end xilinx;
