-------------------------------------------------------------------------------
--
--   File Name:  spi4_to_spi3_burst_storage.vhd
--     Version:  1.0
--        Date:  June 11, 2007
--
-- Description:  The purpose of the spi4_to_spi3_burst_storage.vhd file is to
--               provide an intermediate FIFO between the SPI-4.2 core and the
--               four SPI-3 cores.
--
--      Author:  Zhe Xia
--     Company:  Xilinx Inc.
--
--
--     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
--     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
--     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
--     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
--     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
--     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
--     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
--     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
--     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
--     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
--     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
--     FOR A PARTICULAR PURPOSE.
--
--     (c) Copyright 2003 Xilinx, Inc.
--     All rights reserved.
--
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Library Declarations
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

--------------------------------------------------------------------------------
-- Entity Declaration
--------------------------------------------------------------------------------

entity spi4_to_spi3_burst_storage is
  port (
    --Generic signals
    Reset_n              : in  std_logic;
    SPI4_Clk             : in  std_logic;
    
    --Signals coming from the SPI-4.2 core
    SPI4_Addr            : in  std_logic_vector(7 downto 0);
    SPI4_Data            : in  std_logic_vector(63 downto 0);
    SPI4_Mod             : in  std_logic_vector(2 downto 0);
    SPI4_SOP             : in  std_logic;
    SPI4_EOP             : in  std_logic;
    SPI4_Err             : in  std_logic;
    SPI4_Valid           : in  std_logic;
    SPI4_Dip4Err         : in  std_logic;
	 SPI4_FFBurstErr		 : in  std_logic;
	 SPI4_FFPayloadDIP4	 : in  std_logic;
    SPI4_PayloadErr      : in  std_logic;
    
    --Burst FIFO signals
    SPI3_0_BrstRdEn      : in  std_logic;
    SPI3_0_BrstData      : out std_logic_vector(69 downto 0);
    SPI3_1_BrstRdEn      : in  std_logic;
    SPI3_1_BrstData      : out std_logic_vector(69 downto 0);
    SPI3_2_BrstRdEn      : in  std_logic;
    SPI3_2_BrstData      : out std_logic_vector(69 downto 0);
    SPI3_3_BrstRdEn      : in  std_logic;
    SPI3_3_BrstData      : out std_logic_vector(69 downto 0);
    SPI3_0_Brst_Empty    : out std_logic;
    SPI3_1_Brst_Empty    : out std_logic;
    SPI3_2_Brst_Empty    : out std_logic;
    SPI3_3_Brst_Empty    : out std_logic;
    SPI3_0_Brst_HalfFull : out std_logic;
    SPI3_1_Brst_HalfFull : out std_logic;
    SPI3_2_Brst_HalfFull : out std_logic;
    SPI3_3_Brst_HalfFull : out std_logic
    );
end spi4_to_spi3_burst_storage;

--------------------------------------------------------------------------------
-- Definition of Ports:
--------------------------------------------------------------------------------
--    --Generic signals
--    Reset_n              : System Level Reset (active low)
--    SPI4_Clk             : SPI4 Interface Clock 
--    
--    --Signals coming from the SPI-4.2 core
--    SPI4_Addr            : Address read from the SPI-4.2 core by read module
--    SPI4_Data            : Data read from the SPI-4.2 core by read module
--    SPI4_Mod             : Mod read from the SPI-4.2 core by read module
--    SPI4_SOP             : SOP read from the SPI-4.2 core by read module
--    SPI4_EOP             : EOP read from the SPI-4.2 core by read module
--    SPI4_Err             : Err read from the SPI-4.2 core by read module
--    SPI4_Valid           : Valid signal read from the SPI-4.2 core by read
--                           module
--    SPI4_Dip4Err         : Dip4Err read from the SPI-4.2 core by read module
--    SPI4_PayloadErr      : PayloadErr signal read from the SPI-4.2 core by
--                           read module
--    
--    --Burst FIFO signals
--    SPI3_0_BrstRdEn      : Read enable for burst FIFO 0 from write module
--    SPI3_0_BrstData      : Data from burst FIFO 0 for write module
--    SPI3_1_BrstRdEn      : Read enable for burst FIFO 1 from write module
--    SPI3_1_BrstData      : Data from burst FIFO 1 for write module
--    SPI3_2_BrstRdEn      : Read enable for burst FIFO 2 from write module
--    SPI3_2_BrstData      : Data from burst FIFO 2 for write module
--    SPI3_3_BrstRdEn      : Read enable for burst FIFO 3 from write module
--    SPI3_3_BrstData      : Data from burst FIFO 3 for write module
--    SPI3_0_Brst_Empty    : Indicates that burst FIFO 0 is empty
--    SPI3_1_Brst_Empty    : Indicates that burst FIFO 1 is empty
--    SPI3_2_Brst_Empty    : Indicates that burst FIFO 2 is empty
--    SPI3_3_Brst_Empty    : Indicates that burst FIFO 3 is empty
--    SPI3_0_Brst_HalfFull : Indicates that burst FIFO 0 is half full
--    SPI3_1_Brst_HalfFull : Indicates that burst FIFO 1 is half full
--    SPI3_2_Brst_HalfFull : Indicates that burst FIFO 2 is half full
--    SPI3_3_Brst_HalfFull : Indicates that burst FIFO 3 is half full

--------------------------------------------------------------------------------
-- spi4_to_spi3_read Architecture
--------------------------------------------------------------------------------
architecture xilinx of spi4_to_spi3_burst_storage is

  ------------------------------------------------------------------------------
  -- SPI4 to SPI3 Burst FIFO Logic Component Declaration
  ------------------------------------------------------------------------------

  component generic_sfifo_512x72
  port (
    clk        : in  std_logic;
    sinit      : in  std_logic;
    din        : in  std_logic_VECTOR(71 downto 0);
    wr_en      : in  std_logic;
    rd_en      : in  std_logic;
    dout       : out std_logic_VECTOR(71 downto 0);
    full       : out std_logic;
    empty      : out std_logic;
    data_count : out std_logic_VECTOR(8 downto 0)
    );
  end component;
  
  attribute syn_black_box : boolean;
  attribute syn_black_box of generic_sfifo_512x72: component is true;
      
  signal SPI4_Addr_d1              : std_logic_vector(7 downto 0);
  signal SPI4_Data_d1              : std_logic_vector(63 downto 0);
  signal SPI4_Mod_d1               : std_logic_vector(2 downto 0);
  signal SPI4_SOP_d1               : std_logic;
  signal SPI4_EOP_d1               : std_logic;
  signal SPI4_Err_d1               : std_logic;
  signal SPI4_Valid_d1             : std_logic;
  signal SPI4_Addr_d2              : std_logic_vector(7 downto 0);
  signal SPI4_Data_d2              : std_logic_vector(63 downto 0);
  signal SPI4_Mod_d2               : std_logic_vector(2 downto 0);
  signal SPI4_SOP_d2               : std_logic;
  signal SPI4_EOP_d2               : std_logic;
  signal SPI4_Valid_d2             : std_logic;
  signal BurstFIFO_0_WrEn          : std_logic;
  signal BurstFIFO_1_WrEn          : std_logic;
  signal BurstFIFO_2_WrEn          : std_logic;
  signal BurstFIFO_3_WrEn          : std_logic;
  signal SPI4_Err_d2               : std_logic;
  signal FIFO_wr_data              : std_logic_vector(71 downto 0);
  signal FIFO_0_data_count         : std_logic_vector(8 downto 0);
  signal FIFO_1_data_count         : std_logic_vector(8 downto 0);
  signal FIFO_2_data_count         : std_logic_vector(8 downto 0);
  signal FIFO_3_data_count         : std_logic_vector(8 downto 0);
  signal FIFO_reset                : std_logic;
  signal SPI3_0_BrstData_temp      : std_logic_vector(71 downto 0);
  signal SPI3_1_BrstData_temp      : std_logic_vector(71 downto 0);
  signal SPI3_2_BrstData_temp      : std_logic_vector(71 downto 0);
  signal SPI3_3_BrstData_temp      : std_logic_vector(71 downto 0);


begin
  
  --This pipeline stage is done in order to more easily meet timing.  More
  --pipeline stages can be added if timing problems are encountered after
  --changing the design.
  pipeline_stage: process (Reset_n, SPI4_Clk)
  begin
    if (Reset_n='0') then
      --pipeline stage 1
      SPI4_Addr_d1       <= (others => '0');
      SPI4_Data_d1       <= (others => '0');
      SPI4_Mod_d1        <= (others => '0');
      SPI4_SOP_d1        <= '0';
      SPI4_EOP_d1        <= '0';
      SPI4_Err_d1        <= '0';
      SPI4_Valid_d1      <= '0';
      
      --pipeline stage 2
      SPI4_Addr_d2       <= (others => '0');
      SPI4_Data_d2       <= (others => '0');
      SPI4_Mod_d2        <= (others => '0');
      SPI4_SOP_d2        <= '0';
      SPI4_EOP_d2        <= '0';
      SPI4_Valid_d2      <= '0';
    elsif (SPI4_Clk'event and SPI4_Clk='1') then
      --pipeline stage 1
      SPI4_Addr_d1       <= SPI4_Addr;
      SPI4_Data_d1       <= SPI4_Data;
      SPI4_Mod_d1        <= SPI4_Mod;
      SPI4_SOP_d1        <= SPI4_SOP;
      SPI4_EOP_d1        <= SPI4_EOP;
      SPI4_Err_d1        <= SPI4_Err;    
      SPI4_Valid_d1      <= SPI4_Valid;
      
      --pipeline stage 2
      SPI4_Addr_d2       <= SPI4_Addr_d1;
      SPI4_Data_d2       <= SPI4_Data_d1;
      SPI4_Mod_d2        <= SPI4_Mod_d1;
      SPI4_SOP_d2        <= SPI4_SOP_d1;
      SPI4_EOP_d2        <= SPI4_EOP_d1;
      SPI4_Valid_d2      <= SPI4_Valid_d1;
    end if;
  end process;
  
  --Decode the address in order to produce write enables for the FIFOs
  write_enable_proc: process (Reset_n, SPI4_Clk)
  begin
    if (Reset_n='0') then
      BurstFIFO_0_WrEn <= '0';
      BurstFIFO_1_WrEn <= '0';
      BurstFIFO_2_WrEn <= '0';
      BurstFIFO_3_WrEn <= '0';
    elsif (SPI4_Clk'event and SPI4_Clk='1') then
      BurstFIFO_0_WrEn <= '0';
      BurstFIFO_1_WrEn <= '0';
      BurstFIFO_2_WrEn <= '0';
      BurstFIFO_3_WrEn <= '0';
      if (SPI4_Addr_d1="00000000") then
        BurstFIFO_0_WrEn <= SPI4_Valid_d1;
      elsif (SPI4_Addr_d1="00000001") then
        BurstFIFO_1_WrEn <= SPI4_Valid_d1;
      elsif (SPI4_Addr_d1="00000010") then
        BurstFIFO_2_WrEn <= SPI4_Valid_d1;
      elsif (SPI4_Addr_d1="00000011") then
        BurstFIFO_3_WrEn <= SPI4_Valid_d1;
      end if;
    end if;
  end process;
  
  --A pipeline of the error signal.  If more robust error reporting is required
  --(Dip4, Payload errors), the extra logic can be added to this process.
  error_signal_proc: process (Reset_n, SPI4_Clk)
  begin
    if (Reset_n='0') then
      SPI4_Err_d2 <= '0';
    elsif (SPI4_Clk'event and SPI4_Clk='1') then
      SPI4_Err_d2 <= SPI4_Err_d1;
    end if;
  end process;
  
  FIFO_wr_data <= ("00" & SPI4_Data_d2 & SPI4_SOP_d2 & SPI4_EOP_d2
                    & SPI4_Mod_d2 & SPI4_Err_d2);
  FIFO_reset <= not(Reset_n);

   -----------------------------------------------------------------------------
  -- SPI4 to SPI3 Burst FIFO Logic Component Instantiation 0
  ------------------------------------------------------------------------------
  spi4_to_spi3_burst_fifo0: generic_sfifo_512x72
  port map (
    clk        => SPI4_Clk,
    sinit      => FIFO_reset,
    din        => FIFO_wr_data,
    wr_en      => BurstFIFO_0_WrEn,
    rd_en      => SPI3_0_BrstRdEn,
    dout       => SPI3_0_BrstData_temp,
    full       => OPEN,
    empty      => SPI3_0_Brst_Empty,
    data_count => FIFO_0_data_count
    );
    
  ------------------------------------------------------------------------------
  -- SPI4 to SPI3 Burst FIFO Logic Component Instantiation 1
  ------------------------------------------------------------------------------
  spi4_to_spi3_burst_fifo1: generic_sfifo_512x72
  port map (
    clk        => SPI4_Clk,
    sinit      => FIFO_reset,
    din        => FIFO_wr_data,
    wr_en      => BurstFIFO_1_WrEn,
    rd_en      => SPI3_1_BrstRdEn,
    dout       => SPI3_1_BrstData_temp,
    full       => OPEN,
    empty      => SPI3_1_Brst_Empty,
    data_count => FIFO_1_data_count
    );
    
  ------------------------------------------------------------------------------
  -- SPI4 to SPI3 Burst FIFO Logic Component Instantiation 2
  ------------------------------------------------------------------------------
  spi4_to_spi3_burst_fifo2: generic_sfifo_512x72
  port map (
    clk        => SPI4_Clk,
    sinit      => FIFO_reset,
    din        => FIFO_wr_data,
    wr_en      => BurstFIFO_2_WrEn,
    rd_en      => SPI3_2_BrstRdEn,
    dout       => SPI3_2_BrstData_temp,
    full       => OPEN,
    empty      => SPI3_2_Brst_Empty,
    data_count => FIFO_2_data_count
    );
    
  ------------------------------------------------------------------------------
  -- SPI4 to SPI3 Burst FIFO Logic Component Instantiation 3
  ------------------------------------------------------------------------------
  spi4_to_spi3_burst_fifo3: generic_sfifo_512x72
  port map (
    clk        => SPI4_Clk,
    sinit      => FIFO_reset,
    din        => FIFO_wr_data,
    wr_en      => BurstFIFO_3_WrEn,
    rd_en      => SPI3_3_BrstRdEn,
    dout       => SPI3_3_BrstData_temp,
    full       => OPEN,
    empty      => SPI3_3_Brst_Empty,
    data_count => FIFO_3_data_count
    );
    
  --Combinatorial outputs
  SPI3_0_Brst_HalfFull <= FIFO_0_data_count(8);
  SPI3_1_Brst_HalfFull <= FIFO_1_data_count(8);
  SPI3_2_Brst_HalfFull <= FIFO_2_data_count(8);
  SPI3_3_Brst_HalfFull <= FIFO_3_data_count(8);
  
  SPI3_0_BrstData <= SPI3_0_BrstData_temp(69 downto 0);
  SPI3_1_BrstData <= SPI3_1_BrstData_temp(69 downto 0);
  SPI3_2_BrstData <= SPI3_2_BrstData_temp(69 downto 0);
  SPI3_3_BrstData <= SPI3_3_BrstData_temp(69 downto 0);

end xilinx;
