-------------------------------------------------------------------------------
--
--   File Name:  spi4_to_spi3_core.vhd
--     Version:  1.0
--        Date:  June 11, 2007
--
-- Description:  The purpose of the spi4_to_spi3_core.vhd file is to serve as 
--               the wrapper file for all logic associated with the SPI4 to 
--               SPI3 direction of the Bridge.
--
--      Author:  Zhe Xia
--     Company:  Xilinx Inc.
--
--
--     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
--     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
--     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
--     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
--     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
--     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
--     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
--     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
--     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
--     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
--     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
--     FOR A PARTICULAR PURPOSE.
--
--     (c) Copyright 2003 Xilinx, Inc.
--     All rights reserved.
--
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Library Declarations
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

--------------------------------------------------------------------------------
-- Entity Declaration
--------------------------------------------------------------------------------

entity spi4_to_spi3_core is
  port (
    -- System signals
    Reset_n                 : in  std_logic;
    SPI4_Clk                : in  std_logic;

    -- SPI4 Interface Signals
    SPI4_FFAddr             : in  std_logic_vector(7 downto 0);
    SPI4_FFData             : in  std_logic_vector(63 downto 0);
    SPI4_FFMod              : in  std_logic_vector(2 downto 0);
    SPI4_FFSOP              : in  std_logic;
    SPI4_FFEOP              : in  std_logic;
    SPI4_FFErr              : in  std_logic;
    SPI4_FFValid            : in  std_logic;
    SPI4_FFPayloadErr       : in  std_logic;
    SPI4_FFDIP4Err          : in  std_logic;
	 SPI4_FFBurstErr			 : in  std_logic;
	 SPI4_FFPayloadDIP4		 : in  std_logic;
    SPI4_FFEmpty_n          : in  std_logic;
    SPI4_FFAlmostEmpty_n    : in  std_logic;
    SPI4_FFRdEn_n           : out std_logic;
    
    -- SPI3 0 Interface Signals
    SPI3_0_WrEn_n         : out std_logic;
    SPI3_0_WrAddr           : out std_logic_vector(7 downto 0);
    SPI3_0_WrData           : out std_logic_vector(31 downto 0);
    SPI3_0_WrMod            : out std_logic_vector(1 downto 0);
    SPI3_0_WrSOP            : out std_logic;
    SPI3_0_WrEOP            : out std_logic;
    SPI3_0_WrErr            : out std_logic;
    SPI3_0_TxPTPA           : in  std_logic;
    SPI3_0_TxTAdr           : in  std_logic_vector(7 downto 0);

     -- SPI3 1 Interface Signals
    SPI3_1_WrEn_n         : out std_logic;
    SPI3_1_WrAddr           : out std_logic_vector(7 downto 0);
    SPI3_1_WrData           : out std_logic_vector(31 downto 0);
    SPI3_1_WrMod            : out std_logic_vector(1 downto 0);
    SPI3_1_WrSOP            : out std_logic;
    SPI3_1_WrEOP            : out std_logic;
    SPI3_1_WrErr            : out std_logic;
    SPI3_1_TxPTPA           : in  std_logic;
    SPI3_1_TxTAdr           : in  std_logic_vector(7 downto 0);

    -- SPI3 2 Interface Signals
    SPI3_2_WrEn_n         : out std_logic;
    SPI3_2_WrAddr           : out std_logic_vector(7 downto 0);
    SPI3_2_WrData           : out std_logic_vector(31 downto 0);
    SPI3_2_WrMod            : out std_logic_vector(1 downto 0);
    SPI3_2_WrSOP            : out std_logic;
    SPI3_2_WrEOP            : out std_logic;
    SPI3_2_WrErr            : out std_logic;
    SPI3_2_TxPTPA           : in  std_logic;
    SPI3_2_TxTAdr           : in  std_logic_vector(7 downto 0);

    -- SPI3 3 Interface Signals
    SPI3_3_WrEn_n         : out std_logic;
    SPI3_3_WrAddr           : out std_logic_vector(7 downto 0);
    SPI3_3_WrData           : out std_logic_vector(31 downto 0);
    SPI3_3_WrMod            : out std_logic_vector(1 downto 0);
    SPI3_3_WrSOP            : out std_logic;
    SPI3_3_WrEOP            : out std_logic;
    SPI3_3_WrErr            : out std_logic;
    SPI3_3_TxPTPA           : in  std_logic;
    SPI3_3_TxTAdr           : in  std_logic_vector(7 downto 0);

    -- SPI4 Flow Control
    SPI4_Stat               : out std_logic_vector(7 downto 0);
    SPI4_StatWrEn_n         : out std_logic;

    -- SPI3 Flow Control
    SPI3_0_dst_rdy     : in  std_logic;
    SPI3_1_dst_rdy     : in  std_logic;
    SPI3_2_dst_rdy     : in  std_logic;
    SPI3_3_dst_rdy     : in  std_logic
    );
end spi4_to_spi3_core;

--------------------------------------------------------------------------------
-- Definition of Ports:
--------------------------------------------------------------------------------
-- System signals
--    Reset_n                 : System Level Reset (active low)
--    SPI4_Clk                : SPI4 Interface Clock 
--
-- SPI4 Core Interface Input Signals
--    SPI4_FFAddr           : SPI4 Interface Address
--    SPI4_FFData           : SPI4 Interface Data
--    SPI4_FFMod            : SPI4 Interface MOD
--    SPI4_FFSOP            : SPI4 Interface Start of Packet 
--    SPI4_FFEOP            : SPI4 Interface End of Packet 
--    SPI4_FFErr            : SPI4 Interface Error 
--    SPI4_FFValid          : SPI4 Interface Data is Valid 
--    SPI4_FFPayloadErr     : SPI4 Interface Payload Error (current data was
--                            not preceded by a payload control word
--    SPI4_FFDIP4Err        : SPI4 Interface DIP-4 Error was detected on current
--                            end of burst (either EOP or N-credit boundary)
--    SPI4_FFEmpty_n        : SPI4 Interface is Empty (active low) 
--    SPI4_FFAlmostEmpty_n  : SPI4 Interface is Almost Empty (active low)

-- SPI4 Core Interface Output Signals
--    SPI4_FFRdEn_n         : SPI4 Interface Read Enable (active low)

-- SPI3 Core 0 Interface Write Output Signals
--    SPI3_0_WrEn_n         : SPI3 Interface Write Enable (active low)
--    SPI3_0_Addr           : SPI3 Interface Write Address 
--    SPI3_0_Data           : SPI3 Interface Write Data 
--    SPI3_0_Mod            : SPI3 Interface Write MOD 
--    SPI3_0_SOP            : SPI3 Interface Write Start of Packet 
--    SPI3_0_EOP            : SPI3 Interface Write End of Packet  
--    SPI3_0_Err            : SPI3 Interface Write Error  
--
-- SPI3 Core 1 Interface Write Output Signals
--    SPI3_1_WrEn_n         : SPI3 Interface Write Enable (active low)
--    SPI3_1_Addr           : SPI3 Interface Write Address 
--    SPI3_1_Data           : SPI3 Interface Write Data 
--    SPI3_1_Mod            : SPI3 Interface Write MOD 
--    SPI3_1_SOP            : SPI3 Interface Write Start of Packet 
--    SPI3_1_EOP            : SPI3 Interface Write End of Packet  
--    SPI3_1_Err            : SPI3 Interface Write Error  

-- SPI3 Core 2 Interface Write Output Signals
--    SPI3_2_WrEn_n         : SPI3 Interface Write Enable (active low)
--    SPI3_2_Addr           : SPI3 Interface Write Address 
--    SPI3_2_Data           : SPI3 Interface Write Data 
--    SPI3_2_Mod            : SPI3 Interface Write MOD 
--    SPI3_2_SOP            : SPI3 Interface Write Start of Packet 
--    SPI3_2_EOP            : SPI3 Interface Write End of Packet  
--    SPI3_2_Err            : SPI3 Interface Write Error  
--
-- SPI3 Core 3 Interface Write Output Signals
--    SPI3_3_WrEn_n         : SPI3 Interface Write Enable (active low)
--    SPI3_3_Addr           : SPI3 Interface Write Address 
--    SPI3_3_Data           : SPI3 Interface Write Data 
--    SPI3_3_Mod            : SPI3 Interface Write MOD 
--    SPI3_3_SOP            : SPI3 Interface Write Start of Packet 
--    SPI3_3_EOP            : SPI3 Interface Write End of Packet  
--    SPI3_3_Err            : SPI3 Interface Write Error  
--
-- SPI4 Status Output Signals
--    SPI4_StatWrEn_n           : SPI4 Status Write Enable (active low)
--    SPI4_Stat                 : SPI4 Status - 2-bits per channel
--                                (7:6) = Channel 3, (1:0) = Channel 0
-- SPI3 Core Status Input Signals
--    SPI3_0_dst_rdy       : SPI3 0 is available (active high)
--    SPI3_1_dst_rdy       : SPI3 1 is available (active high) 
--    SPI3_2_dst_rdy       : SPI3 2 is available (active high) 
--    SPI3_3_dst_rdy       : SPI3 3 is available (active high)
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
-- spi4_to_spi3_core Architecture
--------------------------------------------------------------------------------
architecture xilinx of spi4_to_spi3_core is

  ------------------------------------------------------------------------------
  -- SPI4 to SPI3 Read Logic Component Declaration
  ------------------------------------------------------------------------------
  component spi4_to_spi3_read 
    port (
      -- System signals
      Reset_n               : in  std_logic;
      SPI4_Clk              : in  std_logic;
  
      -- SPI4 Core Interface Input Signals
      SPI4_FFAddr           : in  std_logic_vector(7 downto 0);
      SPI4_FFData           : in  std_logic_vector(63 downto 0);
      SPI4_FFMod            : in  std_logic_vector(2 downto 0);
      SPI4_FFSOP            : in  std_logic;
      SPI4_FFEOP            : in  std_logic;
      SPI4_FFErr            : in  std_logic;
      SPI4_FFValid          : in  std_logic;
      SPI4_FFPayloadErr     : in  std_logic;
      SPI4_FFDIP4Err        : in  std_logic;
		SPI4_FFBurstErr		 : in  std_logic;
	   SPI4_FFPayloadDIP4	 : in  std_logic;
      SPI4_FFEmpty_n        : in  std_logic;
      SPI4_FFAlmostEmpty_n  : in  std_logic;

      -- SPI4 Core Interface Output Signals 
      SPI4_FFRdEn_n         : out std_logic;
    
      -- SPI4 Write Logic Interface Output Signals
      SPI4_Addr             : out std_logic_vector(7 downto 0);
      SPI4_Data             : out std_logic_vector(63 downto 0);
      SPI4_Mod              : out std_logic_vector(2 downto 0);
      SPI4_SOP              : out std_logic;
      SPI4_EOP              : out std_logic;
      SPI4_Err              : out std_logic;
      SPI4_Valid            : out std_logic
      );
  end component;

  ------------------------------------------------------------------------------
  -- SPI4 to SPI3 Write Logic Component Declaration
  ------------------------------------------------------------------------------
  component spi4_to_spi3_write
    port (
      -- System signals
      Reset_n               : in  std_logic;
      SPI4_Clk              : in  std_logic;

      -- Signals to/from Burst Storage FIFO
      SPI3_0_BrstRdEn      : out std_logic;
      SPI3_0_BrstData      : in  std_logic_vector(69 downto 0);
      SPI3_1_BrstRdEn      : out std_logic;
      SPI3_1_BrstData      : in  std_logic_vector(69 downto 0);
      SPI3_2_BrstRdEn      : out std_logic;
      SPI3_2_BrstData      : in  std_logic_vector(69 downto 0);
      SPI3_3_BrstRdEn      : out std_logic;
      SPI3_3_BrstData      : in  std_logic_vector(69 downto 0);
      SPI3_0_Brst_Empty    : in  std_logic;
      SPI3_1_Brst_Empty    : in  std_logic;
      SPI3_2_Brst_Empty    : in  std_logic;
      SPI3_3_Brst_Empty    : in  std_logic;
      
      -- SPI3 Status Signals
      SPI3_0_dst_rdy 		: in  std_logic;
      SPI3_1_dst_rdy 		: in  std_logic;
      SPI3_2_dst_rdy			: in  std_logic;
      SPI3_3_dst_rdy 		: in  std_logic;
      
      -- SPI3 0 Write Logic Interface
      SPI3_0_WrEn_n       : out std_logic;
      SPI3_0_WrData       : out std_logic_vector(31 downto 0);
      SPI3_0_WrMod        : out std_logic_vector(1 downto 0);
      SPI3_0_WrSOP        : out std_logic;
      SPI3_0_WrEOP        : out std_logic;
      SPI3_0_WrErr        : out std_logic;
      SPI3_0_TxPTPA       : in  std_logic;
      SPI3_0_TxTAdr       : in  std_logic_vector(7 downto 0);

      -- SPI3 1 Write Logic Interface
      SPI3_1_WrEn_n       : out std_logic;
      SPI3_1_WrData       : out std_logic_vector(31 downto 0);
      SPI3_1_WrMod        : out std_logic_vector(1 downto 0);
      SPI3_1_WrSOP        : out std_logic;
      SPI3_1_WrEOP        : out std_logic;
      SPI3_1_WrErr        : out std_logic;
      SPI3_1_TxPTPA       : in  std_logic;
      SPI3_1_TxTAdr       : in  std_logic_vector(7 downto 0);
      
      -- SPI3 2 Write Logic Interface
      SPI3_2_WrEn_n       : out std_logic;
      SPI3_2_WrData       : out std_logic_vector(31 downto 0);
      SPI3_2_WrMod        : out std_logic_vector(1 downto 0);
      SPI3_2_WrSOP        : out std_logic;
      SPI3_2_WrEOP        : out std_logic;
      SPI3_2_WrErr        : out std_logic;
      SPI3_2_TxPTPA       : in  std_logic;
      SPI3_2_TxTAdr       : in  std_logic_vector(7 downto 0);

      -- SPI3 3 Write Logic Interface
      SPI3_3_WrEn_n       : out std_logic;
      SPI3_3_WrData       : out std_logic_vector(31 downto 0);
      SPI3_3_WrMod        : out std_logic_vector(1 downto 0);
      SPI3_3_WrSOP        : out std_logic;
      SPI3_3_WrEOP        : out std_logic;
      SPI3_3_WrErr        : out std_logic;
      SPI3_3_TxPTPA       : in  std_logic;
      SPI3_3_TxTAdr       : in  std_logic_vector(7 downto 0)
      );
  end component;


  ------------------------------------------------------------------------------
  -- SPI4 to SPI3 Flow Control Logic Component Declaration
  ------------------------------------------------------------------------------
  component spi4_to_spi3_flow_control
    port (
      Reset_n                   : in  std_logic;
      SPI4_Clk                  : in  std_logic;
  
      SPI3_0_dst_rdy       	  : in  std_logic;
      SPI3_1_dst_rdy       	  : in  std_logic;
      SPI3_2_dst_rdy       	  : in  std_logic;
      SPI3_3_dst_rdy       	  : in  std_logic;

      SPI3_0_Brst_HalfFull      : in  std_logic;
      SPI3_1_Brst_HalfFull      : in  std_logic;
      SPI3_2_Brst_HalfFull      : in  std_logic;
      SPI3_3_Brst_HalfFull      : in  std_logic;
  
      SPI4_StatWrEn_n           : out std_logic;
      SPI4_Stat                 : out std_logic_vector(7 downto 0)
      );
  end component;
  
  ------------------------------------------------------------------------------
  -- SPI4 to SPI3 Burst Storage Logic Component Declaration
  ------------------------------------------------------------------------------
  component spi4_to_spi3_burst_storage
  port (
    Reset_n              : in  std_logic;
    SPI4_Clk             : in  std_logic;
    SPI4_Addr            : in  std_logic_vector(7 downto 0);
    SPI4_Data            : in  std_logic_vector(63 downto 0);
    SPI4_Mod             : in  std_logic_vector(2 downto 0);
    SPI4_SOP             : in  std_logic;
    SPI4_EOP             : in  std_logic;
    SPI4_Err             : in  std_logic;
    SPI4_Valid           : in  std_logic;
    SPI4_Dip4Err         : in  std_logic;
	 SPI4_FFBurstErr		 : in  std_logic;
	 SPI4_FFPayloadDIP4	 : in  std_logic;
    SPI4_PayloadErr      : in  std_logic;
    SPI3_0_BrstRdEn      : in  std_logic;
    SPI3_0_BrstData      : out std_logic_vector(69 downto 0);
    SPI3_1_BrstRdEn      : in  std_logic;
    SPI3_1_BrstData      : out std_logic_vector(69 downto 0);
    SPI3_2_BrstRdEn      : in  std_logic;
    SPI3_2_BrstData      : out std_logic_vector(69 downto 0);
    SPI3_3_BrstRdEn      : in  std_logic;
    SPI3_3_BrstData      : out std_logic_vector(69 downto 0);
    SPI3_0_Brst_Empty    : out std_logic;
    SPI3_1_Brst_Empty    : out std_logic;
    SPI3_2_Brst_Empty    : out std_logic;
    SPI3_3_Brst_Empty    : out std_logic;
    SPI3_0_Brst_HalfFull : out std_logic;
    SPI3_1_Brst_HalfFull : out std_logic;
    SPI3_2_Brst_HalfFull : out std_logic;
    SPI3_3_Brst_HalfFull : out std_logic
    );
  end component;
  
  ------------------------------------------------------------------------------
  -- Constant Declarations
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  -- Signal Declarations
  ------------------------------------------------------------------------------

  -- SPI4 Read FSM to SPI4 Write Logic Signals
  signal SPI4_Addr              : std_logic_vector(7 downto 0);
  signal SPI4_Data              : std_logic_vector(63 downto 0);
  signal SPI4_Mod               : std_logic_vector(2 downto 0);
  signal SPI4_SOP               : std_logic;
  signal SPI4_EOP               : std_logic;
  signal SPI4_Err               : std_logic;
  signal SPI4_Valid             : std_logic;
  signal SPI4_Full_n            : std_logic;
  
  signal SPI3_0_BrstData        : std_logic_vector(69 downto 0);
  signal SPI3_1_BrstData        : std_logic_vector(69 downto 0);
  signal SPI3_2_BrstData        : std_logic_vector(69 downto 0);
  signal SPI3_3_BrstData        : std_logic_vector(69 downto 0);
  signal SPI3_0_BrstRdEn        : std_logic;
  signal SPI3_1_BrstRdEn        : std_logic;
  signal SPI3_2_BrstRdEn        : std_logic;
  signal SPI3_3_BrstRdEn        : std_logic;
  signal SPI3_0_Brst_Empty      : std_logic;
  signal SPI3_1_Brst_Empty      : std_logic;
  signal SPI3_2_Brst_Empty      : std_logic;
  signal SPI3_3_Brst_Empty      : std_logic;
  signal SPI3_0_Brst_HalfFull   : std_logic;
  signal SPI3_1_Brst_HalfFull   : std_logic;
  signal SPI3_2_Brst_HalfFull   : std_logic;
  signal SPI3_3_Brst_HalfFull   : std_logic;
  
begin
  
  ------------------------------------------------------------------------------
  -- SPI4 to SPI3 Read Logic Component Instantiation
  ------------------------------------------------------------------------------
  spi4_to_spi3_read0 : spi4_to_spi3_read 
  port map (
    Reset_n              => Reset_n,
    SPI4_Clk             => SPI4_Clk,
    SPI4_FFAddr          => SPI4_FFAddr,
    SPI4_FFData          => SPI4_FFData,
    SPI4_FFMod           => SPI4_FFMod,
    SPI4_FFSOP           => SPI4_FFSOP,
    SPI4_FFEOP           => SPI4_FFEOP,
    SPI4_FFErr           => SPI4_FFErr,
    SPI4_FFValid         => SPI4_FFValid,
    SPI4_FFPayloadErr    => SPI4_FFPayloadErr,
    SPI4_FFDIP4Err       => SPI4_FFDIP4Err,
	 SPI4_FFBurstErr		 => SPI4_FFBurstErr,
	 SPI4_FFPayloadDIP4	 => SPI4_FFPayloadDIP4,
    SPI4_FFEmpty_n       => SPI4_FFEmpty_n,
    SPI4_FFAlmostEmpty_n => SPI4_FFAlmostEmpty_n,
    SPI4_FFRdEn_n        => SPI4_FFRdEn_n,
    SPI4_Addr            => SPI4_Addr,
    SPI4_Data            => SPI4_Data,
    SPI4_Mod             => SPI4_Mod,
    SPI4_SOP             => SPI4_SOP,
    SPI4_EOP             => SPI4_EOP,
    SPI4_Err             => SPI4_Err,
    SPI4_Valid           => SPI4_Valid 
    );
  ------------------------------------------------------------------------------
  -- SPI4 to SPI3 Write Logic Component Instantiation
  ------------------------------------------------------------------------------
  spi4_to_spi3_write0 : spi4_to_spi3_write 
  port map (
    Reset_n             => Reset_n ,
    SPI4_Clk            => SPI4_Clk ,
    SPI3_0_BrstRdEn     => SPI3_0_BrstRdEn ,
    SPI3_0_BrstData     => SPI3_0_BrstData ,
    SPI3_1_BrstRdEn     => SPI3_1_BrstRdEn ,
    SPI3_1_BrstData     => SPI3_1_BrstData ,
    SPI3_2_BrstRdEn     => SPI3_2_BrstRdEn ,
    SPI3_2_BrstData     => SPI3_2_BrstData ,
    SPI3_3_BrstRdEn     => SPI3_3_BrstRdEn ,
    SPI3_3_BrstData     => SPI3_3_BrstData ,
    SPI3_0_Brst_Empty   => SPI3_0_Brst_Empty ,
    SPI3_1_Brst_Empty   => SPI3_1_Brst_Empty ,
    SPI3_2_Brst_Empty   => SPI3_2_Brst_Empty ,
    SPI3_3_Brst_Empty   => SPI3_3_Brst_Empty ,
    SPI3_0_dst_rdy      => SPI3_0_dst_rdy ,
    SPI3_1_dst_rdy      => SPI3_1_dst_rdy ,
    SPI3_2_dst_rdy      => SPI3_2_dst_rdy ,
    SPI3_3_dst_rdy      => SPI3_3_dst_rdy ,
    SPI3_0_WrEn_n       => SPI3_0_WrEn_n ,
    SPI3_0_WrData       => SPI3_0_WrData ,
    SPI3_0_WrMod        => SPI3_0_WrMod ,
    SPI3_0_WrSOP        => SPI3_0_WrSOP ,
    SPI3_0_WrEOP        => SPI3_0_WrEOP ,
    SPI3_0_WrErr        => SPI3_0_WrErr ,
    SPI3_0_TxPTPA       => SPI3_0_TxPTPA ,
    SPI3_0_TxTAdr       => SPI3_0_TxTAdr ,
    SPI3_1_WrEn_n       => SPI3_1_WrEn_n ,
    SPI3_1_WrData       => SPI3_1_WrData ,
    SPI3_1_WrMod        => SPI3_1_WrMod ,
    SPI3_1_WrSOP        => SPI3_1_WrSOP ,
    SPI3_1_WrEOP        => SPI3_1_WrEOP ,
    SPI3_1_WrErr        => SPI3_1_WrErr ,
    SPI3_1_TxPTPA       => SPI3_1_TxPTPA ,
    SPI3_1_TxTAdr       => SPI3_1_TxTAdr ,
    SPI3_2_WrEn_n       => SPI3_2_WrEn_n ,
    SPI3_2_WrData       => SPI3_2_WrData ,
    SPI3_2_WrMod        => SPI3_2_WrMod ,
    SPI3_2_WrSOP        => SPI3_2_WrSOP ,
    SPI3_2_WrEOP        => SPI3_2_WrEOP ,
    SPI3_2_WrErr        => SPI3_2_WrErr ,
    SPI3_2_TxPTPA       => SPI3_2_TxPTPA ,
    SPI3_2_TxTAdr       => SPI3_2_TxTAdr ,
    SPI3_3_WrEn_n       => SPI3_3_WrEn_n ,
    SPI3_3_WrData       => SPI3_3_WrData ,
    SPI3_3_WrMod        => SPI3_3_WrMod ,
    SPI3_3_WrSOP        => SPI3_3_WrSOP ,
    SPI3_3_WrEOP        => SPI3_3_WrEOP ,
    SPI3_3_WrErr        => SPI3_3_WrErr ,
    SPI3_3_TxPTPA       => SPI3_3_TxPTPA ,
    SPI3_3_TxTAdr       => SPI3_3_TxTAdr 
    );                  

  ------------------------------------------------------------------------------
  -- SPI4 to SPI3 Flow Control Logic Component Instantiation
  ------------------------------------------------------------------------------
  spi4_to_spi3_flow_control0 : spi4_to_spi3_flow_control 
  port map (
    Reset_n              => Reset_n ,
    SPI4_Clk             => SPI4_Clk ,
    SPI3_0_dst_rdy  		 => SPI3_0_dst_rdy ,
    SPI3_1_dst_rdy 		 => SPI3_1_dst_rdy ,
    SPI3_2_dst_rdy		 => SPI3_2_dst_rdy ,
    SPI3_3_dst_rdy 		 => SPI3_3_dst_rdy ,
    SPI3_0_Brst_HalfFull => SPI3_0_Brst_HalfFull,
    SPI3_1_Brst_HalfFull => SPI3_1_Brst_HalfFull,
    SPI3_2_Brst_HalfFull => SPI3_2_Brst_HalfFull,
    SPI3_3_Brst_HalfFull => SPI3_3_Brst_HalfFull,
    SPI4_StatWrEn_n      => SPI4_StatWrEn_n ,
    SPI4_Stat            => SPI4_Stat 
    );
    
  ------------------------------------------------------------------------------
  -- SPI4 to SPI3 Burst Storage Logic Component Instantiation
  ------------------------------------------------------------------------------
  spi4_to_spi3_burst_storage0: spi4_to_spi3_burst_storage
  port map (
    Reset_n              => Reset_n ,
    SPI4_Clk             => SPI4_Clk ,
    SPI4_Addr            => SPI4_Addr,
    SPI4_Data            => SPI4_Data,
    SPI4_Mod             => SPI4_Mod ,
    SPI4_SOP             => SPI4_SOP ,
    SPI4_EOP             => SPI4_EOP ,
    SPI4_Err             => SPI4_Err ,
    SPI4_Valid           => SPI4_Valid ,
    SPI4_Dip4Err       => SPI4_FFDip4Err ,
	 SPI4_FFBurstErr		 => SPI4_FFBurstErr,
	 SPI4_FFPayloadDIP4	 	 => SPI4_FFPayloadDIP4,
    SPI4_PayloadErr      => SPI4_FFPayloadErr ,
    SPI3_0_BrstRdEn      => SPI3_0_BrstRdEn ,
    SPI3_0_BrstData      => SPI3_0_BrstData ,
    SPI3_1_BrstRdEn      => SPI3_1_BrstRdEn ,
    SPI3_1_BrstData      => SPI3_1_BrstData ,
    SPI3_2_BrstRdEn      => SPI3_2_BrstRdEn ,
    SPI3_2_BrstData      => SPI3_2_BrstData ,
    SPI3_3_BrstRdEn      => SPI3_3_BrstRdEn ,
    SPI3_3_BrstData      => SPI3_3_BrstData ,
    SPI3_0_Brst_Empty    => SPI3_0_Brst_Empty ,
    SPI3_1_Brst_Empty    => SPI3_1_Brst_Empty ,
    SPI3_2_Brst_Empty    => SPI3_2_Brst_Empty ,
    SPI3_3_Brst_Empty    => SPI3_3_Brst_Empty ,
    SPI3_0_Brst_HalfFull => SPI3_0_Brst_HalfFull ,
    SPI3_1_Brst_HalfFull => SPI3_1_Brst_HalfFull ,
    SPI3_2_Brst_HalfFull => SPI3_2_Brst_HalfFull ,
    SPI3_3_Brst_HalfFull => SPI3_3_Brst_HalfFull
    );
    
  --Only address 0 of the SPI-3 cores is used
  SPI3_0_WrAddr <= (others => '0');
  SPI3_1_WrAddr <= (others => '0');
  SPI3_2_WrAddr <= (others => '0');
  SPI3_3_WrAddr <= (others => '0');

end xilinx;
