-------------------------------------------------------------------------------
--
--   File Name:  spi4_to_spi3_top.vhd
--     Version:  1.0
--        Date:  June 11, 2007
--
-- Description:  The purpose of the spi4_to_spi3_top.vhd file is to serve as 
--               the wrapper file for the SPI4 to SPI3 Bridge Design, as well
--               as instantiating the four SPI3 Cores, and the SPI4.2 Core.
--
--      Author:  Zhe Xia
--     Company:  Xilinx Inc.
--
--
--     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
--     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
--     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
--     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
--     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
--     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
--     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
--     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
--     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
--     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
--     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
--     FOR A PARTICULAR PURPOSE.
--
--     (c) Copyright 2003 Xilinx, Inc.
--     All rights reserved.
--
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Library Declarations
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


-- synthesis translate_off
library unisim;
use unisim.vcomponents.all;
-- synthesis translate_on

--------------------------------------------------------------------------------
-- Entity Declaration
--------------------------------------------------------------------------------

entity spi4_to_spi3_top is
  port (
    -- System signals
    Reset_n                 : in  std_logic;
	 SPI4_FFReset_n			 : in  std_logic;	
    SPI4_Clk                : in  std_logic;
--	 SPI4_Clk_div2				 : in  std_logic;
	 Clk200						 : in  std_logic;

    -- SPI3 0 Interface
    SPI3_TFClk             : in  std_logic;
    SPI3_0_TEnb            : out std_logic;
    SPI3_0_TSx             : out std_logic;
    SPI3_0_TSOP            : out std_logic;
    SPI3_0_TEOP            : out std_logic;
    SPI3_0_TDat            : out std_logic_vector(31 downto 0);
    SPI3_0_TPrty           : out std_logic;
    SPI3_0_TMod            : out std_logic_vector(1 downto 0);
    SPI3_0_TErr            : out std_logic;
    SPI3_0_PTPA            : in  std_logic;
    SPI3_0_TAdr            : out std_logic_vector(7 downto 0);

    -- SPI3 1 Interface
    SPI3_1_TEnb            : out std_logic;
    SPI3_1_TSx             : out std_logic;
    SPI3_1_TSOP            : out std_logic;
    SPI3_1_TEOP            : out std_logic;
    SPI3_1_TDat            : out std_logic_vector(31 downto 0);
    SPI3_1_TPrty           : out std_logic;
    SPI3_1_TMod            : out std_logic_vector(1 downto 0);
    SPI3_1_TErr            : out std_logic;
    SPI3_1_PTPA            : in  std_logic;
    SPI3_1_TAdr            : out std_logic_vector(7 downto 0);

    -- SPI3 2 Interface
    SPI3_2_TEnb            : out std_logic;
    SPI3_2_TSx             : out std_logic;
    SPI3_2_TSOP            : out std_logic;
    SPI3_2_TEOP            : out std_logic;
    SPI3_2_TDat            : out std_logic_vector(31 downto 0);
    SPI3_2_TPrty           : out std_logic;
    SPI3_2_TMod            : out std_logic_vector(1 downto 0);
    SPI3_2_TErr            : out std_logic;
    SPI3_2_PTPA            : in  std_logic;
    SPI3_2_TAdr            : out std_logic_vector(7 downto 0);

    -- SPI3 3 Interface
    SPI3_3_TEnb            : out std_logic;
    SPI3_3_TSx             : out std_logic;
    SPI3_3_TSOP            : out std_logic;
    SPI3_3_TEOP            : out std_logic;
    SPI3_3_TDat            : out std_logic_vector(31 downto 0);
    SPI3_3_TPrty           : out std_logic;
    SPI3_3_TMod            : out std_logic_vector(1 downto 0);
    SPI3_3_TErr            : out std_logic;
    SPI3_3_PTPA            : in  std_logic;
    SPI3_3_TAdr            : out std_logic_vector(7 downto 0);

    -- System clocking input
    SPI3_DcmReset_TFClk     : in  std_logic;
    SPI4_DcmReset_RDClk     : in  std_logic;
    SPI4_Locked_RDClk       : out std_logic;
    
    -- SPI4.2 Interface
    -- SPI4.2 sink bus output
    SPI4_RDClk_P            : in  std_logic;
    SPI4_RDClk_N            : in  std_logic;
    SPI4_RCtl_P             : in  std_logic;
    SPI4_RCtl_N             : in  std_logic;
    SPI4_RDat_P             : in  std_logic_vector(15 downto 0);
    SPI4_RDat_N             : in  std_logic_vector(15 downto 0);

    -- SPI4.2 sink status (LVTTL) input
    SPI4_RSClk              : out std_logic;
    SPI4_RStat              : out std_logic_vector(1 downto 0);
    
    --SPI4.2 Calendar Programming Signals
    SPI4_SnkCalendar_M      : in  std_logic_vector(7 downto 0); 
    SPI4_SnkCalendar_Len    : in  std_logic_vector(8 downto 0); 
--    SPI4_SnkCalClk          : in  std_logic;                    
    SPI4_SnkCalWrEn_n       : in  std_logic;                    
    SPI4_SnkCalAddr         : in  std_logic_vector(8 downto 0); 
    SPI4_SnkCalData         : in  std_logic_vector(7 downto 0); 
    SPI4_SnkCalDataOut      : out std_logic_vector(7 downto 0); 
    
    --SPI4.2 Out of Frame
    SPI4_SnkOof             : out std_logic;
    SPI4_PhaseAlignComplete : out std_logic;
    SPI4_PhaseAlignRequest  : in  std_logic;
    SPI3_TFClk_bufg_out     : out std_logic;
    
    --PL4 sink enable
    SPI4_SnkEn              : in  std_logic;
	  SPI4_SnkTrainValid     : out std_logic;
	  SPI4_SnkDPAFailed      : out std_logic;
	  SPI4_SnkClksRdy        : out std_logic
    );
end spi4_to_spi3_top;

--------------------------------------------------------------------------------
-- Definition of ports:
--------------------------------------------------------------------------------
-- System signals
--    Reset_n                 : System Level Reset (active low)
--    SPI4_Clk                : SPI4 Interface Clock 
--
-- SPI 3 interfaces, # = 0-3
--    SPI3_#_TFClk            : SPI3 Source Clock Input 
--    SPI3_#_TEnb_n           : SPI3 Source Write Enable (active low)
--    SPI3_#_TSx              : SPI3 Source Start of Transfer (active high)
--    SPI3_#_TSOP             : SPI3 Source Start of Packet (active high)
--    SPI3_#_TEOP             : SPI3 Source End of Packet (active high)
--    SPI3_#_TDat             : SPI3 Source Data
--    SPI3_#_TPrty            : SPI3 Source Parity
--    SPI3_#_TMod             : SPI3 Source Word Modulo
--    SPI3_#_TErr             : SPI3 Source Error
--
-- System clocking input
--    SPI3_DcmReset_TFClk     : DCM Reset for the TFClk
--    SPI4_DcmReset_RDClk     : DCM Reset for the RDClk
--    SPI4_Locked_RDClk       : Signal indicating RDClk has locked
--
-- SPI4.2 interfaces
--    SPI4_RDClk_P/N          : SPI4.2 Sink Clock
--    SPI4_RCtl_P/N           : SPI4.2 Sink Control
--    SPI4_RDat_P/N           : SPI4.2 Sink Data
--    SPI4_RSClk              : SPI4.2 Sink Status Clock
--    SPI4_RStat              : SPI4.2 Sink Status 
--
-- SPI4.2 Calendar Programming Signals
--    SPI4_SnkCalendar_M      : Calendar repitition value
--    SPI4_SnkCalendar_Len    : How long is the calendar
--    SPI4_SnkCalClk          : Clock to program the calendar with          
--    SPI4_SnkCalWrEn_n       : Write enable for the calendar (active low)
--    SPI4_SnkCalAddr         : Calendar Address to be written
--    SPI4_SnkCalData         : Data (channel number) to be written into the calendar
--    SPI4_SnkCalDataOut      : Data that is read from the calendar
--    
--    --SPI4.2 Out of Frame
--    SPI4_SnkOof             : Sink core is out of frame
--    SPI4_PhaseAlignComplete : New phase alignment is complete
--    SPI4_PhaseAlignRequest  : Request for new phase alignment
--    SPI3_TFClk_bufg_out     : Clock that is not associated w/ phase alignment
--                              to use for resetting DCMs
--    
--    --PL4 sink enable
--    SPI4_SnkEn              : Enable for the SPI-4.2 sink core
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
-- spi4_to_spi3_top Architecture
--------------------------------------------------------------------------------
architecture xilinx of spi4_to_spi3_top is

  -- SPI3 Source Component Declaration
  component spi3_link_v4_1_spi3_link_tx is
 port (
      RST             : in  std_logic;
      TFCLK           : in  std_logic;
      TENB            : out std_logic;
      TDAT            : out std_logic_vector(31 downto 0);
      TMOD            : out std_logic_vector(1 downto 0);
      TPRTY           : out std_logic;
      TSOP            : out std_logic;
      TEOP            : out std_logic;
      TERR            : out std_logic;
      TSX             : out std_logic;
      TADR            : out std_logic_vector(7 downto 0);
      PTPA            : in  std_logic;
      TX_CLK          : in  std_logic;
      TX_DST_RDY      : out std_logic;
      TX_SRC_RDY      : in  std_logic;
      TX_SOF          : in  std_logic;
      TX_EOF          : in  std_logic;
      TX_ADDR         : in  std_logic_vector(7 downto 0);
      TX_DATA         : in  std_logic_vector(31 downto 0);
      TX_REM          : in  std_logic_vector(1 downto 0);
      TX_ERR          : in  std_logic;
      TX_HALT_TDAT    : in  std_logic;
      TX_CAL_CLK      : in  std_logic;
      TX_CAL_EN       : in  std_logic;
      TX_CAL_WR_EN    : in  std_logic;
      TX_CAL_ADDR     : in  std_logic_vector(8 downto 0);
      TX_CAL_DATA     : in  std_logic_vector(7 downto 0);
      TX_CAL_DATA_OUT : out std_logic_vector(7 downto 0);
      TX_PTPA         : out std_logic;
      TX_TADR         : out std_logic_vector(7 downto 0)
    );
  end component;

  -- SPI4.2 Sink Component Declaration
  component pl4_v8_1_pl4_snk_top
  port (
    -- common signals
    Reset_n             : in  std_logic;
    SnkFifoReset_n      : in  std_logic;
    DcmReset_RDClk      : in  std_logic;
    Locked_RDClk        : out std_logic; 
    DcmLost_RDClk       : out std_logic;
    SnkClksRdy          : out std_logic;
    PhaseAlignRequest   : in std_logic ; 
    PhaseAlignComplete  : out std_logic;
    SnkDPAFailed        : out std_logic; 
    SnkDPARamData       : out std_logic_vector(16 DOWNTO 0);
    SnkDPARamAddr       : out std_logic_vector(6 DOWNTO 0);
    SnkDPARamValid      : out std_logic;
    SnkIdelayRefClk     : in std_logic ;  

    -- sink general purpose clocks
    RDClkDiv_GP       : out std_logic;
    RDClk0_GP         : out std_logic;

    -- Sink Calendar I/O
    SnkCalendar_M     : in  std_logic_vector(7 downto 0);
    SnkCalendar_Len   : in  std_logic_vector(8 downto 0);
    SnkCalClk         : in  std_logic;
    SnkCalWrEn_n      : in  std_logic;
    SnkCalAddr        : in  std_logic_vector(8 downto 0);
    SnkCalData        : in  std_logic_vector(7 downto 0);
    SnkCalDataOut     : out std_logic_vector(7 downto 0);

    SnkStatClk        : in  std_logic;
    SnkStat           : in  std_logic_vector(31 downto 0);
    SnkStatAddr       : in  std_logic_vector(3 downto 0);
    SnkStatWr_n       : in  std_logic;
    SnkStatMask       : in  std_logic_vector(15 downto 0);
 
    -- FIFO I/O
    SnkFFClk          : in  std_logic;
    SnkFFRdEn_n       : in  std_logic;
    SnkFFAddr         : out std_logic_vector(7 downto 0);
    SnkFFData         : out std_logic_vector(63 downto 0);
    SnkFFMod          : out std_logic_vector(2 downto 0);
    SnkFFSOP          : out std_logic;
    SnkFFEOP          : out std_logic;
    SnkFFErr          : out std_logic;
    SnkFFBurstErr     : out std_logic;
    SnkFFDIP4Err      : out std_logic;
    SnkFFPayloadDIP4  : out std_logic;
    SnkFFPayloadErr   : out std_logic;
    SnkFFValid        : out std_logic;
    SnkFFEmpty_n      : out std_logic;
    SnkFFAlmostEmpty_n: out std_logic;
    SnkAlmostFull_n   : out std_logic;
    SnkOverflow_n     : out std_logic;
 
    -- Control and status signals
    SnkEn             : in  std_logic;
    SnkOof            : out std_logic;
    SnkBusErr         : out std_logic;
    SnkBusErrStat     : out std_logic_vector(7 downto 0);
    SnkTrainValid     : out std_logic;
    SnkDIP2ErrRequest : in  std_logic;
 
    -- FIFO status interface
    RSClk             : out std_logic;
    RStat             : out std_logic_vector(1 downto 0);

    -- PL4 sink bus
    RDClk_P           : in  std_logic;
    RDClk_N           : in  std_logic;
    RDat_P            : in  std_logic_vector(15 downto 0);
    RDat_N            : in  std_logic_vector(15 downto 0);
    RCtl_P            : in  std_logic;
    RCtl_N            : in  std_logic;

    -- Configuration signals
    SnkAFThresAssert  : in  std_logic_vector(8 downto 0);
    SnkAFThresNegate  : in  std_logic_vector(8 downto 0);
    FifoAFMode        : in  std_logic_vector(1 downto 0);
    NumDip4Errors     : in  std_logic_vector(3 downto 0);
    NumTrainSequences : in  std_logic_vector(3 downto 0);
    RSClkPhase        : in  std_logic;
    RSClkDiv          : in  std_logic
    );
end component;

  
  -- SPI4.2 to SPI3 Bridge Logic
  component spi4_to_spi3_core
  port (
    Reset_n                 : in  std_logic;
    SPI4_Clk                : in  std_logic;

    SPI4_FFAddr             : in  std_logic_vector(7 downto 0);
    SPI4_FFData             : in  std_logic_vector(63 downto 0);
    SPI4_FFMod              : in  std_logic_vector(2 downto 0);
    SPI4_FFSOP              : in  std_logic;
    SPI4_FFEOP              : in  std_logic;
    SPI4_FFErr              : in  std_logic;
    SPI4_FFValid            : in  std_logic;
    SPI4_FFPayloadErr       : in  std_logic;
    SPI4_FFDIP4Err          : in  std_logic;
	 SPI4_FFBurstErr			 : in  std_logic;
	 SPI4_FFPayloadDIP4		 : in  std_logic;
    SPI4_FFEmpty_n          : in  std_logic;
    SPI4_FFAlmostEmpty_n    : in  std_logic;
    SPI4_FFRdEn_n           : out std_logic;
    
    SPI3_0_WrEn_n           : out std_logic;
    SPI3_0_WrAddr           : out std_logic_vector(7 downto 0);
    SPI3_0_WrData           : out std_logic_vector(31 downto 0);
    SPI3_0_WrMod            : out std_logic_vector(1 downto 0);
    SPI3_0_WrSOP            : out std_logic;
    SPI3_0_WrEOP            : out std_logic;
    SPI3_0_WrErr            : out std_logic;
    SPI3_0_TxPTPA           : in  std_logic;
    SPI3_0_TxTAdr           : in  std_logic_vector(7 downto 0);

    SPI3_1_WrEn_n           : out std_logic;
    SPI3_1_WrAddr           : out std_logic_vector(7 downto 0);
    SPI3_1_WrData           : out std_logic_vector(31 downto 0);
    SPI3_1_WrMod            : out std_logic_vector(1 downto 0);
    SPI3_1_WrSOP            : out std_logic;
    SPI3_1_WrEOP            : out std_logic;
    SPI3_1_WrErr            : out std_logic;
    SPI3_1_TxPTPA           : in  std_logic;
    SPI3_1_TxTAdr           : in  std_logic_vector(7 downto 0);

    SPI3_2_WrEn_n           : out std_logic;
    SPI3_2_WrAddr           : out std_logic_vector(7 downto 0);
    SPI3_2_WrData           : out std_logic_vector(31 downto 0);
    SPI3_2_WrMod            : out std_logic_vector(1 downto 0);
    SPI3_2_WrSOP            : out std_logic;
    SPI3_2_WrEOP            : out std_logic;
    SPI3_2_WrErr            : out std_logic;
    SPI3_2_TxPTPA           : in  std_logic;
    SPI3_2_TxTAdr           : in  std_logic_vector(7 downto 0);

    SPI3_3_WrEn_n           : out std_logic;
    SPI3_3_WrAddr           : out std_logic_vector(7 downto 0);
    SPI3_3_WrData           : out std_logic_vector(31 downto 0);
    SPI3_3_WrMod            : out std_logic_vector(1 downto 0);
    SPI3_3_WrSOP            : out std_logic;
    SPI3_3_WrEOP            : out std_logic;
    SPI3_3_WrErr            : out std_logic;
    SPI3_3_TxPTPA           : in  std_logic;
    SPI3_3_TxTAdr           : in  std_logic_vector(7 downto 0);

    SPI4_Stat               : out std_logic_vector(7 downto 0);
    SPI4_StatWrEn_n         : out std_logic;

    SPI3_0_dst_rdy     : in  std_logic;
    SPI3_1_dst_rdy     : in  std_logic;
    SPI3_2_dst_rdy     : in  std_logic;
    SPI3_3_dst_rdy     : in  std_logic
    );
  end component;

  
  component IBUFG
  port (
    I : in  std_logic;
    O : out std_logic
  );
  end component;

  component BUFG
  port (
    I : in  std_logic;
    O : out std_logic
  );
  end component;

  component IBUF
  port (
    I : in  std_logic;
    O : out std_logic
  );
  end component;

  component OBUF
  port (
    I : in  std_logic;
    O : out std_logic
  );
  end component;

  component DCM
  port (
    CLKIN   : in  std_logic ;
    CLKFB   : in  std_logic ;
    DSSEN   : in  std_logic ;
    PSINCDEC: in  std_logic ;
    PSEN    : in  std_logic ;
    PSCLK   : in  std_logic ;
    RST     : in  std_logic ;
    CLK0    : out std_logic ;
    CLK90   : out std_logic ;
    CLK180  : out std_logic ;
    CLK270  : out std_logic ;
    CLK2X   : out std_logic ;
    CLK2X180: out std_logic ;
    CLKDV   : out std_logic ;
    CLKFX   : out std_logic ;
    CLKFX180: out std_logic ;
    LOCKED  : out std_logic ;
    PSDONE  : out std_logic ;
    STATUS  : out std_logic_vector(7 downto 0));
  end component;

  ------------------------------------------------------------------------------
  -- Constant Declarations
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  -- Signal Declarations
  ------------------------------------------------------------------------------

  -- SPI4.2 Core
  signal SPI4_FFAddr     : std_logic_vector(7 downto 0) ;
  signal SPI4_FFData     : std_logic_vector(63 downto 0) ;
  signal SPI4_FFMod      : std_logic_vector(2 downto 0) ;
  signal SPI4_FFSOP      : std_logic ;
  signal SPI4_FFEOP      : std_logic ;
  signal SPI4_FFErr      : std_logic ;
  signal SPI4_FFValid    : std_logic ;
  signal SPI4_FFPayloadErr : std_logic ;
  signal SPI4_FFDIP4Err  : std_logic ;

  signal SPI4_FFPayloadDIP4: std_logic;
  signal SPI4_FFEmpty_n  : std_logic ;
  signal SPI4_FFAlmostEmpty_n : std_logic ;
  signal SPI4_FFRdEn_n   : std_logic ;

  -- SPI4.2 Static Configuration Signals
  signal SPI4_PhaseAlignEn       : std_logic;
  signal SPI4_PhaseAlignErr      : std_logic;

  -- SPI4.2 Sink Control Signals
  signal SPI4_SnkBusErr          : std_logic;
  signal SPI4_FFBurstErr			: std_logic;
  signal SPI4_SnkAlmostFull_n    : std_logic;
  signal SPI4_SnkOverflow_n      : std_logic;
  signal SPI4_SnkAFThresAssert   : std_logic_vector(8 downto 0);
  signal SPI4_SnkAFThresNegate   : std_logic_vector(8 downto 0);
  signal SPI4_FifoAFMode         : std_logic_vector(1 downto 0);
  signal SPI4_NumDip4Errors      : std_logic_vector(3 downto 0);
  signal SPI4_NumTrainSequences  : std_logic_vector(3 downto 0);
  signal SPI4_NumDip2Errors      : std_logic_vector(3 downto 0);
  signal SPI4_NumDip2Matches     : std_logic_vector(3 downto 0);
  signal SPI4_RSClkPhase         : std_logic;
  signal SPI4_RSClkDiv           : std_logic;
  signal Reset 						: std_logic;
  -- SPI3 0 Core
  signal SPI3_0_WrEn_n         : std_logic ;
  signal SPI3_0_WrAddr         : std_logic_vector(7 downto 0) ;
  signal SPI3_0_WrData         : std_logic_vector(31 downto 0) ;
  signal SPI3_0_WrMod          : std_logic_vector(1 downto 0) ;
  signal SPI3_0_REM          : std_logic_vector(1 downto 0) ;
  signal SPI3_0_WrSOP          : std_logic ;
  signal SPI3_0_WrEOP          : std_logic ;
  signal SPI3_0_WrErr          : std_logic ;
  signal SPI3_0_TxTAdr         : std_logic_vector(7 downto 0) ;
  signal SPI3_0_TxPTPA         : std_logic;
  signal SPI3_0_ParitySelect   : std_logic;
  signal SPI3_0_MaxCalendarLen : std_logic_vector(8 downto 0);
  signal SPI3_0_CalWrEn_n      : std_logic;
  signal SPI3_0_CalAddr        : std_logic_vector(8 downto 0);
  signal SPI3_0_CalData        : std_logic_vector(7 downto 0);
  signal SPI3_0_CalDataOut     : std_logic_vector(7 downto 0);
  signal SPI3_0_HaltTDat       : std_logic;
  signal SPI3_0_Cal_En_n       : std_logic;
  signal SPI3_0_SRC_RDY 		 : std_logic;
  --signal SPI3_0_AFThresAssert  : std_logic_vector(8 downto 0);
  --signal SPI3_0_AFThresNegate  : std_logic_vector(8 downto 0);
  signal SPI3_0_dst_rdy   : std_logic;
  signal SPI3_0_Overflow_n     : std_logic;
      
  -- SPI3 1 Core
  signal SPI3_1_WrEn_n         : std_logic ;
  signal SPI3_1_WrAddr         : std_logic_vector(7 downto 0) ;
  signal SPI3_1_WrData         : std_logic_vector(31 downto 0) ;
  signal SPI3_1_WrMod          : std_logic_vector(1 downto 0) ;
  signal SPI3_1_REM          : std_logic_vector(1 downto 0) ;
  signal SPI3_1_WrSOP          : std_logic ;
  signal SPI3_1_WrEOP          : std_logic ;
  signal SPI3_1_WrErr          : std_logic ;
  signal SPI3_1_TxTAdr         : std_logic_vector(7 downto 0) ;
  signal SPI3_1_TxPTPA         : std_logic;
  signal SPI3_1_ParitySelect   : std_logic;
  signal SPI3_1_MaxCalendarLen : std_logic_vector(8 downto 0);
  signal SPI3_1_CalWrEn_n      : std_logic;
  signal SPI3_1_CalAddr        : std_logic_vector(8 downto 0);
  signal SPI3_1_CalData        : std_logic_vector(7 downto 0);
  signal SPI3_1_CalDataOut     : std_logic_vector(7 downto 0);
  signal SPI3_1_HaltTDat       : std_logic;
  signal SPI3_1_Cal_En_n       : std_logic;
  signal SPI3_1_SRC_RDY 		 : std_logic;
  --signal SPI3_1_AFThresAssert  : std_logic_vector(8 downto 0);
  --signal SPI3_1_AFThresNegate  : std_logic_vector(8 downto 0);
  signal SPI3_1_dst_rdy   : std_logic;
  signal SPI3_1_Overflow_n     : std_logic;
  
  -- SPI3 2 Core
  signal SPI3_2_WrEn_n         : std_logic ;
  signal SPI3_2_WrAddr         : std_logic_vector(7 downto 0) ;
  signal SPI3_2_WrData         : std_logic_vector(31 downto 0) ;
  signal SPI3_2_WrMod          : std_logic_vector(1 downto 0) ;
  signal SPI3_2_REM          : std_logic_vector(1 downto 0) ;
  signal SPI3_2_WrSOP          : std_logic ;
  signal SPI3_2_WrEOP          : std_logic ;
  signal SPI3_2_WrErr          : std_logic ;
  signal SPI3_2_TxTAdr         : std_logic_vector(7 downto 0) ;
  signal SPI3_2_TxPTPA         : std_logic;
  signal SPI3_2_ParitySelect   : std_logic;
  signal SPI3_2_MaxCalendarLen : std_logic_vector(8 downto 0);
  signal SPI3_2_CalWrEn_n      : std_logic;
  signal SPI3_2_CalAddr        : std_logic_vector(8 downto 0);
  signal SPI3_2_CalData        : std_logic_vector(7 downto 0);
  signal SPI3_2_CalDataOut     : std_logic_vector(7 downto 0);
  signal SPI3_2_HaltTDat       : std_logic;
  signal SPI3_2_Cal_En_n       : std_logic;
  signal SPI3_2_SRC_RDY 		 : std_logic;

  --signal SPI3_2_AFThresAssert  : std_logic_vector(8 downto 0);
  --signal SPI3_2_AFThresNegate  : std_logic_vector(8 downto 0);
  signal SPI3_2_dst_rdy   : std_logic;
  signal SPI3_2_Overflow_n     : std_logic;
  
  -- SPI3 3 Core
  signal SPI3_3_WrEn_n         : std_logic ;
  signal SPI3_3_WrAddr         : std_logic_vector(7 downto 0) ;
  signal SPI3_3_WrData         : std_logic_vector(31 downto 0) ;
  signal SPI3_3_WrMod          : std_logic_vector(1 downto 0) ;
  signal SPI3_3_REM          : std_logic_vector(1 downto 0) ;
  signal SPI3_3_WrSOP          : std_logic ;
  signal SPI3_3_WrEOP          : std_logic ;
  signal SPI3_3_WrErr          : std_logic ;
  signal SPI3_3_TxTAdr         : std_logic_vector(7 downto 0) ;
  signal SPI3_3_TxPTPA         : std_logic;
  signal SPI3_3_ParitySelect   : std_logic;
  signal SPI3_3_MaxCalendarLen : std_logic_vector(8 downto 0);
  signal SPI3_3_CalWrEn_n      : std_logic;
  signal SPI3_3_CalAddr        : std_logic_vector(8 downto 0);
  signal SPI3_3_CalData        : std_logic_vector(7 downto 0);
  signal SPI3_3_CalDataOut     : std_logic_vector(7 downto 0);
  signal SPI3_3_HaltTDat       : std_logic;
  signal SPI3_3_Cal_En_n       : std_logic;
  signal SPI3_3_SRC_RDY 		 : std_logic;

  --signal SPI3_3_AFThresAssert  : std_logic_vector(8 downto 0);
  --signal SPI3_3_AFThresNegate  : std_logic_vector(8 downto 0);
  signal SPI3_3_dst_rdy   : std_logic;
  signal SPI3_3_Overflow_n     : std_logic;

  -- SPI3 Clocking
  signal SPI3_TFClk_ibufg       : std_logic;
  signal SPI3_TFClk_dcm         : std_logic;
  signal SPI3_TFClk_bufg        : std_logic;
  signal SPI3_Locked_TFClk      : std_logic;
   
  -- SPI4.2 Sink Status Interface
  signal SPI4_Stat       : std_logic_vector(31 downto 0);
  signal SPI4_Stat_i     : std_logic_vector(7 downto 0);
  signal SPI4_StatAddr   : std_logic_vector(3 downto 0);
  signal SPI4_StatMask   : std_logic_vector(15 downto 0);
  signal SPI4_StatWrEn_n : std_logic;

  -- PL4 Sink Status
  signal SPI4_RStat_i          : std_logic_vector(1 downto 0);
  signal SPI4_RSClk_i          : std_logic;
  
  -- SPI3 Core Resets
  signal SPI3_Reset        : std_logic;
  
  signal GND                   : std_logic;
  signal Refclk					: std_logic;
  
  signal calendar_wr_cnt       : std_logic_vector(1 downto 0);

  -- Synthesis Attributes for SPI4.2 Core
  -- Used to prevent I/O buffers from being instantiated on the SPI4.2 Core
  --
  --------------------------
  -- Synplify Constraints
  --   comment out if using different synthesis tool 
  --------------------------
  attribute syn_black_box      : boolean ;
  attribute black_box_pad_pin : string ;
  
  attribute syn_black_box of pl4_v8_1_pl4_snk_top     : component is true;
--  attribute black_box_pad_pin of pl4_snk_top0 : component is
--      "RDClk_P,RDClk_N,RDat_P(15:0),RDat_N(15:0),RCtl_P,RCtl_N";
  attribute syn_black_box of IBUFG           : component is true;
  attribute syn_black_box of DCM             : component is true;
  attribute syn_black_box of BUFG            : component is true;
  attribute syn_black_box of IBUF            : component is true;
  attribute syn_black_box of OBUF            : component is true;

begin

  GND <= '0';
  Reset <= NOT Reset_n;
  ------------------------------------------------------------------------------
  -- SPI4.2 Dynamic Configuration Signal Assignments
  --   Modify as required 
  ------------------------------------------------------------------------------
  SPI4_NumDip4Errors     <= conv_std_logic_vector( 15 ,4);
  SPI4_NumTrainSequences <= conv_std_logic_vector( 4 ,4);
  SPI4_SnkAFThresAssert  <= conv_std_logic_vector( 32 ,9);
  SPI4_SnkAFThresNegate  <= conv_std_logic_vector( 32 ,9);
  SPI4_FifoAFMode        <= "01";
  SPI4_RSClkPhase        <= '0';
  SPI4_RSClkDiv          <= '0';
  SPI4_PhaseAlignEn      <= '1';
  SPI4_StatAddr          <= conv_std_logic_vector( 0, 4);
  SPI4_StatMask          <= (others => '1');

  -- Convert status into 32-bit vector to interface to SPI4.2 core
  SPI4_Stat(31 downto 8) <= (others => '0');
  SPI4_Stat(7 downto 0)  <= SPI4_Stat_i;

  ------------------------------------------------------------------------------
  -- SPI3 Static Configuration Signal Assignments
  --   Modify as required 
  ------------------------------------------------------------------------------
  -- SPI3 #0
--  SPI3_0_ParitySelect    <= '0';
  SPI3_0_CalWrEn_n       <= '1';
  SPI3_0_Cal_En_n       <= '1';
  SPI3_0_HaltTDat        <= '0';
  SPI3_0_CalWrEn_n		 <= '1';
--  SPI3_0_MaxCalendarLen  <= conv_std_logic_vector( 0, 9);
  SPI3_0_CalAddr         <= conv_std_logic_vector( 0, 9);
  SPI3_0_CalData         <= conv_std_logic_vector( 0, 8);
--  SPI3_0_AFThresAssert   <= conv_std_logic_vector( 32, 9);
--  SPI3_0_AFThresNegate   <= conv_std_logic_vector( 32, 9);

  -- SPI3 #1
--  SPI3_1_ParitySelect    <= '0';
  SPI3_1_CalWrEn_n       <= '1';

  SPI3_1_Cal_En_n       <= '1';
  SPI3_1_HaltTDat        <= '0';
  SPI3_1_CalWrEn_n		 <= '1';
--  SPI3_1_MaxCalendarLen  <= conv_std_logic_vector( 0, 9);
  SPI3_1_CalAddr         <= conv_std_logic_vector( 0, 9);
  SPI3_1_CalData         <= conv_std_logic_vector( 0, 8);
--  SPI3_1_AFThresAssert   <= conv_std_logic_vector( 32, 9);
--  SPI3_1_AFThresNegate   <= conv_std_logic_vector( 32, 9);

  -- SPI3 #2
--  SPI3_2_ParitySelect    <= '0';
  SPI3_2_CalWrEn_n       <= '1';

  SPI3_2_Cal_En_n       <= '1';
  SPI3_2_HaltTDat        <= '0';
  SPI3_2_CalWrEn_n		 <= '1';
--  SPI3_2_MaxCalendarLen  <= conv_std_logic_vector( 0, 9);
  SPI3_2_CalAddr         <= conv_std_logic_vector( 0, 9);
  SPI3_2_CalData         <= conv_std_logic_vector( 0, 8);
--  SPI3_2_AFThresAssert   <= conv_std_logic_vector( 32, 9);
--  SPI3_2_AFThresNegate   <= conv_std_logic_vector( 32, 9);

  -- SPI3 #3
--  SPI3_3_ParitySelect    <= '0';
  SPI3_3_CalWrEn_n       <= '1';

  SPI3_3_Cal_En_n       <= '1';
  SPI3_3_HaltTDat        <= '0';
  SPI3_3_CalWrEn_n		 <= '1';
--  SPI3_3_MaxCalendarLen  <= conv_std_logic_vector( 0, 9);
  SPI3_3_CalAddr         <= conv_std_logic_vector( 0, 9);
  SPI3_3_CalData         <= conv_std_logic_vector( 0, 8);
--  SPI3_3_AFThresAssert   <= conv_std_logic_vector( 32, 9);
--  SPI3_3_AFThresNegate   <= conv_std_logic_vector( 32, 9);
  
  --Note: Since the SPI-3 calendars default to 0 values, no initialization of the
  --calendars is needed.

  ------------------------------------------------------------------------------
  -- SPI3 TFClk
  --   Assumed that all 4 SPI3 interfaces are synchronous to SPI3_RFClk
  ------------------------------------------------------------------------------
  TFClkIBufg: IBUFG
    port map (
      I => SPI3_TFClk,
      O => SPI3_TFClk_ibufg
      );

  tfclk_dcm: DCM
  port map (
        CLKIN      =>  SPI3_TFClk_ibufg,
        CLKFB      =>  SPI3_TFClk_bufg,
        DSSEN      =>  GND,
        PSINCDEC   =>  GND,
        PSEN       =>  GND,
        PSCLK      =>  GND,
        RST        =>  SPI3_DcmReset_TFClk,
        CLK0       =>  SPI3_TFClk_dcm,
        CLK90      =>  OPEN,
        CLK180     =>  OPEN,
        CLK270     =>  OPEN,
        CLK2X      =>  OPEN,
        CLK2X180   =>  OPEN,
        CLKDV      =>  OPEN,
        CLKFX      =>  OPEN,
        CLKFX180   =>  OPEN,
        LOCKED     =>  SPI3_Locked_TFClk,
        PSDONE     =>  OPEN,
        STATUS     =>  OPEN
       ) ;

  TFClkBuf: BUFG
    port map (
      I => SPI3_TFClk_dcm,
      O => SPI3_TFClk_bufg
      );
   
   CLK200_Buf : BUFG
   port map (
      O => Refclk,     -- Clock buffer output
      I => CLK200      -- Clock buffer input
   );
   
  SPI3_TFClk_bufg_out <= SPI3_TFClk_bufg;

  ------------------------------------------------------------------------------
  -- RStat
  ------------------------------------------------------------------------------
  -- Instantiate LVTTL OBUF for the RStat and RSClk pins
  ------------------------------------------------------------------------------
  RStat_obuf0 : OBUF
    port map (
      I => SPI4_RStat_i(0),
      O => SPI4_RStat(0));

  RStat_obuf1 : OBUF
    port map (
      I => SPI4_RStat_i(1),
      O => SPI4_RStat(1));

  RSClk_obuf0 : OBUF
    port map (
      I => SPI4_RSClk_i,
      O => SPI4_RSClk);


  ------------------------------------------------------------------------------
  -- SPI3 0 Source Instantiation
  ------------------------------------------------------------------------------
  --Make sure that DCM has locked before SPI-3 cores are taken out of reset
  SPI3_Reset <= Not (Reset_n and SPI3_Locked_TFClk);
  
  SPI3_0_SRC_RDY <= NOT SPI3_0_WrEn_n;
  SPI3_1_SRC_RDY <= NOT SPI3_1_WrEn_n;
  SPI3_2_SRC_RDY <= NOT SPI3_2_WrEn_n;
  SPI3_3_SRC_RDY <= NOT SPI3_3_WrEn_n;
  SPI3_0_REM     <= NOT SPI3_0_WrMod;
  SPI3_1_REM     <= NOT SPI3_1_WrMod;
  SPI3_2_REM     <= NOT SPI3_2_WrMod;
  SPI3_3_REM     <= NOT SPI3_3_WrMod;
  
  pl3_tx_top0 : spi3_link_v4_1_spi3_link_tx
    port map (
		RST				 => SPI3_Reset,
      TFCLK           => SPI3_TFClk_bufg,
      TENB            => SPI3_0_TEnb,
      TDAT            => SPI3_0_TDat,
      TMOD            => SPI3_0_TMod,
      TPRTY           => SPI3_0_TPrty,
      TSOP            => SPI3_0_TSOP,
      TEOP            => SPI3_0_TEOP,
      TERR            => SPI3_0_TErr,
      TSX             => SPI3_0_TSx,
      TADR            => SPI3_0_TAdr,
      PTPA            => SPI3_0_PTPA,
      TX_CLK          => SPI4_Clk,
      TX_DST_RDY      => SPI3_0_dst_rdy,
      TX_SRC_RDY      => SPI3_0_SRC_RDY,
      TX_SOF          => SPI3_0_WrSOP,
      TX_EOF          => SPI3_0_WrEOP,
      TX_ADDR         => SPI3_0_WrAddr,
      TX_DATA         => SPI3_0_WrData,
      TX_REM          => SPI3_0_REM,
      TX_ERR          => SPI3_0_WrErr,
      TX_HALT_TDAT    => SPI3_0_HaltTDat,	
      TX_CAL_CLK      => SPI4_Clk,
      TX_CAL_EN       => SPI3_0_Cal_En_n,	
      TX_CAL_WR_EN    => SPI3_0_CalWrEn_n,
      TX_CAL_ADDR     => SPI3_0_CalAddr,
      TX_CAL_DATA     => SPI3_0_CalData,
      TX_CAL_DATA_OUT => SPI3_0_CalDataOut,
      TX_PTPA         => SPI3_0_TxPTPA,
      TX_TADR         => SPI3_0_TxTAdr
      );  
  ------------------------------------------------------------------------------
  -- SPI3 1 Source Instantiation
  ------------------------------------------------------------------------------
  pl3_tx_top1 : spi3_link_v4_1_spi3_link_tx
    port map (
		RST				 => SPI3_Reset,
      TFCLK           => SPI3_TFClk_bufg,
      TENB            => SPI3_1_TEnb,
      TDAT            => SPI3_1_TDat,
      TMOD            => SPI3_1_TMod,
      TPRTY           => SPI3_1_TPrty,
      TSOP            => SPI3_1_TSOP,
      TEOP            => SPI3_1_TEOP,
      TERR            => SPI3_1_TErr,
      TSX             => SPI3_1_TSx,
      TADR            => SPI3_1_TAdr,
      PTPA            => SPI3_1_PTPA,
      TX_CLK          => SPI4_Clk,
      TX_DST_RDY      => SPI3_1_dst_rdy,
      TX_SRC_RDY      => SPI3_1_SRC_RDY,
      TX_SOF          => SPI3_1_WrSOP,
      TX_EOF          => SPI3_1_WrEOP,
      TX_ADDR         => SPI3_1_WrAddr,
      TX_DATA         => SPI3_1_WrData,
      TX_REM          => SPI3_1_REM,
      TX_ERR          => SPI3_1_WrErr,
      TX_HALT_TDAT    => SPI3_1_HaltTDat,	
      TX_CAL_CLK      => SPI4_Clk,
      TX_CAL_EN       => SPI3_1_Cal_En_n,	
      TX_CAL_WR_EN    => SPI3_1_CalWrEn_n,
      TX_CAL_ADDR     => SPI3_1_CalAddr,
      TX_CAL_DATA     => SPI3_1_CalData,
      TX_CAL_DATA_OUT => SPI3_1_CalDataOut,
      TX_PTPA         => SPI3_1_TxPTPA,
      TX_TADR         => SPI3_1_TxTAdr
      );

  ------------------------------------------------------------------------------
  -- SPI3 2 Source Instantiation
  ------------------------------------------------------------------------------
  pl3_tx_top2 : spi3_link_v4_1_spi3_link_tx
    port map (
		RST				 => SPI3_Reset,
      TFCLK           => SPI3_TFClk_bufg,
      TENB            => SPI3_2_TEnb,
      TDAT            => SPI3_2_TDat,
      TMOD            => SPI3_2_TMod,
      TPRTY           => SPI3_2_TPrty,
      TSOP            => SPI3_2_TSOP,
      TEOP            => SPI3_2_TEOP,
      TERR            => SPI3_2_TErr,
      TSX             => SPI3_2_TSx,
      TADR            => SPI3_2_TAdr,
      PTPA            => SPI3_2_PTPA,
      TX_CLK          => SPI4_Clk,
      TX_DST_RDY      => SPI3_2_dst_rdy,
      TX_SRC_RDY      => SPI3_2_SRC_RDY,
      TX_SOF          => SPI3_2_WrSOP,
      TX_EOF          => SPI3_2_WrEOP,
      TX_ADDR         => SPI3_2_WrAddr,
      TX_DATA         => SPI3_2_WrData,
      TX_REM          => SPI3_2_REM,
      TX_ERR          => SPI3_2_WrErr,
      TX_HALT_TDAT    => SPI3_2_HaltTDat,	
      TX_CAL_CLK      => SPI4_Clk,
      TX_CAL_EN       => SPI3_2_Cal_En_n,	
      TX_CAL_WR_EN    => SPI3_2_CalWrEn_n,
      TX_CAL_ADDR     => SPI3_2_CalAddr,
      TX_CAL_DATA     => SPI3_2_CalData,
      TX_CAL_DATA_OUT => SPI3_2_CalDataOut,
      TX_PTPA         => SPI3_2_TxPTPA,
      TX_TADR         => SPI3_2_TxTAdr
      );

  ------------------------------------------------------------------------------
  -- SPI3 3 Source Instantiation
  ------------------------------------------------------------------------------
  pl3_tx_top3 : spi3_link_v4_1_spi3_link_tx
    port map (
		RST				 => SPI3_Reset,
      TFCLK           => SPI3_TFClk_bufg,
      TENB            => SPI3_3_TEnb,
      TDAT            => SPI3_3_TDat,
      TMOD            => SPI3_3_TMod,
      TPRTY           => SPI3_3_TPrty,
      TSOP            => SPI3_3_TSOP,
      TEOP            => SPI3_3_TEOP,
      TERR            => SPI3_3_TErr,
      TSX             => SPI3_3_TSx,
      TADR            => SPI3_3_TAdr,
      PTPA            => SPI3_3_PTPA,
      TX_CLK          => SPI4_Clk,
      TX_DST_RDY      => SPI3_3_dst_rdy,
      TX_SRC_RDY      => SPI3_3_SRC_RDY,
      TX_SOF          => SPI3_3_WrSOP,
      TX_EOF          => SPI3_3_WrEOP,
      TX_ADDR         => SPI3_3_WrAddr,
      TX_DATA         => SPI3_3_WrData,
      TX_REM          => SPI3_3_REM,
      TX_ERR          => SPI3_3_WrErr,
      TX_HALT_TDAT    => SPI3_3_HaltTDat,	
      TX_CAL_CLK      => SPI4_Clk,
      TX_CAL_EN       => SPI3_3_Cal_En_n,	
      TX_CAL_WR_EN    => SPI3_3_CalWrEn_n,
      TX_CAL_ADDR     => SPI3_3_CalAddr,
      TX_CAL_DATA     => SPI3_3_CalData,
      TX_CAL_DATA_OUT => SPI3_3_CalDataOut,
      TX_PTPA         => SPI3_3_TxPTPA,
      TX_TADR         => SPI3_3_TxTAdr
      );

  ------------------------------------------------------------------------------
  -- SPI4.2 Sink Instantiation
  ------------------------------------------------------------------------------
    
  pl4_snk_top0 : pl4_v8_1_pl4_snk_top
  port map(

    -- common signals
    Reset_n             => Reset_n,
    SnkFifoReset_n      => SPI4_FFReset_n,
    DcmReset_RDClk      => SPI4_DcmReset_RDClk,
    Locked_RDClk        => SPI4_Locked_RDClk,
--    DcmLost_RDClk       => DcmLost_RDClk,
    SnkClksRdy          => SPI4_SnkClksRdy,
    PhaseAlignRequest   => SPI4_PhaseAlignRequest,
    PhaseAlignComplete  => SPI4_PhaseAlignComplete,
--    SnkDPAFailed        => SnkDPAFailed ,
--    SnkDPARamData       => SnkDPARamData,
--    SnkDPARamAddr       => SnkDPARamAddr,
--    SnkDPARamValid      => SnkDPARamValid,
    SnkDPAFailed        => SPI4_SnkDPAFailed ,
    SnkDPARamData       => open,
    SnkDPARamAddr       => open,
    SnkDPARamValid      => open,
    SnkIdelayRefClk     => Refclk,

    -- sink general purpose clocks
    RDClkDiv_GP       => open,
    RDClk0_GP         => open,
 
    -- Sink Calendar I/O
    SnkCalendar_M     => SPI4_SnkCalendar_M,
    SnkCalendar_Len   => SPI4_SnkCalendar_Len,
    SnkCalClk         => SPI4_Clk,
--    SnkCalClk         => SPI4_Clk_div2,
    SnkCalWrEn_n      => SPI4_SnkCalWrEn_n,
    SnkCalAddr        => SPI4_SnkCalAddr,
    SnkCalData        => SPI4_SnkCalData,
    SnkCalDataOut     => SPI4_SnkCalDataOut,
--    SnkStatClk        => SPI4_Clk_div2,
	 SnkStatClk        => SPI4_Clk,
    SnkStat           => SPI4_Stat,
    SnkStatAddr       => SPI4_StatAddr,
    SnkStatWr_n       => SPI4_StatWrEn_n,
    SnkStatMask       => SPI4_StatMask,

    -- FIFO I/O
    SnkFFClk          => SPI4_Clk,
    SnkFFRdEn_n       => SPI4_FFRdEn_n,
    SnkFFAddr         => SPI4_FFAddr,
    SnkFFData         => SPI4_FFData,
    SnkFFMod          => SPI4_FFMod,
    SnkFFSOP          => SPI4_FFSOP,
    SnkFFEOP          => SPI4_FFEOP,
    SnkFFErr          => SPI4_FFErr,
    SnkFFBurstErr     => SPI4_FFBurstErr,
    SnkFFDIP4Err      => SPI4_FFDIP4Err,
    SnkFFPayloadDIP4  => SPI4_FFPayloadDIP4,
    SnkFFPayloadErr   => SPI4_FFPayloadErr,
    SnkFFValid        => SPI4_FFValid,
    SnkFFEmpty_n      => SPI4_FFEmpty_n,
    SnkFFAlmostEmpty_n => SPI4_FFAlmostEmpty_n,
    SnkAlmostFull_n   => SPI4_SnkAlmostFull_n,
    SnkOverflow_n     => SPI4_SnkOverflow_n,

    -- Control and status signals
    SnkEn             => SPI4_SnkEn,
    SnkOof            => SPI4_SnkOof,
    SnkBusErr         => SPI4_SnkBusErr,
--    SnkBusErrStat     => SnkBusErrStat,
	 SnkBusErrStat     => open,
    SnkTrainValid     => SPI4_SnkTrainValid,
--    SnkDIP2ErrRequest => SnkDIP2ErrRequest,
	 SnkDIP2ErrRequest => GND,
    
    -- FIFO status interface
    RSClk             => SPI4_RSClk_i,
    RStat             => SPI4_RStat_i,

    -- PL4 sink bus
    RDClk_P           => SPI4_RDClk_P,
    RDClk_N           => SPI4_RDClk_N,
    RDat_P            => SPI4_RDat_P,
    RDat_N            => SPI4_RDat_N,
    RCtl_P            => SPI4_RCtl_P,
    RCtl_N            => SPI4_RCtl_N,

    -- Configuration signals
    SnkAFThresAssert  => SPI4_SnkAFThresAssert,
    SnkAFThresNegate  => SPI4_SnkAFThresNegate,
    FifoAFMode        => SPI4_FifoAFMode,
    NumDip4Errors     => SPI4_NumDip4Errors,
    NumTrainSequences => SPI4_NumTrainSequences,
    RSClkPhase        => SPI4_RSClkPhase,
    RSClkDiv          => SPI4_RSClkDiv);


    -- SPI4.2 to SPI3 Core Logic
  spi4_to_spi3_core0: spi4_to_spi3_core
    port map (
      -- System signals
      Reset_n              => Reset_n,
      SPI4_Clk             => SPI4_Clk,
  
      -- SPI4 Interface Signals
      SPI4_FFAddr          => SPI4_FFAddr,
      SPI4_FFData          => SPI4_FFData,
      SPI4_FFMod           => SPI4_FFMod,
      SPI4_FFSOP           => SPI4_FFSOP,
      SPI4_FFEOP           => SPI4_FFEOP,
      SPI4_FFErr           => SPI4_FFErr,
      SPI4_FFValid         => SPI4_FFValid,
      SPI4_FFPayloadErr    => SPI4_FFPayloadErr,
      SPI4_FFDIP4Err       => SPI4_FFDIP4Err,
		SPI4_FFBurstErr      => SPI4_FFBurstErr,
		SPI4_FFPayloadDIP4	=> SPI4_FFPayloadDIP4,
      SPI4_FFEmpty_n       => SPI4_FFEmpty_n,
      SPI4_FFAlmostEmpty_n => SPI4_FFAlmostEmpty_n,
      SPI4_FFRdEn_n        => SPI4_FFRdEn_n,
  
      -- SPI3 0 Interface Signals
      SPI3_0_WrEn_n        => SPI3_0_WrEn_n,
      SPI3_0_WrAddr        => SPI3_0_WrAddr,
      SPI3_0_WrData        => SPI3_0_WrData,
      SPI3_0_WrMod         => SPI3_0_WrMod,
      SPI3_0_WrSOP         => SPI3_0_WrSOP,
      SPI3_0_WrEOP         => SPI3_0_WrEOP,
      SPI3_0_WrErr         => SPI3_0_WrErr,
      SPI3_0_TxPTPA        => SPI3_0_TxPTPA,
      SPI3_0_TxTAdr        => SPI3_0_TxTAdr,

  
      -- SPI3 1 Interface Signals
      SPI3_1_WrEn_n        => SPI3_1_WrEn_n,
      SPI3_1_WrAddr        => SPI3_1_WrAddr,
      SPI3_1_WrData        => SPI3_1_WrData,
      SPI3_1_WrMod         => SPI3_1_WrMod,
      SPI3_1_WrSOP         => SPI3_1_WrSOP,
      SPI3_1_WrEOP         => SPI3_1_WrEOP,
      SPI3_1_WrErr         => SPI3_1_WrErr,
      SPI3_1_TxPTPA        => SPI3_1_TxPTPA,
      SPI3_1_TxTAdr        => SPI3_1_TxTAdr,
  
      -- SPI3 2 Interface Signals
      SPI3_2_WrEn_n        => SPI3_2_WrEn_n,
      SPI3_2_WrAddr        => SPI3_2_WrAddr,
      SPI3_2_WrData        => SPI3_2_WrData,
      SPI3_2_WrMod         => SPI3_2_WrMod,
      SPI3_2_WrSOP         => SPI3_2_WrSOP,
      SPI3_2_WrEOP         => SPI3_2_WrEOP,
      SPI3_2_WrErr         => SPI3_2_WrErr,
      SPI3_2_TxPTPA        => SPI3_2_TxPTPA,
      SPI3_2_TxTAdr        => SPI3_2_TxTAdr,
      
      -- SPI3 3 Interface Signals
      SPI3_3_WrEn_n        => SPI3_3_WrEn_n,
      SPI3_3_WrAddr        => SPI3_3_WrAddr,
      SPI3_3_WrData        => SPI3_3_WrData,
      SPI3_3_WrMod         => SPI3_3_WrMod,
      SPI3_3_WrSOP         => SPI3_3_WrSOP,
      SPI3_3_WrEOP         => SPI3_3_WrEOP,
      SPI3_3_WrErr         => SPI3_3_WrErr,
      SPI3_3_TxPTPA        => SPI3_3_TxPTPA,
      SPI3_3_TxTAdr        => SPI3_3_TxTAdr,
  
  
      -- SPI4.2 Status Interface
      SPI4_Stat            => SPI4_Stat_i,
      SPI4_StatWrEn_n      => SPI4_StatWrEn_n,
  
      -- SPI3 Flow Control
      SPI3_0_dst_rdy  => SPI3_0_dst_rdy,
      SPI3_1_dst_rdy  => SPI3_1_dst_rdy,
      SPI3_2_dst_rdy  => SPI3_2_dst_rdy,
      SPI3_3_dst_rdy  => SPI3_3_dst_rdy
      );
      
end xilinx;
