-------------------------------------------------------------------------------
--
--   File Name:  spi4_to_spi3_write.vhd
--     Version:  1.0
--        Date:  June 11, 2007
--
-- Description:  The purpose of the spi4_to_spi3_write.vhd file is to take the 
--               data from the SPI-4 read logic and write into one of four SPI3
--               Cores.
--
--      Author:  Zhe Xia
--     Company:  Xilinx Inc.
--
--
--     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
--     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
--     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
--     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
--     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
--     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
--     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
--     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
--     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
--     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
--     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
--     FOR A PARTICULAR PURPOSE.
--
--     (c) Copyright 2003 Xilinx, Inc.
--     All rights reserved.
--
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Library Declarations
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

--------------------------------------------------------------------------------
-- Entity Declaration
--------------------------------------------------------------------------------

entity spi4_to_spi3_write is
  port (
    -- System signals
    Reset_n               : in  std_logic;
    SPI4_Clk              : in  std_logic;

    -- Signals to/from Burst Storage FIFO
    SPI3_0_BrstRdEn      : out std_logic;
    SPI3_0_BrstData      : in  std_logic_vector(69 downto 0);
    SPI3_1_BrstRdEn      : out std_logic;
    SPI3_1_BrstData      : in  std_logic_vector(69 downto 0);
    SPI3_2_BrstRdEn      : out std_logic;
    SPI3_2_BrstData      : in  std_logic_vector(69 downto 0);
    SPI3_3_BrstRdEn      : out std_logic;
    SPI3_3_BrstData      : in  std_logic_vector(69 downto 0);
    SPI3_0_Brst_Empty    : in  std_logic;
    SPI3_1_Brst_Empty    : in  std_logic;
    SPI3_2_Brst_Empty    : in  std_logic;
    SPI3_3_Brst_Empty    : in  std_logic;
    
    -- SPI3 Status Signals
    SPI3_0_dst_rdy 	 	 : in  std_logic;
    SPI3_1_dst_rdy 		 : in  std_logic;
    SPI3_2_dst_rdy 		 : in  std_logic;
    SPI3_3_dst_rdy 		 : in  std_logic;

    -- SPI3 0 Write Logic Interface
    SPI3_0_WrEn_n       : out std_logic;
    SPI3_0_WrData       : out std_logic_vector(31 downto 0);
    SPI3_0_WrMod        : out std_logic_vector(1 downto 0);
    SPI3_0_WrSOP        : out std_logic;
    SPI3_0_WrEOP        : out std_logic;
    SPI3_0_WrErr        : out std_logic;
    SPI3_0_TxPTPA       : in  std_logic;
    SPI3_0_TxTAdr       : in  std_logic_vector(7 downto 0);

    -- SPI3 1 Write Logic Interface
    SPI3_1_WrEn_n       : out std_logic;
    SPI3_1_WrData       : out std_logic_vector(31 downto 0);
    SPI3_1_WrMod        : out std_logic_vector(1 downto 0);
    SPI3_1_WrSOP        : out std_logic;
    SPI3_1_WrEOP        : out std_logic;
    SPI3_1_WrErr        : out std_logic;
    SPI3_1_TxPTPA       : in  std_logic;
    SPI3_1_TxTAdr       : in  std_logic_vector(7 downto 0);

    -- SPI3 2 Write Logic Interface
    SPI3_2_WrEn_n       : out std_logic;
    SPI3_2_WrData       : out std_logic_vector(31 downto 0);
    SPI3_2_WrMod        : out std_logic_vector(1 downto 0);
    SPI3_2_WrSOP        : out std_logic;
    SPI3_2_WrEOP        : out std_logic;
    SPI3_2_WrErr        : out std_logic;
    SPI3_2_TxPTPA       : in  std_logic;
    SPI3_2_TxTAdr       : in  std_logic_vector(7 downto 0);

    -- SPI3 3 Write Logic Interface
    SPI3_3_WrEn_n       : out std_logic;
    SPI3_3_WrData       : out std_logic_vector(31 downto 0);
    SPI3_3_WrMod        : out std_logic_vector(1 downto 0);
    SPI3_3_WrSOP        : out std_logic;
    SPI3_3_WrEOP        : out std_logic;
    SPI3_3_WrErr        : out std_logic;
    SPI3_3_TxPTPA       : in  std_logic;
    SPI3_3_TxTAdr       : in  std_logic_vector(7 downto 0)
    );
end spi4_to_spi3_write;

--------------------------------------------------------------------------------
-- Definition of Ports:
--------------------------------------------------------------------------------
--    -- System signals
--    Reset_n               : Active-Low system reset
--    SPI4_Clk              : Clock that is used for the bridge, derived from a
--                            SPI4 core clock
--
--    -- Signals to/from Burst Storage FIFO
--    SPI3_0_BrstRdEn      : Read Enable for Burst FIFO, channel 0
--    SPI3_0_BrstData      : Data received from Burst FIFO, channel 0
--    SPI3_1_BrstRdEn      : Read Enable for Burst FIFO, channel 1
--    SPI3_1_BrstData      : Data received from Burst FIFO, channel 1
--    SPI3_2_BrstRdEn      : Read Enable for Burst FIFO, channel 2
--    SPI3_2_BrstData      : Data received from Burst FIFO, channel 2
--    SPI3_3_BrstRdEn      : Read Enable for Burst FIFO, channel 3
--    SPI3_3_BrstData      : Data received from Burst FIFO, channel 3
--    SPI3_0_Brst_Empty    : Empty indication from Burst FIFO, channel 0
--    SPI3_1_Brst_Empty    : Empty indication from Burst FIFO, channel 1
--    SPI3_2_Brst_Empty    : Empty indication from Burst FIFO, channel 2
--    SPI3_3_Brst_Empty    : Empty indication from Burst FIFO, channel 3
--    
--    -- SPI3 Status Signals
--    SPI3_0_dst_rdy       : SPI3 0 is available (active high)
--    SPI3_1_dst_rdy       : SPI3 1 is available (active high) 
--    SPI3_2_dst_rdy       : SPI3 2 is available (active high) 
--    SPI3_3_dst_rdy       : SPI3 3 is available (active high)
--
--    -- SPI3 0 Write Logic Interface
--    SPI3_0_WrEn_n       : Write Enable signal (active-low) for SPI3 transmit
--                          core 0
--    SPI3_0_WrData       : Data to be written into SPI3 transmit core 0
--    SPI3_0_WrMod        : Mod signal to be written into SPI3 transmit core 0
--    SPI3_0_WrSOP        : SOP signal to be written into SPI3 transmit core 0
--    SPI3_0_WrEOP        : EOP signal to be written into SPI3 transmit core 0
--    SPI3_0_WrErr        : Err signal to be written into SPI3 transmit core 0
--    SPI3_0_TxPTPA       : Polled status from device connected to SPI3 transmit
--                          core 0
--    SPI3_0_TxTAdr       : Polled status address from device connected to SPI3
--                          transmit core 0
--
--    -- SPI3 1 Write Logic Interface
--    SPI3_1_WrEn_n       : Write Enable signal (active-low) for SPI3 transmit
--                          core 1          
--    SPI3_1_WrData       : Data to be written into SPI3 transmit core 1
--    SPI3_1_WrMod        : Mod signal to be written into SPI3 transmit core 1                 
--    SPI3_1_WrSOP        : SOP signal to be written into SPI3 transmit core 1                 
--    SPI3_1_WrEOP        : EOP signal to be written into SPI3 transmit core 1                 
--    SPI3_1_WrErr        : Err signal to be written into SPI3 transmit core 1                 
--    SPI3_1_TxPTPA       : Polled status from device connected to SPI3 transmit
--                          core 1        
--    SPI3_1_TxTAdr       : Polled status address from device connected to SPI3
--                          transmit core 1
--
--    -- SPI3 2 Write Logic Interface
--    SPI3_2_WrEn_n       : Write Enable signal (active-low) for SPI3 transmit
--                          core 2          
--    SPI3_2_WrData       : Data to be written into SPI3 transmit core 2                       
--    SPI3_2_WrMod        : Mod signal to be written into SPI3 transmit core 2                 
--    SPI3_2_WrSOP        : SOP signal to be written into SPI3 transmit core 2                 
--    SPI3_2_WrEOP        : EOP signal to be written into SPI3 transmit core 2                 
--    SPI3_2_WrErr        : Err signal to be written into SPI3 transmit core 2                 
--    SPI3_2_TxPTPA       : Polled status from device connected to SPI3 transmit
--                          core 2        
--    SPI3_2_TxTAdr       : Polled status address from device connected to SPI3
--                          transmit core 2
--
--    -- SPI3 3 Write Logic Interface
--    SPI3_3_WrEn_n       : Write Enable signal (active-low) for SPI3 transmit
--                          core 3          
--    SPI3_3_WrData       : Data to be written into SPI3 transmit core 3                       
--    SPI3_3_WrMod        : Mod signal to be written into SPI3 transmit core 3                 
--    SPI3_3_WrSOP        : SOP signal to be written into SPI3 transmit core 3                 
--    SPI3_3_WrEOP        : EOP signal to be written into SPI3 transmit core 3                 
--    SPI3_3_WrErr        : Err signal to be written into SPI3 transmit core 3                 
--    SPI3_3_TxPTPA       : Polled status from device connected to SPI3 transmit
--                          core 3        
--    SPI3_3_TxTAdr       : Polled status address from device connected to SPI3
--                          transmit core 3
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
-- spi4_to_spi3_write Architecture
--------------------------------------------------------------------------------
architecture xilinx of spi4_to_spi3_write is

  ------------------------------------------------------------------------------
  -- Constant Declarations
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  -- Signal Declarations
  ------------------------------------------------------------------------------
  signal SPI3_0_BrstRdEn_int    : std_logic;
  signal SPI3_0_BrstRdEn_d1     : std_logic;
  signal wait_for_conversion0   : std_logic_vector(1 downto 0);
  signal pl3_0_conversion_state : std_logic_vector(1 downto 0);
  signal Holding_Data0          : std_logic_vector(31 downto 0);
  --signal Holding_SOP0           : std_logic;
  signal Holding_EOP0           : std_logic;
  signal Holding_Mod0           : std_logic_vector(1 downto 0);
  signal Holding_Err0           : std_logic;
  signal SPI3_0_WrEOP_int				: std_logic;
--  signal SPI3_0_WrErr_int  			: std_logic;
--  signal SPI3_0_WrMod_int				: std_logic;
  
  signal SPI3_1_BrstRdEn_int    : std_logic;
  signal SPI3_1_BrstRdEn_d1     : std_logic;
  signal wait_for_conversion1   : std_logic_vector(1 downto 0);
  signal pl3_1_conversion_state : std_logic_vector(1 downto 0);
  signal Holding_Data1          : std_logic_vector(31 downto 0);
  --signal Holding_SOP1           : std_logic;
  signal Holding_EOP1           : std_logic;
  signal Holding_Mod1           : std_logic_vector(1 downto 0);
  signal Holding_Err1           : std_logic;
  signal SPI3_1_WrEOP_int				: std_logic;
--  signal SPI3_1_WrErr_int  			: std_logic;
--  signal SPI3_1_WrMod_int				: std_logic;
  
  signal SPI3_2_BrstRdEn_int    : std_logic;
  signal SPI3_2_BrstRdEn_d1     : std_logic;
  signal wait_for_conversion2   : std_logic_vector(1 downto 0);
  signal pl3_2_conversion_state : std_logic_vector(1 downto 0);
  signal Holding_Data2          : std_logic_vector(31 downto 0);
  --signal Holding_SOP2           : std_logic;
  signal Holding_EOP2           : std_logic;
  signal Holding_Mod2           : std_logic_vector(1 downto 0);
  signal Holding_Err2           : std_logic;
  signal SPI3_2_WrEOP_int				: std_logic;
--  signal SPI3_2_WrErr_int  			: std_logic;
--  signal SPI3_2_WrMod_int				: std_logic;
  
  signal SPI3_3_BrstRdEn_int    : std_logic;
  signal SPI3_3_BrstRdEn_d1     : std_logic;
  signal wait_for_conversion3   : std_logic_vector(1 downto 0);
  signal pl3_3_conversion_state : std_logic_vector(1 downto 0);
  signal Holding_Data3          : std_logic_vector(31 downto 0);
  --signal Holding_SOP3           : std_logic;
  signal Holding_EOP3           : std_logic;
  signal Holding_Mod3           : std_logic_vector(1 downto 0);
  signal Holding_Err3           : std_logic;
  signal SPI3_0_polled_status   : std_logic;
  signal SPI3_1_polled_status   : std_logic;
  signal SPI3_2_polled_status   : std_logic;
  signal SPI3_3_polled_status   : std_logic;
  signal SPI3_3_WrEOP_int				: std_logic;
--  signal SPI3_3_WrErr_int  			: std_logic;
--  signal SPI3_3_WrMod_int				: std_logic;

begin

  --Keep track of the status coming from each core
  status_tracking_proc: process (Reset_n, SPI4_Clk)
  begin
    if (Reset_n='0') then
      SPI3_0_polled_status <= '0';
      SPI3_1_polled_status <= '0';
      SPI3_2_polled_status <= '0';
      SPI3_3_polled_status <= '0';
    elsif (SPI4_Clk'event and SPI4_Clk='1') then
      if SPI3_0_TxTAdr="00000000" then
        SPI3_0_polled_status <= SPI3_0_TxPTPA;
      end if;
      if SPI3_1_TxTAdr="00000000" then
        SPI3_1_polled_status <= SPI3_1_TxPTPA;
      end if;
      if SPI3_2_TxTAdr="00000000" then
        SPI3_2_polled_status <= SPI3_2_TxPTPA;
      end if;
      if SPI3_3_TxTAdr="00000000" then
        SPI3_3_polled_status <= SPI3_3_TxPTPA;
      end if;
    end if;
  end process;

  ------------------------------------------------------------------------------
  -- Write SPI-3 core 0 Process
  --
  -- Note: the purpose of this process is to write data into the SPI-3 source
  -- core, instantiation 0
  ------------------------------------------------------------------------------
  --If the PL3 core isn't full and the burst FIFO isn't empty
  --then read data
         write_pl3_core_0_proc: process (Reset_n, SPI4_Clk)
  begin
    if (Reset_n='0') then
      SPI3_0_BrstRdEn_int <= '0';
      SPI3_0_BrstRdEn_d1 <= '0';
      wait_for_conversion0 <= "00";
    elsif (SPI4_Clk'event and SPI4_Clk='1') then
      SPI3_0_BrstRdEn_d1 <= SPI3_0_BrstRdEn_int;
      if (SPI3_0_dst_rdy='1' and SPI3_0_Brst_Empty='0' and
          wait_for_conversion0= "00" and SPI3_0_polled_status='1') then
        SPI3_0_BrstRdEn_int <= '1';
        wait_for_conversion0 <= "01";
      elsif (SPI3_0_dst_rdy='1' and wait_for_conversion0 <= "01") then
      	SPI3_0_BrstRdEn_int <= '0';
        wait_for_conversion0 <= "10";
      elsif (SPI3_0_dst_rdy='1' and wait_for_conversion0 <= "10") then
        SPI3_0_BrstRdEn_int <= '0';
        wait_for_conversion0 <= "00";
       else
       SPI3_0_BrstRdEn_int <= '0';
       wait_for_conversion0 <= wait_for_conversion0;
      end if;
    end if;
  end process;
  
  pl3_0_data_conversion: process (Reset_n, SPI4_Clk)
  begin
    if (Reset_n='0') then
      pl3_0_conversion_state <= "00";
      SPI3_0_WrData <= (others => '0');
      SPI3_0_WrEn_n <= '1';
      SPI3_0_WrSOP <= '0';
      SPI3_0_WrEOP_int <= '0';
      SPI3_0_WrMod <= "00";
      SPI3_0_WrErr <= '0';
      Holding_Data0 <= (others => '0');
      Holding_EOP0 <= '0';
      Holding_Mod0 <= "00";
      Holding_Err0 <= '0';
    elsif (SPI4_Clk'event and SPI4_Clk='1') then
      case pl3_0_conversion_state is
        when "00" =>
          if (SPI3_0_BrstRdEn_d1='1') then
          	if (SPI3_0_dst_rdy = '1') then
            	pl3_0_conversion_state <= "01";
            	SPI3_0_WrData <= SPI3_0_BrstData(69 downto 38);
            	SPI3_0_WrEn_n <= '0';
            	SPI3_0_WrSOP  <= SPI3_0_BrstData(5);
            	Holding_Data0 <= SPI3_0_BrstData(37 downto 6);
         
           		--Look at Mod stuff here
            	--If there is an EOP, Mod must be looked at
           	 	if (SPI3_0_BrstData(4)='1') then
              case SPI3_0_BrstData(3 downto 1) is
                --"000" means data bits 63-0 have valid data
                when "000" => 
                  SPI3_0_WrEOP_int <= '0';
                  SPI3_0_WrErr <= '0';
                  SPI3_0_WrMod <= "00";
                  Holding_EOP0 <= '1';
                  Holding_Mod0 <= "00";
                  Holding_Err0 <= SPI3_0_BrstData(0);
                  pl3_0_conversion_state <= "01";
                --"111" means data bits 63-8 have valid data
                when "111" =>
                  SPI3_0_WrEOP_int <= '0';
                  SPI3_0_WrErr <= '0';
                  SPI3_0_WrMod <= "00";
                  Holding_EOP0 <= '1';
                  Holding_Mod0 <= "01";
                  Holding_Err0 <= SPI3_0_BrstData(0);
                  pl3_0_conversion_state <= "01";
                --"110" means data bits 63-16 have valid data
                when "110" =>
                  SPI3_0_WrEOP_int <= '0';
                  SPI3_0_WrErr <= '0';
                  SPI3_0_WrMod <= "00";
                  Holding_EOP0 <= '1';
                  Holding_Mod0 <= "10";
                  Holding_Err0 <= SPI3_0_BrstData(0);
                  pl3_0_conversion_state <= "01";
                --"101" means data bits 63-24 have valid data
                when "101" =>
                  SPI3_0_WrEOP_int <= '0';
                  SPI3_0_WrErr <= '0';
                  SPI3_0_WrMod <= "00";
                  Holding_EOP0 <= '1';
                  Holding_Mod0 <= "11";
                  Holding_Err0 <= SPI3_0_BrstData(0);
                  pl3_0_conversion_state <= "01";
                --"100" means data bits 63-32 have valid data
                when "100" =>
                  SPI3_0_WrEOP_int <= '1';
                  SPI3_0_WrErr <= SPI3_0_BrstData(0);
                  SPI3_0_WrMod <= "00";
                  Holding_EOP0 <= '0';
                  Holding_Mod0 <= "00";
                  Holding_Err0 <= '0';
                  pl3_0_conversion_state <= "10";
                --"011" means data bits 63-40 have valid data
                when "011" =>
                  SPI3_0_WrEOP_int <= '1';
                  SPI3_0_WrErr <= SPI3_0_BrstData(0);
                  SPI3_0_WrMod <= "01";
                  Holding_EOP0 <= '0';
                  Holding_Mod0 <= "00";
                  Holding_Err0 <= '0';
                  pl3_0_conversion_state <= "10";
                --"010" means data bits 63-48 have valid data
                when "010" =>
                  SPI3_0_WrEOP_int <= '1';
                  SPI3_0_WrErr <= SPI3_0_BrstData(0);
                  SPI3_0_WrMod <= "10";
                  Holding_EOP0 <= '0';
                  Holding_Mod0 <= "00";
                  Holding_Err0 <= '0';
                  pl3_0_conversion_state <= "10";
                --"001" means data bits 63-56 have valid data
                when "001" =>
                  SPI3_0_WrEOP_int <= '1';
                  SPI3_0_WrErr <= SPI3_0_BrstData(0);
                  SPI3_0_WrMod <= "11";
                  Holding_EOP0 <= '0';
                  Holding_Mod0 <= "00";
                  Holding_Err0 <= '0';
                  pl3_0_conversion_state <= "10";
                --We have covered all the cases except X's & U's, so leave null
                when others =>
                  NULL;
              end case;
              
            else
              SPI3_0_WrEOP_int <= '0';
              SPI3_0_WrErr <= '0';
              SPI3_0_WrMod <= "00";
              Holding_EOP0 <= '0';
              Holding_Mod0 <= "00";
              Holding_Err0 <= '0';
              if (SPI3_0_dst_rdy='1') then
                  pl3_0_conversion_state <= "01";
                  else
                  pl3_0_conversion_state <= "00";
                  end if;
            end if;
            else
              	pl3_0_conversion_state <= "11";
              end if;
            
          else
            pl3_0_conversion_state <= "00";
            SPI3_0_WrEn_n <= '1';
          end if;
        when "01" =>
        	 if (SPI3_0_dst_rdy = '1') then
          pl3_0_conversion_state <= "00";
          SPI3_0_WrData <= Holding_Data0;
          SPI3_0_WrEn_n <= '0';
          SPI3_0_WrSOP  <= '0';
          SPI3_0_WrEOP_int  <= Holding_EOP0;
          SPI3_0_WrMod  <= Holding_Mod0;
          SPI3_0_WrErr  <= Holding_Err0;
          end if;
          
        when "10" =>
        	if (SPI3_0_dst_rdy = '1') then
          pl3_0_conversion_state <= "00";
          SPI3_0_WrEn_n <= '1';
          end if;
        when "11" =>      
          if (SPI3_0_dst_rdy = '1') then
            	pl3_0_conversion_state <= "01";
            	SPI3_0_WrData <= SPI3_0_BrstData(69 downto 38);
            	SPI3_0_WrEn_n <= '0';
            	SPI3_0_WrSOP  <= SPI3_0_BrstData(5);
            	Holding_Data0 <= SPI3_0_BrstData(37 downto 6);
         
           		--Look at Mod stuff here
            	--If there is an EOP, Mod must be looked at
           	 	if (SPI3_0_BrstData(4)='1') then
              case SPI3_0_BrstData(3 downto 1) is
                --"000" means data bits 63-0 have valid data
                when "000" => 
                  SPI3_0_WrEOP_int <= '0';
                  SPI3_0_WrErr <= '0';
                  SPI3_0_WrMod <= "00";
                  Holding_EOP0 <= '1';
                  Holding_Mod0 <= "00";
                  Holding_Err0 <= SPI3_0_BrstData(0);
                  pl3_0_conversion_state <= "01";
                --"111" means data bits 63-8 have valid data
                when "111" =>
                  SPI3_0_WrEOP_int <= '0';
                  SPI3_0_WrErr <= '0';
                  SPI3_0_WrMod <= "00";
                  Holding_EOP0 <= '1';
                  Holding_Mod0 <= "01";
                  Holding_Err0 <= SPI3_0_BrstData(0);
                  pl3_0_conversion_state <= "01";
                --"110" means data bits 63-16 have valid data
                when "110" =>
                  SPI3_0_WrEOP_int <= '0';
                  SPI3_0_WrErr <= '0';
                  SPI3_0_WrMod <= "00";
                  Holding_EOP0 <= '1';
                  Holding_Mod0 <= "10";
                  Holding_Err0 <= SPI3_0_BrstData(0);
                  pl3_0_conversion_state <= "01";
                --"101" means data bits 63-24 have valid data
                when "101" =>
                  SPI3_0_WrEOP_int <= '0';
                  SPI3_0_WrErr <= '0';
                  SPI3_0_WrMod <= "00";
                  Holding_EOP0 <= '1';
                  Holding_Mod0 <= "11";
                  Holding_Err0 <= SPI3_0_BrstData(0);
                  pl3_0_conversion_state <= "01";
                --"100" means data bits 63-32 have valid data
                when "100" =>
                  SPI3_0_WrEOP_int <= '1';
                  SPI3_0_WrErr <= SPI3_0_BrstData(0);
                  SPI3_0_WrMod <= "00";
                  Holding_EOP0 <= '0';
                  Holding_Mod0 <= "00";
                  Holding_Err0 <= '0';
                  pl3_0_conversion_state <= "10";
                --"011" means data bits 63-40 have valid data
                when "011" =>
                  SPI3_0_WrEOP_int <= '1';
                  SPI3_0_WrErr <= SPI3_0_BrstData(0);
                  SPI3_0_WrMod <= "01";
                  Holding_EOP0 <= '0';
                  Holding_Mod0 <= "00";
                  Holding_Err0 <= '0';
                  pl3_0_conversion_state <= "10";
                --"010" means data bits 63-48 have valid data
                when "010" =>
                  SPI3_0_WrEOP_int <= '1';
                  SPI3_0_WrErr <= SPI3_0_BrstData(0);
                  SPI3_0_WrMod <= "10";
                  Holding_EOP0 <= '0';
                  Holding_Mod0 <= "00";
                  Holding_Err0 <= '0';
                  pl3_0_conversion_state <= "10";
                --"001" means data bits 63-56 have valid data
                when "001" =>
                  SPI3_0_WrEOP_int <= '1';
                  SPI3_0_WrErr <= SPI3_0_BrstData(0);
                  SPI3_0_WrMod <= "11";
                  Holding_EOP0 <= '0';
                  Holding_Mod0 <= "00";
                  Holding_Err0 <= '0';
                  pl3_0_conversion_state <= "10";
                --We have covered all the cases except X's & U's, so leave null
                when others =>
                  NULL;
              end case;
              end if;
              else
              	pl3_0_conversion_state <= "11";
              
           end if;
        when others =>
          pl3_0_conversion_state <= "00";
          SPI3_0_WrEn_n <= '1';
      end case;
    end if;
  end process;
  
  SPI3_0_BrstRdEn <= SPI3_0_BrstRdEn_int;
  SPI3_0_WrEOP  <= SPI3_0_WrEOP_int;

  
  ------------------------------------------------------------------------------
  -- Write SPI-3 core 1 Process
  --
  -- Note: the purpose of this process is to write data into the SPI-3 source
  -- core, instantiation 1
  -- Note: this is a repeat of the above processes
  ------------------------------------------------------------------------------
  
           write_pl3_core_1_proc: process (Reset_n, SPI4_Clk)
  begin
    if (Reset_n='0') then
      SPI3_1_BrstRdEn_int <= '0';
      SPI3_1_BrstRdEn_d1 <= '0';
      wait_for_conversion1 <= "00";
    elsif (SPI4_Clk'event and SPI4_Clk='1') then
      SPI3_1_BrstRdEn_d1 <= SPI3_1_BrstRdEn_int;
      if (SPI3_1_dst_rdy='1' and SPI3_1_Brst_Empty='0' and
          wait_for_conversion1= "00" and SPI3_1_polled_status='1') then
        SPI3_1_BrstRdEn_int <= '1';
        wait_for_conversion1 <= "01";
      elsif (SPI3_1_dst_rdy='1' and wait_for_conversion1 <= "01") then
      	SPI3_1_BrstRdEn_int <= '0';
        wait_for_conversion1 <= "10";
      elsif (SPI3_1_dst_rdy='1' and wait_for_conversion1 <= "10") then
        SPI3_1_BrstRdEn_int <= '0';
        wait_for_conversion1 <= "00";
       else
       SPI3_1_BrstRdEn_int <= '0';
       wait_for_conversion1 <= wait_for_conversion1;
      end if;
    end if;
  end process;
  
  pl3_1_data_conversion: process (Reset_n, SPI4_Clk)
  begin
    if (Reset_n='0') then
      pl3_1_conversion_state <= "00";
      SPI3_1_WrData <= (others => '0');
      SPI3_1_WrEn_n <= '1';
      SPI3_1_WrSOP <= '0';
      SPI3_1_WrEOP_int <= '0';
      SPI3_1_WrMod <= "00";
      SPI3_1_WrErr <= '0';
      Holding_Data1 <= (others => '0');
      Holding_EOP1 <= '0';
      Holding_Mod1 <= "00";
      Holding_Err1 <= '0';
    elsif (SPI4_Clk'event and SPI4_Clk='1') then
      case pl3_1_conversion_state is
        when "00" =>
          if (SPI3_1_BrstRdEn_d1='1') then
          	if (SPI3_1_dst_rdy = '1') then
            	pl3_1_conversion_state <= "01";
            	SPI3_1_WrData <= SPI3_1_BrstData(69 downto 38);
            	SPI3_1_WrEn_n <= '0';
            	SPI3_1_WrSOP  <= SPI3_1_BrstData(5);
            	Holding_Data1 <= SPI3_1_BrstData(37 downto 6);
         
           		--Look at Mod stuff here
            	--If there is an EOP, Mod must be looked at
           	 	if (SPI3_1_BrstData(4)='1') then
              case SPI3_1_BrstData(3 downto 1) is
                --"000" means data bits 63-0 have valid data
                when "000" => 
                  SPI3_1_WrEOP_int <= '0';
                  SPI3_1_WrErr <= '0';
                  SPI3_1_WrMod <= "00";
                  Holding_EOP1 <= '1';
                  Holding_Mod1 <= "00";
                  Holding_Err1 <= SPI3_1_BrstData(0);
                  pl3_1_conversion_state <= "01";
                --"111" means data bits 63-8 have valid data
                when "111" =>
                  SPI3_1_WrEOP_int <= '0';
                  SPI3_1_WrErr <= '0';
                  SPI3_1_WrMod <= "00";
                  Holding_EOP1 <= '1';
                  Holding_Mod1 <= "01";
                  Holding_Err1 <= SPI3_1_BrstData(0);
                  pl3_1_conversion_state <= "01";
                --"110" means data bits 63-16 have valid data
                when "110" =>
                  SPI3_1_WrEOP_int <= '0';
                  SPI3_1_WrErr <= '0';
                  SPI3_1_WrMod <= "00";
                  Holding_EOP1 <= '1';
                  Holding_Mod1 <= "10";
                  Holding_Err1 <= SPI3_1_BrstData(0);
                  pl3_1_conversion_state <= "01";
                --"101" means data bits 63-24 have valid data
                when "101" =>
                  SPI3_1_WrEOP_int <= '0';
                  SPI3_1_WrErr <= '0';
                  SPI3_1_WrMod <= "00";
                  Holding_EOP1 <= '1';
                  Holding_Mod1 <= "11";
                  Holding_Err1 <= SPI3_1_BrstData(0);
                  pl3_1_conversion_state <= "01";
                --"100" means data bits 63-32 have valid data
                when "100" =>
                  SPI3_1_WrEOP_int <= '1';
                  SPI3_1_WrErr <= SPI3_1_BrstData(0);
                  SPI3_1_WrMod <= "00";
                  Holding_EOP1 <= '0';
                  Holding_Mod1 <= "00";
                  Holding_Err1 <= '0';
                  pl3_1_conversion_state <= "10";
                --"011" means data bits 63-40 have valid data
                when "011" =>
                  SPI3_1_WrEOP_int <= '1';
                  SPI3_1_WrErr <= SPI3_1_BrstData(0);
                  SPI3_1_WrMod <= "01";
                  Holding_EOP1 <= '0';
                  Holding_Mod1 <= "00";
                  Holding_Err1 <= '0';
                  pl3_1_conversion_state <= "10";
                --"010" means data bits 63-48 have valid data
                when "010" =>
                  SPI3_1_WrEOP_int <= '1';
                  SPI3_1_WrErr <= SPI3_1_BrstData(0);
                  SPI3_1_WrMod <= "10";
                  Holding_EOP1 <= '0';
                  Holding_Mod1 <= "00";
                  Holding_Err1 <= '0';
                  pl3_1_conversion_state <= "10";
                --"001" means data bits 63-56 have valid data
                when "001" =>
                  SPI3_1_WrEOP_int <= '1';
                  SPI3_1_WrErr <= SPI3_1_BrstData(0);
                  SPI3_1_WrMod <= "11";
                  Holding_EOP1 <= '0';
                  Holding_Mod1 <= "00";
                  Holding_Err1 <= '0';
                  pl3_1_conversion_state <= "10";
                --We have covered all the cases except X's & U's, so leave null
                when others =>
                  NULL;
              end case;
              
            else
              SPI3_1_WrEOP_int <= '0';
              SPI3_1_WrErr <= '0';
              SPI3_1_WrMod <= "00";
              Holding_EOP1 <= '0';
              Holding_Mod1 <= "00";
              Holding_Err1 <= '0';
              if (SPI3_1_dst_rdy='1') then
                  pl3_1_conversion_state <= "01";
                  else
                  pl3_1_conversion_state <= "00";
                  end if;
            end if;
            else
              	pl3_1_conversion_state <= "11";
              end if;
            
          else
            pl3_1_conversion_state <= "00";
            SPI3_1_WrEn_n <= '1';
          end if;
        when "01" =>
        	 if (SPI3_1_dst_rdy = '1') then
          pl3_1_conversion_state <= "00";
          SPI3_1_WrData <= Holding_Data1;
          SPI3_1_WrEn_n <= '0';
          SPI3_1_WrSOP  <= '0';
          SPI3_1_WrEOP_int  <= Holding_EOP1;
          SPI3_1_WrMod  <= Holding_Mod1;
          SPI3_1_WrErr  <= Holding_Err1;
          end if;
          
        when "10" =>
        	if (SPI3_1_dst_rdy = '1') then
          pl3_1_conversion_state <= "00";
          SPI3_1_WrEn_n <= '1';
          end if;
        when "11" =>      
          if (SPI3_1_dst_rdy = '1') then
            	pl3_1_conversion_state <= "01";
            	SPI3_1_WrData <= SPI3_1_BrstData(69 downto 38);
            	SPI3_1_WrEn_n <= '0';
            	SPI3_1_WrSOP  <= SPI3_1_BrstData(5);
            	Holding_Data1 <= SPI3_1_BrstData(37 downto 6);
         
           		--Look at Mod stuff here
            	--If there is an EOP, Mod must be looked at
           	 	if (SPI3_1_BrstData(4)='1') then
              case SPI3_1_BrstData(3 downto 1) is
                --"000" means data bits 63-0 have valid data
                when "000" => 
                  SPI3_1_WrEOP_int <= '0';
                  SPI3_1_WrErr <= '0';
                  SPI3_1_WrMod <= "00";
                  Holding_EOP1 <= '1';
                  Holding_Mod1 <= "00";
                  Holding_Err1 <= SPI3_1_BrstData(0);
                  pl3_1_conversion_state <= "01";
                --"111" means data bits 63-8 have valid data
                when "111" =>
                  SPI3_1_WrEOP_int <= '0';
                  SPI3_1_WrErr <= '0';
                  SPI3_1_WrMod <= "00";
                  Holding_EOP1 <= '1';
                  Holding_Mod1 <= "01";
                  Holding_Err1 <= SPI3_1_BrstData(0);
                  pl3_1_conversion_state <= "01";
                --"110" means data bits 63-16 have valid data
                when "110" =>
                  SPI3_1_WrEOP_int <= '0';
                  SPI3_1_WrErr <= '0';
                  SPI3_1_WrMod <= "00";
                  Holding_EOP1 <= '1';
                  Holding_Mod1 <= "10";
                  Holding_Err1 <= SPI3_1_BrstData(0);
                  pl3_1_conversion_state <= "01";
                --"101" means data bits 63-24 have valid data
                when "101" =>
                  SPI3_1_WrEOP_int <= '0';
                  SPI3_1_WrErr <= '0';
                  SPI3_1_WrMod <= "00";
                  Holding_EOP1 <= '1';
                  Holding_Mod1 <= "11";
                  Holding_Err1 <= SPI3_1_BrstData(0);
                  pl3_1_conversion_state <= "01";
                --"100" means data bits 63-32 have valid data
                when "100" =>
                  SPI3_1_WrEOP_int <= '1';
                  SPI3_1_WrErr <= SPI3_1_BrstData(0);
                  SPI3_1_WrMod <= "00";
                  Holding_EOP1 <= '0';
                  Holding_Mod1 <= "00";
                  Holding_Err1 <= '0';
                  pl3_1_conversion_state <= "10";
                --"011" means data bits 63-40 have valid data
                when "011" =>
                  SPI3_1_WrEOP_int <= '1';
                  SPI3_1_WrErr <= SPI3_1_BrstData(0);
                  SPI3_1_WrMod <= "01";
                  Holding_EOP1 <= '0';
                  Holding_Mod1 <= "00";
                  Holding_Err1 <= '0';
                  pl3_1_conversion_state <= "10";
                --"010" means data bits 63-48 have valid data
                when "010" =>
                  SPI3_1_WrEOP_int <= '1';
                  SPI3_1_WrErr <= SPI3_1_BrstData(0);
                  SPI3_1_WrMod <= "10";
                  Holding_EOP1 <= '0';
                  Holding_Mod1 <= "00";
                  Holding_Err1 <= '0';
                  pl3_1_conversion_state <= "10";
                --"001" means data bits 63-56 have valid data
                when "001" =>
                  SPI3_1_WrEOP_int <= '1';
                  SPI3_1_WrErr <= SPI3_1_BrstData(0);
                  SPI3_1_WrMod <= "11";
                  Holding_EOP1 <= '0';
                  Holding_Mod1 <= "00";
                  Holding_Err1 <= '0';
                  pl3_1_conversion_state <= "10";
                --We have covered all the cases except X's & U's, so leave null
                when others =>
                  NULL;
              end case;
              end if;
              else
              	pl3_1_conversion_state <= "11";
              
           end if;
        when others =>
          pl3_1_conversion_state <= "00";
          SPI3_1_WrEn_n <= '1';
      end case;
    end if;
  end process;
  
  SPI3_1_BrstRdEn <= SPI3_1_BrstRdEn_int;
  SPI3_1_WrEOP  <= SPI3_1_WrEOP_int;

  
  ------------------------------------------------------------------------------
  -- Write SPI-3 core 2 Process
  --
  -- Note: the purpose of this process is to write data into the SPI-3 source
  -- core, instantiation 2
  -- Note: this is a repeat of the above processes
  ------------------------------------------------------------------------------
  
           write_pl3_core_2_proc: process (Reset_n, SPI4_Clk)
  begin
    if (Reset_n='0') then
      SPI3_2_BrstRdEn_int <= '0';
      SPI3_2_BrstRdEn_d1 <= '0';
      wait_for_conversion2 <= "00";
    elsif (SPI4_Clk'event and SPI4_Clk='1') then
      SPI3_2_BrstRdEn_d1 <= SPI3_2_BrstRdEn_int;
      if (SPI3_2_dst_rdy='1' and SPI3_2_Brst_Empty='0' and
          wait_for_conversion2= "00" and SPI3_2_polled_status='1') then
        SPI3_2_BrstRdEn_int <= '1';
        wait_for_conversion2 <= "01";
      elsif (SPI3_2_dst_rdy='1' and wait_for_conversion2 <= "01") then
      	SPI3_2_BrstRdEn_int <= '0';
        wait_for_conversion2 <= "10";
      elsif (SPI3_2_dst_rdy='1' and wait_for_conversion2 <= "10") then
        SPI3_2_BrstRdEn_int <= '0';
        wait_for_conversion2 <= "00";
       else
       SPI3_2_BrstRdEn_int <= '0';
       wait_for_conversion2 <= wait_for_conversion2;
      end if;
    end if;
  end process;
  
  pl3_2_data_conversion: process (Reset_n, SPI4_Clk)
  begin
    if (Reset_n='0') then
      pl3_2_conversion_state <= "00";
      SPI3_2_WrData <= (others => '0');
      SPI3_2_WrEn_n <= '1';
      SPI3_2_WrSOP <= '0';
      SPI3_2_WrEOP_int <= '0';
      SPI3_2_WrMod <= "00";
      SPI3_2_WrErr <= '0';
      Holding_Data2 <= (others => '0');
      Holding_EOP2 <= '0';
      Holding_Mod2 <= "00";
      Holding_Err2 <= '0';
    elsif (SPI4_Clk'event and SPI4_Clk='1') then
      case pl3_2_conversion_state is
        when "00" =>
          if (SPI3_2_BrstRdEn_d1='1') then
          	if (SPI3_2_dst_rdy = '1') then
            	pl3_2_conversion_state <= "01";
            	SPI3_2_WrData <= SPI3_2_BrstData(69 downto 38);
            	SPI3_2_WrEn_n <= '0';
            	SPI3_2_WrSOP  <= SPI3_2_BrstData(5);
            	Holding_Data2 <= SPI3_2_BrstData(37 downto 6);
         
           		--Look at Mod stuff here
            	--If there is an EOP, Mod must be looked at
           	 	if (SPI3_2_BrstData(4)='1') then
              case SPI3_2_BrstData(3 downto 1) is
                --"000" means data bits 63-0 have valid data
                when "000" => 
                  SPI3_2_WrEOP_int <= '0';
                  SPI3_2_WrErr <= '0';
                  SPI3_2_WrMod <= "00";
                  Holding_EOP2 <= '1';
                  Holding_Mod2 <= "00";
                  Holding_Err2 <= SPI3_2_BrstData(0);
                  pl3_2_conversion_state <= "01";
                --"111" means data bits 63-8 have valid data
                when "111" =>
                  SPI3_2_WrEOP_int <= '0';
                  SPI3_2_WrErr <= '0';
                  SPI3_2_WrMod <= "00";
                  Holding_EOP2 <= '1';
                  Holding_Mod2 <= "01";
                  Holding_Err2 <= SPI3_2_BrstData(0);
                  pl3_2_conversion_state <= "01";
                --"110" means data bits 63-16 have valid data
                when "110" =>
                  SPI3_2_WrEOP_int <= '0';
                  SPI3_2_WrErr <= '0';
                  SPI3_2_WrMod <= "00";
                  Holding_EOP2 <= '1';
                  Holding_Mod2 <= "10";
                  Holding_Err2 <= SPI3_2_BrstData(0);
                  pl3_2_conversion_state <= "01";
                --"101" means data bits 63-24 have valid data
                when "101" =>
                  SPI3_2_WrEOP_int <= '0';
                  SPI3_2_WrErr <= '0';
                  SPI3_2_WrMod <= "00";
                  Holding_EOP2 <= '1';
                  Holding_Mod2 <= "11";
                  Holding_Err2 <= SPI3_2_BrstData(0);
                  pl3_2_conversion_state <= "01";
                --"100" means data bits 63-32 have valid data
                when "100" =>
                  SPI3_2_WrEOP_int <= '1';
                  SPI3_2_WrErr <= SPI3_2_BrstData(0);
                  SPI3_2_WrMod <= "00";
                  Holding_EOP2 <= '0';
                  Holding_Mod2 <= "00";
                  Holding_Err2 <= '0';
                  pl3_2_conversion_state <= "10";
                --"011" means data bits 63-40 have valid data
                when "011" =>
                  SPI3_2_WrEOP_int <= '1';
                  SPI3_2_WrErr <= SPI3_2_BrstData(0);
                  SPI3_2_WrMod <= "01";
                  Holding_EOP2 <= '0';
                  Holding_Mod2 <= "00";
                  Holding_Err2 <= '0';
                  pl3_2_conversion_state <= "10";
                --"010" means data bits 63-48 have valid data
                when "010" =>
                  SPI3_2_WrEOP_int <= '1';
                  SPI3_2_WrErr <= SPI3_2_BrstData(0);
                  SPI3_2_WrMod <= "10";
                  Holding_EOP2 <= '0';
                  Holding_Mod2 <= "00";
                  Holding_Err2 <= '0';
                  pl3_2_conversion_state <= "10";
                --"001" means data bits 63-56 have valid data
                when "001" =>
                  SPI3_2_WrEOP_int <= '1';
                  SPI3_2_WrErr <= SPI3_2_BrstData(0);
                  SPI3_2_WrMod <= "11";
                  Holding_EOP2 <= '0';
                  Holding_Mod2 <= "00";
                  Holding_Err2 <= '0';
                  pl3_2_conversion_state <= "10";
                --We have covered all the cases except X's & U's, so leave null
                when others =>
                  NULL;
              end case;
              
            else
              SPI3_2_WrEOP_int <= '0';
              SPI3_2_WrErr <= '0';
              SPI3_2_WrMod <= "00";
              Holding_EOP2 <= '0';
              Holding_Mod2 <= "00";
              Holding_Err2 <= '0';
              if (SPI3_2_dst_rdy='1') then
                  pl3_2_conversion_state <= "01";
                  else
                  pl3_2_conversion_state <= "00";
                  end if;
            end if;
            else
              	pl3_2_conversion_state <= "11";
              end if;
            
          else
            pl3_2_conversion_state <= "00";
            SPI3_2_WrEn_n <= '1';
          end if;
        when "01" =>
        	 if (SPI3_2_dst_rdy = '1') then
          pl3_2_conversion_state <= "00";
          SPI3_2_WrData <= Holding_Data2;
          SPI3_2_WrEn_n <= '0';
          SPI3_2_WrSOP  <= '0';
          SPI3_2_WrEOP_int  <= Holding_EOP2;
          SPI3_2_WrMod  <= Holding_Mod2;
          SPI3_2_WrErr  <= Holding_Err2;
          end if;
          
        when "10" =>
        	if (SPI3_2_dst_rdy = '1') then
          pl3_2_conversion_state <= "00";
          SPI3_2_WrEn_n <= '1';
          end if;
        when "11" =>      
          if (SPI3_2_dst_rdy = '1') then
            	pl3_2_conversion_state <= "01";
            	SPI3_2_WrData <= SPI3_2_BrstData(69 downto 38);
            	SPI3_2_WrEn_n <= '0';
            	SPI3_2_WrSOP  <= SPI3_2_BrstData(5);
            	Holding_Data2 <= SPI3_2_BrstData(37 downto 6);
         
           		--Look at Mod stuff here
            	--If there is an EOP, Mod must be looked at
           	 	if (SPI3_2_BrstData(4)='1') then
              case SPI3_2_BrstData(3 downto 1) is
                --"000" means data bits 63-0 have valid data
                when "000" => 
                  SPI3_2_WrEOP_int <= '0';
                  SPI3_2_WrErr <= '0';
                  SPI3_2_WrMod <= "00";
                  Holding_EOP2 <= '1';
                  Holding_Mod2 <= "00";
                  Holding_Err2 <= SPI3_2_BrstData(0);
                  pl3_2_conversion_state <= "01";
                --"111" means data bits 63-8 have valid data
                when "111" =>
                  SPI3_2_WrEOP_int <= '0';
                  SPI3_2_WrErr <= '0';
                  SPI3_2_WrMod <= "00";
                  Holding_EOP2 <= '1';
                  Holding_Mod2 <= "01";
                  Holding_Err2 <= SPI3_2_BrstData(0);
                  pl3_2_conversion_state <= "01";
                --"110" means data bits 63-16 have valid data
                when "110" =>
                  SPI3_2_WrEOP_int <= '0';
                  SPI3_2_WrErr <= '0';
                  SPI3_2_WrMod <= "00";
                  Holding_EOP2 <= '1';
                  Holding_Mod2 <= "10";
                  Holding_Err2 <= SPI3_2_BrstData(0);
                  pl3_2_conversion_state <= "01";
                --"101" means data bits 63-24 have valid data
                when "101" =>
                  SPI3_2_WrEOP_int <= '0';
                  SPI3_2_WrErr <= '0';
                  SPI3_2_WrMod <= "00";
                  Holding_EOP2 <= '1';
                  Holding_Mod2 <= "11";
                  Holding_Err2 <= SPI3_2_BrstData(0);
                  pl3_2_conversion_state <= "01";
                --"100" means data bits 63-32 have valid data
                when "100" =>
                  SPI3_2_WrEOP_int <= '1';
                  SPI3_2_WrErr <= SPI3_2_BrstData(0);
                  SPI3_2_WrMod <= "00";
                  Holding_EOP2 <= '0';
                  Holding_Mod2 <= "00";
                  Holding_Err2 <= '0';
                  pl3_2_conversion_state <= "10";
                --"011" means data bits 63-40 have valid data
                when "011" =>
                  SPI3_2_WrEOP_int <= '1';
                  SPI3_2_WrErr <= SPI3_2_BrstData(0);
                  SPI3_2_WrMod <= "01";
                  Holding_EOP2 <= '0';
                  Holding_Mod2 <= "00";
                  Holding_Err2 <= '0';
                  pl3_2_conversion_state <= "10";
                --"010" means data bits 63-48 have valid data
                when "010" =>
                  SPI3_2_WrEOP_int <= '1';
                  SPI3_2_WrErr <= SPI3_2_BrstData(0);
                  SPI3_2_WrMod <= "10";
                  Holding_EOP2 <= '0';
                  Holding_Mod2 <= "00";
                  Holding_Err2 <= '0';
                  pl3_2_conversion_state <= "10";
                --"001" means data bits 63-56 have valid data
                when "001" =>
                  SPI3_2_WrEOP_int <= '1';
                  SPI3_2_WrErr <= SPI3_2_BrstData(0);
                  SPI3_2_WrMod <= "11";
                  Holding_EOP2 <= '0';
                  Holding_Mod2 <= "00";
                  Holding_Err2 <= '0';
                  pl3_2_conversion_state <= "10";
                --We have covered all the cases except X's & U's, so leave null
                when others =>
                  NULL;
              end case;
              end if;
              else
              	pl3_2_conversion_state <= "11";
              
           end if;
        when others =>
          pl3_2_conversion_state <= "00";
          SPI3_2_WrEn_n <= '1';
      end case;
    end if;
  end process;
  
  SPI3_2_BrstRdEn <= SPI3_2_BrstRdEn_int;
  SPI3_2_WrEOP  <= SPI3_2_WrEOP_int;

  
  ------------------------------------------------------------------------------
  -- Write SPI-3 core 3 Process
  --
  -- Note: the purpose of this process is to write data into the SPI-3 source
  -- core, instantiation 3
  -- Note: this is a repeat of the above processes
  ------------------------------------------------------------------------------
  
  write_pl3_core_3_proc: process (Reset_n, SPI4_Clk)
  begin
    if (Reset_n='0') then
      SPI3_3_BrstRdEn_int <= '0';
      SPI3_3_BrstRdEn_d1 <= '0';
      wait_for_conversion3 <= "00";
    elsif (SPI4_Clk'event and SPI4_Clk='1') then
      SPI3_3_BrstRdEn_d1 <= SPI3_3_BrstRdEn_int;
      if (SPI3_3_dst_rdy='1' and SPI3_3_Brst_Empty='0' and
          wait_for_conversion3= "00" and SPI3_3_polled_status='1') then
        SPI3_3_BrstRdEn_int <= '1';
        wait_for_conversion3 <= "01";
      elsif (SPI3_3_dst_rdy='1' and wait_for_conversion3 <= "01") then
      	SPI3_3_BrstRdEn_int <= '0';
        wait_for_conversion3 <= "10";
      elsif (SPI3_3_dst_rdy='1' and wait_for_conversion3 <= "10") then
        SPI3_3_BrstRdEn_int <= '0';
        wait_for_conversion3 <= "00";
       else
       SPI3_3_BrstRdEn_int <= '0';
       wait_for_conversion3 <= wait_for_conversion3;
      end if;
    end if;
  end process;
  
  pl3_3_data_conversion: process (Reset_n, SPI4_Clk)
  begin
    if (Reset_n='0') then
      pl3_3_conversion_state <= "00";
      SPI3_3_WrData <= (others => '0');
      SPI3_3_WrEn_n <= '1';
      SPI3_3_WrSOP <= '0';
      SPI3_3_WrEOP_int <= '0';
      SPI3_3_WrMod <= "00";
      SPI3_3_WrErr <= '0';
      Holding_Data3 <= (others => '0');
      Holding_EOP3 <= '0';
      Holding_Mod3 <= "00";
      Holding_Err3 <= '0';
    elsif (SPI4_Clk'event and SPI4_Clk='1') then
      case pl3_3_conversion_state is
        when "00" =>
          if (SPI3_3_BrstRdEn_d1='1') then
          	if (SPI3_3_dst_rdy = '1') then
            	pl3_3_conversion_state <= "01";
            	SPI3_3_WrData <= SPI3_3_BrstData(69 downto 38);
            	SPI3_3_WrEn_n <= '0';
            	SPI3_3_WrSOP  <= SPI3_3_BrstData(5);
            	Holding_Data3 <= SPI3_3_BrstData(37 downto 6);
         
           		--Look at Mod stuff here
            	--If there is an EOP, Mod must be looked at
           	 	if (SPI3_3_BrstData(4)='1') then
              case SPI3_3_BrstData(3 downto 1) is
                --"000" means data bits 63-0 have valid data
                when "000" => 
                  SPI3_3_WrEOP_int <= '0';
                  SPI3_3_WrErr <= '0';
                  SPI3_3_WrMod <= "00";
                  Holding_EOP3 <= '1';
                  Holding_Mod3 <= "00";
                  Holding_Err3 <= SPI3_3_BrstData(0);
                  pl3_3_conversion_state <= "01";
                --"111" means data bits 63-8 have valid data
                when "111" =>
                  SPI3_3_WrEOP_int <= '0';
                  SPI3_3_WrErr <= '0';
                  SPI3_3_WrMod <= "00";
                  Holding_EOP3 <= '1';
                  Holding_Mod3 <= "01";
                  Holding_Err3 <= SPI3_3_BrstData(0);
                  pl3_3_conversion_state <= "01";
                --"110" means data bits 63-16 have valid data
                when "110" =>
                  SPI3_3_WrEOP_int <= '0';
                  SPI3_3_WrErr <= '0';
                  SPI3_3_WrMod <= "00";
                  Holding_EOP3 <= '1';
                  Holding_Mod3 <= "10";
                  Holding_Err3 <= SPI3_3_BrstData(0);
                  pl3_3_conversion_state <= "01";
                --"101" means data bits 63-24 have valid data
                when "101" =>
                  SPI3_3_WrEOP_int <= '0';
                  SPI3_3_WrErr <= '0';
                  SPI3_3_WrMod <= "00";
                  Holding_EOP3 <= '1';
                  Holding_Mod3 <= "11";
                  Holding_Err3 <= SPI3_3_BrstData(0);
                  pl3_3_conversion_state <= "01";
                --"100" means data bits 63-32 have valid data
                when "100" =>
                  SPI3_3_WrEOP_int <= '1';
                  SPI3_3_WrErr <= SPI3_3_BrstData(0);
                  SPI3_3_WrMod <= "00";
                  Holding_EOP3 <= '0';
                  Holding_Mod3 <= "00";
                  Holding_Err3 <= '0';
                  pl3_3_conversion_state <= "10";
                --"011" means data bits 63-40 have valid data
                when "011" =>
                  SPI3_3_WrEOP_int <= '1';
                  SPI3_3_WrErr <= SPI3_3_BrstData(0);
                  SPI3_3_WrMod <= "01";
                  Holding_EOP3 <= '0';
                  Holding_Mod3 <= "00";
                  Holding_Err3 <= '0';
                  pl3_3_conversion_state <= "10";
                --"010" means data bits 63-48 have valid data
                when "010" =>
                  SPI3_3_WrEOP_int <= '1';
                  SPI3_3_WrErr <= SPI3_3_BrstData(0);
                  SPI3_3_WrMod <= "10";
                  Holding_EOP3 <= '0';
                  Holding_Mod3 <= "00";
                  Holding_Err3 <= '0';
                  pl3_3_conversion_state <= "10";
                --"001" means data bits 63-56 have valid data
                when "001" =>
                  SPI3_3_WrEOP_int <= '1';
                  SPI3_3_WrErr <= SPI3_3_BrstData(0);
                  SPI3_3_WrMod <= "11";
                  Holding_EOP3 <= '0';
                  Holding_Mod3 <= "00";
                  Holding_Err3 <= '0';
                  pl3_3_conversion_state <= "10";
                --We have covered all the cases except X's & U's, so leave null
                when others =>
                  NULL;
              end case;
              
            else
              SPI3_3_WrEOP_int <= '0';
              SPI3_3_WrErr <= '0';
              SPI3_3_WrMod <= "00";
              Holding_EOP3 <= '0';
              Holding_Mod3 <= "00";
              Holding_Err3 <= '0';
              if (SPI3_3_dst_rdy='1') then
                  pl3_3_conversion_state <= "01";
                  else
                  pl3_3_conversion_state <= "00";
                  end if;
            end if;
            else
              	pl3_3_conversion_state <= "11";
              end if;
            
          else
            pl3_3_conversion_state <= "00";
            SPI3_3_WrEn_n <= '1';
          end if;
        when "01" =>
        	 if (SPI3_3_dst_rdy = '1') then
          pl3_3_conversion_state <= "00";
          SPI3_3_WrData <= Holding_Data3;
          SPI3_3_WrEn_n <= '0';
          SPI3_3_WrSOP  <= '0';
          SPI3_3_WrEOP_int  <= Holding_EOP3;
          SPI3_3_WrMod  <= Holding_Mod3;
          SPI3_3_WrErr  <= Holding_Err3;
          end if;
          
        when "10" =>
        	if (SPI3_3_dst_rdy = '1') then
          pl3_3_conversion_state <= "00";
          SPI3_3_WrEn_n <= '1';
          end if;
        when "11" =>      
          if (SPI3_3_dst_rdy = '1') then
            	pl3_3_conversion_state <= "01";
            	SPI3_3_WrData <= SPI3_3_BrstData(69 downto 38);
            	SPI3_3_WrEn_n <= '0';
            	SPI3_3_WrSOP  <= SPI3_3_BrstData(5);
            	Holding_Data3 <= SPI3_3_BrstData(37 downto 6);
         
           		--Look at Mod stuff here
            	--If there is an EOP, Mod must be looked at
           	 	if (SPI3_3_BrstData(4)='1') then
              case SPI3_3_BrstData(3 downto 1) is
                --"000" means data bits 63-0 have valid data
                when "000" => 
                  SPI3_3_WrEOP_int <= '0';
                  SPI3_3_WrErr <= '0';
                  SPI3_3_WrMod <= "00";
                  Holding_EOP3 <= '1';
                  Holding_Mod3 <= "00";
                  Holding_Err3 <= SPI3_3_BrstData(0);
                  pl3_3_conversion_state <= "01";
                --"111" means data bits 63-8 have valid data
                when "111" =>
                  SPI3_3_WrEOP_int <= '0';
                  SPI3_3_WrErr <= '0';
                  SPI3_3_WrMod <= "00";
                  Holding_EOP3 <= '1';
                  Holding_Mod3 <= "01";
                  Holding_Err3 <= SPI3_3_BrstData(0);
                  pl3_3_conversion_state <= "01";
                --"110" means data bits 63-16 have valid data
                when "110" =>
                  SPI3_3_WrEOP_int <= '0';
                  SPI3_3_WrErr <= '0';
                  SPI3_3_WrMod <= "00";
                  Holding_EOP3 <= '1';
                  Holding_Mod3 <= "10";
                  Holding_Err3 <= SPI3_3_BrstData(0);
                  pl3_3_conversion_state <= "01";
                --"101" means data bits 63-24 have valid data
                when "101" =>
                  SPI3_3_WrEOP_int <= '0';
                  SPI3_3_WrErr <= '0';
                  SPI3_3_WrMod <= "00";
                  Holding_EOP3 <= '1';
                  Holding_Mod3 <= "11";
                  Holding_Err3 <= SPI3_3_BrstData(0);
                  pl3_3_conversion_state <= "01";
                --"100" means data bits 63-32 have valid data
                when "100" =>
                  SPI3_3_WrEOP_int <= '1';
                  SPI3_3_WrErr <= SPI3_3_BrstData(0);
                  SPI3_3_WrMod <= "00";
                  Holding_EOP3 <= '0';
                  Holding_Mod3 <= "00";
                  Holding_Err3 <= '0';
                  pl3_3_conversion_state <= "10";
                --"011" means data bits 63-40 have valid data
                when "011" =>
                  SPI3_3_WrEOP_int <= '1';
                  SPI3_3_WrErr <= SPI3_3_BrstData(0);
                  SPI3_3_WrMod <= "01";
                  Holding_EOP3 <= '0';
                  Holding_Mod3 <= "00";
                  Holding_Err3 <= '0';
                  pl3_3_conversion_state <= "10";
                --"010" means data bits 63-48 have valid data
                when "010" =>
                  SPI3_3_WrEOP_int <= '1';
                  SPI3_3_WrErr <= SPI3_3_BrstData(0);
                  SPI3_3_WrMod <= "10";
                  Holding_EOP3 <= '0';
                  Holding_Mod3 <= "00";
                  Holding_Err3 <= '0';
                  pl3_3_conversion_state <= "10";
                --"001" means data bits 63-56 have valid data
                when "001" =>
                  SPI3_3_WrEOP_int <= '1';
                  SPI3_3_WrErr <= SPI3_3_BrstData(0);
                  SPI3_3_WrMod <= "11";
                  Holding_EOP3 <= '0';
                  Holding_Mod3 <= "00";
                  Holding_Err3 <= '0';
                  pl3_3_conversion_state <= "10";
                --We have covered all the cases except X's & U's, so leave null
                when others =>
                  NULL;
              end case;
              end if;
              else
              	pl3_3_conversion_state <= "11";
              
           end if;
        when others =>
          pl3_3_conversion_state <= "00";
          SPI3_3_WrEn_n <= '1';
      end case;
    end if;
  end process;
  
  SPI3_3_BrstRdEn <= SPI3_3_BrstRdEn_int;
  SPI3_3_WrEOP  <= SPI3_3_WrEOP_int;



end xilinx;
