-------------------------------------------------------------------------------
--
--   File Name:  spi_pkg.vhd
--     Version:  1.0
--        Date:  June 11, 2007
--
-- Description:  This file contains constants for the SPI4.2 to SPI3 bridge 
--
--      Author:  Zhe Xia
--     Company:  Xilinx Inc.
--
--
--     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
--     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
--     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
--     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
--     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
--     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
--     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
--     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
--     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
--     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
--     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
--     FOR A PARTICULAR PURPOSE.
--
--     (c) Copyright 2003 Xilinx, Inc.
--     All rights reserved.
--
--------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- Library Declarations
-------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_misc.all;


-------------------------------------------------------------------------------
-- Package Declaration
-------------------------------------------------------------------------------
package spi_pkg is

  -----------------------------------------------------------------------------
  -- SPI3 to SPI4 Burst Storage FIFO Almost Empty Threshold 
  -- This value must be > (16*MaxBurst1 and 16*MaxBurst2)
  -- Units : Bytes
  -----------------------------------------------------------------------------
  
  constant SPI3_SPI4BrstAE_Thresh : integer := 255;  --2040 Bytes

  
  -----------------------------------------------------------------------------
  -- SPI4 MaxBurst1 and MaxBurst2 parameters
  -- MaxBurst1 : This parameter defines number of credits transfered
  --             when SPI4 Status for the channel is Starving ("00")
  -- MaxBurst2 : This parameter defines number of credits transfered
  --             when SPI4 Status for the channel is Hungry ("01")
  -- Units     : SPI4 Credits (16 bytes)
  -- NOTE: SPI3_SPI4BrstAE_Thresh must be >  (16*MaxBurst1 and 16*MaxBurst2)
  -----------------------------------------------------------------------------
  constant MaxBurst1 : integer := 8;  --128 Bytes
  constant MaxBurst2 : integer := 4;  --64 Bytes

  
  -----------------------------------------------------------------------------
  -- Simulation Constants
  -----------------------------------------------------------------------------
  constant TFF             : time := 0.2 ns;

end spi_pkg;

-------------------------------------------------------------------------------
-- Package Body
-------------------------------------------------------------------------------
package body spi_pkg is
end spi_pkg;
