/******************************************************************************
**
**   File Name:  master_clocks.v
**     Version:  2.0
**        Date:  August 31, 2004
**
** Description:  This file provides the external clocks necessary to drive the
**               SPI3 to SPI4 and SPI4 to SPI3 bridges.
**
**      Author:  IP Solutions Division
**     Company:  Xilinx Inc.
**
**
**     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
**     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
**     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
**     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
**     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
**     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
**     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
**     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
**     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
**     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
**     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
**     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
**     FOR A PARTICULAR PURPOSE.
**
**     (c) Copyright 2003 Xilinx, Inc.
**     All rights reserved.
**
******************************************************************************/

`timescale 1 ps / 1 ps

/******************************************************************************
 * Module Declarations
 *****************************************************************************/
module master_clocks (
		      SimulationComplete,

		      SPI3_RFClk,
		      SPI3_TFClk,

		      SPI4_SysClk_P,
			SPI4_SysClk_N,
			CLK200
                      );

	input      SimulationComplete;

	output      SPI3_RFClk;
	output      SPI3_TFClk;

	output      SPI4_SysClk_P;
	output	SPI4_SysClk_N;
	output CLK200;


/******************************************************************************
 * Reg and Wire Declarations
 *****************************************************************************/
	reg 		spi3_rfclk_reg;
	reg		spi3_tfclk_reg;
	reg		CLK200_int;
	reg		spi4_sysclk;
	reg		SimulationComplete_reg;

/******************************************************************************
 * Parameter Declarations
 *****************************************************************************/
   parameter NS                 = 1000; //Multiplier to convert ns to ps
   parameter SPI3_RFCLK_PERIOD  = 9615; // 104 MHz
   parameter SPI3_TFCLK_PERIOD  = 9615; // 104 MHz
   parameter SPI4_SYSCLK_PERIOD = 2857; // 350 MHz
   parameter SPI4_SYSCLK_PERIOD_DIV_2 = SPI4_SYSCLK_PERIOD/2;
   parameter SPI3_RFCLK_PERIOD_DIV_2 = SPI3_RFCLK_PERIOD/2;
   parameter SPI3_TFCLK_PERIOD_DIV_2 = SPI3_TFCLK_PERIOD/2;
   parameter CLK200_PERIOD = 5000; // 200 MHz
   parameter CLK200_PERIOD_DIV_2 = CLK200_PERIOD/2;
 
   initial
    begin
	    spi3_rfclk_reg = 0;
   	    spi3_tfclk_reg = 0;
  	    spi4_sysclk = 0;
	    SimulationComplete_reg = 0;
    end

/******************************************************************************
** Generate SPI3 RX Reference Clock(s)
 *****************************************************************************/

  always
  begin
    if (SimulationComplete_reg == 0)
     begin
      spi3_rfclk_reg <= 0;
      # SPI3_RFCLK_PERIOD_DIV_2
      spi3_rfclk_reg <= 1;
      # SPI3_RFCLK_PERIOD_DIV_2;
     end
    else if (SimulationComplete_reg == 1)
     begin
      spi3_rfclk_reg <= 0;
     end
  end 
  
  assign SPI3_RFClk = spi3_rfclk_reg;

/******************************************************************************
** Generate SPI3 TX Reference Clock(s) 
*****************************************************************************/

  always
  begin
    if (SimulationComplete_reg == 0)
     begin
      spi3_tfclk_reg <= 0;
      # SPI3_TFCLK_PERIOD_DIV_2
      spi3_tfclk_reg <= 1;
      # SPI3_TFCLK_PERIOD_DIV_2;
     end
    else if (SimulationComplete_reg == 1)
     begin
      spi3_tfclk_reg <= 0;
     end
  end

  assign SPI3_TFClk = spi3_tfclk_reg;

/******************************************************************************
** Generate SPI4 Source Core Reference Clock
*****************************************************************************/

 always
  begin
    if (SimulationComplete_reg == 0)
     begin
      spi4_sysclk <= 0;
      # SPI4_SYSCLK_PERIOD_DIV_2
      spi4_sysclk <= 1;
      # SPI4_SYSCLK_PERIOD_DIV_2;
     end
    else if (SimulationComplete_reg == 1)
     begin
      spi4_sysclk <= 0;
     end
  end

  assign SimulationComplete = SimulationComplete_reg;
  assign SPI4_SysClk_P = spi4_sysclk;
  assign SPI4_SysClk_N = ~spi4_sysclk;
  


/******************************************************************************
** Generate SPI4 Idelay Reference Clock
*****************************************************************************/

 always
  begin
    if (SimulationComplete_reg == 0)
     begin
      CLK200_int <= 0;
      # CLK200_PERIOD_DIV_2
      CLK200_int <= 1;
      # CLK200_PERIOD_DIV_2;
     end
    else if (SimulationComplete_reg == 1)
     begin
      CLK200_int <= 0;
     end
  end

  assign CLK200 = CLK200_int;

endmodule
