/******************************************************************************
**
**   File Name:  spi4_to_4spi3_tb.v
**     Version:  2.0
**        Date:  October 15, 2004
**
** Description:  This file instantiates the two bridge directions (SPI3 to
**               SPI4.2, and SPI4.2 to SPI3), as well as the test environment.
**               The testbench is comprised of four PL3 PHY emulators, which
**               source data into the four PL3 interfaces. The SPI4.2 interface
**               loops data from the SPI4.2 source to the SPI4.2 sink, and
**               loops status in the opposite direction.
**
**      Author:  IP Solutions Division
**     Company:  Xilinx Inc.
**
**
**     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
**     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
**     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
**     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
**     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
**     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
**     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
**     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
**     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
**     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
**     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
**     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
**     FOR A PARTICULAR PURPOSE.
**
**     (c) Copyright 2003 Xilinx, Inc.
**     All rights reserved.
**
******************************************************************************/

`timescale 1ns / 1ps

/******************************************************************************
** Module Declaration
******************************************************************************/

module spi4_to_4spi3_tb ();

  /****************************************************************************
  ** Wire and Reg Declarations
  ****************************************************************************/
  reg 		 EmReset_n;
  reg 		 BridgeReset_n;
  
  // SPI3 Sink Interfaces
  wire 		 SPI3_RFClk;
  wire 		 SPI3_0_RVal;
  wire 		 SPI3_0_REnb_n;
  wire 		 SPI3_0_RSx;
  wire 		 SPI3_0_RSOP;
  wire 		 SPI3_0_REOP;
  wire 	[31:0] SPI3_0_RDat;
  wire 		 SPI3_0_RPrty;
  wire 	[1:0]  SPI3_0_RMod;
  wire 		 SPI3_0_RErr;
  wire 		 SPI3_1_RVal;
  wire 		 SPI3_1_REnb_n;
  wire 		 SPI3_1_RSx;
  wire 		 SPI3_1_RSOP;
  wire 		 SPI3_1_REOP;
  wire 	[31:0] SPI3_1_RDat;
  wire 		 SPI3_1_RPrty;
  wire 	[1:0]  SPI3_1_RMod;
  wire 		 SPI3_1_RErr;
  wire 		 SPI3_2_RVal;
  wire 		 SPI3_2_REnb_n;
  wire 		 SPI3_2_RSx;
  wire 		 SPI3_2_RSOP;
  wire 		 SPI3_2_REOP;
  wire 	[31:0] SPI3_2_RDat;
  wire 		 SPI3_2_RPrty;
  wire 	[1:0]  SPI3_2_RMod;
  wire 		 SPI3_2_RErr;
  wire 		 SPI3_3_RVal;
  wire 		 SPI3_3_REnb_n;
  wire 		 SPI3_3_RSx;
  wire 		 SPI3_3_RSOP;
  wire 		 SPI3_3_REOP;
  wire 	[31:0] SPI3_3_RDat;
  wire 		 SPI3_3_RPrty;
  wire 	[1:0]	 SPI3_3_RMod;
  wire 		 SPI3_3_RErr;
  wire 		 SPI3_ParityErr;
  
  // SPI4 Source/Sink Interfaces
  wire			 CLK200;
  wire 		 SPI4_SysClk_P;
  wire 		 SPI4_SysClk_N;
  wire 		 SPI4_TDClk_P;
  wire 		 SPI4_TDClk_N;
  wire 		 SPI4_TCtl_P;
  wire 		 SPI4_TCtl_N;
  wire 	[15:0] SPI4_TDat_P;
  wire 	[15:0] SPI4_TDat_N;
  wire 		 SPI4_TSClk;
  wire 	[1:0]  SPI4_TStat;
  wire 		 SPI4_RDClk_P;
  wire 		 SPI4_RDClk_N;
  wire 		 SPI4_RCtl_P;
  wire 		 SPI4_RCtl_N;
  wire 	[15:0] SPI4_RDat_P;
  wire 	[15:0] SPI4_RDat_N;
  wire 		 SPI4_RSClk;
  wire 	[1:0]  SPI4_RStat;
  
  // SPI3 Source Interfaces
  wire 		 SPI3_TFClk;
  wire 		 SPI3_0_TEnb_n;
  wire 		 SPI3_0_TSx;
  wire 		 SPI3_0_TSOP;
  wire 		 SPI3_0_TEOP;
  wire 	[31:0] SPI3_0_TDat;
  wire 		 SPI3_0_TPrty;
  wire 	[1:0]  SPI3_0_TMod;
  wire 		 SPI3_0_TErr;
  wire 		 SPI3_0_PTPA;
  wire 	[7:0]  SPI3_0_TAdr;
  wire 		 SPI3_1_TEnb_n;
  wire 		 SPI3_1_TSx;
  wire 		 SPI3_1_TSOP;
  wire 		 SPI3_1_TEOP;
  wire 	[31:0] SPI3_1_TDat;
  wire 		 SPI3_1_TPrty;
  wire 	[1:0]  SPI3_1_TMod;
  wire 		 SPI3_1_TErr;
  wire 		 SPI3_1_PTPA;
  wire 	[7:0]  SPI3_1_TAdr;
  wire 		 SPI3_2_TEnb_n;
  wire 		 SPI3_2_TSx;
  wire 		 SPI3_2_TSOP;
  wire 		 SPI3_2_TEOP;
  wire 	[31:0] SPI3_2_TDat;
  wire 		 SPI3_2_TPrty;
  wire 	[1:0]  SPI3_2_TMod;
  wire 		 SPI3_2_TErr;
  wire 		 SPI3_2_PTPA;
  wire 	[7:0]  SPI3_2_TAdr;
  wire 		 SPI3_3_TEnb_n;
  wire 		 SPI3_3_TSx;
  wire 		 SPI3_3_TSOP;
  wire 		 SPI3_3_TEOP;
  wire 	[31:0] SPI3_3_TDat;
  wire 		 SPI3_3_TPrty;
  wire 	[1:0]  SPI3_3_TMod;
  wire 		 SPI3_3_TErr;
  wire 		 SPI3_3_PTPA;
  wire 	[7:0]  SPI3_3_TAdr;

  reg 	 	 SimulationComplete;
  
 
  // Combined SPI4 to SPI3 and SPI3 to SPI4 Bridge Top Level File
  bridge_top dut (
      // System signals
      .Reset_n(BridgeReset_n),

      // SPI4.2 Sink signals
      .Ref_Clk(CLK200),
      .SPI4_RDClk_P(SPI4_RDClk_P),
      .SPI4_RDClk_N(SPI4_RDClk_N),
      .SPI4_RCtl_P(SPI4_RCtl_P),
      .SPI4_RCtl_N(SPI4_RCtl_N),
      .SPI4_RDat_P(SPI4_RDat_P),
      .SPI4_RDat_N(SPI4_RDat_N),

      // SPI4.2 Sink Status (LVTTL) input
      .SPI4_RSClk(SPI4_RSClk),
      .SPI4_RStat(SPI4_RStat),

      // SPI4.2 Source signals
      .SPI4_TDClk_P(SPI4_TDClk_P),
      .SPI4_TDClk_N(SPI4_TDClk_N),
      .SPI4_TCtl_P(SPI4_TCtl_P),
      .SPI4_TCtl_N(SPI4_TCtl_N),
      .SPI4_TDat_P(SPI4_TDat_P),
      .SPI4_TDat_N(SPI4_TDat_N),
      .SPI4_SysClk_P(SPI4_SysClk_P),
      .SPI4_SysClk_N(SPI4_SysClk_N),
    
      // SPI4.2 Source status (LVTTL) input
      .SPI4_TSClk(SPI4_TSClk),
      .SPI4_TStat(SPI4_TStat),

      // SPI3 Clocks - All four SPI3 interfaces share common Rx, Tx Clocks.
      .SPI3_RFClk(SPI3_RFClk),

      // SPI3 0 Sink Interface
      .SPI3_0_RVal(SPI3_0_RVal),
      .SPI3_0_REnb(SPI3_0_REnb_n),
      .SPI3_0_RSx(SPI3_0_RSx),
      .SPI3_0_RSOP(SPI3_0_RSOP),
      .SPI3_0_REOP(SPI3_0_REOP),
      .SPI3_0_RDat(SPI3_0_RDat),
      .SPI3_0_RPrty(SPI3_0_RPrty),
      .SPI3_0_RMod(SPI3_0_RMod),
      .SPI3_0_RErr(SPI3_0_RErr),

      // SPI3 0 Source Interface
      .SPI3_TFClk(SPI3_TFClk),
      .SPI3_0_TEnb(SPI3_0_TEnb_n),
      .SPI3_0_TSx(SPI3_0_TSx),
      .SPI3_0_TSOP(SPI3_0_TSOP),
      .SPI3_0_TEOP(SPI3_0_TEOP),
      .SPI3_0_TDat(SPI3_0_TDat),
      .SPI3_0_TPrty(SPI3_0_TPrty),
      .SPI3_0_TMod(SPI3_0_TMod),
      .SPI3_0_TErr(SPI3_0_TErr),
      .SPI3_0_PTPA(SPI3_0_PTPA),
      .SPI3_0_TAdr(SPI3_0_TAdr),

      // SPI3 1 Sink Interface
      .SPI3_1_RVal(SPI3_1_RVal),
      .SPI3_1_REnb(SPI3_1_REnb_n),
      .SPI3_1_RSx(SPI3_1_RSx),
      .SPI3_1_RSOP(SPI3_1_RSOP),
      .SPI3_1_REOP(SPI3_1_REOP),
      .SPI3_1_RDat(SPI3_1_RDat),
      .SPI3_1_RPrty(SPI3_1_RPrty),
      .SPI3_1_RMod(SPI3_1_RMod),
      .SPI3_1_RErr(SPI3_1_RErr),

      // SPI3 1 Source Interface
      .SPI3_1_TEnb(SPI3_1_TEnb_n),
      .SPI3_1_TSx(SPI3_1_TSx),
      .SPI3_1_TSOP(SPI3_1_TSOP),
      .SPI3_1_TEOP(SPI3_1_TEOP),
      .SPI3_1_TDat(SPI3_1_TDat),
      .SPI3_1_TPrty (SPI3_1_TPrty),
      .SPI3_1_TMod(SPI3_1_TMod),
      .SPI3_1_TErr(SPI3_1_TErr),
      .SPI3_1_PTPA(SPI3_1_PTPA),
      .SPI3_1_TAdr(SPI3_1_TAdr),


      // SPI3 2 Sink Interface
      .SPI3_2_RVal(SPI3_2_RVal),
      .SPI3_2_REnb(SPI3_2_REnb_n),
      .SPI3_2_RSx(SPI3_2_RSx),
      .SPI3_2_RSOP(SPI3_2_RSOP),
      .SPI3_2_REOP(SPI3_2_REOP),
      .SPI3_2_RDat(SPI3_2_RDat),
      .SPI3_2_RPrty(SPI3_2_RPrty),
      .SPI3_2_RMod(SPI3_2_RMod),
      .SPI3_2_RErr(SPI3_2_RErr),

      // SPI3 2 Source Interface
      .SPI3_2_TEnb(SPI3_2_TEnb_n),
      .SPI3_2_TSx(SPI3_2_TSx),
      .SPI3_2_TSOP(SPI3_2_TSOP),
      .SPI3_2_TEOP(SPI3_2_TEOP),
      .SPI3_2_TDat(SPI3_2_TDat),
      .SPI3_2_TPrty(SPI3_2_TPrty),
      .SPI3_2_TMod(SPI3_2_TMod),
      .SPI3_2_TErr(SPI3_2_TErr),
      .SPI3_2_PTPA(SPI3_2_PTPA),
      .SPI3_2_TAdr(SPI3_2_TAdr),
    
      // SPI3 3 Sink Interface
      .SPI3_3_RVal(SPI3_3_RVal),
      .SPI3_3_REnb(SPI3_3_REnb_n),
      .SPI3_3_RSx(SPI3_3_RSx),
      .SPI3_3_RSOP(SPI3_3_RSOP),
      .SPI3_3_REOP(SPI3_3_REOP),
      .SPI3_3_RDat(SPI3_3_RDat),
      .SPI3_3_RPrty(SPI3_3_RPrty),
      .SPI3_3_RMod(SPI3_3_RMod),
      .SPI3_3_RErr(SPI3_3_RErr),

      // SPI3 3 Source Interface
      .SPI3_3_TEnb(SPI3_3_TEnb_n),
      .SPI3_3_TSx(SPI3_3_TSx),
      .SPI3_3_TSOP(SPI3_3_TSOP),
      .SPI3_3_TEOP(SPI3_3_TEOP),
      .SPI3_3_TDat(SPI3_3_TDat),
      .SPI3_3_TPrty(SPI3_3_TPrty),
      .SPI3_3_TMod(SPI3_3_TMod),
      .SPI3_3_TErr(SPI3_3_TErr),
      .SPI3_3_PTPA(SPI3_3_PTPA),
      .SPI3_3_TAdr(SPI3_3_TAdr),
      .SPI3_ParityErr(SPI3_ParityErr)
    );

  // Loopback the SPI4 interfaces
  assign SPI4_RDClk_P = SPI4_TDClk_P;
  assign SPI4_RDClk_N = SPI4_TDClk_N;
  assign SPI4_RCtl_P  = SPI4_TCtl_P;
  assign SPI4_RCtl_N  = SPI4_TCtl_N;
  assign SPI4_RDat_P  = SPI4_TDat_P;
  assign SPI4_RDat_N  = SPI4_TDat_N;

  assign SPI4_TSClk   = SPI4_RSClk;
  assign SPI4_TStat   = SPI4_RStat;

  // SPI3 Emulator #0
  spi3_emulator_phy spi3_emulator_phy0 (
      .SPI3_EMUL_RST_n(EmReset_n),

      .SPI3_RFClk(SPI3_RFClk),
      .SPI3_RENB_n(SPI3_0_REnb_n),
      .SPI3_RDAT(SPI3_0_RDat),
      .SPI3_RSOP(SPI3_0_RSOP),
      .SPI3_REOP(SPI3_0_REOP),
      .SPI3_RMOD(SPI3_0_RMod),
      .SPI3_RPRTY(SPI3_0_RPrty),
      .SPI3_RERR(SPI3_0_RErr),
      .SPI3_RVAL(SPI3_0_RVal),
      .SPI3_RSX(SPI3_0_RSx),

      .SPI3_TFClk(SPI3_TFClk),
      .SPI3_TENB_n(SPI3_0_TEnb_n),
      .SPI3_TDAT(SPI3_0_TDat),
      .SPI3_TSOP(SPI3_0_TSOP),
      .SPI3_TEOP(SPI3_0_TEOP),
      .SPI3_TMOD(SPI3_0_TMod),
      .SPI3_TPRTY(SPI3_0_TPrty),
      .SPI3_TERR(SPI3_0_TErr),
      .SPI3_TSX(SPI3_0_TSx),
      .SPI3_PTPA(SPI3_0_PTPA),
      .SPI3_TADR(SPI3_0_TAdr)
      );
      
  // SPI3 Emulator #1
  spi3_emulator_phy spi3_emulator_phy1 (
      .SPI3_EMUL_RST_n(EmReset_n),

      .SPI3_RFClk(SPI3_RFClk),
      .SPI3_RENB_n(SPI3_1_REnb_n),
      .SPI3_RDAT(SPI3_1_RDat),
      .SPI3_RSOP(SPI3_1_RSOP),
      .SPI3_REOP(SPI3_1_REOP),
      .SPI3_RMOD(SPI3_1_RMod),
      .SPI3_RPRTY(SPI3_1_RPrty),
      .SPI3_RERR(SPI3_1_RErr),
      .SPI3_RVAL(SPI3_1_RVal),
      .SPI3_RSX(SPI3_1_RSx),

      .SPI3_TFClk(SPI3_TFClk),
      .SPI3_TENB_n(SPI3_1_TEnb_n),
      .SPI3_TDAT(SPI3_1_TDat),
      .SPI3_TSOP(SPI3_1_TSOP),
      .SPI3_TEOP(SPI3_1_TEOP),
      .SPI3_TMOD(SPI3_1_TMod),
      .SPI3_TPRTY(SPI3_1_TPrty),
      .SPI3_TERR(SPI3_1_TErr),
      .SPI3_TSX(SPI3_1_TSx),
      .SPI3_PTPA(SPI3_1_PTPA),
      .SPI3_TADR(SPI3_1_TAdr)
      );

  // SPI3 Emulator #2
  spi3_emulator_phy spi3_emulator_phy2 (
      .SPI3_EMUL_RST_n (EmReset_n),
      .SPI3_RFClk(SPI3_RFClk),
      .SPI3_RENB_n(SPI3_2_REnb_n),
      .SPI3_RDAT(SPI3_2_RDat),
      .SPI3_RSOP(SPI3_2_RSOP),
      .SPI3_REOP(SPI3_2_REOP),
      .SPI3_RMOD(SPI3_2_RMod),
      .SPI3_RPRTY(SPI3_2_RPrty),
      .SPI3_RERR(SPI3_2_RErr),
      .SPI3_RVAL(SPI3_2_RVal),
      .SPI3_RSX(SPI3_2_RSx),

      .SPI3_TFClk(SPI3_TFClk),
      .SPI3_TENB_n(SPI3_2_TEnb_n),
      .SPI3_TDAT(SPI3_2_TDat),
      .SPI3_TSOP(SPI3_2_TSOP),
      .SPI3_TEOP(SPI3_2_TEOP),
      .SPI3_TMOD(SPI3_2_TMod),
      .SPI3_TPRTY(SPI3_2_TPrty),
      .SPI3_TERR(SPI3_2_TErr),
      .SPI3_TSX(SPI3_2_TSx),
      .SPI3_PTPA(SPI3_2_PTPA),
      .SPI3_TADR(SPI3_2_TAdr)
      );

  // SPI3 Emulator #3
  spi3_emulator_phy spi3_emulator_phy3 (
      .SPI3_EMUL_RST_n(EmReset_n),

      .SPI3_RFClk(SPI3_RFClk),
      .SPI3_RENB_n(SPI3_3_REnb_n),
      .SPI3_RDAT(SPI3_3_RDat),
      .SPI3_RSOP(SPI3_3_RSOP),
      .SPI3_REOP(SPI3_3_REOP),
      .SPI3_RMOD(SPI3_3_RMod),
      .SPI3_RPRTY(SPI3_3_RPrty),
      .SPI3_RERR(SPI3_3_RErr),
      .SPI3_RVAL(SPI3_3_RVal),
      .SPI3_RSX(SPI3_3_RSx),

      .SPI3_TFClk(SPI3_TFClk),
      .SPI3_TENB_n(SPI3_3_TEnb_n),
      .SPI3_TDAT(SPI3_3_TDat),
      .SPI3_TSOP(SPI3_3_TSOP),
      .SPI3_TEOP(SPI3_3_TEOP),
      .SPI3_TMOD(SPI3_3_TMod),
      .SPI3_TPRTY(SPI3_3_TPrty),
      .SPI3_TERR(SPI3_3_TErr),
      .SPI3_TSX(SPI3_3_TSx),
      .SPI3_PTPA(SPI3_3_PTPA),
      .SPI3_TADR(SPI3_3_TAdr)
      );

  // Master Clocks  Instantiation
  master_clocks master_clocks0 (
      .SimulationComplete(SimulationComplete),    
      .SPI3_RFClk(SPI3_RFClk),
      .SPI3_TFClk(SPI3_TFClk),
      .SPI4_SysClk_P(SPI4_SysClk_P),
      .SPI4_SysClk_N(SPI4_SysClk_N),
	.CLK200(CLK200)
      );

  // Main Simulation Controller process
  always
   begin
     // Reset the Emulators
    EmReset_n          <= 0;

    // Reset the Bridges
    BridgeReset_n      <= 0;

    // Clear SimulationComplete
    SimulationComplete <= 0;

     #500  //wait for 500 ns

    BridgeReset_n      <= 1;

    #1000    //wait for 1000 ns - Allow the Bridge DCMs to lock.

    // Finally release the reset for the SPI3 Emulators.
    EmReset_n          <= 1;

    // Run the simulation for a while
    #10000	//wait for 10 us
    
    // End the simulation.
    SimulationComplete <= 1;
   end 

  
endmodule
