-------------------------------------------------------------------------------
--
--   File Name:  master_clocks.vhd
--     Version:  1.0
--        Date:  April 11, 2003
--
-- Description:  This file provides the external clocks necessary to drive the
--               SPI3 to SPI4 and SPI4 to SPI3 bridges.
--
--      Author:  IP Solutions Division
--     Company:  Xilinx Inc.
--
--
--     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
--     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
--     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
--     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
--     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
--     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
--     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
--     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
--     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
--     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
--     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
--     FOR A PARTICULAR PURPOSE.
--
--     (c) Copyright 2003 Xilinx, Inc.
--     All rights reserved.
--
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- Library Declarations
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

-------------------------------------------------------------------------------
-- Entity Declaration
-------------------------------------------------------------------------------
entity master_clocks is
  port (
    SimulationComplete : in  std_logic;
    
    SPI3_RFClk         : out std_logic;
    SPI3_TFClk         : out std_logic;
    Ref_Clk		   : out std_logic;
    SPI4_SysClk_P      : out std_logic;
    SPI4_SysClk_N      : out std_logic

    );
end master_clocks;

-------------------------------------------------------------------------------
-- Architecture Declaration
-------------------------------------------------------------------------------
architecture behavioral of master_clocks is

-------------------------------------------------------------------------------
-- Constant Declarations
-------------------------------------------------------------------------------
  constant SPI3_RFCLK_PERIOD  : time      := 9.615 ns;  -- 104 MHz
  constant SPI3_TFCLK_PERIOD  : time      := 9.615 ns;  -- 104 MHz
  constant SPI4_SYSCLK_PERIOD : time      := 2.857 ns;  -- 350 MHz
  constant SPI4_REFCLK_PERIOD : time      := 5 ns;  -- 200 MHz

-------------------------------------------------------------------------------
-- Signal Declarations
-------------------------------------------------------------------------------
  signal spi3_rfclk_reg       : std_logic := '0';
  signal spi3_tfclk_reg       : std_logic := '0';
  signal spi4_sysclk          : std_logic := '0';
  signal Ref_Clk_reg		  : std_logic := '0';
begin  -- behavioral


-------------------------------------------------------------------------------
-- Generate SPI4 Reference Clock(s) 
-------------------------------------------------------------------------------
  SPI4_RFClk : process
  begin
    if (SimulationComplete = '0') then
      Ref_Clk_reg <= '0';
      wait for SPI4_REFCLK_PERIOD/2;
      Ref_Clk_reg <= '1';
      wait for SPI4_REFCLK_PERIOD/2;
    elsif (SimulationComplete = '1') then
      Ref_Clk_reg <= '0';
      wait;
    end if;
  end process SPI4_RFClk;
  
  Ref_Clk <= Ref_Clk_reg;

-------------------------------------------------------------------------------
-- Generate SPI3 RX Reference Clock(s) 
-------------------------------------------------------------------------------
  SPI3_RFClk_Proc : process
  begin
    if (SimulationComplete = '0') then
      spi3_rfclk_reg <= '0';
      wait for SPI3_RFCLK_PERIOD/2;
      spi3_rfclk_reg <= '1';
      wait for SPI3_RFCLK_PERIOD/2;
    elsif (SimulationComplete = '1') then
      spi3_rfclk_reg <= '0';
      wait;
    end if;
  end process SPI3_RFClk_Proc;
  
  SPI3_RFClk <= spi3_rfclk_reg;


-------------------------------------------------------------------------------
-- Generate SPI3 TX Reference Clock(s) 
------------------------------------------------------------------------------- 
  SPI3_TFClk_Proc : process
  begin
    if (SimulationComplete = '0') then
      spi3_tfclk_reg <= '0';
      wait for SPI3_TFCLK_PERIOD/2;
      spi3_tfclk_reg <= '1';
      wait for SPI3_TFCLK_PERIOD/2;
    elsif (SimulationComplete = '1') then
      spi3_tfclk_reg <= '0';
      wait;
    end if;
  end process SPI3_TFClk_Proc;

  SPI3_TFClk <= spi3_tfclk_reg;

-------------------------------------------------------------------------------
-- Generate SPI4 Source Core Reference Clock
-------------------------------------------------------------------------------
  SPI4_SysClk_Gen : process
  begin
    if (SimulationComplete = '0') then
      spi4_sysclk <= '0';
      wait for SPI4_SYSCLK_PERIOD/2;
      spi4_sysclk <= '1';
      wait for SPI4_SYSCLK_PERIOD/2;
    elsif (SimulationComplete = '1') then
      spi4_sysclk <= '0';
      wait;
    end if;
  end process SPI4_SysClk_Gen;
  SPI4_SysClk_P <= spi4_sysclk;
  SPI4_SysClk_N <= not spi4_sysclk;
  
end behavioral;
