-------------------------------------------------------------------------------
--
--   File Name:  spi4_to_4spi3_tb.vhd
--     Version:  1.0
--        Date:  April 11, 2003
--
-- Description:  This file instantiates the two bridge directions (SPI3 to
--               SPI4.2, and SPI4.2 to SPI3), as well as the test environment.
--               The testbench is comprised of four PL3 PHY emulators, which
--               source data into the four PL3 interfaces. The SPI4.2 interface
--               loops data from the SPI4.2 source to the SPI4.2 sink, and
--               loops status in the opposite direction.
--
--      Author:  IP Solutions Division
--     Company:  Xilinx Inc.
--
--
--     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
--     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
--     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
--     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
--     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
--     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
--     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
--     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
--     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
--     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
--     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
--     FOR A PARTICULAR PURPOSE.
--
--     (c) Copyright 2003 Xilinx, Inc.
--     All rights reserved.
--
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- Library Declarations
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

library unisim;
use unisim.vcomponents.all;

-------------------------------------------------------------------------------
-- Entity Declaration
-------------------------------------------------------------------------------

entity spi4_to_4spi3_tb is
end spi4_to_4spi3_tb;

-------------------------------------------------------------------------------
-- spi4_to_4spi3_tb Architecture
-------------------------------------------------------------------------------
architecture demo of spi4_to_4spi3_tb is

  -- Combined SPI4 to SPI3 and SPI3 to SPI4 Bridge Top Level File
  component bridge_top
    port (
      -- System signals
      Reset_n                : in  std_logic;
      Ref_Clk			  : in  std_logic;

      -- SPI4.2 Sink signals
      SPI4_RDClk_P           : in  std_logic;
      SPI4_RDClk_N           : in  std_logic;
      SPI4_RCtl_P            : in  std_logic;
      SPI4_RCtl_N            : in  std_logic;
      SPI4_RDat_P            : in  std_logic_vector(15 downto 0);
      SPI4_RDat_N            : in  std_logic_vector(15 downto 0);

      -- SPI4.2 Sink Status (LVTTL) input
      SPI4_RSClk             : out std_logic;
      SPI4_RStat             : out std_logic_vector(1 downto 0);

      -- SPI4.2 Source signals
      SPI4_TDClk_P           : out std_logic;
      SPI4_TDClk_N           : out std_logic;
      SPI4_TCtl_P            : out std_logic;
      SPI4_TCtl_N            : out std_logic;
      SPI4_TDat_P            : out std_logic_vector(15 downto 0);
      SPI4_TDat_N            : out std_logic_vector(15 downto 0);
      SPI4_SysClk_P          : in  std_logic;
      SPI4_SysClk_N          : in  std_logic;
    
      -- SPI4.2 Source status (LVTTL) input
      SPI4_TSClk             : in  std_logic;
      SPI4_TStat             : in  std_logic_vector(1 downto 0);

      -- SPI3 Clocks - All four SPI3 interfaces share common Rx, Tx Clocks.
      SPI3_RFClk             : in  std_logic;

      -- SPI3 0 Sink Interface
      SPI3_0_RVal            : in  std_logic;
      SPI3_0_REnb          : out std_logic;
      SPI3_0_RSx             : in  std_logic;
      SPI3_0_RSOP            : in  std_logic;
      SPI3_0_REOP            : in  std_logic;
      SPI3_0_RDat            : in  std_logic_vector(31 downto 0);
      SPI3_0_RPrty           : in  std_logic;
      SPI3_0_RMod            : in  std_logic_vector(1 downto 0);
      SPI3_0_RErr            : in  std_logic;

      -- SPI3 0 Source Interface
      SPI3_TFClk             : in  std_logic;
      SPI3_0_TEnb          : out std_logic;
      SPI3_0_TSx             : out std_logic;
      SPI3_0_TSOP            : out std_logic;
      SPI3_0_TEOP            : out std_logic;
      SPI3_0_TDat            : out std_logic_vector(31 downto 0);
      SPI3_0_TPrty           : out std_logic;
      SPI3_0_TMod            : out std_logic_vector(1 downto 0);
      SPI3_0_TErr            : out std_logic;
      SPI3_0_PTPA            : in  std_logic;
      SPI3_0_TAdr            : out std_logic_vector(7 downto 0);

      -- SPI3 1 Sink Interface
      SPI3_1_RVal            : in  std_logic;
      SPI3_1_REnb          : out std_logic;
      SPI3_1_RSx             : in  std_logic;
      SPI3_1_RSOP            : in  std_logic;
      SPI3_1_REOP            : in  std_logic;
      SPI3_1_RDat            : in  std_logic_vector(31 downto 0);
      SPI3_1_RPrty           : in  std_logic;
      SPI3_1_RMod            : in  std_logic_vector(1 downto 0);
      SPI3_1_RErr            : in  std_logic;

      -- SPI3 1 Source Interface
      SPI3_1_TEnb          : out std_logic;
      SPI3_1_TSx             : out std_logic;
      SPI3_1_TSOP            : out std_logic;
      SPI3_1_TEOP            : out std_logic;
      SPI3_1_TDat            : out std_logic_vector(31 downto 0);
      SPI3_1_TPrty           : out std_logic;
      SPI3_1_TMod            : out std_logic_vector(1 downto 0);
      SPI3_1_TErr            : out std_logic;
      SPI3_1_PTPA            : in  std_logic;
      SPI3_1_TAdr            : out std_logic_vector(7 downto 0);

      -- SPI3 2 Sink Interface
      SPI3_2_RVal            : in  std_logic;
      SPI3_2_REnb          : out std_logic;
      SPI3_2_RSx             : in  std_logic;
      SPI3_2_RSOP            : in  std_logic;
      SPI3_2_REOP            : in  std_logic;
      SPI3_2_RDat            : in  std_logic_vector(31 downto 0);
      SPI3_2_RPrty           : in  std_logic;
      SPI3_2_RMod            : in  std_logic_vector(1 downto 0);
      SPI3_2_RErr            : in  std_logic;

      -- SPI3 2 Source Interface
      SPI3_2_TEnb            : out std_logic;
      SPI3_2_TSx             : out std_logic;
      SPI3_2_TSOP            : out std_logic;
      SPI3_2_TEOP            : out std_logic;
      SPI3_2_TDat            : out std_logic_vector(31 downto 0);
      SPI3_2_TPrty           : out std_logic;
      SPI3_2_TMod            : out std_logic_vector(1 downto 0);
      SPI3_2_TErr            : out std_logic;
      SPI3_2_PTPA            : in  std_logic;
      SPI3_2_TAdr            : out std_logic_vector(7 downto 0);
    
      -- SPI3 3 Sink Interface
      SPI3_3_RVal            : in  std_logic;
      SPI3_3_REnb          : out std_logic;
      SPI3_3_RSx             : in  std_logic;
      SPI3_3_RSOP            : in  std_logic;
      SPI3_3_REOP            : in  std_logic;
      SPI3_3_RDat            : in  std_logic_vector(31 downto 0);
      SPI3_3_RPrty           : in  std_logic;
      SPI3_3_RMod            : in  std_logic_vector(1 downto 0);
      SPI3_3_RErr            : in  std_logic;

      -- SPI3 3 Source Interface
      SPI3_3_TEnb          : out std_logic;
      SPI3_3_TSx             : out std_logic;
      SPI3_3_TSOP            : out std_logic;
      SPI3_3_TEOP            : out std_logic;
      SPI3_3_TDat            : out std_logic_vector(31 downto 0);
      SPI3_3_TPrty           : out std_logic;
      SPI3_3_TMod            : out std_logic_vector(1 downto 0);
      SPI3_3_TErr            : out std_logic;
      SPI3_3_PTPA            : in  std_logic;
      SPI3_3_TAdr            : out std_logic_vector(7 downto 0);
    
      -- Misc (can be removed, was added only to keep the signal from being
      -- optimized)
      SPI3_ParityErr         : out std_logic
    );
  end component;

  -- SPI3 PHY Emulator Component Instantiation
  component spi3_emulator_phy
    generic (
      DataOffset      : integer
      );
    port (
      SPI3_emul_rst_n : in std_logic;

      SPI3_RFClk      : in std_logic;
      SPI3_REnb_n     : in std_logic;
      SPI3_RDat       : out std_logic_vector(31 downto 0);
      SPI3_RSOP       : out std_logic;
      SPI3_REOP       : out std_logic;
      SPI3_RMod       : out std_logic_vector(1 downto 0);
      SPI3_RPrty      : out std_logic;
      SPI3_RErr       : out std_logic;
      SPI3_RVal       : out std_logic;
      SPI3_RSx        : out std_logic;

      SPI3_TFClk      : in std_logic;
      SPI3_TEnb_n     : in std_logic;
      SPI3_TDat       : in std_logic_vector(31 downto 0);
      SPI3_TSOP       : in std_logic;
      SPI3_TEOP       : in std_logic;
      SPI3_TMod       : in std_logic_vector(1 downto 0);
      SPI3_TPrty      : in std_logic;
      SPI3_TErr       : in std_logic;
      SPI3_TSx        : in std_logic;
      SPI3_PTPA       : out std_logic;
      SPI3_TAdr       : in std_logic_vector(7 downto 0)
      );
  end component;

  -- Master Clocks Component Declaration
  component master_clocks
    port (
      SimulationComplete : in  std_logic;
    
      SPI3_RFClk         : out std_logic;
      SPI3_TFClk         : out std_logic;
      Ref_Clk		    : out std_logic;
      SPI4_SysClk_P      : out std_logic;
      SPI4_SysClk_N      : out std_logic

      );
  end component;

  -----------------------------------------------------------------------------
  -- Signal Declarations
  -----------------------------------------------------------------------------
  signal EmReset_n           : std_logic;
  signal BridgeReset_n       : std_logic;
  signal Ref_Clk				  : std_logic;
  -- SPI3 Sink Interfaces
  signal SPI3_RFClk          : std_logic;
  signal SPI3_0_RVal         : std_logic;
  signal SPI3_0_REnb_n       : std_logic;
  signal SPI3_0_REnb         : std_logic;
  signal SPI3_0_RSx          : std_logic;
  signal SPI3_0_RSOP         : std_logic;
  signal SPI3_0_REOP         : std_logic;
  signal SPI3_0_RDat         : std_logic_vector(31 downto 0);
  signal SPI3_0_RPrty        : std_logic;
  signal SPI3_0_RMod         : std_logic_vector(1 downto 0);
  signal SPI3_0_RErr         : std_logic;
  signal SPI3_1_RVal         : std_logic;
  signal SPI3_1_REnb_n       : std_logic;
  signal SPI3_1_REnb         : std_logic;
  signal SPI3_1_RSx          : std_logic;
  signal SPI3_1_RSOP         : std_logic;
  signal SPI3_1_REOP         : std_logic;
  signal SPI3_1_RDat         : std_logic_vector(31 downto 0);
  signal SPI3_1_RPrty        : std_logic;
  signal SPI3_1_RMod         : std_logic_vector(1 downto 0);
  signal SPI3_1_RErr         : std_logic;
  signal SPI3_2_RVal         : std_logic;
  signal SPI3_2_REnb         : std_logic;
  signal SPI3_2_REnb_n       : std_logic;
  signal SPI3_2_RSx          : std_logic;
  signal SPI3_2_RSOP         : std_logic;
  signal SPI3_2_REOP         : std_logic;
  signal SPI3_2_RDat         : std_logic_vector(31 downto 0);
  signal SPI3_2_RPrty        : std_logic;
  signal SPI3_2_RMod         : std_logic_vector(1 downto 0);
  signal SPI3_2_RErr         : std_logic;
  signal SPI3_3_RVal         : std_logic;
  signal SPI3_3_REnb         : std_logic;
  signal SPI3_3_REnb_n       : std_logic;
  signal SPI3_3_RSx          : std_logic;
  signal SPI3_3_RSOP         : std_logic;
  signal SPI3_3_REOP         : std_logic;
  signal SPI3_3_RDat         : std_logic_vector(31 downto 0);
  signal SPI3_3_RPrty        : std_logic;
  signal SPI3_3_RMod         : std_logic_vector(1 downto 0);
  signal SPI3_3_RErr         : std_logic;
  signal SPI3_ParityErr      : std_logic;
  
  -- SPI4 Source/Sink Interfaces
  signal SPI4_SysClk_P        : std_logic;
  signal SPI4_SysClk_N        : std_logic;
  signal SPI4_TDClk_P         : std_logic;
  signal SPI4_TDClk_N         : std_logic;
  signal SPI4_TCtl_P          : std_logic;
  signal SPI4_TCtl_N          : std_logic;
  signal SPI4_TDat_P          : std_logic_vector(15 downto 0);
  signal SPI4_TDat_N          : std_logic_vector(15 downto 0);
  signal SPI4_TSClk           : std_logic;
  signal SPI4_TStat           : std_logic_vector(1 downto 0);
  signal SPI4_RDClk_P         : std_logic;
  signal SPI4_RDClk_N         : std_logic;
  signal SPI4_RCtl_P          : std_logic;
  signal SPI4_RCtl_N          : std_logic;
  signal SPI4_RDat_P          : std_logic_vector(15 downto 0);
  signal SPI4_RDat_N          : std_logic_vector(15 downto 0);
  signal SPI4_RSClk           : std_logic;
  signal SPI4_RStat           : std_logic_vector(1 downto 0);
  
  -- SPI3 Source Interfaces
  signal SPI3_TFClk          : std_logic;
  signal SPI3_0_TEnb         : std_logic;
  signal SPI3_0_TEnb_n       : std_logic;
  signal SPI3_0_TSx          : std_logic;
  signal SPI3_0_TSOP         : std_logic;
  signal SPI3_0_TEOP         : std_logic;
  signal SPI3_0_TDat         : std_logic_vector(31 downto 0);
  signal SPI3_0_TPrty        : std_logic;
  signal SPI3_0_TMod         : std_logic_vector(1 downto 0);
  signal SPI3_0_TErr         : std_logic;
  signal SPI3_0_PTPA         : std_logic;
  signal SPI3_0_TAdr         : std_logic_vector(7 downto 0);
  signal SPI3_1_TEnb         : std_logic;
  signal SPI3_1_TEnb_n       : std_logic;
  signal SPI3_1_TSx          : std_logic;
  signal SPI3_1_TSOP         : std_logic;
  signal SPI3_1_TEOP         : std_logic;
  signal SPI3_1_TDat         : std_logic_vector(31 downto 0);
  signal SPI3_1_TPrty        : std_logic;
  signal SPI3_1_TMod         : std_logic_vector(1 downto 0);
  signal SPI3_1_TErr         : std_logic;
  signal SPI3_1_PTPA         : std_logic;
  signal SPI3_1_TAdr         : std_logic_vector(7 downto 0);
  signal SPI3_2_TEnb         : std_logic;
  signal SPI3_2_TEnb_n       : std_logic;
  signal SPI3_2_TSx          : std_logic;
  signal SPI3_2_TSOP         : std_logic;
  signal SPI3_2_TEOP         : std_logic;
  signal SPI3_2_TDat         : std_logic_vector(31 downto 0);
  signal SPI3_2_TPrty        : std_logic;
  signal SPI3_2_TMod         : std_logic_vector(1 downto 0);
  signal SPI3_2_TErr         : std_logic;
  signal SPI3_2_PTPA         : std_logic;
  signal SPI3_2_TAdr         : std_logic_vector(7 downto 0);
  signal SPI3_3_TEnb         : std_logic;
  signal SPI3_3_TEnb_n       : std_logic;
  signal SPI3_3_TSx          : std_logic;
  signal SPI3_3_TSOP         : std_logic;
  signal SPI3_3_TEOP         : std_logic;
  signal SPI3_3_TDat         : std_logic_vector(31 downto 0);
  signal SPI3_3_TPrty        : std_logic;
  signal SPI3_3_TMod         : std_logic_vector(1 downto 0);
  signal SPI3_3_TErr         : std_logic;
  signal SPI3_3_PTPA         : std_logic;
  signal SPI3_3_TAdr         : std_logic_vector(7 downto 0);

  signal SimulationComplete  : std_logic := '0';
  
  begin

  SPI3_0_REnb_n <=  SPI3_0_REnb;
  SPI3_1_REnb_n <=  SPI3_1_REnb;
  SPI3_2_REnb_n <=  SPI3_2_REnb;
  SPI3_3_REnb_n <=  SPI3_3_REnb;
  SPI3_0_TEnb_n <=  SPI3_0_TEnb;
  SPI3_1_TEnb_n <=  SPI3_1_TEnb;
  SPI3_2_TEnb_n <=  SPI3_2_TEnb;
  SPI3_3_TEnb_n <=  SPI3_3_TEnb;
    

  -- Combined SPI4 to SPI3 and SPI3 to SPI4 Bridge Top Level File
  dut : bridge_top
    port map (
      -- System signals
      Reset_n                => BridgeReset_n,
      Ref_CLK                => Ref_CLk,

      -- SPI4.2 Sink signals
      SPI4_RDClk_P           => SPI4_RDClk_P,
      SPI4_RDClk_N           => SPI4_RDClk_N,
      SPI4_RCtl_P            => SPI4_RCtl_P,
      SPI4_RCtl_N            => SPI4_RCtl_N,
      SPI4_RDat_P            => SPI4_RDat_P,
      SPI4_RDat_N            => SPI4_RDat_N,

      -- SPI4.2 Sink Status (LVTTL) input
      SPI4_RSClk             => SPI4_RSClk,
      SPI4_RStat             => SPI4_RStat,

      -- SPI4.2 Source signals
      SPI4_TDClk_P           => SPI4_TDClk_P,
      SPI4_TDClk_N           => SPI4_TDClk_N,
      SPI4_TCtl_P            => SPI4_TCtl_P,
      SPI4_TCtl_N            => SPI4_TCtl_N,
      SPI4_TDat_P            => SPI4_TDat_P,
      SPI4_TDat_N            => SPI4_TDat_N,
      SPI4_SysClk_P          => SPI4_SysClk_P,
      SPI4_SysClk_N          => SPI4_SysClk_N,
    
      -- SPI4.2 Source status (LVTTL) input
      SPI4_TSClk             => SPI4_TSClk,
      SPI4_TStat             => SPI4_TStat,

      -- SPI3 Clocks - All four SPI3 interfaces share common Rx, Tx Clocks.
      SPI3_RFClk             => SPI3_RFClk,

      -- SPI3 0 Sink Interface
      SPI3_0_RVal            => SPI3_0_RVal,
      SPI3_0_REnb            => SPI3_0_REnb,
      SPI3_0_RSx             => SPI3_0_RSx,
      SPI3_0_RSOP            => SPI3_0_RSOP,
      SPI3_0_REOP            => SPI3_0_REOP,
      SPI3_0_RDat            => SPI3_0_RDat,
      SPI3_0_RPrty           => SPI3_0_RPrty,
      SPI3_0_RMod            => SPI3_0_RMod,
      SPI3_0_RErr            => SPI3_0_RErr,

      -- SPI3 0 Source Interface
      SPI3_TFClk             => SPI3_TFClk,
      SPI3_0_TEnb            => SPI3_0_TEnb,
      SPI3_0_TSx             => SPI3_0_TSx,
      SPI3_0_TSOP            => SPI3_0_TSOP,
      SPI3_0_TEOP            => SPI3_0_TEOP,
      SPI3_0_TDat            => SPI3_0_TDat,
      SPI3_0_TPrty           => SPI3_0_TPrty,
      SPI3_0_TMod            => SPI3_0_TMod,
      SPI3_0_TErr            => SPI3_0_TErr,
      SPI3_0_PTPA            => SPI3_0_PTPA,
      SPI3_0_TAdr            => SPI3_0_TAdr,

      -- SPI3 1 Sink Interface
      SPI3_1_RVal            => SPI3_1_RVal,
      SPI3_1_REnb            => SPI3_1_REnb,
      SPI3_1_RSx             => SPI3_1_RSx,
      SPI3_1_RSOP            => SPI3_1_RSOP,
      SPI3_1_REOP            => SPI3_1_REOP,
      SPI3_1_RDat            => SPI3_1_RDat,
      SPI3_1_RPrty           => SPI3_1_RPrty,
      SPI3_1_RMod            => SPI3_1_RMod,
      SPI3_1_RErr            => SPI3_1_RErr,

      -- SPI3 1 Source Interface
      SPI3_1_TEnb            => SPI3_1_TEnb,
      SPI3_1_TSx             => SPI3_1_TSx,
      SPI3_1_TSOP            => SPI3_1_TSOP,
      SPI3_1_TEOP            => SPI3_1_TEOP,
      SPI3_1_TDat            => SPI3_1_TDat,
      SPI3_1_TPrty           => SPI3_1_TPrty,
      SPI3_1_TMod            => SPI3_1_TMod,
      SPI3_1_TErr            => SPI3_1_TErr,
      SPI3_1_PTPA            => SPI3_1_PTPA,
      SPI3_1_TAdr            => SPI3_1_TAdr,


      -- SPI3 2 Sink Interface
      SPI3_2_RVal            => SPI3_2_RVal,
      SPI3_2_REnb            => SPI3_2_REnb,
      SPI3_2_RSx             => SPI3_2_RSx,
      SPI3_2_RSOP            => SPI3_2_RSOP,
      SPI3_2_REOP            => SPI3_2_REOP,
      SPI3_2_RDat            => SPI3_2_RDat,
      SPI3_2_RPrty           => SPI3_2_RPrty,
      SPI3_2_RMod            => SPI3_2_RMod,
      SPI3_2_RErr            => SPI3_2_RErr,

      -- SPI3 2 Source Interface
      SPI3_2_TEnb           => SPI3_2_TEnb,
      SPI3_2_TSx             => SPI3_2_TSx,
      SPI3_2_TSOP            => SPI3_2_TSOP,
      SPI3_2_TEOP            => SPI3_2_TEOP,
      SPI3_2_TDat            => SPI3_2_TDat,
      SPI3_2_TPrty           => SPI3_2_TPrty,
      SPI3_2_TMod            => SPI3_2_TMod,
      SPI3_2_TErr            => SPI3_2_TErr,
      SPI3_2_PTPA            => SPI3_2_PTPA,
      SPI3_2_TAdr            => SPI3_2_TAdr,
    
      -- SPI3 3 Sink Interface
      SPI3_3_RVal            => SPI3_3_RVal,
      SPI3_3_REnb            => SPI3_3_REnb,
      SPI3_3_RSx             => SPI3_3_RSx,
      SPI3_3_RSOP            => SPI3_3_RSOP,
      SPI3_3_REOP            => SPI3_3_REOP,
      SPI3_3_RDat            => SPI3_3_RDat,
      SPI3_3_RPrty           => SPI3_3_RPrty,
      SPI3_3_RMod            => SPI3_3_RMod,
      SPI3_3_RErr            => SPI3_3_RErr,

      -- SPI3 3 Source Interface
      SPI3_3_TEnb            => SPI3_3_TEnb,
      SPI3_3_TSx             => SPI3_3_TSx,
      SPI3_3_TSOP            => SPI3_3_TSOP,
      SPI3_3_TEOP            => SPI3_3_TEOP,
      SPI3_3_TDat            => SPI3_3_TDat,
      SPI3_3_TPrty           => SPI3_3_TPrty,
      SPI3_3_TMod            => SPI3_3_TMod,
      SPI3_3_TErr            => SPI3_3_TErr,
      SPI3_3_PTPA            => SPI3_3_PTPA,
      SPI3_3_TAdr            => SPI3_3_TAdr,
      SPI3_ParityErr         => SPI3_ParityErr
    );

  -- Loopback the SPI4 interfaces
  SPI4_RDClk_P <= SPI4_TDClk_P;
  SPI4_RDClk_N <= SPI4_TDClk_N;
  SPI4_RCtl_P  <= SPI4_TCtl_P;
  SPI4_RCtl_N  <= SPI4_TCtl_N;
  SPI4_RDat_P  <= SPI4_TDat_P;
  SPI4_RDat_N  <= SPI4_TDat_N;

  SPI4_TSClk   <= SPI4_RSClk;
  SPI4_TStat   <= SPI4_RStat;

  -- SPI3 Emulator #0
  spi3_emulator_phy0 : spi3_emulator_phy
    generic map (
      DataOffset      => 0
      )
    port map (
      SPI3_emul_rst_n => EmReset_n,

      SPI3_RFClk      => SPI3_RFClk,
      SPI3_REnb_n     => SPI3_0_REnb_n,
      SPI3_RDat       => SPI3_0_RDat,
      SPI3_RSOP       => SPI3_0_RSOP,
      SPI3_REOP       => SPI3_0_REOP,
      SPI3_RMod       => SPI3_0_RMod,
      SPI3_RPrty      => SPI3_0_RPrty,
      SPI3_RErr       => SPI3_0_RErr,
      SPI3_RVal       => SPI3_0_RVal,
      SPI3_RSx        => SPI3_0_RSx,

      SPI3_TFClk      => SPI3_TFClk,
      SPI3_TEnb_n     => SPI3_0_TEnb_n,
      SPI3_TDat       => SPI3_0_TDat,
      SPI3_TSOP       => SPI3_0_TSOP,
      SPI3_TEOP       => SPI3_0_TEOP,
      SPI3_TMod       => SPI3_0_TMod,
      SPI3_TPrty      => SPI3_0_TPrty,
      SPI3_TErr       => SPI3_0_TErr,
      SPI3_TSx        => SPI3_0_TSx,
      SPI3_PTPA       => SPI3_0_PTPA,
      SPI3_TAdr       => SPI3_0_TAdr
      );
      
  -- SPI3 Emulator #1
  spi3_emulator_phy1 : spi3_emulator_phy
    generic map (
      DataOffset      => 10
      )
    port map (
      SPI3_emul_rst_n => EmReset_n,

      SPI3_RFClk      => SPI3_RFClk,
      SPI3_REnb_n     => SPI3_1_REnb_n,
      SPI3_RDat       => SPI3_1_RDat,
      SPI3_RSOP       => SPI3_1_RSOP,
      SPI3_REOP       => SPI3_1_REOP,
      SPI3_RMod       => SPI3_1_RMod,
      SPI3_RPrty      => SPI3_1_RPrty,
      SPI3_RErr       => SPI3_1_RErr,
      SPI3_RVal       => SPI3_1_RVal,
      SPI3_RSx        => SPI3_1_RSx,

      SPI3_TFClk      => SPI3_TFClk,
      SPI3_TEnb_n     => SPI3_1_TEnb_n,
      SPI3_TDat       => SPI3_1_TDat,
      SPI3_TSOP       => SPI3_1_TSOP,
      SPI3_TEOP       => SPI3_1_TEOP,
      SPI3_TMod       => SPI3_1_TMod,
      SPI3_TPrty      => SPI3_1_TPrty,
      SPI3_TErr       => SPI3_1_TErr,
      SPI3_TSx        => SPI3_1_TSx,
      SPI3_PTPA       => SPI3_1_PTPA,
      SPI3_TAdr       => SPI3_1_TAdr
      );

  -- SPI3 Emulator #2
  spi3_emulator_phy2 : spi3_emulator_phy
    generic map (
      DataOffset      => 20
      )
    port map (
      SPI3_emul_rst_n => EmReset_n,

      SPI3_RFClk      => SPI3_RFClk,
      SPI3_REnb_n     => SPI3_2_REnb_n,
      SPI3_RDat       => SPI3_2_RDat,
      SPI3_RSOP       => SPI3_2_RSOP,
      SPI3_REOP       => SPI3_2_REOP,
      SPI3_RMod       => SPI3_2_RMod,
      SPI3_RPrty      => SPI3_2_RPrty,
      SPI3_RErr       => SPI3_2_RErr,
      SPI3_RVal       => SPI3_2_RVal,
      SPI3_RSx        => SPI3_2_RSx,

      SPI3_TFClk      => SPI3_TFClk,
      SPI3_TEnb_n     => SPI3_2_TEnb_n,
      SPI3_TDat       => SPI3_2_TDat,
      SPI3_TSOP       => SPI3_2_TSOP,
      SPI3_TEOP       => SPI3_2_TEOP,
      SPI3_TMod       => SPI3_2_TMod,
      SPI3_TPrty      => SPI3_2_TPrty,
      SPI3_TErr       => SPI3_2_TErr,
      SPI3_TSx        => SPI3_2_TSx,
      SPI3_PTPA       => SPI3_2_PTPA,
      SPI3_TAdr       => SPI3_2_TAdr
      );

  -- SPI3 Emulator #3
  spi3_emulator_phy3 : spi3_emulator_phy
    generic map (
      DataOffset      => 30
      )
    port map (
      SPI3_emul_rst_n => EmReset_n,

      SPI3_RFClk      => SPI3_RFClk,
      SPI3_REnb_n     => SPI3_3_REnb_n,
      SPI3_RDat       => SPI3_3_RDat,
      SPI3_RSOP       => SPI3_3_RSOP,
      SPI3_REOP       => SPI3_3_REOP,
      SPI3_RMod       => SPI3_3_RMod,
      SPI3_RPrty      => SPI3_3_RPrty,
      SPI3_RErr       => SPI3_3_RErr,
      SPI3_RVal       => SPI3_3_RVal,
      SPI3_RSx        => SPI3_3_RSx,

      SPI3_TFClk      => SPI3_TFClk,
      SPI3_TEnb_n     => SPI3_3_TEnb_n,
      SPI3_TDat       => SPI3_3_TDat,
      SPI3_TSOP       => SPI3_3_TSOP,
      SPI3_TEOP       => SPI3_3_TEOP,
      SPI3_TMod       => SPI3_3_TMod,
      SPI3_TPrty      => SPI3_3_TPrty,
      SPI3_TErr       => SPI3_3_TErr,
      SPI3_TSx        => SPI3_3_TSx,
      SPI3_PTPA       => SPI3_3_PTPA,
      SPI3_TAdr       => SPI3_3_TAdr
      );

  -- Master Clocks  Instantiation
  master_clocks0 : master_clocks
    port map (
      SimulationComplete  => SimulationComplete,
    
      SPI3_RFClk           => SPI3_RFClk,
      SPI3_TFClk           => SPI3_TFClk,
      Ref_Clk					=> Ref_Clk,
      SPI4_SysClk_P        => SPI4_SysClk_P,
      SPI4_SysClk_N        => SPI4_SysClk_N
      );

  -- Main Simulation Controller process
  Stimulus_proc: process
  begin
    -- Reset the Emulators
    EmReset_n          <= '0';
    -- Reset the Bridges
    BridgeReset_n      <= '0';
    
    wait for 500 ns;

    BridgeReset_n      <= '1';

    wait for 1000 ns;                   -- Allow the Bridge DCMs to lock.

    -- Finally release the reset for the SPI3 Emulators.
    EmReset_n          <= '1';

    -- Run the simulation for a while
    wait for 10 us;
    
    -- End the simulation.
    SimulationComplete <= '1';
    wait;
  end process Stimulus_proc;
  
end demo;
